#!/bin/sh

version=3.0
revision=1

if [ ! -x $JAVA_HOME/bin/javac ]; then
    echo JAVA_HOME not set properly
    exit 1
fi
cd `dirname $0`/../../..
chmod +x ./Octopus-src/Makefile
(cd Octopus-src&&make clean)
cat >octopus.spec <<EOF
Summary: An ETL tool
Name: octopus
Version: $version
Release: $revision
URL: http://octopus.enhydra.org
Source: %{name}-%{version}-${revision}.src.tar.gz
License: GPL
Group: Development/Tools

AutoReqProv: no
BuildRoot: /tmp/%name-%version.build_root

# Relocatable version:
Prefix: /usr/local
Requires: rpm > 4.0.2

%description
Octopus is a simple Java-based Extraction, Transformation,
and Loading (ETL) tool.
It may connect any JDBC data source and perform a transformation
defined from an XML file.
Octopus supports Ant and JUnit.

%prep
%setup -q

%build
    while
        echo -e "\n\nPlease enter path to your "\
            "Java Development Kit - JDK directory [/usr/java/j2sdk1.4.1_01] \n\n"
        read jdkdir
        jdkdir=\${jdkdir:=/usr/java/j2sdk1.4.1_01}
        [ ! -d \$jdkdir ]
    do
        echo "Warning: No such directory: \$jdkdir"
    done
    JAVA_HOME=\${jdkdir}
    MAVEN_HOME=\${PWD}/maven
    export JAVA_HOME MAVEN_HOME
    export PATH=\$PATH:\$JAVA_HOME/bin
    chmod +x ./Octopus-src/configure
    cd Octopus-src
    ./configure -jdkhome \$JAVA_HOME -prefix %prefix
    make buildAll

%install
    rm -rf \$RPM_BUILD_ROOT
    mkdir -p \$RPM_BUILD_ROOT/usr/local/%{name}-%{version}
    cd Octopus-src/output
    cp -R %{name}-%{version} \$RPM_BUILD_ROOT/usr/local
    rm -f \$(find \$RPM_BUILD_ROOT/usr/local/%{name}-%{version} -type f -name '*.cmd')
    rm -f \$(find \$RPM_BUILD_ROOT/usr/local/%{name}-%{version} -type f -name '*.cmd.DIST')

%clean
rm -rf \$RPM_BUILD_ROOT

%pre
    rm -rf /usr/local/octopus-%version

%post
    RPM_INSTALL_PREFIX=\${RPM_INSTALL_PREFIX:=/usr/local}
    if
        [ -f \$RPM_INSTALL_PREFIX/build.properties.rpmorig ]
    then
        conf_file=\$RPM_INSTALL_PREFIX/build.properties.rpmorig
        default_jdk=\$(grep "jdk.dir=" \$conf_file)
        default_jdk=\$(echo \$default_jdk | sed 's/^jdk.dir=//')
    fi
    default_jdk=\${default_jdk:=/usr/java/j2sdk1.4.1_01}
    while
        echo -e "\n\nPlease enter path to your "\
            "Java Development Kit - JDK directory [\$default_jdk] \c"
        read jdkdir </dev/tty
        jdkdir=\${jdkdir:=\$default_jdk}
        [ ! -d \$jdkdir ]
    do
        echo "Warning: No such directory: \$jdkdir"
    done

    cd \$RPM_INSTALL_PREFIX/octopus-%version
    echo "jdk.dir=\$jdkdir" >configure.properties
    PATH=\$jdkdir/bin:\$PATH
    export PATH
    chmod +x configure
    if
        [ -x configure ]
    then
        ./configure
    fi

%files
%defattr(-,root,root)
/usr/local/%{name}-%{version}
EOF

if [ ! -w  ${RPM_ROOT:=/usr/src/redhat}/SOURCES ]; then
    echo Cannot write into $RPM_ROOT/SOURCES
    exit 1
fi
rm -fr distribution
mkdir -p distribution/Source
mkdir -p distribution/Unix
rm -fr /tmp/build_/octopus-$version
mkdir -p /tmp/build_/octopus-$version
cp -R . /tmp/build_/octopus-$version
rm -f ${RPM_ROOT}/SOURCES/octopus-$version-$revision.src.tar.gz
(cd /tmp/build_&&tar czf ${RPM_ROOT}/SOURCES/octopus-$version-$revision.src.tar.gz --exclude '*~' --exclude 'distribution' --exclude 'CVS' octopus-$version)



echo $JAVA_HOME|rpmbuild -ba --target noarch octopus.spec

mv ${RPM_ROOT}/SOURCES/octopus-${version}-${revision}.src.tar.gz distribution/Source
mv ${RPM_ROOT}/SRPMS/octopus-${version}-${revision}.src.rpm distribution/Source
mkdir -p tmp
cd tmp
rpm2cpio ${RPM_ROOT}/RPMS/noarch/octopus-${version}-${revision}.noarch.rpm|cpio -id

find . -type f  -exec chmod a+r {} \;
find . -type f -a \( -name '*.sh' -o -name 'configure*' \)  -exec chmod a+rx {} \;
find . -type d -a -exec chmod a+rx {} \;

tar czf ../distribution/Unix/octopus-${version}-${revision}.tar.gz . --exclude '*.cmd' --exclude 'configure.bat'
cd ..
rm -fr tmp
mv ${RPM_ROOT}/RPMS/noarch/octopus-${version}-${revision}.noarch.rpm distribution/Unix
