;-----------------------------------------------------------------------------------------------
; Octopus installation script
;	All output messages will be written to file octopus.log in project root directory
;-----------------------------------------------------------------------------------------------
!define NAME "Octopus"

#!define VERSION "2.3" 	;only for testing with makensisw.exe
#!define RELEASE "1" 		;only for testing with makensisw.exe
;--------------------------------
;Version Information
  VIProductVersion "${VERSION}.0.0"
;--------------------------------
Name "${NAME} ${VERSION}" ;Define your own software name here

!define MUI_ICON "${NSISDIR}\Contrib\Icons\octopus.ico"
!define MUI_UNICON "${NSISDIR}\Contrib\Icons\octopus-uninstall.ico"

!include "MUI.nsh"
;--------------------------------------------------------------------------------
;Configuration
;--------------------------------------------------------------------------------
; General

	OutFile "..\..\..\distribution\octopus-${VERSION}-${RELEASE}.setup.exe"	; The file to write

; Folder-selection page
  InstallDir "C:\octopus${VERSION}"

  ;Remember install folder
  InstallDirRegKey HKCU "Software\${NAME} ${VERSION}" ""

  !define MUI_HEADERIMAGE
    !define MUI_HEADERIMAGE_BITMAP "${NSISDIR}\Contrib\Icons\octopus.bmp"
    !define MUI_HEADERIMAGE_UNBITMAP "${NSISDIR}\Contrib\Icons\octopus.bmp"
  !define MUI_ABORTWARNING

  ;Remember the Start Menu Folder
  !define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKCU"
  !define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\${NAME} ${VERSION}"
  !define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"

#  ShowInstDetails show

; Compress
;------------
#SetCompress          auto
#SetCompressor        bzip2
#SetDatablockOptimize on
#SetDateSave          on

;--------------------------------
;Modern UI Configuration

  !define MUI_CUSTOMPAGECOMMANDS

  !define MUI_WELCOMEPAGE
  !define MUI_WELCOMEFINISHPAGE_BITMAP "${NSISDIR}\Contrib\Icons\octopus-wizard.bmp"
  !define MUI_LICENSEPAGE
  !define MUI_DIRECTORYPAGE
  !define MUI_STARTMENUPAGE
  !define MUI_FINISHPAGE
    !define MUI_FINISHPAGE_NOAUTOCLOSE
    !define MUI_FINISHPAGE_RUN "$INSTDIR/bin/OctopusGenerator.cmd"

	  !define MUI_FINISHPAGE_SHOWREADME
    !define MUI_FINISHPAGE_SHOWREADME_TEXT "Show Release Notes"
    !define MUI_FINISHPAGE_SHOWREADME_FUNCTION "RunReleaseNotes"

    !define MUI_FINISHPAGE_LINK "Visit Octopus home page"
    !define MUI_FINISHPAGE_LINK_LOCATION "http://octopus.enhydra.org/"

  !define MUI_UNINSTALLER
  !define MUI_UNCONFIRMPAGE

;--------------------------------
;Variables

  Var STARTMENU_FOLDER
  Var MUI_TEMP
  Var TEMP1
  Var JAVAHOME

;--------------------------------
;Languages
  !insertmacro MUI_LANGUAGE "English"
   BrandingText "Octopus"

   LicenseLangString license_text ${LANG_ENGLISH} "${NSISDIR}\Contrib\License\LicenseEnglish.txt"
   LicenseForceSelection checkbox

;--------------------------------
;Language Strings

  VIAddVersionKey /LANG=${LANG_ENGLISH} "FileVersion" "${VERSION}.${RELEASE}"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "LegalCopyright" " Together Teamlsungen GmbH"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "FileDescription" "Octopus"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "ProductName" "Octopus - WebDocWf Octopus"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "Comments" "WebDocWf Octopus loads data from a JDBC source database into JDBC target databases"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "CompanyName" "Together Teamlsungen GmbH"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "LegalTrademarks" "Octopus Application is a trademark of Together Teamlsungen GmbH"
  VIAddVersionKey /LANG=${LANG_ENGLISH} "Current version" "${VERSION}"

# Install Options pages
# LANG_ENGLISH
# Header
    LangString TEXT_IO_TITLE ${LANG_ENGLISH} "Please select the appropriate directories"
    LangString TEXT_IO_SUBTITLE ${LANG_ENGLISH} "Click button on right side to set the appropriate directories :"
# Window titles
    LangString TEXT_IO_PAGESUBTITLE_A ${LANG_ENGLISH} "Select Java Home directory :"

;---------------------------------
  LangString MUI_TEXT_INSTALLING_TITLE ${LANG_ENGLISH}  "Installing"
  LangString MUI_TEXT_INSTALLING_SUBTITLE ${LANG_ENGLISH}  "Please wait while $(^Name) is being installed."
  LangString MUI_TEXT_FINISH_TITLE ${LANG_ENGLISH}  "Installation Complete"
  LangString MUI_TEXT_FINISH_SUBTITLE ${LANG_ENGLISH}  "Setup was completed successfully."
  LangString MUI_TEXT_ABORT_TITLE ${LANG_ENGLISH}  "Installation Aborted"
  LangString MUI_UNTEXT_UNINSTALLING_TITLE ${LANG_ENGLISH}  "Uninstalling"
  LangString MUI_UNTEXT_UNINSTALLING_SUBTITLE ${LANG_ENGLISH}  "Please wait while $(^Name) is being uninstalled."
  LangString MUI_UNTEXT_FINISH_TITLE ${LANG_ENGLISH}  "Uninstallation Complete"
  LangString MUI_UNTEXT_FINISH_SUBTITLE ${LANG_ENGLISH}  "Uninstall was completed successfully."
  LangString MUI_UNTEXT_ABORT_TITLE ${LANG_ENGLISH}  "Uninstallation Aborted"

  LangString TEXT_IO_JAVATITLE ${LANG_ENGLISH} "Java Setup"
  LangString TEXT_IO_JAVASUBTITLE ${LANG_ENGLISH} "Java virtual machine"

  LangString TEXT_IO_TITLE_A ${LANG_ENGLISH} "$(^Name) Setup"
  LangString TEXT_IO_CANCEL_A ${LANG_ENGLISH} "&Cancel"
  LangString TEXT_IO_NEXT_A ${LANG_ENGLISH} "&Next >"
  LangString TEXT_IO_BACK_A ${LANG_ENGLISH} "< &Back"

;--------------------------------
;Pages Define our own pages

	!insertmacro MUI_PAGE_WELCOME
  !insertmacro MUI_PAGE_LICENSE "$(license_text)"
   Page custom SetJavaPage
  !insertmacro MUI_PAGE_DIRECTORY
  !insertmacro MUI_PAGE_STARTMENU Application $STARTMENU_FOLDER
  !insertmacro MUI_PAGE_INSTFILES
  !insertmacro MUI_PAGE_FINISH

;--------------------------------

# Things that need to be extracted on startup (keep these lines before any File command!)
# Only useful for BZIP2 compression
# Use ReserveFile for your own Install Options ini files too!
  !insertmacro MUI_RESERVEFILE_INSTALLOPTIONS
  ReserveFile "javapage.ini"
;----------------------------------------------------------------------------------------
;Modern UI System
;----------------------------------------------------------------------------------------
#!insertmacro MUI_SYSTEM
;--------------------------------------------------------------------------------
;Installer Sections
;--------------------------------------------------------------------------------
Section Install

  SetOutPath "$INSTDIR"
  File /r ".\..\..\OctopusWindowsInstall\*.*"

  StrCpy $1 $TEMP1
  ; replace char '\' with char '/' at JDK directory string
  Push "$1"
  Push "\"
  Push "/"
	Call ReplaceChar
	Pop $1

	; set value for jdk.dir in file build.properties
	FileOpen $8 $INSTDIR\configure.properties w
	FileWrite $8 "jdk.dir=$1$\r$\n"
	FileClose $8

  ClearErrors
	nsExec::ExecToLog '"$INSTDIR\configure.bat"'
	Pop $0                                                ; return value/error/timeout
	Pop $1                                                ; return value/error/timeout
	StrCmp $0 "0" success

	DetailPrint "${NAME} installation failed (output: $0)"
	Goto end

	success:
	DetailPrint "${NAME} installation succeded (output: $0)"

  ; Write the installation path into the registry
  WriteRegStr HKLM "Software\${NAME} ${VERSION}" "InstDir" "$INSTDIR"

  !insertmacro MUI_STARTMENU_WRITE_BEGIN Application
	; Create Start Menu Shortcuts
  CreateDirectory "$SMPROGRAMS\$STARTMENU_FOLDER"

  CreateShortCut 	"$SMPROGRAMS\$STARTMENU_FOLDER\Documentation.lnk" \
  								"$INSTDIR\docs\index.html" \
  								"" \
  								"" 0
  CreateShortCut 	"$SMPROGRAMS\$STARTMENU_FOLDER\Enhydra Octopus.lnk" \
  								"$INSTDIR\bin\OctopusGenerator.cmd" \
  								"" \
  								"$INSTDIR\octopus.ico" 0

  CreateShortCut 	"$SMPROGRAMS\$STARTMENU_FOLDER\Uninstall.lnk" \
  								"$INSTDIR\uninstall.exe" \
  								"" \
  								"" 0
  !insertmacro MUI_STARTMENU_WRITE_END

  ; Write the uninstall keys for Windows
  WriteRegStr HKLM 	"Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${VERSION}" \
                    "DisplayName" "${NAME} ${VERSION} (remove only)"
  WriteRegStr HKLM 	"Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${VERSION}" \
  									"UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteUninstaller "uninstall.exe"

  ; section to associate the file

  ; File Association - back up old value of .olj
  ReadRegStr $1 HKCR ".olj" ""
  StrCmp $1 "" NoBackup
    StrCmp $1 "OptionsFile" NoBackup
    WriteRegStr HKCR ".olj" "backup_val" $1
  NoBackup:
  WriteRegStr HKCR ".olj" "" "OptionsFile"
  ReadRegStr $0 HKCR "OptionsFile" ""
  StrCmp $0 "" 0 skipOPTAssoc
	WriteRegStr HKCR "OptionsFile" "" "Octopus File"
	WriteRegStr HKCR "OptionsFile\shell" "" "open"
	WriteRegStr HKCR "OptionsFile\DefaultIcon" "" $INSTDIR\octopus.ico,0
  skipOPTAssoc:
  WriteRegStr HKCR "OptionsFile\shell\open\command" "" \
    '$INSTDIR\bin\OctopusGenerator.cmd -sol "%1"'
  WriteRegStr HKCR "OptionsFile\shell\edit" "" "Edit Octopus File"
  WriteRegStr HKCR "OptionsFile\shell\edit\command" "" \
    '$INSTDIR\execute.exe "%1"'

	; File Association - back up old value of .opf
  ReadRegStr $1 HKCR ".opf" ""
  StrCmp $1 "" NoBackup1
    StrCmp $1 "OptionsFile1" NoBackup1
    WriteRegStr HKCR ".opf" "backup_val" $1
  NoBackup1:
  WriteRegStr HKCR ".opf" "" "OptionsFile1"
  ReadRegStr $0 HKCR "OptionsFile1" ""
  StrCmp $0 "" 0 skipOPTAssoc1
	WriteRegStr HKCR "OptionsFile1" "" "Octopus Project File"
	WriteRegStr HKCR "OptionsFile1\shell" "" "open"
	WriteRegStr HKCR "OptionsFile1\DefaultIcon" "" $INSTDIR\octopus.ico,0
  skipOPTAssoc1:
  WriteRegStr HKCR "OptionsFile1\shell\open\command" "" \
    '$INSTDIR\bin\OctopusGenerator.cmd -sop "%1"'
  WriteRegStr HKCR "OptionsFile1\shell\edit" "" "Edit Octopus File"
  WriteRegStr HKCR "OptionsFile1\shell\edit\command" "" \
    '$INSTDIR\execute.exe "%1"'

	end:
SectionEnd
;--------------------------------------------------------------------------------
;Installer Functions
;--------------------------------------------------------------------------------
Function .onInit

# start splash screen
# the plugins dir is automatically deleted when the installer exits
		InitPluginsDir
		File /oname=$PLUGINSDIR\splash.bmp "${NSISDIR}\Modern UI\splash_octopus.bmp"
#optional
#File /oname=$PLUGINSDIR\splash.wav "C:\myprog\sound.wav"
		advsplash::show 3000 600 400 -1 $PLUGINSDIR\splash
		Pop $0 ; $0 has '1' if the user closed the splash screen early,
		                ; '0' if everything closed normal, and '-1' if some error occured.
# end splash screen

  ;Extract Install Options INI Files
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "javapage.ini"

  StrCpy $JAVAHOME ""

FunctionEnd
;--------------------------------------------------------------------------------
;Descriptions
;--------------------------------------------------------------------------------
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES
;--------------------------------------------------------------------------------
;Uninstaller Section
;--------------------------------------------------------------------------------
Section "Uninstall"

  ; MUST REMOVE UNINSTALLER, too
  Delete $INSTDIR\uninstall.exe

  RMDir /r "$INSTDIR"

  !insertmacro MUI_STARTMENU_GETFOLDER Application $MUI_TEMP

  Delete "$SMPROGRAMS\$MUI_TEMP\Uninstall.lnk"
  Delete "$SMPROGRAMS\$MUI_TEMP\Documentation.lnk"
  Delete "$SMPROGRAMS\$MUI_TEMP\Enhydra Octopus.lnk"

  ;Delete empty start menu parent diretories
  StrCpy $MUI_TEMP "$SMPROGRAMS\$MUI_TEMP"

  startMenuDeleteLoop:
    RMDir $MUI_TEMP
    GetFullPathName $MUI_TEMP "$MUI_TEMP\.."

    IfErrors startMenuDeleteLoopDone

    StrCmp $MUI_TEMP $SMPROGRAMS startMenuDeleteLoopDone startMenuDeleteLoop
  startMenuDeleteLoopDone:

  ; remove registry keys
  DeleteRegKey HKCU "Software\${NAME} ${VERSION}"
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${VERSION}"
  DeleteRegKey HKLM "Software\${NAME} ${VERSION}"

 ;start of restore File Association
  ReadRegStr $1 HKCR ".olj" ""
  StrCmp $1 "OptionsFile" 0 NoOwn ; only do this if we own it
    ReadRegStr $1 HKCR ".olj" "backup_val"
    StrCmp $1 "" 0 RestoreBackup ; if backup == "" then delete the whole key
      DeleteRegKey HKCR ".olj"
    Goto NoOwn
    RestoreBackup:
      WriteRegStr HKCR ".olj" "" $1
      DeleteRegValue HKCR ".olj" "backup_val"
    DeleteRegKey HKCR "OptionsFile" ;Delete key with association settings
  NoOwn:

  ReadRegStr $1 HKCR ".opf" ""
  StrCmp $1 "OptionsFile1" 0 NoOwn1 ; only do this if we own it
    ReadRegStr $1 HKCR ".opf" "backup_val"
    StrCmp $1 "" 0 RestoreBackup1 ; if backup == "" then delete the whole key
      DeleteRegKey HKCR ".opf"
    Goto NoOwn1
    RestoreBackup1:
      WriteRegStr HKCR ".opf" "" $1
      DeleteRegValue HKCR ".opf" "backup_val"
    DeleteRegKey HKCR "OptionsFile1" ;Delete key with association settings
  NoOwn1:

SectionEnd
;--------------------------------------------------------------------------------
; Call on installation failed
;--------------------------------------------------------------------------------
Function .onInstFailed
	Call Cleanup
FunctionEnd
;------------------------------------------------------------------------------
; Call when installation failed or when uninstall started
;------------------------------------------------------------------------------
Function Cleanup

  ; MUST REMOVE UNINSTALLER, too
  Delete $INSTDIR\uninstall.exe

  RMDir /r "$INSTDIR"

  !insertmacro MUI_STARTMENU_GETFOLDER Application $MUI_TEMP

  Delete "$SMPROGRAMS\$MUI_TEMP\Uninstall.lnk"
  Delete "$SMPROGRAMS\$MUI_TEMP\Documentation.lnk"
  Delete "$SMPROGRAMS\$MUI_TEMP\Enhydra Octopus.lnk"

  ;Delete empty start menu parent diretories
  StrCpy $MUI_TEMP "$SMPROGRAMS\$MUI_TEMP"

  startMenuDeleteLoop:
    RMDir $MUI_TEMP
    GetFullPathName $MUI_TEMP "$MUI_TEMP\.."

    IfErrors startMenuDeleteLoopDone

    StrCmp $MUI_TEMP $SMPROGRAMS startMenuDeleteLoopDone startMenuDeleteLoop
  startMenuDeleteLoopDone:

  ; remove registry keys
  DeleteRegKey HKCU "Software\${NAME} ${VERSION}"
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME} ${VERSION}"
  DeleteRegKey HKLM "Software\${NAME} ${VERSION}"

FunctionEnd
;------------------------------------------------------------------------------
; ReplaceChar
; input, input string  (C:\JBuilder\jdk1.3.1)
;        char to need replaced (\)
;        char to with replaced (/)
; output, top of stack (C:/JBuilder/jdk1.3.1)
; modifies no other variables.
;
; Usage:
;   Push $1 ; "C:\JBuilder\jdk1.3.1"
;   Push "\"
;   Push "/"
;   Call ReplaceChar
;   Pop $0
;   ; at this point $0 will equal "C:/JBuilder/jdk1.3.1"
;------------------------------------------------------------------------------
Function ReplaceChar
					; stack="/","\","C:\JBuilder\jdk1.3.1", ... (from top to down)
  Exch $2 ; $2="/", stack="2-gi param","\","C:\JBuilder\jdk1.3.1", ...
  Exch  	; stack="\","2-ti param","C:\JBuilder\jdk1.3.1", ...
  Exch $1 ; $1="\", stack="1-vi param","2-gi param","C:\JBuilder\jdk1.3.1", ...
  Exch    ; stack="2-gi param","1-vi param","C:\JBuilder\jdk1.3.1", ...
	Exch 2	; stack="C:\JBuilder\jdk1.3.1","1-vi param","2-gi param", ...
  Exch $0 ; $0="C:\JBuilder\jdk1.3.1", stack="0-ti param","1-vi param","2-gi param", ...
  Exch 2  ; stack="2-gi param","1-vi param","0-ti param", ...

  Push $3	; Len("C:\JBuilder\jdk1.3.1")
  Push $4	; index of string
  Push $5 ; tmp - form strat to found char witch need to change
  Push $6 ; tmp - from found char witch need to change + 1 to end ($3)
  Push $7 ; tmp - position where found char witch need to change + 1
  StrLen $3 $0
  StrCpy $4 1
  loop:
    StrCpy $5 $0 1 $4
    StrCmp $5 "" exit
    StrCmp $5 $1 change
    IntOp $4 $4 + 1
  Goto loop
  change:
  	StrCpy $6 $4				; pozition where found char witch need to change
  	StrCpy $5 $0 $6   	; $5="C:"
  	StrCpy $5 "$5$2"   	; $5="C:/"
  	StrCpy $7 $4				; pozition where found char witch need to change
  	IntOp $7 $7 + 1 		; next pozition
  	StrCpy $6 $0 $3 $7  ; $6="JBuilder\jdk1.3.1"
  	StrCpy $0 $5				; $0="C:/"
  	StrCpy $0 "$0$6" 		; $0="C:/JBuilder\jdk1.3.1"
  Goto loop
  exit:
    Pop $7
    Pop $6
    Pop $5
    Pop $4
    Pop $3
    Pop $2
    Pop $1
    Exch $0 ; put $0 on top of stack, restore $0 to original value
FunctionEnd
;-----------------------------------------------------------------------
Function SetJavaPage

  start:
  call GetJavaVersion

  !insertmacro MUI_HEADER_TEXT "$(TEXT_IO_JAVATITLE)" "$(TEXT_IO_JAVASUBTITLE)"
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY_RETURN "javapage.ini"
	Pop $R0

  StrCmp $R0 "cancel" end
  StrCmp $R0 "back" end
  StrCmp $R0 "success" "" error

  ; check Browse for folder
  !insertmacro MUI_INSTALLOPTIONS_READ $TEMP1 "javapage.ini" "Field 8" "State"
  StrCmp $TEMP1 "" checkdir
  Goto check

	checkdir:
  ; check JAVA_HOME environment variable & choose the directory which will be set as JAVA HOME directory
  !insertmacro MUI_INSTALLOPTIONS_READ $TEMP1 "javapage.ini" "Field 6" "State" ; read choosed java
  StrCmp $TEMP1 "" "" check
  !insertmacro MUI_INSTALLOPTIONS_READ $TEMP1 "javapage.ini" "Field 4" "State" ; read JAVA_HOME enviroment variable
  StrCmp $TEMP1 "Not Defined" "" check
	MessageBox MB_OK "Please choose the directory which will be set as JAVA HOME directory!"
  Goto start

  check:
	IfFileExists $TEMP1\bin\java.exe continue	; check if java.exe exist
	MessageBox MB_OK "File java.exe does not exist in directory $TEMP1\bin"
	Goto start
	continue:
	IfFileExists '$TEMP1\jre\lib\*.*' continue1	checkJDK ; check if JDK 1.4.x
	continue1:
	IfFileExists '$TEMP1\jre\lib\jsse.jar' end	; check if JDK 1.4.x
	MessageBox MB_OK "File jsse.jar does not exist in directory $TEMP1\jre\lib"
	Goto start
	checkJDK:
	IfFileExists '$TEMP1\lib\jsse.jar' javaFound	; check if JDK 1.4.x
	MessageBox MB_OK "File jsse.jar does not exist in directory $TEMP1\lib"
	Goto start
  error:
  MessageBox MB_OK|MB_ICONSTOP "InstallOptions error:$\r$\n$TEMP1"
  Goto end

  javaFound:
  StrCpy $JAVAHOME $TEMP1
  end:
FunctionEnd
;-----------------------------------------------------------------------
Function GetJavaVersion

  ;No value by default
  StrCpy $1 ""
  StrCpy $4 ""

  ClearErrors
  ; READ JAVA_HOME from all users
  ReadEnvStr $4 "JAVA_HOME"
  IfErrors noAllUsersJavaHome
  ; Set the JAVA_HOME value into the ini file
  WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 4" "State" $4

  noAllUsersJavaHome:
  ClearErrors
  ; READ JAVA_HOME from the user
  ReadRegStr $4 HKCU "Environment" "JAVA_HOME"
  IfErrors noUserJavaHome
  ; Set the JAVA_HOME value into the ini file
  WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 4" "State" $4

  noUserJavaHome:
  ClearErrors
  ; Read the Sun JDK value from the registry
  ReadRegStr $2 HKLM "SOFTWARE\JavaSoft\Java Development Kit" "CurrentVersion"
  ReadRegStr $1 HKLM "SOFTWARE\JavaSoft\Java Development Kit\$2" "JavaHome"

  IfErrors noSunJDK
  StrCpy $R9 0   ; count EnumRegKey from "SOFTWARE\JavaSoft\Java Development Kit"
  WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 6" "ListItems" ""
  loop:
    EnumRegKey $R1 HKLM "SOFTWARE\JavaSoft\Java Development Kit" $R9
    StrCmp $R1 "" noSunJDK
    ReadRegStr $R2 HKLM "SOFTWARE\JavaSoft\Java Development Kit\$R1" "JavaHome"

    ; check if java version >= 1.4.x
  	StrCpy $1 $R1 1 ; major version
    StrCpy $2 $R1 1 2 ; minor version
    IntFmt $3 "%u" $1
    IntFmt $4 "%u" $2
    IntCmp $4 4 ok 0 ok
    goto increment
    ok:

    ; Read item from combo
    ReadINIStr $3 "$PLUGINSDIR\javapage.ini" "Field 6" "State"
    ReadINIStr $4 "$PLUGINSDIR\javapage.ini" "Field 6" "ListItems"
    ; add or append to existing value ?
    StrCmp $3 "" setJDK appendJDK

    setJDK:
      WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 6" "State" $R2
      WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 6" "ListItems" $R2
    goto increment

    appendJDK:
      ; check if path exist
      Push $4
      Push $R2
      Call StrStr
      Pop $R0
      StrCmp $R0 -1 0 increment
      WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 6" "ListItems" $4|$R2

    increment:
      IntOp $R9 $R9 + 1
  goto loop

  noSunJDK:

  ClearErrors
  ; Read the Sun JRE value from the registry
  ReadRegStr $2 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment" "CurrentVersion"
  ReadRegStr $1 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment\$2" "JavaHome"

  IfErrors noSunJRE
  StrCpy $R9 0   ; count EnumRegKey from "SOFTWARE\JavaSoft\Java Runtime Environment"
  loopJRE:
    EnumRegKey $R1 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment" $R9
    StrCmp $R1 "" noSunJRE
    ReadRegStr $R2 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment\$R1" "JavaHome"

    ; check if java version >= 1.4.x
  	StrCpy $1 $R1 1 ; major version
    StrCpy $2 $R1 1 2 ; minor version
    IntFmt $3 "%u" $1
    IntFmt $4 "%u" $2
    IntCmp $4 4 okJRE 0 okJRE
    goto incrementJRE
    okJRE:

    ; Read item from combo
    ReadINIStr $3 "$PLUGINSDIR\javapage.ini" "Field 6" "State"
    ReadINIStr $4 "$PLUGINSDIR\javapage.ini" "Field 6" "ListItems"
    ; add or append to existing value ?
    StrCmp $3 "" setJRE appendJRE

    setJRE:
      WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 6" "State" $R2
      WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 6" "ListItems" $R2
    goto incrementJRE

    appendJRE:
      ; check if path exist
      Push $4
      Push $R2
      Call StrStr
      Pop $R0
      StrCmp $R0 -1 0 incrementJRE
      WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 6" "ListItems" $4|$R2
    incrementJRE:
      IntOp $R9 $R9 + 1
  goto loopJRE

  noSunJRE:
      WriteINIStr "$PLUGINSDIR\javapage.ini" "Field 6" "State" ""
FunctionEnd
;====================================================
; StrStr - Finds a given string in another given string.
;               Returns -1 if not found and the pos if found.
;          Input: head of the stack - string to find
;                      second in the stack - string to find in
;          Output: head of the stack
;====================================================
Function StrStr
  Push $0
  Exch
  Pop $0 ; $0 now have the string to find
  Push $1
  Exch 2
  Pop $1 ; $1 now have the string to find in
  Exch
  Push $2
  Push $3
  Push $4
  Push $5

  StrCpy $2 -1
  StrLen $3 $0
  StrLen $4 $1
  IntOp $4 $4 - $3

  unStrStr_loop:
    IntOp $2 $2 + 1
    IntCmp $2 $4 0 0 unStrStrReturn_notFound
    StrCpy $5 $1 $3 $2
    StrCmp $5 $0 unStrStr_done unStrStr_loop

  unStrStrReturn_notFound:
    StrCpy $2 -1

  unStrStr_done:
    Pop $5
    Pop $4
    Pop $3
    Exch $2
    Exch 2
    Pop $0
    Pop $1
FunctionEnd
;------------------------------------------------------------------------------
; Run Release Notes
;------------------------------------------------------------------------------
Function RunReleaseNotes

  Exec '"$PROGRAMFILES\Internet Explorer\iexplore.exe" "$INSTDIR\ReleaseNotes.txt"'

FunctionEnd
