/**
    Copyright (C) 2002-2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


package org.relique.jdbc.csv;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.io.InputStream;
import java.io.Reader;
import java.sql.Ref;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Array;
import java.sql.ResultSetMetaData;
import java.util.Calendar;
import java.net.URL;
import java.sql.ParameterMetaData;
import java.sql.SQLWarning;
import java.sql.Connection;
import java.sql.DriverManager;
import java.io.File;
import java.util.*;

/**
 * This class implements the PreparedStatement interface for the CsvJdbc driver.
 *
 * @author     Zoran Milakovic
 */
public class CsvPreparedStatement implements PreparedStatement {

  private CsvConnection connection;
  private Vector resultSets = new Vector();
  private String sqlForPrepare = "";
  private String sqlPrepared = "";
  private int paramCount = 0;
  private ArrayList parameters = new ArrayList();
  private ArrayList binaryStreamParameters = new ArrayList();
  private CsvWriter writeCsv;
/** Name used for prepared statement parameters */
  public static String PREPARE_SEPARATOR = "~@#NEXTPARAMETER#@~";
  private String sql;

  public CsvPreparedStatement(CsvConnection connection, String preparedSql) {
    DriverManager.println("CsvJdbc - CsvStatement() - connection=" + connection);
    this.sqlForPrepare = preparedSql;
    this.sqlPrepared = preparedSql;
    this.paramCount = countParameters(preparedSql);
    for ( int i = 0; i < paramCount; i++ )
      parameters.add(null);
    this.connection = connection;
    try {
      if(!connection.getAutoCommit())
        writeCsv=new CsvWriter(
              null,
              connection.getSeperator(),
              connection.getExtension(),
              connection.getMaxFileSize()
              );
    }
    catch (Exception ex) {
    ex.printStackTrace();
    }
  }

  private int countParameters(String sql) {
    int count = 0;
    int index = sql.indexOf(PREPARE_SEPARATOR);
    while( index != -1 ) {
      count++;
      sql = sql.substring(index+1);
      index = sql.indexOf(PREPARE_SEPARATOR);
    }
    return count;
  }

  private boolean prepareSql() throws SQLException {
    boolean retVal = true;
    for (int i = 0; i < parameters.size(); i++) {
      int index = sqlPrepared.indexOf(PREPARE_SEPARATOR);
      String val;
      if (index != -1) {
        if( parameters.get(i) == null )
          val = "null";
        else
          val = parameters.get(i).toString();
        sqlPrepared = sqlPrepared.substring(0, index) +
            val +
            sqlPrepared.substring(index + PREPARE_SEPARATOR.length());
      }
    }
    if (sqlPrepared.indexOf(PREPARE_SEPARATOR) != -1)
      throw new SQLException(
          "All ? in prepared query has to be replaced with values.");
    else if (parameters.size() < this.paramCount)
      throw new SQLException(
          "Number of setted parameters is less than number of parameters in statement.");
    else if (parameters.size() > this.paramCount)
      throw new SQLException(
          "Number of setted parameters is greater than number of parameters in statement.");

    return retVal;
  }

  /**
     *Sets the maxFieldSize attribute of the CsvStatement object
     *
     * @param  p0                The new maxFieldSize value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setMaxFieldSize(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the maxRows attribute of the CsvStatement object
     *
     * @param  p0                The new maxRows value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setMaxRows(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the escapeProcessing attribute of the CsvStatement object
     *
     * @param  p0                The new escapeProcessing value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setEscapeProcessing(boolean p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the queryTimeout attribute of the CsvStatement object
     *
     * @param  p0                The new queryTimeout value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setQueryTimeout(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the cursorName attribute of the CsvStatement object
     *
     * @param  p0                The new cursorName value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setCursorName(String p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the fetchDirection attribute of the CsvStatement object
     *
     * @param  p0                The new fetchDirection value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setFetchDirection(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the fetchSize attribute of the CsvStatement object
     *
     * @param  p0                The new fetchSize value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setFetchSize(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the maxFieldSize attribute of the CsvStatement object
     *
     * @return                   The maxFieldSize value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getMaxFieldSize() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the maxRows attribute of the CsvStatement object
     *
     * @return                   The maxRows value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getMaxRows() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the queryTimeout attribute of the CsvStatement object
     *
     * @return                   The queryTimeout value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getQueryTimeout() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the warnings attribute of the CsvStatement object
     *
     * @return                   The warnings value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public SQLWarning getWarnings() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the resultSet attribute of the CsvStatement object
     *
     * @return                   The resultSet value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public ResultSet getResultSet() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the updateCount attribute of the CsvStatement object
     *
     * @return                   The updateCount value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getUpdateCount() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the moreResults attribute of the CsvStatement object
     *
     * @return                   The moreResults value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public boolean getMoreResults() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the fetchDirection attribute of the CsvStatement object
     *
     * @return                   The fetchDirection value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getFetchDirection() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the fetchSize attribute of the CsvStatement object
     *
     * @return                   The fetchSize value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getFetchSize() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the resultSetConcurrency attribute of the CsvStatement object
     *
     * @return                   The resultSetConcurrency value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getResultSetConcurrency() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the resultSetType attribute of the CsvStatement object
     *
     * @return                   The resultSetType value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getResultSetType() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the connection attribute of the CsvStatement object
     *
     * @return                   The connection value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public Connection getConnection() throws SQLException
    {
        return connection;
    }


    /**
     *Description of the Method
     *
     * @param  sql               Description of Parameter
     * @return                   Description of the Returned Value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public ResultSet executeQuery(String sql) throws SQLException
    {
        DriverManager.println("CsvJdbc - CsvStatement:executeQuery() - sql= " + sql);
        CsvSqlParser parser = new CsvSqlParser();
        if( binaryStreamParameters.size() != 0 )
          parser.setBinaryStreamList( binaryStreamParameters );

        this.sql = sql;
        try
            {
                parser.parse(this);
            }
        catch (Exception e)
            {
                throw new SQLException("Syntax Error. " + e.getMessage());
            }

    String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
    File checkFile = new File(fileName);
    if (!checkFile.exists())
    {
      throw new SQLException("Cannot open data file '" + fileName + "'  !");
    }

    if (!checkFile.canRead())
    {
      throw new SQLException("Data file '" + fileName + "'  not readable !");
    }
    CsvReader reader;
    try
    {
      reader = new CsvReader(fileName,
                             connection.getSeperator(),
                             connection.isSuppressHeaders(),
                             connection.getCharset(),
                             connection.getExtension(),
                             connection.getLineBreakEscape(),
//                             connection.getDoubleQuotesEscape(),
                             connection.getCarriageReturnEscape()
                             );
      String[] xxx = parser.getColumnNames();
      String[] yyy = reader.getColumnNames();
      boolean isOK = true;
      for(int i=0; i< xxx.length; i++) {
      if(!xxx[i].endsWith("*")) {
       out:
      for(int j=0; j< yyy.length; j++) {
         if(xxx[i].equalsIgnoreCase( yyy[j] )) {
            isOK=true;
            break out;
          }
          else
            isOK=false;
        }
       if(!isOK)
        throw new SQLException("Column '" + xxx[i] + "' not found.");
      }
      }
    }
    catch (Exception e)
    {
      throw new SQLException("Error reading data file. Message was: " + e);
    }
    CsvResultSet resultSet = new CsvResultSet(this, reader,
            parser.getTableName(), parser.getColumnNames(), parser.getWhereColumnNames(),
            parser.getWhereColumnValues(), reader.getColumnTypes());
    resultSets.add(resultSet);
    return resultSet;
  }



    /**
     *Description of the Method
     *
     * @param  sql               Description of Parameter
     * @return                   Description of the Returned Value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int executeUpdate(String sql) throws SQLException
    {
    int updated=0;
    DriverManager.println("CsvJdbc - CsvStatement:executeUpdate() - sql= " + sql);
    CsvSqlParser parser = new CsvSqlParser();
    if( binaryStreamParameters.size() != 0 )
          parser.setBinaryStreamList( binaryStreamParameters );
    this.sql = sql;
    try
    {
      parser.parse(this);
    }
    catch (Exception e)
    {
      throw new SQLException("Syntax Error. " + e.getMessage());
    }
      if(parser.sqlType.equals(parser.SELECT))
              throw new SQLException("Not supported SELECT statement - use executeQuery method");
      else if (parser.sqlType.equals(parser.CREATE_TABLE)) {
                  throw new SQLException("Not supported CREATE TABLE statement - use execute method");
      }
      else if (parser.sqlType.equals(parser.INSERT)) {
            String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
            File checkFile = new File(fileName);

            if (!checkFile.exists())
            {
              throw new SQLException("Cannot open data file '" + fileName + "'  !");
            }

            if (!checkFile.canWrite())
            {
              throw new SQLException("Data file '" + fileName + "'  is read only !");
            }
//           CsvWriter writeCsv;
            try
            {
              if(connection.getAutoCommit())
                writeCsv = new CsvWriter(
                  fileName,
                  connection.getSeperator(),
                  connection.getExtension(),
                  connection.getMaxFileSize()
                  );
              else {
                writeCsv.setFileName(fileName);
                writeCsv.fillTableColumnNames();
              }

                String[] xxx = parser.getColumnNames();
                String[] yyy = writeCsv.getColumnNames();
                boolean isOK = true;
                for(int i=0; i< xxx.length; i++) {
                  if(!xxx[i].endsWith("*")) {
                  out:
                  for(int j=0; j< yyy.length; j++) {
                    if(xxx[i].equalsIgnoreCase( yyy[j] )) {
                      isOK=true;
                      break out;
                    }
                  else
                    isOK=false;
                  }
                if(!isOK)
                  throw new SQLException("Column '" + xxx[i] + "' not found.");
                }
              }
              writeCsv.newLine(parser.columnNames, parser.columnValues);

            }
            catch (Exception e)
            {
              e.printStackTrace();
              throw new SQLException("Error reading data file. Message was: " + e);
            }

      }
      else if (parser.sqlType.equals(parser.UPDATE)) {

            String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
            File checkFile = new File(fileName);

            if (!checkFile.exists())
            {
              throw new SQLException("Cannot open data file '" + fileName + "'  !");
            }

            if (!checkFile.canWrite())
            {
              throw new SQLException("Data file '" + fileName + "'  is read only !");
            }
//           CsvWriter writeCsv;
            try
            {
              if(connection.getAutoCommit())
                writeCsv = new CsvWriter(
              fileName,
              connection.getSeperator(),
              connection.getExtension(),
              connection.getMaxFileSize()
              );
              else{
               writeCsv.setFileName(fileName);
               writeCsv.fillTableColumnNames();
              }

                String[] xxx = parser.getColumnNames();
                String[] yyy = writeCsv.getColumnNames();
                boolean isOK = true;
                for(int i=0; i< xxx.length; i++) {
                  if(!xxx[i].endsWith("*")) {
                    out:
                  for(int j=0; j< yyy.length; j++) {
                    if(xxx[i].equalsIgnoreCase( yyy[j] )) {
                      isOK=true;
                      break out;
                    }
                  else
                    isOK=false;
                }
                if(!isOK)
                  throw new SQLException("Column '" + xxx[i] + "' not found.");
                  }
                }
                if(!writeCsv.updateFields(parser.columnNames, parser.columnValues, parser.columnWhereNames, parser.columnWhereValues))
                  updated = -1;
            }
            catch (Exception e)
            {
              throw new SQLException("Error reading data file. Message was: " + e);
            }

        }
      return updated;

  }


    /**
     * Releases this <code>Statement</code> object's database
     * and JDBC resources immediately instead of waiting for
     * this to happen when it is automatically closed.
     * It is generally good practice to release resources as soon as
     * you are finished with them to avoid tying up database
     * resources.
     * <P>
     * Calling the method <code>close</code> on a <code>Statement</code>
     * object that is already closed has no effect.
     * <P>
     * <B>Note:</B> A <code>Statement</code> object is automatically closed
     * when it is garbage collected. When a <code>Statement</code> object is
     * closed, its current <code>ResultSet</code> object, if one exists, is
     * also closed.
     *
     * @exception SQLException if a database access error occurs
     */
    public void close() throws SQLException {
      // close all result sets
      for(Enumeration i = resultSets.elements(); i.hasMoreElements(); ) {
        CsvResultSet resultSet = (CsvResultSet) i.nextElement();
        resultSet.close();
      }
      try {
        this.writeCsv.close();
      }
      catch(Exception e) {
        e.printStackTrace();
        throw new SQLException(e.getMessage());
      }
    }

    /**
     *Description of the Method
     *
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void cancel() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Description of the Method
     *
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void clearWarnings() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Description of the Method
     *
     * @param  sql               Description of Parameter
     * @return                   Description of the Returned Value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public boolean execute(String sql) throws SQLException
  {
    CsvSqlParser parser = new CsvSqlParser();
    if( binaryStreamParameters.size() != 0 )
          parser.setBinaryStreamList( binaryStreamParameters );
    this.sql = sql;
    try
    {
      parser.parse(this);
    }
    catch (Exception e)
    {
      throw new SQLException("Syntax Error. " + e.getMessage());
    }
      if(parser.sqlType.equals(parser.SELECT))
              throw new SQLException("Not supported SELECT statement - use executeQuery method");
      else if (parser.sqlType.equals(parser.INSERT))
              executeUpdate(sql);
       else if (parser.sqlType.equals(parser.CREATE_TABLE)) {
             String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
            File checkFile = new File(fileName);

            if (checkFile.exists())
            {
              throw new SQLException("Data file '" + fileName + "'already exists  !");
            }

//           CsvWriter writeCsv;
            try
            {
              if(connection.getAutoCommit())
                writeCsv = new CsvWriter(
              fileName,
              connection.getSeperator(),
              connection.getExtension(),
              connection.getMaxFileSize()
              );
              else{
                writeCsv.setFileName(fileName);
                writeCsv.fillTableColumnNames();
              }

              writeCsv.createTable(parser.columnNames, fileName);
            }
            catch (Exception e)
            {
              throw new SQLException("Error reading data file. Message was: " + e);
            }
        }
        return true;

  }

  /**
   * Set String as parameter in sql statement.
   * @param parameterIndex
   * @param value
   * @throws SQLException
   */
  public void setString(int parameterIndex, String value) throws SQLException {
    if( value == null ) {
      CsvDriver.log("value = null object");
      parameters.set(parameterIndex - 1, value);
    } else {
      value = Utils.replaceAll(value, "'", CsvSqlParser.QUOTE_ESCAPE);
      CsvDriver.log("value = " + value);
      parameters.set(parameterIndex - 1, "'" + value + "'");
    }
  }


  /**
   *
   * @param parameterIndex
   * @param value
   * @throws SQLException
   */
  public void setBytes(int parameterIndex, byte[] value) throws SQLException {
    binaryStreamParameters.add(Utils.bytesToHexString(value));
    String paramName = CsvSqlParser.BINARY_STREAM_OBJECT+""+binaryStreamParameters.size();
    parameters.set(parameterIndex-1, paramName);
  }

    /**
     *Adds a feature to the Batch attribute of the CsvStatement object
     *
     * @param  p0                The feature to be added to the Batch attribute
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void addBatch(String p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Description of the Method
     *
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void clearBatch() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Description of the Method
     *
     * @return                   Description of the Returned Value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int[] executeBatch() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }

    //---------------------------------------------------------------------
    // JDBC 3.0
    //---------------------------------------------------------------------

    public boolean getMoreResults(int current) throws SQLException {
        throw new UnsupportedOperationException("Statement.getMoreResults(int) unsupported");
    }

    public ResultSet getGeneratedKeys() throws SQLException {
        throw new UnsupportedOperationException("Statement.getGeneratedKeys() unsupported");
    }

    public int executeUpdate(String sql, int autoGeneratedKeys) throws SQLException {
        throw new UnsupportedOperationException("Statement.executeUpdate(String,int) unsupported");
    }

    public int executeUpdate(String sql, int[] columnIndexes) throws SQLException {
        throw new UnsupportedOperationException("Statement.executeUpdate(String,int[]) unsupported");
    }

    public int executeUpdate(String sql, String[] columnNames) throws SQLException {
        throw new UnsupportedOperationException("Statement.executeUpdate(String,String[]) unsupported");
    }

    public boolean execute(String sql, int autoGeneratedKeys) throws SQLException {
        throw new UnsupportedOperationException("Statement.execute(String,int) unsupported");
    }

    public boolean execute(String sql, int[] columnIndexes) throws SQLException {
        throw new UnsupportedOperationException("Statement.execute(String,int[]) unsupported");
    }

    public boolean execute(String sql, String[] columnNames) throws SQLException {
        throw new UnsupportedOperationException("Statement.execute(String,String[]) unsupported");
    }

    public int getResultSetHoldability() throws SQLException {
        throw new UnsupportedOperationException("Statement.getResultSetHoldability() unsupported");
    }



  public ResultSet executeQuery() throws SQLException {
    if( !prepareSql())
        throw new SQLException("Error with prepared statement  !");
    return executeQuery(this.sqlPrepared);

  }
  public int executeUpdate() throws SQLException {
    if( !prepareSql())
        throw new SQLException("Error with prepared statement  !");
    executeUpdate(this.sqlPrepared);
    return 0;
  }





  public void setNull(int parameterIndex, int sqlType) throws SQLException {
    this.setString(parameterIndex, null );
  }

  public void setBoolean(int parameterIndex, boolean value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setByte(int parameterIndex, byte value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setShort(int parameterIndex, short value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setInt(int parameterIndex, int value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setLong(int parameterIndex, long value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setFloat(int parameterIndex, float value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setDouble(int parameterIndex, double value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setBigDecimal(int parameterIndex, BigDecimal value) throws SQLException {
    this.setString( parameterIndex, value.toString() );
  }
  public void setDate(int parameterIndex, Date x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setDate() not yet implemented.");
  }
  public void setTime(int parameterIndex, Time x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setTime() not yet implemented.");
  }
  public void setTimestamp(int parameterIndex, Timestamp x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setTimestamp() not yet implemented.");
  }
  public void setAsciiStream(int parameterIndex, InputStream x, int length) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setAsciiStream() not yet implemented.");
  }
  public void setUnicodeStream(int parameterIndex, InputStream x, int length) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setUnicodeStream() not yet implemented.");
  }

  public void clearParameters() throws SQLException {
    this.sqlPrepared = this.sqlForPrepare;
    for(int i = 0; i < parameters.size(); i++)
      parameters.set(i, null);
  }
  public void setObject(int parameterIndex, Object x, int targetSqlType, int scale) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setObject() not yet implemented.");
  }
  public void setObject(int parameterIndex, Object x, int targetSqlType) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setObject() not yet implemented.");
  }
  public void setObject(int parameterIndex, Object x) throws SQLException {
    if( x == null )
       x = new String("null");
     setString( parameterIndex, x.toString() );
  }
  public boolean execute() throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method execute() not yet implemented.");
  }
  public void addBatch() throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method addBatch() not yet implemented.");
  }
  public void setCharacterStream(int parameterIndex, Reader reader, int length) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setCharacterStream() not yet implemented.");
  }
  public void setRef(int i, Ref x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setRef() not yet implemented.");
  }

  public void setClob(int i, Clob x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setClob() not yet implemented.");
  }
  public void setArray(int i, Array x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setArray() not yet implemented.");
  }
  public ResultSetMetaData getMetaData() throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method getMetaData() not yet implemented.");
  }
  public void setDate(int parameterIndex, Date x, Calendar cal) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setDate() not yet implemented.");
  }
  public void setTime(int parameterIndex, Time x, Calendar cal) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setTime() not yet implemented.");
  }
  public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setTimestamp() not yet implemented.");
  }
  public void setNull(int paramIndex, int sqlType, String typeName) throws SQLException {
    this.setString(paramIndex, null );
  }

  public void setURL(int parameterIndex, URL x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException(
        "Method setURL() not yet implemented.");
  }

  public ParameterMetaData getParameterMetaData() throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException(
        "Method getParameterMetaData() not yet implemented.");
  }

  public void setBinaryStream(int parameterIndex, InputStream value, int length) throws
      SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException(
        "Method setBinaryStream() not yet implemented.");
  }

  public void setBlob(int parameterIndex, Blob value) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException(
        "Method setBlob() not yet implemented.");
  }

  public String getSqlStatement() {
      return sql;
    }

}