/**
  ConfigReader - Read vales from config files for specified database.
    Copyright (C) 2002-2003  Together
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ConfigReader.java
 Date: 20.5.2003.
 */

package org.webdocwf.util.loader;

import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.webdocwf.util.loader.logging.*;
import java.util.Hashtable;

/**
 * Class read configuration parameters from configuration XML file.
 *
 * @author     Zoran Milakovic, Radoslav Dutina
 * @version    1.1
 */
public class ConfigReader {

  private Logger logger;
  private String strVendorFileName = null;
  private String strDriverClassName = "";
  private int iFirstColumnResult = 1;
  private boolean bRequiredUser = false;
  private boolean bEnableJumpResult = false;
  private boolean bAfterLastRow = true;
  private boolean bEnableOrderBy = false;
  private boolean bRowCountEnabled = false;
  private boolean bSetFetchSizeEnabled = false;
  private String oidDbType = "";
  private String versionDbType = "";
  private String oidColumnName = "oid";
  private String versionColumnName = "version";
  //This is default date format.
  //private String dateFormat = "MM/dd/yyyy hh:mm:ss";
  private String dateFormat = "yyyy-MM-dd";

  private boolean bSetCursorNameEnabled = false;
  private boolean bSetEmptyStringAsNull = false;
  private boolean bReadingOrderRelevant = false;
  private boolean bGetColumnsSupported = false;
  private boolean bSetMaxRowsSupported = false;
  private String bConnectionPrefix = "";
  private boolean bFileSystemDatabase = false;
  private String confJarStructure = "";
  private boolean useSeparateConfFiles = false;
  private Hashtable javaTypeMapp = new Hashtable();
//ZK added this 6.5.2004
  private Hashtable isNumberMapp = new Hashtable();
  private Hashtable isBinaryObjectMap = new Hashtable();
  private Hashtable isDateMap = new Hashtable();
//end
  private String currentDriverName = "";
  private String currentDatabaseName = "";

  private static final String BYTE_ARRAY="1";
  private static final String JAVA_MATH_BIGDECIMAL="2";
  private static final String JAVA_LANG_DOUBLE="3";
  private static final String JAVA_LANG_FLOAT="4";
  private static final String JAVA_LANG_INTEGER="5";
  private static final String JAVA_LANG_LONG="6";
  private static final String JAVA_LANG_SHORT="7";
  private static final String JAVA_LANG_STRING="8";
  private static final String JAVA_SQL_DATE="9";
  private static final String JAVA_SQL_TIME="10";
  private static final String JAVA_SQL_TIMESTAMP="11";
  private static final String JAVA_LANG_BOOLEAN="12";
  private static final String JAVA_LANG_BYTE="13";
  private static final String JAVA_LANG_OBJECT="14";

  /**Method readConfigValues read specific values for desired database(dbVendor) and
   * puts them into global variables. Third parameter in this method describes which database is analysed (target or source)
   * Method reads values from Loader.conf configuration file (XML format).
   * @param dbVendor - String - type of source database (table);
   * @param strType - String - type of Loader database (source or target);
   * @param driverName is name of the driver
   * @throws LoaderException
   */
  public void readConfigValues(String dbVendor, String driverName,
                               String strType) throws LoaderException {
    Document doc = null;
    this.logger.write("full", "\treadConfigValues method is started.");
    if(this.javaTypeMapp.size()>0)
      this.javaTypeMapp.clear();
      try {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        db = dbf.newDocumentBuilder();

        String OCTOPUS_HOME = System.getProperty("OCTOPUS_HOME");
        String databaseConfFile = null;
        InputStream confFile = null;
        File configFile = null;

//sinisa - Octopus supports mssql and msql values for dbVendor element
        if (dbVendor.equalsIgnoreCase("mssql"))
          dbVendor = "MSQL";
        if (dbVendor.equalsIgnoreCase("Hsqldb"))
          dbVendor = "HypersonicSQL";

        if (this.strVendorFileName != null && !this.strVendorFileName.equals("")) {
          configFile = new File(this.strVendorFileName);
          if (configFile.exists()) {
            doc = db.parse(configFile);
          }
        } else {
          if (OCTOPUS_HOME != null) {
            if (! (OCTOPUS_HOME.endsWith("\\") || OCTOPUS_HOME.endsWith("/")))
              OCTOPUS_HOME += "/";
            configFile = new File(OCTOPUS_HOME + "conf/OctopusDBVendors.xml");
            if (configFile.exists())
              doc = db.parse(configFile);
          } else {
            if (this.useSeparateConfFiles) {
              confFile = this.getClass().getClassLoader().getResourceAsStream(
                         this.confJarStructure + "/OctopusDBVendors.xml");
            } else {
              confFile = this.getClass().getClassLoader().getResourceAsStream(
                         "xml/conf/OctopusDBVendors.xml");
            }
            if (confFile != null)
              doc = db.parse(confFile);
          }
        }
        if (doc != null) {
          NodeList tagConfFile = doc.getElementsByTagName("Vendor");
          out:for (int i = 0; i < tagConfFile.getLength(); i++) {
            Node nodeMain = tagConfFile.item(i);
            NamedNodeMap attrs = nodeMain.getAttributes();
            Node nodeDriver = attrs.getNamedItem("name");
            if (nodeDriver != null &&
                nodeDriver.getFirstChild().getNodeValue().
                equalsIgnoreCase(dbVendor)) {
              databaseConfFile = nodeMain.getFirstChild().getNodeValue();
              break out;
            }
          }
        } else {
          //if driver name don't exists use default for source i target
          if (strType.equalsIgnoreCase("source"))
            databaseConfFile = "CsvConf.xml";
          else
            databaseConfFile = "MSQLConf.xml";
        }

        doc = null;
        if (OCTOPUS_HOME == null) {
          if (this.useSeparateConfFiles) {
            confFile = getClass().getClassLoader().getResourceAsStream(
                       this.confJarStructure + "/" + databaseConfFile);
          } else {
            confFile = getClass().getClassLoader().getResourceAsStream(
                       "xml/conf/" + databaseConfFile);
          }
          if (confFile != null)
            doc = db.parse(confFile);
        } else {
          if (databaseConfFile != null)
            configFile = new File(databaseConfFile);
          else
            configFile = new File("null");

          if (configFile.exists()) {
            doc = db.parse(configFile);
          }
        }

        if (doc == null) {
          if (strType.equalsIgnoreCase("source")) {
            this.logger.write("normal",
                "Failed to load config file for source database, load default configuration.");
            this.strDriverClassName = "org.relique.jdbc.csv.CsvDriver";
            this.iFirstColumnResult = 1;
            this.bRequiredUser = false;
            this.bEnableJumpResult = false;
            this.bAfterLastRow = true;
            this.bEnableOrderBy = false;
            if (driverName == null || driverName.equals(""))
              driverName = "csv";
            this.bFileSystemDatabase = true;
          } else {
            this.logger.write("normal",
                "Failed to load config file for target database, load default configuration.");
            this.strDriverClassName = "com.newatlanta.jturbo.driver.Driver";
            this.iFirstColumnResult = 1;
            this.bRequiredUser = true;
            if (driverName == null || driverName.equals(""))
              driverName = "jTurbo";
          }
        }
      }
      catch (Exception e) {
        this.logger.write("normal", "Sorry, an error occurred: " + e.getMessage());
        LoaderException le = new LoaderException("Exception: ", (Throwable)e);
        throw le;
      }

      String strDriverName = "";
      if (doc != null) {
        if (driverName == null || driverName.equalsIgnoreCase("")) {
          if (dbVendor.equalsIgnoreCase("MSQL"))
            driverName = "jTurbo";
        }
        //oid db type
        NodeList tagOidDbType = doc.getElementsByTagName("OidDbType");
        this.oidDbType = tagOidDbType.item(0).getFirstChild().getNodeValue();

        NodeList tagVersionDbType = doc.getElementsByTagName("VersionDbType");
        this.versionDbType = tagVersionDbType.item(0).getFirstChild().getNodeValue();

        NodeList tagOidColumnName = doc.getElementsByTagName("OidDbColumnName");
        this.oidColumnName = tagOidColumnName.item(0).getFirstChild().getNodeValue();

        NodeList tagVersionColumnName = doc.getElementsByTagName("VersionDbColumnName");
        this.versionColumnName = tagVersionColumnName.item(0).getFirstChild().getNodeValue();


		NodeList tagDateFormat = doc.getElementsByTagName("DateFormat");
		Node tempChildNode = tagDateFormat.item(0).getFirstChild();

		if (tempChildNode != null ){
			String tmpDate = tagDateFormat.item(0).getFirstChild().getNodeValue();
            this.dateFormat = tmpDate;
		}


        //if(strType.equalsIgnoreCase("target")){
          NodeList tagJavaType = doc.getElementsByTagName("SQLType");
          Node nodeJavaTMain = tagJavaType.item(0);
          NodeList dataTypeNodes = nodeJavaTMain.getChildNodes();
					String nodeAttrIsNumber = "";
					String nodeAttrIsBinary = "";
					String nodeAttrIsDate = "";
          for (int i = 0; i < dataTypeNodes.getLength(); i++) {
            if (dataTypeNodes.item(i).getNodeType() == Node.ELEMENT_NODE) {
              String nodeName = dataTypeNodes.item(i).getNodeName();
              String nodeAttr = dataTypeNodes.item(i).getAttributes()
                                .getNamedItem("javaType").getNodeValue();
//ZK added this 6.5.2004
				if (dataTypeNodes.item(i).getAttributes().getNamedItem("isNumber")!=null){
					nodeAttrIsNumber = dataTypeNodes.item(i).getAttributes().getNamedItem("isNumber").getNodeValue();
				}else{
					nodeAttrIsNumber = "false";
				}
				if (dataTypeNodes.item(i).getAttributes().getNamedItem("isBinary")!=null){
					nodeAttrIsBinary = dataTypeNodes.item(i).getAttributes().getNamedItem("isBinary").getNodeValue();
				}else{
					nodeAttrIsBinary = "false";
				}
				if (dataTypeNodes.item(i).getAttributes().getNamedItem("isDate")!=null){
					nodeAttrIsDate = dataTypeNodes.item(i).getAttributes().getNamedItem("isDate").getNodeValue();
				}else{
					nodeAttrIsDate = "false";
				}


//end
              String nodeAttrInt;
              if (nodeAttr.equalsIgnoreCase("byte[]"))
                nodeAttrInt = BYTE_ARRAY;
              else if (nodeAttr.equalsIgnoreCase("java.math.BigDecimal"))
                nodeAttrInt = JAVA_MATH_BIGDECIMAL;
              else if (nodeAttr.equalsIgnoreCase("java.lang.Double"))
                nodeAttrInt = JAVA_LANG_DOUBLE;
              else if (nodeAttr.equalsIgnoreCase("java.lang.Float"))
                nodeAttrInt = JAVA_LANG_FLOAT;
              else if (nodeAttr.equalsIgnoreCase("java.lang.Integer"))
                nodeAttrInt = JAVA_LANG_INTEGER;
              else if (nodeAttr.equalsIgnoreCase("java.lang.Long"))
                nodeAttrInt = JAVA_LANG_LONG;
              else if (nodeAttr.equalsIgnoreCase("java.lang.Short"))
                nodeAttrInt = JAVA_LANG_SHORT;
              else if (nodeAttr.equalsIgnoreCase("java.lang.String"))
                nodeAttrInt = JAVA_LANG_STRING;
              else if (nodeAttr.equalsIgnoreCase("java.sql.Date"))
                nodeAttrInt = JAVA_SQL_DATE;
              else if (nodeAttr.equalsIgnoreCase("java.sql.Time"))
                nodeAttrInt = JAVA_SQL_TIME;
              else if (nodeAttr.equalsIgnoreCase("java.sql.Timestamp"))
                nodeAttrInt = JAVA_SQL_TIMESTAMP;
              else if (nodeAttr.equalsIgnoreCase("java.lang.Boolean"))
                nodeAttrInt = JAVA_LANG_BOOLEAN;
              else if (nodeAttr.equalsIgnoreCase("java.lang.Byte"))
                nodeAttrInt = JAVA_LANG_BYTE;
              else
                nodeAttrInt = JAVA_LANG_OBJECT;
//ZK added this 6.5.2004

					isNumberMapp.put(nodeName.toLowerCase(), nodeAttrIsNumber);

				  isBinaryObjectMap.put(nodeName.toLowerCase(), nodeAttrIsBinary);

					isDateMap.put(nodeName.toLowerCase(), nodeAttrIsDate);

//end
//TODO Add this part of code for replacing "_" into LONG_VARCHAR for DB2 
//use something like s = Utils.replaceAll(s,"_"," ");
              javaTypeMapp.put(nodeName, nodeAttrInt);
            }
          }
        //}
        NodeList tagDbVendor = doc.getElementsByTagName("Driver");
        out:for (int i = 0; i < tagDbVendor.getLength(); i++) {
          Node nodeMain = tagDbVendor.item(i);
          NamedNodeMap attrs = nodeMain.getAttributes();
          Node nodeDriver = attrs.getNamedItem("name");
          if (nodeDriver != null) {
            strDriverName = nodeDriver.getNodeValue();
            if (driverName == null || driverName.equalsIgnoreCase(""))
              driverName = strDriverName;
            if (! (driverName == null || driverName.equalsIgnoreCase(""))) {
              if (strDriverName.equalsIgnoreCase(driverName)) {
                NodeList configNodes = nodeMain.getChildNodes();
                for (int j = 0; j < configNodes.getLength(); j++) {
                  if (configNodes.item(j).getNodeName().equalsIgnoreCase("FirstColumnResult")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value")) != null) {
                        this.iFirstColumnResult = Integer.parseInt(nodeValue.getNodeValue());
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase("RequiredUser")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value")) != null) {
                        this.bRequiredUser = (new Boolean(nodeValue.getNodeValue())).booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase(
                      "EnableJumpInResult")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bEnableJumpResult = (new Boolean(nodeValue.
                                                 getNodeValue())).booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
                      "Connection")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bConnectionPrefix = nodeValue.getNodeValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
                      "AfterLastRow")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bAfterLastRow = (new Boolean(nodeValue.getNodeValue())).
                                             booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase(
                      "EnableOrderBy")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bEnableOrderBy = (new Boolean(nodeValue.getNodeValue())).
                                              booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase(
                      "ClassName")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.strDriverClassName = (nodeValue.getNodeValue()).
                                                  toString();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
                      "RowCountEnabled")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bRowCountEnabled = (new Boolean(nodeValue.
                                                getNodeValue())).booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
                      "SetFetchSizeEnabled")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bSetFetchSizeEnabled = (new Boolean(nodeValue.
                                                    getNodeValue())).booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
                      "SetCursorNameEnabled")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bSetCursorNameEnabled = (new Boolean(nodeValue.
                                                     getNodeValue())).booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
                      "SetEmptyStringAsNull")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bSetEmptyStringAsNull = (new Boolean(nodeValue.
                                                     getNodeValue())).booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
                      "ReadingOrderRelevant")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bReadingOrderRelevant = (new Boolean(nodeValue.
                                                     getNodeValue())).booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
                      "FileSystemDatabase")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
                                        != null) {
                        this.bFileSystemDatabase = (new Boolean(nodeValue.
                                                   getNodeValue())).booleanValue();
                      }
                    }
                  } else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
                      "GetColumnsSupported")) {
                    NamedNodeMap configAttributes = null;
                    Node nodeValue = null;
                    configAttributes = configNodes.item(j).getAttributes();
                    if (configAttributes != null) {
                      if ( (nodeValue = configAttributes.getNamedItem("value"))
					  														!= null) {
	               							 this.bGetColumnsSupported = (new Boolean(nodeValue.
			       									 getNodeValue())).booleanValue();
	             							 }
                    	}
		  						} else if (configNodes.item(j).getNodeName().equalsIgnoreCase( //
										"SetMaxRowsSupported")) {
									  NamedNodeMap configAttributes = null;
									  Node nodeValue = null;
									  configAttributes = configNodes.item(j).getAttributes();
									  if (configAttributes != null) {
											if ( (nodeValue = configAttributes.getNamedItem("value"))
																		  != null) {
										   				this.bSetMaxRowsSupported = (new Boolean(nodeValue.
												  		getNodeValue())).booleanValue();
										   }
					 					 }
								  }

                } //configNodes.getLength()
                break out;
            }
          }
        }
      }
    }
    this.logger.write("full", "\treadConfigValues method is finished.");
  }

  /**
   * This method read value of javaTypeMapp parameter
   * @return value of javaTypeMapp attribute.
   */
  public Hashtable getJavaTypeMapings() {
    return this.javaTypeMapp;
  }
  /**
	 * This method read value of isNumberMapp parameter
	 * @return value of javaTypeMapp attribute.
	 */
	public Hashtable getIsNumberMapp() {
	  return this.isNumberMapp;
	}

  /**
   * Method isNumber is used for checking column type.
   * @param s String that represents column type.
   * @return true if it is numeric and false if it is not.
   */
	public boolean isNumber(String s) throws LoaderException{
		if( s != null )
				s = s.toLowerCase();
			boolean isNumber = false;
//		TODO TEST THIS!!!ZK added next line because of problems with LONG VARCHAR on DB2
					 //s = Utils.replaceAll(s," ","_");
//		TODO ZK added next line because of problems with LONG VARCHAR on DB2
				   if (s.equalsIgnoreCase("LONG VARCHAR")){

				   s = "long_varchar";
				   }
			if (this.isNumberMapp.containsKey(s)){
				Object bValue = this.isNumberMapp.get(s);
				if( bValue != null && (bValue.toString()).equalsIgnoreCase("true")) {
					//return (new Boolean(bValue.toString()).booleanValue());
					return true;
				}
			}else{
				LoaderException le = new LoaderException("Exception:",new Exception("Type "+s+" is not supported. You must add it in conf file for this database vendor."));
				throw le;
			}
		return isNumber;

	}
	/**
	  * Method isNumber is used for checking column type.
	  * @param s String that represents column type.
	  * @return true if it is numeric and false if it is not.
	  */
	public boolean isBinaryObject(String s) throws LoaderException{
	   if( s != null )
			 s = s.toLowerCase();
		   boolean isBinaryObject = false;
		   //TODO TEST THIS!!!ZK added next line because of problems with LONG VARCHAR on DB2
				//s = Utils.replaceAll(s," ","_");
		   //TODO ZK added next line because of problems with LONG VARCHAR on DB2
			 if (s.equalsIgnoreCase("LONG VARCHAR")){

			 s = "long_varchar";
			 }
			 //end
		   if (this.isBinaryObjectMap.containsKey(s)){

			   Object bValue = this.isBinaryObjectMap.get(s);
			   if( bValue != null && (bValue.toString()).equalsIgnoreCase("true")) {
				   //return (new Boolean(bValue.toString()).booleanValue());
				   return true;
			   }
		   }else{
					LoaderException le = new LoaderException("Exception:",new Exception("Type "+s+" is not supported. You must add it in conf file for this database vendor."));
					throw le;
		}
	   return isBinaryObject;

  }
	/**
		  * Method isDate is used for checking column type.
		  * @param s String that represents column type.
		  * @return true if it is numeric and false if it is not.
		  */
	public boolean isDate(String s) throws LoaderException{
		if (s != null)
			s = s.toLowerCase();
			boolean isDate  = false;
		//TODO TEST THIS!!!ZK added next line because of problems with LONG VARCHAR on DB2
			//s = Utils.replaceAll(s," ","_");
		//TODO ZK added next line because of problems with LONG VARCHAR on DB2
			if (s.equalsIgnoreCase("LONG VARCHAR")){

				s = "long_varchar";
			}
		//end
			if (this.isDateMap.containsKey(s)){
					Object dValue  = this.isDateMap.get(s);
					if ((dValue != null) && (dValue.toString()).equalsIgnoreCase("true")){
						//return (new Boolean(dValue.toString()).booleanValue());
						return true;
					}

			}else{
				LoaderException le = new LoaderException("Exception:",new Exception("Type "+s+" is not supported. You must add it in conf file for this database vendor."));
				throw le;
			}
			return isDate;

	}
  /**
	 * This method read value of bGetColumnsSupported parameter
	 * @return value of bGetColumnsSupported attribute.
	 */
	public boolean getColumnsSupported() {
	  return this.bGetColumnsSupported;
	}
	/**
		* This method read value of bSetMaxRowsSupported parameter
		* @return value of bSetMaxRowsSupported attribute.
		*/
	   public boolean getMaxRowsSupported() {
		 return this.bSetMaxRowsSupported;
	   }

  /**
   * This method read value of oidColumnName parameter
   * @return value of oidColumnName attribute.
   */
  public String getOidColumnName() {
    return this.oidColumnName;
  }

  /**
   * This method read value of versionColumnName parameter
   * @return value of versionColumnName attribute.
   */
  public String getVersionColumnName() {
    return this.versionColumnName;
  }

  /**
   * This method read value of bConnectionPrefix parameter
   * @return value of bConnectionPrefix attribute.
   */
  public String getConnectionPrefix() {
    return this.bConnectionPrefix;
  }

  /**
   * This method read value of bReadingOrderRelevant parameter
   * @return value of bReadingOrderRelevant attribute.
   */
  public boolean getReadingOrderRelevant() {
    return this.bReadingOrderRelevant;
  }

  /**
   * This method read value of bSetEmptyStringAsNull parameter
   * @return value of bSetEmptyStringAsNull attribute.
   */
  public boolean getSetEmptyStringAsNull() {
    return this.bSetEmptyStringAsNull;
  }

  /**
   * This method read value of bSetCursorNameEnabled parameter
   * @return value of bSetCursorNameEnabled attribute.
   */
  public boolean getSetCursorNameEnabled() {
    return this.bSetCursorNameEnabled;
  }

  /**
   * This method read value of bSetFetchSizeEnabled parameter
   * @return value of bSetFetchSizeEnabled attribute.
   */
  public boolean getSetFetchSizeEnabled() {
    return this.bSetFetchSizeEnabled;
  }

  /**
   * This method read value of bRowCountEnabled parameter
   * @return value of bRowCountEnabled attribute.
   */
  public boolean getRowCountEnabled() {
    return this.bRowCountEnabled;
  }

  /**
   * This method read value of bEnableOrderBy parameter
   * @return value of EnableOrderBy attribute.
   */
  public boolean getEnableOrderBy() {
    return this.bEnableOrderBy;
  }

  /**
   * This method read value of bAfterLastRow parameter
   * @return value of AfterLastRow attribute.
   */
  public boolean getAfterLastRow() {
    return this.bAfterLastRow;
  }

  /**
   * This method read value of bFileSystemDatabase parameter
   * @return value of bFileSystemDatabase attribute.
   */
  public boolean getFileSystemDatabase() {
    return this.bFileSystemDatabase;
  }

  /**
   * This method read value of bEnableJumpResult parameter
   * @return value of EnableJumpResult attribute.
   */
  public boolean getEnableJumpResult() {
    return this.bEnableJumpResult;
  }

  /**
   * This method read value of bRequiredUser parameter
   * @return value of RequiredUser attribute.
   */
  public boolean getRequiredUser() {
    return this.bRequiredUser;
  }

  /**
   * This method read value of iFirstColumnResult parameter
   * @return value of FirstColumnResult attribute.
   */
  public int getFirstColumnResult() {
    return this.iFirstColumnResult;
  }

  /**
   * This method read value of strDriverClassName parameter
   * @return value of SourceDriverClassName attribute.
   */
  public String getDriverClassName() {
    return this.strDriverClassName;
  }

  /**
   * This method read value of strVendorFileName parameter
   * @return value of VendorFileName attribute.
   */
  public String getVendorFileName() {
    return this.strVendorFileName;
  }

  /**
   * This method set value of strVendorFileName parameter
   * @param fileName set dbVendorFileName which is used for reading separated database conf files.
   */
  public void setVendorFileName(String fileName) {
    this.strVendorFileName = fileName;
  }

  /**
   * This method set value of echo parameter
   * @param logger set Logger which is used for log file.
   */
  public void setLogger(Logger logger) {
    this.logger = logger;
  }

  /**
   * This method read value from parameter
   * @return value of parameter
   */
  public String getOidDbType() {
    return this.oidDbType;
  }

  /**
   * This method read value from parameter
   * @return value of parameter
   */
  public String getVersionDbType() {
    return this.versionDbType;
  }

  /**
	 * This method read value from parameter
	 * @return value of parameter
	 */
	public String getDateFormat() {
	  return this.dateFormat;
	}

  /**
   * This method set value of confJarStructure parameter
   * @param confJarStructure is value of parameter
   */
  public void setConfJarStructure(String confJarStructure) {
    if (confJarStructure != null && !confJarStructure.equalsIgnoreCase("")) {
      if (confJarStructure.endsWith("/") || confJarStructure.endsWith("\\"))
        confJarStructure = confJarStructure.substring(0,
                           confJarStructure.length() - 1);

      this.confJarStructure = confJarStructure;
      this.useSeparateConfFiles = true;
    }
  }

}
