/**
  CounterColumns - insert auto increment values in target counter columns.
    Copyright (C) 2002-2003  Together
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 CounterColumns.java
 Date: 11.03.2003.
 @version 1.1.0
 @author: Milosevic Sinisa sinisa@prozone.co.yu
 @author: Radoslav Dutina rale@prozone.co.yu
 */

package org.webdocwf.util.loader;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import java.sql.Connection;
import java.math.BigDecimal;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;
import org.webdocwf.util.loader.logging.Logger;

/**
 *
 * CounterColumns - insert auto increment values in target counter columns.
 */
public class CounterColumns {

  private String counterTableName = null;
  private String counterNameColumn = null;
  private String counterValueColumn = null;

  private Hashtable counterName;
  private Hashtable counterStartValue;
  private Hashtable counterIncrement;
  private Hashtable targetColumnName;
  private Hashtable valueMode;
  private Hashtable counterStartValueReset;
  private Hashtable targetColumnTyp;

//subCounter columns
  private Hashtable subCounterName;
  private Hashtable subCounterStartValue;
  private Hashtable subCounterIncrement;
  private Hashtable subTargetColumnName;
  private Hashtable subValueMode;
  private Hashtable subCounterKeyColumns;
  private Hashtable subCounterKeyValues;
  private Hashtable subCounterKeyColumnsTyp;
  private Hashtable subTargetColumnTyp;

  private Vector vecCounterName = new Vector();
  private Vector vecCounterIncrement = new Vector();
  private Vector vecCounterStartValue = new Vector();
  private Vector vecTargetColumnName = new Vector();
  private Vector vecValueMode = new Vector();
  private Vector vecCounterStartValueReset = new Vector();
  private Vector vecTargetTableName = new Vector();
  private Vector vecTargetTableID = new Vector();

  private Vector vecSubCounterName = new Vector();
  private Vector vecSubCounterIncrement = new Vector();
  private Vector vecSubCounterStartValue = new Vector();
  private Vector vecSubTargetTableName = new Vector();
  private Vector vecSubTargetTableID = new Vector();
  private Vector vecSubTargetColumnName = new Vector();
  private Vector vecSubValueMode = new Vector();
  private Vector vecSubKeyColumns = new Vector();
  private Vector vecSubKeyColumnsTyp = new Vector();

  private Hashtable currentCounterValue;

  //caching the subcounter value
  private Hashtable subCounterCache = new Hashtable();
  private Logger logger;

  /**
   * Empty constructor of CounterColumns class
   */
  public CounterColumns() {

    this.counterTableName = null;
    this.counterNameColumn = null;
    this.counterValueColumn = null;
    this.counterName = new Hashtable();
    this.counterStartValue = new Hashtable();
    this.counterIncrement = new Hashtable();
    this.targetColumnName = new Hashtable();
    this.targetColumnTyp = new Hashtable();
    this.valueMode = new Hashtable();
    this.counterStartValueReset = new Hashtable();

    this.subCounterName = new Hashtable();
    this.subCounterStartValue = new Hashtable();
    this.subCounterIncrement = new Hashtable();
    this.subTargetColumnName = new Hashtable();
    this.subValueMode = new Hashtable();
    this.subCounterKeyColumns = new Hashtable();
    this.subCounterKeyValues = new Hashtable();
    this.subCounterKeyColumnsTyp = new Hashtable();
    this.subTargetColumnTyp = new Hashtable();

    this.vecCounterName = new Vector();
    this.vecCounterIncrement = new Vector();
    this.vecCounterStartValue = new Vector();
    this.vecTargetColumnName = new Vector();
    this.vecValueMode = new Vector();
    this.vecCounterStartValueReset = new Vector();
    this.vecTargetTableName = new Vector();
    this.vecTargetTableID = new Vector();
    this.currentCounterValue = new Hashtable();

    this.vecSubCounterName = new Vector();
    this.vecSubCounterIncrement = new Vector();
    this.vecSubCounterStartValue = new Vector();
    this.vecSubTargetTableName = new Vector();
    this.vecSubTargetTableID = new Vector();
    this.vecSubTargetColumnName = new Vector();
    this.vecSubValueMode = new Vector();
    this.vecSubKeyColumns = new Vector();
    this.vecSubKeyColumnsTyp = new Vector();
  }

  /**
   * This method set logger object
   */
  public void setLogger(Logger logger) {
    this.logger = logger;
  }

  /**
   * This method read value of counterTableName parameter
   * @return string name of counter table
   */
  public String getCounterTableName() {
    return this.counterTableName;
  }

  /**
   * This method read value of counterValueColumn parameter
   * @return String name of column in counter table with current values of each counter
   */
  public String getCounterValueColumn() {
    return this.counterValueColumn;
  }

  /**
   * This method read value of counterNameColumn parameter
   * @return String name of column in counter table with counter names.
   */
  public String getCounterNameColumn() {
    return this.counterNameColumn;
  }

  /**
   * This method sets the value of counterNameColumn parameter
   * @param nameColumn is name of the column
   */
  public void setCounterNameColumn(String nameColumn) {
    this.counterNameColumn = nameColumn;
  }

  /**
   * This method sets the value of counterValueColumn parameter
   * @param valueColumn is value of the column
   */
  public void setCounterValueColumn(String valueColumn) {
    this.counterValueColumn = valueColumn;
  }

  /**
   * This method sets the value of counterTableName parameter
   * @param tableName is table name
   */
  public void setCounterTableName(String tableName) {
    this.counterTableName = tableName;
  }

  /**
   * This method sets the value of:
   * counterTableName, counterNameColumn and counterValueColumn parameters
   * @param doc represents Object document
   * @param importJob represents current import job
   */
  public void readConstantColumnAttributes(Document doc, int importJob) {
    this.counterTableName = importAttributeValue(doc, "counterColumns", "counterTableName",
                            importJob);
    this.counterNameColumn = importAttributeValue(doc, "counterColumns", "counterNameColumn",
                             importJob);
    this.counterValueColumn = importAttributeValue(doc, "counterColumns", "counterValueColumn",
                              importJob);
  }

  /**
   * This method sets the value of next parameters:
   * vecCounterName, vecCounterIncrement, vecCounterStartValue, vecTargetColumnName,
   * vecValueMode, vecCounterStartValueReset, vecTargetTableName, vecTargetTableID,
   * vecSubCounterName, vecSubCounterIncrement, vecSubCounterStartValue,
   * vecSubTargetTableName, vecSubTargetTableID, vecSubTargetColumnName,
   * vecSubValueMode and vecSubKeyColumns
   * @param doc represents Object document
   * @param importJob represents current import job
   */
  public void readConstantColumnsParameters(Document doc, int importJob) {
    this.vecCounterName = importValue(doc, "counterColumn", "counterName", importJob, null);
    this.vecCounterIncrement = importValue(doc, "counterColumn", "counterIncrement", importJob, "1");
    this.vecCounterStartValue = importValue(doc, "counterColumn", "counterStartValue", importJob,
                                "0");
    this.vecTargetColumnName = importValue(doc, "counterColumn", "targetColumnName", importJob, null);
    this.vecValueMode = importValue(doc, "counterColumn", "valueMode", importJob, "setIfCreated");
    this.vecCounterStartValueReset = importValue(doc, "counterColumn", "counterStartValueReset",
                                     importJob, (new Boolean(false)).toString());
    this.vecTargetTableName = importValue(doc, "counterColumn", "targetTableName", importJob, null);
    this.vecTargetTableID = importValue(doc, "counterColumn", "targetTableID", importJob, null);

    this.vecSubCounterName = importValue(doc, "subCounterColumn", "counterName", importJob, null);
    this.vecSubCounterIncrement = importValue(doc, "subCounterColumn", "counterIncrement",
                                  importJob, "1");
    this.vecSubCounterStartValue = importValue(doc, "subCounterColumn", "counterStartValue",
                                   importJob, "0");
    this.vecSubTargetTableName = importValue(doc, "subCounterColumn", "targetTableName", importJob, null);
    this.vecSubTargetTableID = importValue(doc, "subCounterColumn", "targetTableID", importJob, null);
    this.vecSubTargetColumnName = importValue(doc, "subCounterColumn", "targetColumnName",
                                  importJob, null);
    this.vecSubValueMode = importValue(doc, "subCounterColumn", "valueMode", importJob,
                           "setIfCreated");

    for (int i = 0; i < this.vecSubCounterName.size(); i++) {
      this.vecSubKeyColumns.addElement(importSubCounterKeyValue(doc, "keyColumnName", importJob, i));
    }

  }

  /**
   * This method sets the value of Constant Columns parameters
   * @param tableName is table name
   * @param tableID is table ID
   */
  public void setConstantColumnsParameters(String tableName, String tableID) {

    Vector targetCounterName = new Vector();
    Vector targetCounterIncrement = new Vector();
    Vector targetCounterStartValue = new Vector();
    Vector targetTargetColumnName = new Vector();
    Vector targetValueMode = new Vector();
    Vector targetCounterStartValueReset = new Vector();

    Vector targetSubCounterName = new Vector();
    Vector targetSubCounterIncrement = new Vector();
    Vector targetSubCounterStartValue = new Vector();
    Vector targetSubTargetColumnName = new Vector();
    Vector targetSubValueMode = new Vector();
    Vector targetSubKeyColumns = new Vector();
    Vector subKeyColumnsKey = new Vector();

    for (int i = 0; i < this.vecTargetTableName.size(); i++) {
      if (tableName.equalsIgnoreCase(this.vecTargetTableName.get(i).toString()) &&
          tableID.equalsIgnoreCase(this.vecTargetTableID.get(i).toString())) {
        if (this.vecCounterName.get(i) != null)
          targetCounterName.addElement(this.vecCounterName.get(i).toString());
        else
          targetCounterName.addElement(null);

        if (this.vecCounterStartValue.get(i) != null)
          targetCounterStartValue.addElement(this.vecCounterStartValue.get(i).toString());
        else
          targetCounterStartValue.addElement(null);

        if (this.vecCounterIncrement.get(i) != null)
          targetCounterIncrement.addElement(this.vecCounterIncrement.get(i).toString());
        else
          targetCounterIncrement.addElement(null);

        if (this.vecTargetColumnName.get(i) != null)
          targetTargetColumnName.addElement(this.vecTargetColumnName.get(i).toString());
        else
          targetTargetColumnName.addElement(null);

        if (this.vecValueMode.get(i) != null)
          targetValueMode.addElement(this.vecValueMode.get(i).toString());
        else
          targetValueMode.addElement(null);

        if (this.vecCounterStartValueReset.get(i) != null)
          targetCounterStartValueReset.addElement(this.vecCounterStartValueReset.get(i).toString());
        else
          targetCounterStartValueReset.addElement(null);

//sub counter columns
      }
    }
    for (int i = 0; i < this.vecSubTargetTableName.size(); i++) {
      if (tableName.equalsIgnoreCase(this.vecSubTargetTableName.get(i).toString()) &&
          tableID.equalsIgnoreCase(this.vecSubTargetTableID.get(i).toString())) {
        if (this.vecSubCounterName.get(i) != null) {
          targetSubCounterName.addElement(this.vecSubCounterName.get(i).toString());
        } else
          targetSubCounterName.addElement(null);
        if (this.vecSubCounterStartValue.get(i) != null)
          targetSubCounterStartValue.addElement(this.vecSubCounterStartValue.get(i).toString());
        else
          targetSubCounterStartValue.addElement(null);
        if (this.vecSubCounterIncrement.get(i) != null)
          targetSubCounterIncrement.addElement(this.vecSubCounterIncrement.get(i).toString());
        else
          targetSubCounterIncrement.addElement(null);
        if (this.vecSubTargetColumnName.get(i) != null)
          targetSubTargetColumnName.addElement(this.vecSubTargetColumnName.get(i).toString());
        else
          targetSubTargetColumnName.addElement(null);
        if (this.vecSubValueMode.get(i) != null)
          targetSubValueMode.addElement(this.vecSubValueMode.get(i).toString());
        else
          targetSubValueMode.addElement(null);
        if (this.vecSubKeyColumns.get(i) != null)
          targetSubKeyColumns.addElement(this.vecSubKeyColumns.get(i));
        else
          targetSubKeyColumns.addElement(null);

      }
    }
    this.counterName.put(tableName + "_" + tableID, targetCounterName);
    this.counterIncrement.put(tableName + "_" + tableID, targetCounterIncrement);
    this.counterStartValue.put(tableName + "_" + tableID, targetCounterStartValue);
    this.targetColumnName.put(tableName + "_" + tableID, targetTargetColumnName);
    this.valueMode.put(tableName + "_" + tableID, targetValueMode);
    this.counterStartValueReset.put(tableName + "_" + tableID, targetCounterStartValueReset);

// subCounter columns
    this.subCounterName.put(tableName + "_" + tableID, targetSubCounterName);
    this.subCounterIncrement.put(tableName + "_" + tableID, targetSubCounterIncrement);
    this.subCounterStartValue.put(tableName + "_" + tableID, targetSubCounterStartValue);
    this.subTargetColumnName.put(tableName + "_" + tableID, targetSubTargetColumnName);
    this.subValueMode.put(tableName + "_" + tableID, targetSubValueMode);
    this.subCounterKeyColumns.put(tableName + "_" + tableID, targetSubKeyColumns);

  }

  private int getVectorsSize() {
    if (this.vecCounterName != null)
      return this.vecCounterName.size();
    else
      return 0;
  }

  /**
   * This method read value of counterNames parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getCounterName(String tableName, String tableID) {
    return (Vector)this.counterName.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of counterIncrement parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return Vector of increment values for all counter columns in table tableName.
   */
  public Vector getCounterIncrement(String tableName, String tableID) {
    return (Vector)this.counterIncrement.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of counterStartValue parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return Vector of start values for all counter columns in table tableName.
   */
  public Vector getCounterStartValue(String tableName, String tableID) {
    return (Vector)this.counterStartValue.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of targetColumnName parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return Vector of counter column names for all counter columns in table tableName.
   */
  public Vector getTargetColumnName(String tableName, String tableID) {
    return (Vector)this.targetColumnName.get(tableName + "_" + tableID);

  }

  /**
   * This method read value of targetColumnTyp parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return Vector of counter column types for all counter columns in table tableName.
   */
  public Vector getTargetColumnTyp(String tableName, String tableID) {
    return (Vector)this.targetColumnTyp.get(tableName + "_" + tableID);

  }

  /**
   * This method read value of valueMode parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getValueMode(String tableName, String tableID) {
    return (Vector)this.valueMode.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of counterStartValueReset parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getCounterStartValueReset(String tableName, String tableID) {
    return (Vector)this.counterStartValueReset.get(tableName + "_" + tableID);
  }

//subCounter columns
  /**
   * This method read value of subCounterName parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getSubCounterName(String tableName, String tableID) {
    return (Vector)this.subCounterName.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of getSubCounterIncrement parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getSubCounterIncrement(String tableName, String tableID) {
    return (Vector)this.subCounterIncrement.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of getSubCounterStartValue parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getSubCounterStartValue(String tableName, String tableID) {
    return (Vector)this.subCounterStartValue.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of subTargetColumnName parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getSubTargetColumnName(String tableName, String tableID) {
    return (Vector)this.subTargetColumnName.get(tableName + "_" + tableID);

  }

  /**
   * This method read value of subTargetColumnTyp parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getSubTargetColumnTyp(String tableName, String tableID) {
    return (Vector)this.subTargetColumnTyp.get(tableName + "_" + tableID);

  }

  /**
   * This method read value of subValueMode parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getSubValueMode(String tableName, String tableID) {
    return (Vector)this.subValueMode.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of subCounterKeyColumns parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getSubCounterKeyColumns(String tableName, String tableID) {
    return (Vector)this.subCounterKeyColumns.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of subCounterKeyColumnsTyp parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getSubCounterKeyColumnsTyp(String tableName, String tableID) {
    return (Vector)this.subCounterKeyColumnsTyp.get(tableName + "_" + tableID);
  }

  /**
   * This method read value of subCounterKeyValues parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return vector
   */
  public Vector getSubCounterKeyValues(String tableName, String tableID) {
    return (Vector)this.subCounterKeyValues.get(tableName + "_" + tableID);
  }

  /**
   * This method set value of subCounterKeyValues parameter
   * @param tableName defines table name
   * @param tableID defines table ID
   * @param values is hashtable new key values
   */
  public void setSubCounterKeyValues(String tableName, String tableID, Hashtable values) {

    Enumeration keys = values.keys();
    Vector vecColumns = (Vector)this.subCounterKeyColumns.get(tableName + "_" + tableID);
    Vector vecValues = new Vector(vecColumns.size());

    String column;
    for (int i = 0; i < vecColumns.size(); i++) {
      Vector subValues = new Vector();
      for (int j = 0; j < ( (Vector)vecColumns.get(i)).size(); j++) {
        column = ( (Vector)vecColumns.get(i)).get(j).toString();
        subValues.add(values.get(column));
      }
      vecValues.add(subValues);
    }
    this.subCounterKeyValues.put(tableName + "_" + tableID, vecValues);
  }

  /**
   * This method set values for Counter columns
   * @param tableName is table name
   * @param tableID is table ID
   */
  public void setCounterValue(String tableName, String tableID) {
    Vector targetCounterValue = new Vector();
    Vector oldValues = getCounterValue(tableName, tableID);
    Vector counterIncrement = getCounterIncrement(tableName, tableID);
    for (int i = 0; i < counterIncrement.size(); i++) {
      BigDecimal newValue = new BigDecimal(oldValues.get(i).toString());
      newValue = newValue.add(new BigDecimal(counterIncrement.get(i).toString()));
      targetCounterValue.addElement(newValue);
    }
    this.currentCounterValue.put(tableName + "_" + tableID, targetCounterValue);
  }

  /**
   * This method set values for currentCounterValue parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @return Vector of current counter values for table - tableName
   */
  public Vector getCounterValue(String tableName, String tableID) {
    return (Vector)this.currentCounterValue.get(tableName + "_" + tableID);
  }

  /**
   * This method set values for currentCounterValue parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @param conn is connection to target database
   * @param firstColumn is first column
   * @throws SQLException
   */
  public void setTargetColumnStartValues(String tableName, String tableID, Connection conn,
      int firstColumn) throws SQLException {
    Vector values = new Vector();
    //TODO ZORAN
	  Vector valuesFromTable = readCounter(tableName, tableID, conn, firstColumn);
    for (int i = 0; i < this.getCounterName(tableName, tableID).size(); i++) {
      if (this.getCounterStartValueReset(tableName,
          tableID).get(i).toString().equalsIgnoreCase("true")) {
        if (this.getCounterStartValue(tableName, tableID).get(i) != null) {
          BigDecimal newValue = new BigDecimal( (this.getCounterStartValue(tableName,
                                tableID)).get(i).toString());
          values.addElement(newValue);
        } else
          values.addElement(null);
      } else if (this.getCounterStartValueReset(tableName,
          tableID).get(i).toString().equalsIgnoreCase("false")) {
        if (this.getCounterStartValue(tableName, tableID).get(i) != null) {
          BigDecimal newValue = new BigDecimal(valuesFromTable.get(i).toString());
          values.addElement(newValue);
        } else
          values.addElement(null);
      }
    }
    this.currentCounterValue.put(tableName + "_" + tableID, values);
  }

  /**
   * Method importValue reads values from desired XML tag and puts them into Vector.
   * @param doc Parsed import XML file.
   * @param tagName The name of XML tag.
   * @param strAttrName The name of tag attribute which reads input strValue.
   * @param iImportJobItem Number of ImportDefinition tag which is processed.
   * @param defaultValue The default value of strattrname attribute.
   * @return Vector of importing values.
   */
  private Vector importValue(Document doc, String tagName, String strAttrName,
      int iImportJobItem, String defaultValue) {
    Vector strValue = new Vector();
    NodeList tagBasic = doc.getElementsByTagName("importDefinition");
    if (tagBasic.getLength() != 0) {
      Element docFragment = (Element)tagBasic.item(iImportJobItem);
      NodeList tag = docFragment.getElementsByTagName(tagName);
      for (int i = 0; i < tag.getLength(); i++) {
        String nodeValue = "";
        if (strAttrName != null) {
          NamedNodeMap attrs = tag.item(i).getAttributes();
          Node nodeResult = attrs.getNamedItem(strAttrName);
          if (nodeResult != null)
            nodeValue = nodeResult.getNodeValue();
          else
            nodeValue = defaultValue;
          strValue.addElement(nodeValue);
        } else {
          NodeList nodeText = tag.item(i).getChildNodes();
          if (nodeText.item(0) != null) {
            nodeValue = nodeText.item(0).getNodeValue();
            strValue.addElement(nodeValue);
          }
        }
      }
    }
    return strValue;
  }

  /**
   * Method importSubCounterKeyValue reads values from strAttrName attribute
   * XML tag and puts them into Vector.
   * @param doc Parsed import XML file.
   * @param strAttrName The name of tag attribute which reads input strValue.
   * @param iImportJobItem Number of ImportDefinition tag which is processed.
   * @param iSubCounterItem Number of subCounterColumn tag which is processed.
   * @return Vector of importing values.
   */
  private Vector importSubCounterKeyValue(Document doc, String strAttrName,
      int iImportJobItem, int iSubCounterItem) {
    Vector strValue = null;
    NodeList tagBasic = doc.getElementsByTagName("importDefinition");
    if (tagBasic.getLength() != 0) {
      Element docFragment = (Element)tagBasic.item(iImportJobItem);
      NodeList tag = docFragment.getElementsByTagName("subCounterColumn");
      if (tag.getLength() != 0) {
        Element docCounterColumn = (Element)tag.item(iSubCounterItem);
        NodeList tagSubCounterColumn = docCounterColumn.getElementsByTagName("subCounterKeyColumn");
        strValue = new Vector();
        for (int i = 0; i < tagSubCounterColumn.getLength(); i++) {
          String nodeValue = "";
          if (strAttrName != null) {
            NamedNodeMap attrs = tagSubCounterColumn.item(i).getAttributes();
            Node nodeResult = attrs.getNamedItem(strAttrName);
            if (nodeResult != null) {
              nodeValue = nodeResult.getNodeValue();
            } else
              nodeValue = null;
            strValue.addElement(nodeValue);

          } else {
            NodeList nodeText = tag.item(i).getChildNodes();
            if (nodeText.item(0) != null) {
              nodeValue = nodeText.item(0).getNodeValue();
              strValue.addElement(nodeValue);
            }
          }
        }
      }
    }
    return strValue;
  }

//  /**
//   * This method check the update operation
//   * @param tableName is table name
//   * @param tableID is table ID
//   * @param conn is connection
//   * @return true if update operation OK, false otherwise
//   * @throws SQLException
//   */
//  public boolean updateCounter(String tableName, String tableID,
//      Connection conn) throws SQLException {
//
//    String strQuery = null;
//
//    Vector counterNames = this.getCounterName(tableName, tableID);
//    Vector counterValues = this.getCounterValue(tableName, tableID);
//    Vector counterIncrements = this.getCounterIncrement(tableName, tableID);
//    for (int i = 0; i < counterNames.size(); i++) {
//      Statement stmt = conn.createStatement();
//      BigDecimal newValue = new BigDecimal(counterValues.get(i).toString());
//      strQuery = queryUpdateCounter(counterNames.get(i).toString(), newValue);
//      stmt.executeUpdate(strQuery);
//      stmt.close();
//    }
//    return true;
//  }

	/**
	  * This method check the update operation
	  * @param conn is connection
	  * @throws SQLException
	  */    
  public void updateCounter(Connection conn) throws SQLException{
  	String strQuery = "";
  	for (int i=0;i<this.vecTargetTableName.size();i++){
  		String tableName = this.vecTargetTableName.get(i).toString();
			String tableID = this.vecTargetTableID.get(i).toString();
			Vector counterNames = this.getCounterName(tableName, tableID);
			Vector counterValues = this.getCounterValue(tableName, tableID);
			Vector counterIncrements = this.getCounterIncrement(tableName, tableID);
			for (int j = 0; j < counterNames.size(); j++) {
	 			Statement stmt = conn.createStatement();
	 			BigDecimal newValue = new BigDecimal(counterValues.get(i).toString());
	 			strQuery = queryUpdateCounter(counterNames.get(i).toString(), newValue);
	 			stmt.executeUpdate(strQuery);
	 			stmt.close();
			}
  	}	
  }

  /**
   * This method sets the value of counter parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @param conn is connection
   * @return true
   * @throws SQLException
   */
  private boolean insertCounter(String tableName, String tableID,
      Connection conn) throws SQLException {

    String strQuery = null;

    Vector counterNames = this.getCounterName(tableName, tableID);
    Vector startValues = this.getCounterIncrement(tableName, tableID);
    for (int i = 0; i < counterNames.size(); i++) {
      Statement stmt = conn.createStatement();
      BigDecimal newValue = new BigDecimal(startValues.get(i).toString());
      strQuery = queryInsertCounter(counterNames.get(i).toString(), newValue);
      stmt.execute(strQuery);
      stmt.close();
    }
    return true;
  }

  /**
   * This method read value of counter parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @param conn is connection
   * @param firstColumn is first column
   * @return true
   * @throws SQLException
   */
  private Vector readCounter(String tableName, String tableID, Connection conn,
      int firstColumn) throws SQLException {

    String strQuery = null;
    BigDecimal value = new BigDecimal(0);
    Vector startValues = new Vector();
    Vector counterNames = this.getCounterName(tableName, tableID);
    Vector counterValues = this.getCounterValue(tableName, tableID);
    Vector counterIncrements = this.getCounterIncrement(tableName, tableID);
    Vector defaultStartValues = this.getCounterIncrement(tableName, tableID);

    for (int i = 0; i < counterNames.size(); i++) {
      Statement stmt = conn.createStatement();
      strQuery = querySelectCounter(counterNames.get(i).toString());
	  	this.logger.write("full", "\tQuery '" + strQuery + "' will be executed");
      ResultSet rset = stmt.executeQuery(strQuery);
      if (rset.next()) {
        if (firstColumn == 0)
          value = new BigDecimal(rset.getString(0));
        else
          value = new BigDecimal(rset.getString(1));
      } else {
        insertCounter(tableName, tableID, conn);
        value = new BigDecimal(defaultStartValues.get(i).toString());
      }
      stmt.close();
      rset.close();
      startValues.addElement(value);

    }
    return startValues;
  }

  /**
   *
   * @param counterColumnName is name of the column
   * @return query
   */
  private String querySelectCounter(String counterColumnName) {
    String query = "select ";

    query += this.counterValueColumn + " from " + this.counterTableName + " where " +
        this.counterNameColumn + " = '" + counterColumnName + "'";
    return query;
  }

  /**
   *
   * @param counterColumnName is column name
   * @param counterColumnValue is column value
   * @return query
   */
  private String queryInsertCounter(String counterColumnName, BigDecimal counterColumnValue) {
    String query = "INSERT into ";

    query += this.counterTableName + " (" + this.counterNameColumn + ", " + this.counterValueColumn
        + ") VALUES('" +
        counterColumnName + "', " + counterColumnValue.intValue() + ")";
    return query;
  }

  /**
   *
   * @param counterColumnName is colulmn name
   * @param newValue is column value
   * @return query
   */

  private String queryUpdateCounter(String counterColumnName, BigDecimal newValue) {
    String query = "update ";
    query += this.counterTableName + " set " +
        this.counterValueColumn + " = " + newValue.toString() + " where " +
        this.counterNameColumn + " = '" + counterColumnName + "'";
    return query;
  }

  /**
   * This method read value of sub counter parameter
   * @param tableName is table name
   * @param tableID is table ID
   * @param conn is connection
   * @param firstColumn is first column
   * @return vector
   * @throws SQLException
   */
  public Vector readSubCounterValue(String tableName, String tableID, Connection conn,
      int firstColumn, String tableMode, ConfigReader targetConfigReader) throws SQLException, LoaderException {

    BigDecimal value = new BigDecimal(0);
    Vector startValues = new Vector();

    this.logger.write("full", "\treadSubCounterValue method is started");

    Vector columns = this.getSubCounterKeyColumns(tableName, tableID);
    Vector typs = this.getSubCounterKeyColumnsTyp(tableName, tableID);
    Vector subColumnNames = this.getSubTargetColumnName(tableName, tableID);
    Vector subIncrement = this.getSubCounterIncrement(tableName, tableID);
    Vector subCounterStartValues = this.getSubCounterStartValue(tableName, tableID);
    Vector subCounterKeyColumnValues = this.getSubCounterKeyValues(tableName, tableID);

    for (int i = 0; i < subColumnNames.size(); i++) {

      String strQuery = "select ";
      Statement stmt = conn.createStatement();
      strQuery += subColumnNames.get(i).toString() + " from " + tableName + " where ";
      for (int j = 0; j < ( (Vector)columns.get(i)).size(); j++) {
        if ( ( (Vector)subCounterKeyColumnValues.get(i)).get(j) != null &&
            !( (Vector)subCounterKeyColumnValues.get(i)).get(j).toString().equalsIgnoreCase("")) {
//ZK change this from CheckType to targetConfigReader
         try {
            if (!targetConfigReader.isNumber( ( (Vector)typs.get(i)).get(j).toString()))
                strQuery += ( (Vector)columns.get(i)).get(j).toString() + " = '"
                    + ( (Vector)subCounterKeyColumnValues.get(i)).get(j).toString() + "' and ";
              else
                strQuery += ( (Vector)columns.get(i)).get(j).toString() + " = "
                    + ( (Vector)subCounterKeyColumnValues.get(i)).get(j).toString() + " and ";
        } catch (LoaderException e) {
           LoaderException le= new LoaderException("Exception:This sql type isn't present in conf file for target database. Yuo must add it into conf file.",(Throwable)e);
           throw le;
        }
        } else
          strQuery += ( (Vector)columns.get(i)).get(j).toString() + " is null and ";
      }

      if (strQuery.endsWith(" and "))
        strQuery = strQuery.substring(0, strQuery.length() - 5);

      if (tableMode.equalsIgnoreCase("cache")) {
        //caching the subcounter value
        Object obj = subCounterCache.get(strQuery);
        if (obj == null) { //query isn't cached before
          this.logger.write("full", "\tQuery '" + strQuery + "' will be executed");
          ResultSet rset = stmt.executeQuery(strQuery);
          BigDecimal currentValue = new BigDecimal(0);
          if (rset.next()) {
            if (firstColumn == 0) {
              value = new BigDecimal(rset.getString(0));
              currentValue = value;
              while (rset.next()) {
                value = new BigDecimal(rset.getString(0));
                if (currentValue.intValue() < value.intValue())
                  currentValue = value;
              }
            } else { //firstColumnResult==1
              value = new BigDecimal(rset.getString(1));
              currentValue = value;
              while (rset.next()) {
                value = new BigDecimal(rset.getString(1));
                if (currentValue.intValue() < value.intValue())
                  currentValue = value;
              }
            }
            value = value.add(new BigDecimal(subIncrement.get(i).toString()));
          } else {
            value = new BigDecimal(subCounterStartValues.get(i).toString());
          }
          subCounterCache.put(strQuery, value);
          stmt.close();
        } else { //query is cached before
          BigDecimal oldValue = (BigDecimal)obj;
          value = oldValue.add(new BigDecimal(subIncrement.get(i).toString()));
          subCounterCache.remove(strQuery);
          subCounterCache.put(strQuery, value);
        }
      } else {//tableMode=query
        this.logger.write("full", "\tQuery '" + strQuery + "' will be executed");
        ResultSet rset = stmt.executeQuery(strQuery);
        BigDecimal currentValue = new BigDecimal(0);
        if (rset.next()) {
          if (firstColumn == 0) {
            value = new BigDecimal(rset.getString(0));
            currentValue = value;
            while (rset.next()) {
              value = new BigDecimal(rset.getString(0));
              if (currentValue.intValue() < value.intValue())
                currentValue = value;
            }
          } else { //firstColumnResult==1
            value = new BigDecimal(rset.getString(1));
            currentValue = value;
            while (rset.next()) {
              value = new BigDecimal(rset.getString(1));
              if (currentValue.intValue() < value.intValue())
                currentValue = value;
            }
          }
          value = value.add(new BigDecimal(subIncrement.get(i).toString()));
        } else {
          value = new BigDecimal(subCounterStartValues.get(i).toString());
        }
        stmt.close();

      }
      startValues.addElement(value);
    }
    this.logger.write("full", "\treadSubCounterValue method is finished");
    return startValues;
  }

  /**
   * This method reset cach for subcounter
   */
  public void resetSubCounterCache(){
    if(this.subCounterCache.size()>0)
      this.subCounterCache.clear();
  }


  /**
   * Method importAttributeValue reads value for strAttrName attribute in strTagName tag.
   * This method return this value.
   * @param doc Parsed import XML file.
   * @param strTagName The name of tag where attribute is situated.
   * @param strAttrName The name of tag attribute which reads input value.
   * @param iImportJobItem Number of ImportDefinition tag which is processed.
   * @return String - importing value.
   */
  private String importAttributeValue(Document doc, String strTagName, String strAttrName,
      int iImportJobItem) {
    String strValue = "";
    NodeList tagBasic = doc.getElementsByTagName("importDefinition");
    if (tagBasic.getLength() != 0) {
      Element docFragment = (Element)tagBasic.item(iImportJobItem);
//      NodeList tag = docFragment.getElementsByTagName(tagName);
//      for (int i = 0; i < tag.getLength(); i++) {

      tagBasic = docFragment.getElementsByTagName(strTagName);
      if (tagBasic.getLength() != 0) {
        docFragment = (Element)tagBasic.item(0);
        if (docFragment != null)
          strValue = docFragment.getAttribute(strAttrName);
      }
    }
    return strValue;
  }

  /**
   * Method counterColumnTypes is used to put types of constant columns into
   * global vector sorted in target tables. If there is an error, Exception
   * "SQLException" or "NullPointerException" is thrown.
   * @param c Connection to target database.
   * @param tableName is table name
   * @param tableID is table ID
   * @param firstColumn is first column
   * @throws SQLException Constructs an SQLException object with a reason.
   * @throws NullPointerException Constructs a NullPointerException with the specified detail message.
   */
  public void counterColumnTypes(String tableName, String tableID, Connection c,
      int firstColumn, boolean columnsSuportedTarget, ConfigReader configReaderTarget) throws SQLException, NullPointerException {
    int iCnt = 0;
    try {
      Vector columnNames = this.getSubCounterKeyColumns(tableName, tableID);
      Statement stmtConstant = c.createStatement();
      Vector typs = new Vector();
      Vector subTyps = new Vector();
      String strQuery = "select ";
	  	ResultSet rsetConstant=null;
	  
      if (columnNames.size() != 0) {
        for (int i = 0; i < columnNames.size(); i++) {
          for (int j = 0; j < ( (Vector)columnNames.get(i)).size(); j++) {
            strQuery += ( (Vector)columnNames.get(i)).get(j).toString() +
                ", ";
          }
          strQuery = strQuery.substring(0, strQuery.length() - 2);
          strQuery += " from " + tableName;
//ZK change this. Because of problems with getColumnTypeName()method. Some drivers doesn't support it.
//start 
    		if (columnsSuportedTarget){
    			  
						rsetConstant = c.getMetaData().getColumns( c.getCatalog(), null, tableName, "%" );
						String columnName = "";
						String columnType = "";
						while(rsetConstant.next()){
							columnName = rsetConstant.getString(3+firstColumn);
							columnType = rsetConstant.getString(5+firstColumn);
							Vector temp = (Vector)columnNames.get(i); 				
							for (int j = 0; j < temp.size(); j++) {									
									if( temp.get(j).toString().equalsIgnoreCase( columnName ) ){
										typs.add(columnType);
									}
							}
						}		
        }else{//TODO ZK ADDED stmtConstant.setMaxRows(1). Place this as parameter in conf file, like maxRowsSuported
					if (configReaderTarget.getMaxRowsSupported()){
        		 stmtConstant.setMaxRows(1);
    			}
             rsetConstant = stmtConstant.executeQuery(strQuery);
             
			 for (int j = 0; j < ( (Vector)columnNames.get(i)).size(); j++) {

			  typs.add(rsetConstant.getMetaData().getColumnTypeName(j + firstColumn));
          }

		  }		
          rsetConstant.close();
          subTyps.addElement(typs);
        }
      }      
      this.subCounterKeyColumnsTyp.put(tableName + "_" + tableID, subTyps);
      stmtConstant.close();

    }
    catch (SQLException ex) {
      throw ex;
    }
    catch (NullPointerException ex) {
      throw ex;
    }
  }

  /**
   * This method reset all variables
   */
  public void reset() {
    this.counterTableName = null;
    this.counterNameColumn = null;
    this.counterValueColumn = null;
    this.counterName = new Hashtable();
    this.counterStartValue = new Hashtable();
    this.counterIncrement = new Hashtable();
    this.targetColumnName = new Hashtable();
    this.targetColumnTyp = new Hashtable();
    this.valueMode = new Hashtable();
    this.counterStartValueReset = new Hashtable();

    this.subCounterName = new Hashtable();
    this.subCounterStartValue = new Hashtable();
    this.subCounterIncrement = new Hashtable();
    this.subTargetColumnName = new Hashtable();
    this.subValueMode = new Hashtable();
    this.subCounterKeyColumns = new Hashtable();
    this.subCounterKeyValues = new Hashtable();
    this.subCounterKeyColumnsTyp = new Hashtable();
    this.subTargetColumnTyp = new Hashtable();

    this.vecCounterName = new Vector();
    this.vecCounterIncrement = new Vector();
    this.vecCounterStartValue = new Vector();
    this.vecTargetColumnName = new Vector();
    this.vecValueMode = new Vector();
    this.vecCounterStartValueReset = new Vector();
    this.vecTargetTableName = new Vector();
    this.vecTargetTableID = new Vector();
    this.currentCounterValue = new Hashtable();

    this.vecSubCounterName = new Vector();
    this.vecSubCounterIncrement = new Vector();
    this.vecSubCounterStartValue = new Vector();
    this.vecSubTargetTableName = new Vector();
    this.vecSubTargetTableID = new Vector();
    this.vecSubTargetColumnName = new Vector();
    this.vecSubValueMode = new Vector();
    this.vecSubKeyColumns = new Vector();
    this.vecSubKeyColumnsTyp = new Vector();
    this.subCounterCache.clear();
  }
}
