/*

  Loader - tool for transfering data from one JDBC source to another and
  doing transformations during copy.

    Copyright (C) 2002-2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 Loader.java
 Date: 03.03.2003.
 @version 2.1 alpha
 @author:
 Radoslav Dutina rale@prozone.co.yu

*/
package org.webdocwf.util.loader;

import java.sql.*;
import java.io.*;
import java.util.*;
import org.webdocwf.util.loader.logging.*;

/**
 *
 * DataCleaning class contain method for executing data cleaning process
 * @author Radoslav Dutina
 * @version 1.0
 */
public class DataCleaning {

  private Logger logger;
  private Statement stmt;
  private ResultSet rsetTarget;
  private ResultSet rs;
  private Hashtable colNamesDataTypes=new Hashtable();
  private Hashtable colNamesDataLenght=new Hashtable();
  private String currentTableName="";

  private Hashtable colNamesDataTypesUpdate=new Hashtable();
  private Hashtable colNamesDataLenghtUpdate=new Hashtable();
  private ResultSet rsUpdate;

  // Default values for log table
  private String logTableName="LOGTABLENAME";
  private String logTable="LOGTABLE";
  private String logColumnName="LOGCOLUMNNAME";
  private String logRowNumber="LOGROWNUMBER";
  private String logOriginalValue="LOGORIGINALVALUE";
  private String logNewValue="LOGNEWVALUE";
  private String logImportDefinitionName="LOGIMPORTDEFINITIONNAME";
  private String logOperationName="LOGOPERATIONNAME";
  private String logTypeName="LOGTYPENAME";

  private boolean logTableExists=false;
  private boolean checkIsDone=false;
  
  private ConfigReader configReader;


  /**
   * Constructor of DataCleaning class, without parameters.
   */
  public DataCleaning(ConfigReader configReader) {
    this.configReader = configReader;
  }

  /**
   * This method set the value of parameter logTableName
   * @param logTableName is value of parameter
   */
  public void setLogTableName(String logTableName){
    this.logTableName=logTableName;
  }

  /**
   * This method set the value of parameter logTable
   * @param logTable is value of parameter
   */
  public void setLogTable(String logTable){
    this.logTable=logTable;
  }

  /**
   * This method set the value of parameter logColumnName
   * @param logColumnName is value of parmeter
   */
  public void setLogColumnName(String logColumnName){
    this.logColumnName=logColumnName;
  }

  /**
   * This method set the value of parameter logRowNumber
   * @param logRowNumber is value of parameter
   */
  public void setLogRowNumber(String logRowNumber){
    this.logRowNumber=logRowNumber;
  }

  /**
   * This method set the value of parameter logOriginalValue
   * @param logOriginalValue is value of parameter
   */
  public void setLogOriginalValue(String logOriginalValue){
    this.logOriginalValue=logOriginalValue;
  }

  /**
   * This method set the value of parameter logNewValue
   * @param logNewValue is value of parameter
   */
  public void setLogNewValue(String logNewValue){
    this.logNewValue=logNewValue;
  }

  /**
   * This method set the value of parameter logImportDefinitionName
   * @param logImportDefinitionName is value of parameter
   */
  public void setLogImportDefinitionName(String logImportDefinitionName){
    this.logImportDefinitionName=logImportDefinitionName;
  }

  /**
   * This method set the value of parameter logOperationName
   * @param logOperationName is value of parameter
   */
  public void setLogOperationName(String logOperationName){
    this.logOperationName=logOperationName;
  }

  /**
   * This method set the value of parameter logTypeName
   * @param logTypeName is value of parameter
   */
  public void setLogTypeName(String logTypeName){
    this.logTypeName=logTypeName;
  }

  /**
   * This method read value from parameter logTableName
   * @return value of parameter
   */
  public String getLogTableName(){
    return this.logTableName;
  }

  /**
   * This method read value from parameter logTable
   * @return value of parameter
   */
  public String getLogTable(){
    return this.logTable;
  }

  /**
   * This method read value from parameter logColumnName
   * @return value of parameter
   */
  public String getLogColumnName(){
    return this.logColumnName;
  }

  /**
   * This method read value from parameter logRowNumber
   * @return value of parameter
   */
  public String getLogRowNumber(){
    return this.logRowNumber;
  }

  /**
   * This method read value from parameter logOriginalValue
   * @return value of parameter
   */
  public String getLogOriginalValue(){
    return this.logOriginalValue;
  }

  /**
   * This method read value from parameter logNewValue
   * @return value of parameter
   */
  public String getLogNewValue(){
    return this.logNewValue;
  }

  /**
   * This method read value from parameter logImportDefinitionName
   * @return value of parameter
   */
  public String getLogImportDefinitionName(){
    return this.logImportDefinitionName;
  }

  /**
   * This method read value from parameter logImportDefinitionName
   * @return value of parameter
   */
  public String getLogOperationName(){
    return this.logOperationName;
  }

  /**
   * This method read value from parameter logTypeName
   * @return value of parameter
   */
  public String getLogTypeName(){
    return this.logTypeName;
  }

  /**
   * This method write message in to log table if insert/update fails
   * @param tableName is name of the table which is in the process
   * @param conn is connection to target database (table)
   * @param rowNumber is current row number
   * @param typeOfInsert is the parameter which may be 'insert' or 'update'
   * @param msg is error message
   * @param importDefinitionName is name of the import definition job
   */
  public void cleaningInsert(String tableName, Connection conn, int rowNumber, String typeOfInsert,
                             String msg, String importDefinitionName) {
    this.logger.write("full","\tBecause DataCleaning features is turn on, some replacement were made:");
    this.logger.write("full","\t "+typeOfInsert+":FAILS: in table "+tableName+", in row "+(rowNumber+1)+" . "+
                         msg );
    try{
      String[] types = {"TABLE"};
      String catalogName=conn.getCatalog();
      stmt=conn.createStatement();
//      msg=msg.replaceAll("'","''");
      msg=Utils.replaceAll(msg, "'","''");
      ResultSet check=null;
      try{
        if(!checkIsDone){
          check = conn.getMetaData().getTables(catalogName, null, getLogTableName(), types);
          this.checkIsDone = true;
          this.logTableExists = check.next();
          check.close();
        }
      }catch(UnsupportedOperationException ex){
        String message="Error while trying to get meta data from target table."+"\n"+
            "\tMethod getMetaDatata().getTables() is not supported.";
        throw new SQLException(message);
      }
      if(this.logTableExists){
        stmt.executeUpdate("INSERT INTO "+getLogTableName()+" ("+
        getLogImportDefinitionName()+", "+getLogOperationName()+", "+getLogTypeName()+", "+
        getLogTable()+", "+getLogColumnName()+", "+getLogRowNumber()+", "+
        getLogOriginalValue()+", "+getLogNewValue()+
        ")"+
        " VALUES ("+
        "'"+importDefinitionName+"','"+typeOfInsert+"','ERROR','"+
        tableName+"','','"+(rowNumber+1)+"','"+
        msg+"','"+typeOfInsert+" FAILS'"+
        ")");
      }else{
        this.logger.write("full","\t "+getLogTableName()+" does not exists, or is invalid!");
      }
//      check.close();
      stmt.close();
    }catch(SQLException ex){
      this.logger.write("full","\t Error: "+ex.getMessage());
    }

  }


  /**
   * This method write message in to log table if some data in sql statements has value
   * @param tableName is name of the table which is in the process
   * @param columnName is the name of column in current table
   * @param replacement is new value of data
   * @param rowNumber is current row number
   * @param conn is connection to target database (table)
   * @param typeOfInsert is the parameter which may be 'insert' or 'update'
   * @param importDefinitionName is name of the import definition job
   */
  public void cleaningColumnValues(String tableName, String columnName, String replacement,
                                   int rowNumber, Connection conn, String typeOfInsert,
                                   String importDefinitionName){
    this.logger.write("full","\tBecause DataCleaning features is turn on, some replacement were made:");
    this.logger.write("full","\t "+typeOfInsert+":REPLACED VALUES: In table "+tableName+", column "+columnName+", value 'null' were replaced" );
    this.logger.write("full","\t with value '"+replacement+"' (row number "+(rowNumber+1)+")" );
    try{
      String[] types = {"TABLE"};
      String catalogName=conn.getCatalog();
      stmt=conn.createStatement();
      ResultSet check=null;
      try{
        if(!checkIsDone){
          check = conn.getMetaData().getTables(catalogName, null, getLogTableName(), types);
          this.checkIsDone = true;
          this.logTableExists = check.next();
          check.close();
        }
      }catch(UnsupportedOperationException ex){
        String message="Error while trying to get meta data from target table.";
        throw new SQLException(message);
      }
      if(this.logTableExists){
        stmt.executeUpdate("INSERT INTO "+getLogTableName()+" ("+
                                     getLogImportDefinitionName()+", "+getLogOperationName()+", "+getLogTypeName()+", "+
                                     getLogTable()+", "+getLogColumnName()+", "+getLogRowNumber()+", "+
                                     getLogOriginalValue()+", "+getLogNewValue()+
                                     ")"+
                                     " VALUES ("+
                                     "'"+importDefinitionName+"','"+typeOfInsert+"','REPLACE NULL VALUES','"+
                                     tableName+"','"+columnName+"','"+(rowNumber+1)+"','"+
                                     "null"+"','"+replacement+"'"+
                                     ")");
      }else
        this.logger.write("full","\t "+getLogTableName()+" does not exists, or is invalid!");
//      check.close();
      stmt.close();
    }catch(SQLException ex){
      this.logger.write("full","\t Error:"+ ex.getMessage());
    }
  }

  /**
   * This method write message in to log table if relations between table faild to
   * insert/update
   * @param tableName is name of the table which is in the process
   * @param columnName is the name of column in current table
   * @param replacement is new value of data
   * @param dataType is type of relation column
   * @param rowNumber is current row number
   * @param conn is connection to target database (table)
   * @param typeOfInsert represents type of operation (insert or update)
   * @param currentVersion represents update version
   * @param oid define if the oid logic is present
   * @param importDefinitionName is name of the import definition job
   * @return value of parameter
   */
  public String cleaningRelationValues(String tableName, String columnName,String replacement,
                                       String dataType,int rowNumber, Connection conn,
                                       String typeOfInsert, int currentVersion, boolean oid,
                                       String importDefinitionName, String versionColumnName)throws LoaderException{

    String[] types = {"TABLE"};
    String relValue="";
    String addins="";
    String prefix="";
    String oidVersion="";
    if(oid){
//      oidVersion=", version="+currentVersion;
      oidVersion=", "+versionColumnName+"="+currentVersion;
    }
    if(typeOfInsert.equalsIgnoreCase("update")){
      addins=oidVersion+" where ";
      prefix=" = ";
    }else{
      addins="";
    }
    try{
      String catalogName=conn.getCatalog();
      stmt=conn.createStatement();
      ResultSet relations=null;
      ResultSet check=null;
      try{
        relations=conn.getMetaData().getColumns(catalogName,null,tableName,"%");

        if(!checkIsDone){
          check = conn.getMetaData().getTables(catalogName, null, getLogTableName(), types);
          this.checkIsDone = true;
          this.logTableExists = check.next();
          check.close();
        }
      }catch(UnsupportedOperationException ex){
        String message="Error while trying to get meta data from target table.";
        throw new SQLException(message);
      }

      while(relations.next()){
        if(columnName.equalsIgnoreCase(relations.getString(4))){
//ZK change from CheckType to targetConfigReader 7.5.2004

           try {
            if(configReader.isNumber(dataType)){
                  relValue=prefix+replacement+addins;
               }else{
                 relValue=prefix+"'"+replacement+"'"+addins;
               }
        	}catch (LoaderException e) {
           
            LoaderException le = new LoaderException("Exception:",e);
            throw le;
        	}
          if(this.logTableExists)
          stmt.executeUpdate("INSERT INTO "+getLogTableName()+" ("+
                                             getLogImportDefinitionName()+", "+getLogOperationName()+", "+getLogTypeName()+", "+
                                             getLogTable()+", "+getLogColumnName()+", "+getLogRowNumber()+", "+
                                             getLogOriginalValue()+", "+getLogNewValue()+
                                             ")"+
                                             " VALUES ("+
                                             "'"+importDefinitionName+"','"+typeOfInsert+"','CLEANING RELATIONS','"+
                                             tableName+"','"+columnName+"','"+(rowNumber+1)+"',"+
                                             "null,'"+replacement+"'"+
                                             ")");

          else{
            this.logger.write("full","\t "+getLogTableName()+" does not exists, or is invalid!");
          }
          this.logger.write("full","\tBecause DataCleaning features is turn on, some replacement were made:");
          this.logger.write("full","\t "+typeOfInsert+":RELATIONS: In table "+tableName+", column "+columnName+", value null were replaced" );
          this.logger.write("full","\t with value "+replacement+" (row number "+(rowNumber+1)+")" );

        }
      }
      relations.close();
      stmt.close();
    }catch (SQLException ex){
      this.logger.write("full","\t Error:"+ ex.getMessage());
    }
    return relValue;
  }

  /**
   * This method write message in to log table if some data must be truncated
   * @param tableName is name of the table which is in the process
   * @param conn is connection to target database (table)
   * @param strQuery is string representing sql statement
   * @param rowNumber is current row number
   * @param update is the parameter which may be 'insert' or 'update'
   * @param onErrorContinue is value of onErrorContinue attribute
   * @param importDefinitionName is name of the import definition job
   * @throws SQLException
   */
  public void cutingDataLenghtUpdate(String tableName, Connection conn, String strQuery, int rowNumber,
                                     String update, String onErrorContinue,
                                     String importDefinitionName) throws SQLException, LoaderException{

    boolean end=false;
    boolean endTemp=false;
    SqlParser sqlParser=new SqlParser();

    try{
      try{
        sqlParser.parse(update+strQuery);
      }catch(Exception e){
        throw new SQLException(e.getMessage());
      }
      String[] columnNames=sqlParser.getColumnNames();
      String[] columnValues=sqlParser.getColumnValues();

      String[] types = {"TABLE"};
      String catalogName=conn.getCatalog();
      stmt=conn.createStatement();

      if (!this.currentTableName.equalsIgnoreCase(tableName)) {
        try {
          rsUpdate = conn.getMetaData().getColumns(catalogName, null, tableName,
              "%");
        }
        catch (UnsupportedOperationException ex) {
          String message ="Error while trying to get meta data from target table.";
          throw new SQLException(message);
        }
        colNamesDataTypesUpdate.clear();
        colNamesDataLenghtUpdate.clear();
        this.currentTableName = tableName;
        while (rsUpdate.next()) {
          String columnName = rsUpdate.getString(4).toUpperCase();
          colNamesDataTypesUpdate.put(columnName, rsUpdate.getString(6));
          colNamesDataLenghtUpdate.put(columnName, rsUpdate.getString(7));
        }
        rsUpdate.close();
      }

      for (int i = 0; i < columnNames.length; i++) {
        String dataNameInQyery = columnNames[i];
        String dataValueInQyery = columnValues[i];
        String dataTypeName = (String) colNamesDataTypesUpdate.get(dataNameInQyery.toUpperCase());
        String dataTypeLenght = (String) colNamesDataLenghtUpdate.get(dataNameInQyery.toUpperCase());
        if (!dataValueInQyery.equalsIgnoreCase("null")) {
//			ZK change from CheckType to targetConfigReader 7.5.2004
         try{
          
          if (!configReader.isNumber(dataTypeName)) {
            if ((dataValueInQyery.length()) > Integer.parseInt(dataTypeLenght)) {
              String original = dataValueInQyery.substring(0,dataValueInQyery.length());
              String replacement = replaceQuote(original,Integer.parseInt(dataTypeLenght));
              update = replaceFirst(update, dataValueInQyery, replacement);
              ResultSet check = null;
              try {
                if(!checkIsDone){
                  check = conn.getMetaData().getTables(catalogName, null, getLogTableName(), types);
                  this.checkIsDone = true;
                  this.logTableExists = check.next();
                  check.close();
                }
              }
              catch (UnsupportedOperationException ex) {
                String message ="Error while trying to get meta data from target table.";
                throw new SQLException(message);
              }
              if (this.logTableExists) {
                stmt.executeUpdate("INSERT INTO " + getLogTableName() + " (" +
                                   getLogImportDefinitionName() + ", " +
                                   getLogOperationName() + ", " +
                                   getLogTypeName() + ", " +
                                   getLogTable() + ", " + getLogColumnName() +
                                   ", " + getLogRowNumber() + ", " +
                                   getLogOriginalValue() + ", " +
                                   getLogNewValue() +
                                   ")" +
                                   " VALUES (" +
                                   "'" + importDefinitionName +
                                   "','UPDATE','CUT OFF DATA','" +
                                   tableName + "','" + dataNameInQyery + "','" +
                                   (rowNumber+1) + "','" +
                                   original + "','" + replacement + "'" +
                                   ")");

              }
              else {
                this.logger.write("full","\t " + getLogTableName() + " does not exists, or is invalid");
              }
              this.logger.write("full","\t Because dataCutOff attribute is true, some replacement were made:");
              this.logger.write("full","\t UPDATE:CutOff: In table "+tableName+", column "+dataNameInQyery+", value "+dataValueInQyery+" were replaced" );
              this.logger.write("full","\t with value " + replacement + " (row number " +(rowNumber+1) + ")");

            }
          }
         }catch (LoaderException e){
         	LoaderException le = new LoaderException("Exception:",(Throwable)e);
         	throw le;
         	
         }
        }
      }
      stmt.executeUpdate(update+strQuery);
      stmt.close();

    }catch(SQLException ex){
      if(onErrorContinue.equalsIgnoreCase("true"))
        cleaningInsert(tableName,conn,rowNumber,"UPDATE","Update fails. Message: "+ex.getMessage(),importDefinitionName);
      else{
        LoaderException le = new LoaderException("SQLException: ",(Throwable)ex);
        this.logger.write("full", "\tError in SQL statement: " +le.getCause());
//        this.logger.write("full", "\tError : Because OnErrorContinue Attribute is false, application is terminated");
        throw  ex;

      }
    }
  }

  /**
   * This method write message in to log table if some data must be truncated
   * @param tableName is name of the table which is in the process
   * @param conn is connection to target database (table)
   * @param strQuery is string represented sql statement
   * @param rowNumber is current row number
   * @param onErrorContinue represents onErrorContinue tag
   * @param msg represents error message
   * @param importDefinitionName is name of the import definition job
   * @throws SQLException
   */
  public void cutingDataLenght(String tableName, Connection conn, String strQuery,
                               int rowNumber, String onErrorContinue, String msg,
                               String importDefinitionName)throws SQLException{
    boolean end=false;
    boolean endTemp=false;
    SqlParser sqlParser=new SqlParser();

    try{
      try{
        sqlParser.parse(strQuery);
      }catch(Exception e){
        throw new SQLException(e.getMessage());
      }
      String[] columnNames=sqlParser.getColumnNames();
      String[] columnValues=sqlParser.getColumnValues();

      String[] types = {"TABLE"};
      String catalogName=conn.getCatalog();
      stmt=conn.createStatement();

      if(!this.currentTableName.equalsIgnoreCase(tableName)){
        try{
          rs=conn.getMetaData().getColumns(catalogName,null,tableName,"%");
        }catch(UnsupportedOperationException ex){
          String message="Error while trying to get meta data from target table.";
          throw new SQLException(message);
        }
        colNamesDataTypes.clear();
        colNamesDataLenght.clear();
        this.currentTableName=tableName;
        while (rs.next()){
          String columnName=rs.getString(4).toUpperCase();
          colNamesDataTypes.put(columnName,rs.getString(6));
          colNamesDataLenght.put(columnName,rs.getString(7));
        }
      }

      for (int i = 0; i < columnNames.length; i++) {
        String dataNameInQyery=columnNames[i];
        String dataValueInQyery=columnValues[i];
        String dataTypeName=(String)colNamesDataTypes.get(dataNameInQyery.toUpperCase());
        String dataTypeLenght=(String)colNamesDataLenght.get(dataNameInQyery.toUpperCase());
        if(!dataValueInQyery.equalsIgnoreCase("null")){
//			ZK change from CheckType to targetConfigReader 7.5.2004

         try {
                     if(!configReader.isNumber(dataTypeName)){
                        if((dataValueInQyery.length())>Integer.parseInt(dataTypeLenght)){
                          String original=dataValueInQyery.substring(0,dataValueInQyery.length());
                          String replacement=replaceQuote(original,Integer.parseInt(dataTypeLenght));
                          strQuery=replaceFirst(strQuery ,dataValueInQyery,replacement);
                          ResultSet check=null;
                          try{
                            if(!checkIsDone){
                              check = conn.getMetaData().getTables(catalogName, null, getLogTableName(), types);
                              this.checkIsDone=true;
                              this.logTableExists=check.next();
                              check.close();
                            }
                          }catch(UnsupportedOperationException ex){
                            String message="Error while trying to get meta data from target table.";
                            throw new SQLException(message);
                          }
                          if(this.logTableExists){
                          stmt.executeUpdate("INSERT INTO "+getLogTableName()+" ("+
                                                       getLogImportDefinitionName()+", "+getLogOperationName()+", "+getLogTypeName()+", "+
                                                       getLogTable()+", "+getLogColumnName()+", "+getLogRowNumber()+", "+
                                                       getLogOriginalValue()+", "+getLogNewValue()+
                                                       ")"+
                                                       " VALUES ("+
                                                       "'"+importDefinitionName+"','INSERT','CUT OFF DATA','"+
                                                       tableName+"','"+dataNameInQyery+"','"+(rowNumber+1)+"','"+
                                                       original+"','"+replacement+"'"+
                                                       ")");
            
                          }else{
                            this.logger.write("full","\t "+getLogTableName()+" does not exists, or is invalid");
            
                          }
            //              check.close();
                          this.logger.write("full","\t Because dataCutOff attribute is true, some replacement were made:");
                          this.logger.write("full","\t INSERT:CutOff: In table "+tableName+", column "+dataNameInQyery+", value "+dataValueInQyery+" were replaced" );
                          this.logger.write("full","\t with value "+replacement+" (row number "+(rowNumber+1)+")" );
                        }
                      }
        
        } catch (LoaderException e) {
        	LoaderException le = new LoaderException("Exception:",(Throwable)e);
           
        }
        }
      }

      stmt.executeUpdate(strQuery);
      stmt.close();
    }catch(SQLException ex){
      if(onErrorContinue.equalsIgnoreCase("true"))
        cleaningInsert(tableName,conn,rowNumber,"INSERT",msg, importDefinitionName);
      else{
        LoaderException le = new LoaderException("SQLException: ",(Throwable)ex);
        this.logger.write("full", "\tError in SQL statement: " +le.getCause());
//        this.logger.write("full", "\tError : Because OnErrorContinue Attribute is false, application is terminated");
        throw  ex;

      }
    }
  }

  private String replaceQuote(String replacement, int length){
    replacement=Utils.replaceAll(replacement,"''","'");
    replacement=replacement.substring(0,length);
    int index=replacement.indexOf("'");
    if(index!=-1){
      replacement=Utils.replaceAll(replacement,"'","''");
    }
    return replacement;
  }

  private String replaceFirst(String input, String forReplace, String replaceWith) {
    String retVal = input;
    int start = input.indexOf( forReplace );
    int end = start + forReplace.length();
    if( start != -1 ) {
      retVal = input.substring( 0,start ) + replaceWith + input.substring( end );
    }
    return retVal;
  }

  /**
   * Set Logger object
   * @param logger Logger object which is used for log file
   */
  public void setLogger(Logger logger) {
    this.logger = logger;
  }
}