/*
  Loader - tool for transfering data from one JDBC source to another and
  doing transformations during copy.
    Copyright (C) 2002-2003  Together
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 Loader.java
 Date: 03.03.2003.
 @version 2.1 alpha
 @author:
 Radoslav Dutina  rale@prozone.co.yu
 */

package org.webdocwf.util.loader;

import java.sql.*;
import java.math.BigDecimal;
import org.webdocwf.util.loader.logging.*;

/**
 *
 * <p>DataTransmition class is used for handling the operation with restore counter table. </p>
 * @author Radoslav Dutina
 * @version 1.0
 */
public class DataTransmition {
  private ImportDefinitionElement importDefinitionElement;
  private Logger logger;

  public DataTransmition(ImportDefinitionElement importDefinitionElement) {
    this.importDefinitionElement = importDefinitionElement;
  }

  /**
   * This method create restore counter table
   * @param conn defines the connection object of target database
   * @throws LoaderException
   */
  public void createRestartCounterTable(Connection conn) throws
      LoaderException {

    this.logger.write("full", "\tcreateRestartCounterTable method is started.");
    try {
      Statement stmt;
      stmt = conn.createStatement();

      String sqlStmt = "create table " +
                       this.importDefinitionElement.strRestartCounterTableName
                       + " (" +
                       this.importDefinitionElement.strRestartCounterImportDefinitionName +
                       " VARCHAR(50) NOT NULL, "
                       + this.importDefinitionElement.strRestartCounterValue + " DECIMAL(19,0));";

      this.logger.write("full","\tQuery '" + sqlStmt + "' will be executed");
      stmt.execute(sqlStmt);
      this.importDefinitionElement.bRestartAutoCreate = false;
      conn.commit();
      stmt.close();
    }
    catch (SQLException e) {
      LoaderException le = new LoaderException("SQLException: ", (Throwable)e);
      this.logger.write("normal", le.getCause().toString());
      throw le;
    }
        this.logger.write("full", "\tcreateRestartCounterTable method is finished.");
  }

  /**
   * This method is used for checking the restart counter table
   * @param c defines the connection object to target database
   * @param rset defines the ResultSet object of source database
   * @param jobName defines the current import job name
   * @param iTargetFirstColumnResult is parameter from conf file
   * @return value of counter field from restart counter table
   * @throws SQLException
   */
  public BigDecimal checkDataTransmition(Connection c,
      ResultSet rset, String jobName, int iTargetFirstColumnResult) throws SQLException {

    String strQuery = "";
    BigDecimal bdecRestartCounter = null;
    String valueOfRestartColumn = "";

    this.logger.write("full", "\tcheckDataTransmition method is started.");

    if (jobName.equalsIgnoreCase("importDefinition")) {
      valueOfRestartColumn = this.importDefinitionElement.strImportDefinitionName;
    } else if (jobName.equalsIgnoreCase("copyTable")) {
      valueOfRestartColumn = this.importDefinitionElement.strCopyTableName;
    }

    try {
      strQuery = "SELECT " + this.importDefinitionElement.strRestartCounterValue +
                 " FROM " +
                 this.importDefinitionElement.strRestartCounterTableName + " WHERE " +
                 this.importDefinitionElement.strRestartCounterImportDefinitionName
                 + " = '" + valueOfRestartColumn + "'";

      this.logger.write("full", "\tQuery '" + strQuery + "' will be executed");
      Statement stmtCountT = c.createStatement();
      ResultSet rsetCountT = stmtCountT.executeQuery(strQuery);

      //check if row (current import definition) exists in restart counter table
      if (!rsetCountT.next()) {
        strQuery = "INSERT into " +
                   this.importDefinitionElement.strRestartCounterTableName
                   + " (" +
                   this.importDefinitionElement.strRestartCounterImportDefinitionName
                   + ", " + this.importDefinitionElement.strRestartCounterValue + ") " +
                   " VALUES ('"
                   + valueOfRestartColumn + "', null)";
        this.logger.write("full", "\tQuery '" + strQuery + "' will be executed");
        stmtCountT.executeUpdate(strQuery);
        c.commit();
        bdecRestartCounter = null;

      } else { //row exist
        if (iTargetFirstColumnResult == 1) {
          bdecRestartCounter = new BigDecimal(Integer.parseInt(rsetCountT.getString(1)));
        } else { //iTargetFirstColumnResult == 0
          bdecRestartCounter = new BigDecimal(Integer.parseInt(rsetCountT.getString(0)));
        }
        if (false) {
          rset.relative(bdecRestartCounter.intValue());
        } else {
          BigDecimal kl = new BigDecimal(0);
          while (kl.compareTo(bdecRestartCounter) == -1) {
            rset.next();
            kl = kl.add(new BigDecimal(1));
          }
        }
      }
      rsetCountT.close();
      stmtCountT.close();
    }
    catch (SQLException ex) {
      ex.printStackTrace();
    }
    return bdecRestartCounter;
  }

  /**
   * This method is used for inserting the appropriate values into restart counter
   * table
   * @param jobName defines the current import job name
   * @param bdecCount defines the number of rows which are commited
   * @param conn defines the connection object to target database
   * @throws SQLException
   */
  public void insertCounter(String jobName, BigDecimal bdecCount,
      Connection conn) throws SQLException {

    String valueOfRestartColumn = "";

    this.logger.write("full", "\tinsertCounter method is started.");

    if (jobName.equalsIgnoreCase("importDefinition")) {
      valueOfRestartColumn = this.importDefinitionElement.strImportDefinitionName;
    } else if (jobName.equalsIgnoreCase("copyTable")) {
      valueOfRestartColumn = this.importDefinitionElement.strCopyTableName;
    }

    String strQueryUpdate = "update " +
                            this.importDefinitionElement.strRestartCounterTableName
                            + " set " + this.importDefinitionElement.strRestartCounterValue + " = " +
                            bdecCount
                            + " where " +
                            this.importDefinitionElement.strRestartCounterImportDefinitionName +
                            " = '" + valueOfRestartColumn + "'";

    try {
      this.logger.write("full", "\tQuery '" + strQueryUpdate + "' will be executed");
      Statement stmtChange = conn.createStatement();
      int num = stmtChange.executeUpdate(strQueryUpdate);
      stmtChange.close();
      conn.commit();
    }
    catch (SQLException ex) {
      throw ex;
    }
    this.logger.write("full", "\tinsertCounter method is finished.");
  }

  /**
   * This method is used to set '0' into counter column from counter table, if the
   * current import job was finished successfuly
   * @param conn defines the connection object of target database
   * @param jobName defines the current import job name
   * @throws SQLException
   */
  public void resetRestartCounter(Connection conn, String jobName) throws SQLException {

    String valueOfRestartColumn = "";

    this.logger.write("full", "\tresetRestartCounter method is started.");

    if (jobName.equalsIgnoreCase("importDefinition")) {
      valueOfRestartColumn = this.importDefinitionElement.strImportDefinitionName;
    } else if (jobName.equalsIgnoreCase("copyTable")) {
      valueOfRestartColumn = this.importDefinitionElement.strCopyTableName;
    }

    String strQueryReset = "update " +
                           this.importDefinitionElement.strRestartCounterTableName +
                           " set " + this.importDefinitionElement.strRestartCounterValue +
                           " = 0 where " +
                           this.importDefinitionElement.strRestartCounterImportDefinitionName
                           + " = '" + valueOfRestartColumn + "'";
    try {
      Statement stmtReset = conn.createStatement();
      this.logger.write("full", "\tQuery '" + strQueryReset + "' will be executed");
      int num = stmtReset.executeUpdate(strQueryReset);
      stmtReset.close();
      conn.commit();
    }
    catch (SQLException ex) {
      throw ex;
    }
    this.logger.write("full", "\tresetRestartCounter method is finished.");
  }

  /**
   * This method is used to set current Looger
   * @param logger is the currently used logger
   */
  public void setLogger(Logger logger) {
    this.logger = logger;
  }

}