package org.webdocwf.util.loader;

/**
  JdbcParametersElement - Element in parallel to importDefinitions and sql's.

    Copyright (C) 2002-2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 JdbcParametersElement.java
 Date: 20.5.2003.
 @version 1.0.0
 @author: Zoran Milakovic zoran@prozone.co.yu
 */

import java.io.*;
import java.util.*;

import javax.xml.parsers.*;

import org.w3c.dom.*;
import org.webdocwf.util.loader.logging.*;

/**
 *
 * JdbcParametersElement class sets the jdbc parameters
 * @author Radoslav Dutina
 * @version 1.0
 */
public class JdbcParametersElement {

  private String strJDBCSourceParameterDriver = "";
  private String strJDBCTargetParameterDriver = "";
  private String strJDBCSourceParameterConnection = "";
  private String strJDBCTargetParameterConnection = "";
  private String strJDBCSourceParameterUser = "";
  private String strJDBCTargetParameterUser = "";
  private String strJDBCSourceParameterPassword = "";
  private String strJDBCTargetParameterPassword = "";

  private String strJDBCDefaultSourceParameterDriver = "";
  private String strJDBCDefaultTargetParameterDriver = "";
  private String strJDBCDefaultSourceParameterConnection = "";
  private String strJDBCDefaultTargetParameterConnection = "";
  private String strJDBCDefaultSourceParameterUser = "";
  private String strJDBCDefaultTargetParameterUser = "";
  private String strJDBCDefaultSourceParameterPassword = "";
  private String strJDBCDefaultTargetParameterPassword = "";

  private String strDbVendor = "";
  private String strDriverName = "";

  private String strTargetDbVendor = "";
  private String strTargetDriverName = "";

  private String strDbVendorDefault = "";
  private String strDriverNameDefault = "";

  private String strTargetDbVendorDefault = "";
  private String strTargetDriverNameDefault = "";

  private Logger logger;
  private Vector targetConnections;

  private String loaderJobFile="";
  private ConfigReader configReaderSource;


  /**
   * Method parseTargetJDBCParameters is used to analyse import XML file
   * @param inStream Data from inputXML file which is converted into InputStream.
   * @throws LoaderException
   */
  public void parseTargetJDBCParameters(InputStream inStream) throws LoaderException{
    Document doc = null;
    targetConnections=new Vector();
    try {
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = null;
      db = dbf.newDocumentBuilder();
      doc = db.parse(inStream);
    } catch (Exception e) {
      this.logger.write("normal", "Sorry, an error occurred: " + e);
      LoaderException le = new LoaderException("Exception: ",
                                               (Throwable)e);
      throw le;
    }
    if (doc != null) {
      NodeList targetList = doc.getElementsByTagName( "jdbcTargetParameters" );
      int position=0;
      if(targetList.getLength()!=0){
        for(int i=0;i<targetList.getLength();i++){

          targetConnections.add(position,"");
          targetConnections.add(position+1,"");
          targetConnections.add(position+2,"");
          targetConnections.add(position+3,"");

          NodeList childs=targetList.item(i).getChildNodes();
          for(int k=0;k<childs.getLength();k++){
            if(childs.item(k).getNodeType()==3){
              //
            }else{
              NamedNodeMap attributes= childs.item(k).getAttributes();
              String name=attributes.getNamedItem("name").getNodeValue();
              if(name.equalsIgnoreCase("JdbcDriver"))
                targetConnections.set(position,attributes.getNamedItem("value").getNodeValue());
              else if(name.equalsIgnoreCase("Connection.Url")){
                targetConnections.set(position+1,attributes.getNamedItem("value").getNodeValue());
              }else if(name.equalsIgnoreCase("User"))
                targetConnections.set(position+2,attributes.getNamedItem("value").getNodeValue());
              else if(name.equalsIgnoreCase("Password"))
                targetConnections.set(position+3,attributes.getNamedItem("value").getNodeValue());

            }
          }
          position=position+4;
        }
      }
    }
    try {
      inStream.reset();
    } catch (IOException e) {
      this.logger.write("normal", "Sorry, an error occurred: " + e);
      LoaderException le = new LoaderException("IOException: ",(Throwable)e);
      throw le;
  }
  }

  /**
   * This method read the value of targetConnections parameter
   * @return value of parameter
   */
  public Vector getTargetConnections(){
    return targetConnections;
  }

  /**
   * Method parseImportJDBCParameters is used to analyse import XML file
   * about JDBC parameters tags. Puts values of first importDefinition's JDBC parameters in global variables.
   * @param inStream Data from inputXML file which is converted into InputStream.
   * @param number order number of importDefinition or sql tag which is proceeded.
   * @param tagName name of tag which is proceeded
   * @throws LoaderException
   */
  public void parseImportJDBCParameters (InputStream inStream,int number,String tagName) throws LoaderException{
    this.strJDBCSourceParameterDriver = "";
    this.strJDBCTargetParameterDriver = "";
    this.strJDBCSourceParameterConnection = "";
    this.strJDBCTargetParameterConnection = "";
    this.strJDBCSourceParameterUser = "";
    this.strJDBCTargetParameterUser = "";
    this.strJDBCSourceParameterPassword = "";
    this.strJDBCTargetParameterPassword = "";
    this.strDbVendor = "";
    this.strDriverName = "";
    this.strTargetDbVendor = "";
    this.strTargetDriverName = "";
    number--;
    Document doc = null;
    Vector vecJDBCTargetValue = new Vector();
    Vector vecJDBCTargetName = new Vector();
    Vector vecJDBCSourceValue = new Vector();
    Vector vecJDBCSourceName = new Vector();
    boolean isOK = false;

    this.logger.write("full", "\tparseImportJDBCParameters method is started.");
    try {
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = null;
      db = dbf.newDocumentBuilder();
      doc = db.parse(inStream);
    } catch (Exception e) {
      this.logger.write("normal", "Sorry, an error occurred: " + e);
      LoaderException le = new LoaderException("Exception: ",
                                               (Throwable)e);
      throw le;
//            System.exit(1);
    }
    if (doc != null) {
      NodeList tagDefault = doc.getElementsByTagName( tagName );
      if (tagDefault.getLength() != 0) {
        Element docFragment = (Element)tagDefault.item( number );
        //source parameters
        NodeList tag = docFragment.getElementsByTagName("jdbcSourceParameters");
        if (tag.getLength() != 0) {
          NamedNodeMap jdbc = tag.item(0).getAttributes();
          Node nodeJdbc = jdbc.getNamedItem("dbVendor");
          if (nodeJdbc != null)
            this.strDbVendor = nodeJdbc.getNodeValue();
          nodeJdbc = jdbc.getNamedItem("driverName");
          if (nodeJdbc != null)
            this.strDriverName = nodeJdbc.getNodeValue();
        }
        //target parameters
        tag = docFragment.getElementsByTagName("jdbcTargetParameters");
        if (tag.getLength() != 0) {
          NamedNodeMap jdbc = tag.item(0).getAttributes();
          Node nodeJdbc = jdbc.getNamedItem("dbVendor");
          if (nodeJdbc != null)
            this.strTargetDbVendor = nodeJdbc.getNodeValue();
          nodeJdbc = jdbc.getNamedItem("driverName");
          if (nodeJdbc != null)
            this.strTargetDriverName = nodeJdbc.getNodeValue();
        }

        tag = docFragment.getElementsByTagName("jdbcSourceParameter");
        for (int i = 0; i < tag.getLength(); i++) {
          String nodeValueValue = "";
          String nodeNameValue = "";
          NamedNodeMap attrs = tag.item(i).getAttributes();
          Node nodeValue = attrs.getNamedItem("value");
          Node nodeName = attrs.getNamedItem("name");
          if (nodeValue != null && nodeName != null) {
            nodeValueValue = nodeValue.getNodeValue();
            nodeNameValue = nodeName.getNodeValue();
          }
          vecJDBCSourceValue.addElement(nodeValueValue);
          vecJDBCSourceName.addElement(nodeNameValue);
        }
        for (int i = 0; i < vecJDBCSourceValue.size(); i++) {
          if (vecJDBCSourceName.get(i).toString().equalsIgnoreCase("JdbcDriver"))
            this.strJDBCSourceParameterDriver = vecJDBCSourceValue.get(i).toString();
          else if (vecJDBCSourceName.get(i).toString().equalsIgnoreCase("Connection.Url")) {
            //Testing SelectMethod parameter in microsoft MSSQL driver
            if(vecJDBCSourceValue.get(i).toString().indexOf("jdbc:microsoft:sqlserver")!=-1) {
              if(vecJDBCSourceValue.get(i).toString().indexOf("SelectMethod")==-1) {
                this.strJDBCSourceParameterConnection = vecJDBCSourceValue.get(i).toString()+";SelectMethod=cursor";
              } else {
                if(vecJDBCSourceValue.get(i).toString().indexOf("cursor")!=-1) {
                  this.strJDBCSourceParameterConnection = vecJDBCSourceValue.get(i).toString();
                } else {
                  this.logger.write("normal", "Sorry, an error occurred: value of Connection.Url perameter SelectMethod has to be cursor" );
                  LoaderException le = new LoaderException("Exception:", new Exception("value of Connection.Url perameter SelectMethod has to be cursor"));
                  throw le;
                }
              }
            } else {
              this.strJDBCSourceParameterConnection = vecJDBCSourceValue.get(i).toString();
            }
          }
          else if (vecJDBCSourceName.get(i).toString().equalsIgnoreCase("User"))
            this.strJDBCSourceParameterUser = vecJDBCSourceValue.get(i).toString();
          else if (vecJDBCSourceName.get(i).toString().equalsIgnoreCase("Password"))
            this.strJDBCSourceParameterPassword = vecJDBCSourceValue.get(i).toString();
        }
        tag = docFragment.getElementsByTagName("jdbcTargetParameter");
        for (int i = 0; i < tag.getLength(); i++) {
          String nodeValueValue = "";
          String nodeNameValue = "";
          NamedNodeMap attrs = tag.item(i).getAttributes();
          Node nodeValue = attrs.getNamedItem("value");
          Node nodeName = attrs.getNamedItem("name");
          if (nodeValueValue != null && nodeName != null) {
            nodeValueValue = nodeValue.getNodeValue();
            nodeNameValue = nodeName.getNodeValue();
          }
          vecJDBCTargetValue.addElement(nodeValueValue);
          vecJDBCTargetName.addElement(nodeNameValue);
        }
        for (int i = 0; i < vecJDBCTargetValue.size(); i++) {
          if (vecJDBCTargetName.get(i).toString().equalsIgnoreCase("JdbcDriver"))
            this.strJDBCTargetParameterDriver = vecJDBCTargetValue.get(i).toString();
          else if (vecJDBCTargetName.get(i).toString().equalsIgnoreCase("Connection.Url")) {
            //Testing SelectMethod parameter in microsoft MSSQL driver
            if(vecJDBCTargetValue.get(i).toString().indexOf("jdbc:microsoft:sqlserver")!=-1) {
              if(vecJDBCTargetValue.get(i).toString().indexOf("SelectMethod")==-1) {
                this.strJDBCTargetParameterConnection = vecJDBCTargetValue.get(i).toString()+";SelectMethod=cursor";
              } else {
                if(vecJDBCTargetValue.get(i).toString().indexOf("cursor")!=-1) {
                  this.strJDBCTargetParameterConnection = vecJDBCTargetValue.get(i).toString();
                } else {
                  this.logger.write("normal", "Sorry, an error occurred: value of Connection.Url perameter SelectMethod has to be cursor" );
                  LoaderException le = new LoaderException("Exception:", new Exception("value of Connection.Url perameter SelectMethod has to be cursor"));
                  throw le;
                }
              }
            } else {
              this.strJDBCTargetParameterConnection = vecJDBCTargetValue.get(i).toString();
            }
          }
          else if (vecJDBCTargetName.get(i).toString().equalsIgnoreCase("User"))
            this.strJDBCTargetParameterUser = vecJDBCTargetValue.get(i).toString();
          else if (vecJDBCTargetName.get(i).toString().equalsIgnoreCase("Password"))
            this.strJDBCTargetParameterPassword = vecJDBCTargetValue.get(i).toString();
        }
      }
      isOK=true;
    }
    try {
      inStream.reset();
    } catch (IOException e) {
      this.logger.write("normal", "Sorry, an error occurred: " + e);
      LoaderException le = new LoaderException("IOException: ",
                                               (Throwable)e);
      throw le;
    }
    this.logger.write("full", "\tparseImportJDBCDefaultParameters method is finished.");
  }


  /**
   * Method parseImportJDBCDefaultParameters is used to analyse import XML file
   * about Default JDBC parameters tags.
   * @param inStream Data from inputXML file which is converted into InputStream.
   * @return boolean - true default jdbc parameters exists, false otherwise
   * @throws LoaderException
   */
  public boolean parseImportJDBCDefaultParameters (InputStream inStream) throws LoaderException{
    Document doc = null;
    Vector vecJDBCTargetValue = new Vector();
    Vector vecJDBCTargetName = new Vector();
    Vector vecJDBCSourceValue = new Vector();
    Vector vecJDBCSourceName = new Vector();
    boolean isOK = false;

    this.logger.write("full", "\tparseImportJDBCDefaultParameters method is started.");
    try {
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = null;
      db = dbf.newDocumentBuilder();
      doc = db.parse(inStream);
    } catch (Exception e) {
      this.logger.write("normal", "Sorry, an error occurred: " + e);
      LoaderException le = new LoaderException("Exception: ",
                                               (Throwable)e);
      throw le;
    }
    if (doc != null) {
      NodeList tagDefault = doc.getElementsByTagName("jdbcDefaultParameters");
      if (tagDefault.getLength() != 0) {
        Element docFragment = (Element)tagDefault.item(0);
        NodeList tag = docFragment.getElementsByTagName("jdbcSourceParameters");
        if (tag.getLength() != 0) {
          NamedNodeMap jdbc = tag.item(0).getAttributes();
          Node nodeJdbc = jdbc.getNamedItem("dbVendor");
          if (nodeJdbc != null)
            this.strDbVendorDefault = nodeJdbc.getNodeValue();
          nodeJdbc = jdbc.getNamedItem("driverName");
          if (nodeJdbc != null)
            this.strDriverNameDefault = nodeJdbc.getNodeValue();
        }
        tag = docFragment.getElementsByTagName("jdbcTargetParameters");
        if (tag.getLength() != 0) {
          NamedNodeMap jdbc = tag.item(0).getAttributes();
          Node nodeJdbc = jdbc.getNamedItem("dbVendor");
          if (nodeJdbc != null)
            this.strTargetDbVendorDefault = nodeJdbc.getNodeValue();
          nodeJdbc = jdbc.getNamedItem("driverName");
          if (nodeJdbc != null)
            this.strTargetDriverNameDefault = nodeJdbc.getNodeValue();
        }
        tag = docFragment.getElementsByTagName("jdbcSourceParameter");
        for (int i = 0; i < tag.getLength(); i++) {
          String nodeValueValue = "";
          String nodeNameValue = "";
          NamedNodeMap attrs = tag.item(i).getAttributes();
          Node nodeValue = attrs.getNamedItem("value");
          Node nodeName = attrs.getNamedItem("name");
          if (nodeValue != null && nodeName != null) {
            nodeValueValue = nodeValue.getNodeValue();
            nodeNameValue = nodeName.getNodeValue();
          }
          vecJDBCSourceValue.addElement(nodeValueValue);
          vecJDBCSourceName.addElement(nodeNameValue);
        }
        for (int i = 0; i < vecJDBCSourceValue.size(); i++) {
          if (vecJDBCSourceName.get(i).toString().equalsIgnoreCase("JdbcDriver"))
            this.strJDBCDefaultSourceParameterDriver = vecJDBCSourceValue.get(i).toString();
          else if (vecJDBCSourceName.get(i).toString().equalsIgnoreCase("Connection.Url")) {
            //Testing SelectMethod parameter in microsoft MSSQL driver
            if(vecJDBCSourceValue.get(i).toString().indexOf("jdbc:microsoft:sqlserver")!=-1) {
              if(vecJDBCSourceValue.get(i).toString().indexOf("SelectMethod")==-1) {
                this.strJDBCDefaultSourceParameterConnection = vecJDBCSourceValue.get(i).toString()+";SelectMethod=cursor";
              } else {
                if(vecJDBCSourceValue.get(i).toString().indexOf("cursor")!=-1) {
                  this.strJDBCDefaultSourceParameterConnection = vecJDBCSourceValue.get(i).toString();
                } else {
                  this.logger.write("normal", "Sorry, an error occurred: value of Connection.Url perameter SelectMethod has to be cursor" );
                  LoaderException le = new LoaderException("Exception:", new Exception("value of Connection.Url perameter SelectMethod has to be cursor"));
                  throw le;
                }
              }
            } else {
              this.strJDBCDefaultSourceParameterConnection = vecJDBCSourceValue.get(i).toString();
            }
          }
          else if (vecJDBCSourceName.get(i).toString().equalsIgnoreCase("User"))
            this.strJDBCDefaultSourceParameterUser = vecJDBCSourceValue.get(i).toString();
          else if (vecJDBCSourceName.get(i).toString().equalsIgnoreCase("Password"))
            this.strJDBCDefaultSourceParameterPassword = vecJDBCSourceValue.get(i).toString();
        }
        tag = docFragment.getElementsByTagName("jdbcTargetParameter");
        for (int i = 0; i < tag.getLength(); i++) {
          String nodeValueValue = "";
          String nodeNameValue = "";
          NamedNodeMap attrs = tag.item(i).getAttributes();
          Node nodeValue = attrs.getNamedItem("value");
          Node nodeName = attrs.getNamedItem("name");
          if (nodeValueValue != null && nodeName != null) {
            nodeValueValue = nodeValue.getNodeValue();
            nodeNameValue = nodeName.getNodeValue();
          }
          vecJDBCTargetValue.addElement(nodeValueValue);
          vecJDBCTargetName.addElement(nodeNameValue);
        }
        for (int i = 0; i < vecJDBCTargetValue.size(); i++) {
          if (vecJDBCTargetName.get(i).toString().equalsIgnoreCase("JdbcDriver"))
            this.strJDBCDefaultTargetParameterDriver = vecJDBCTargetValue.get(i).toString();
          else if (vecJDBCTargetName.get(i).toString().equalsIgnoreCase("Connection.Url")) {
            //Testing SelectMethod parameter in microsoft MSSQL driver
            if(vecJDBCTargetValue.get(i).toString().indexOf("jdbc:microsoft:sqlserver")!=-1) {
              if(vecJDBCTargetValue.get(i).toString().indexOf("SelectMethod")==-1) {
                this.strJDBCDefaultTargetParameterConnection = vecJDBCTargetValue.get(i).toString()+";SelectMethod=cursor";
              } else {
                if(vecJDBCTargetValue.get(i).toString().indexOf("cursor")!=-1) {
                  this.strJDBCDefaultTargetParameterConnection = vecJDBCTargetValue.get(i).toString();
                } else {
                  this.logger.write("normal", "Sorry, an error occurred: value of Connection.Url perameter SelectMethod has to be cursor" );
                  LoaderException le = new LoaderException("Exception:", new Exception("value of Connection.Url perameter SelectMethod has to be cursor"));
                  throw le;
                }
              }
            } else {
              this.strJDBCDefaultTargetParameterConnection = vecJDBCTargetValue.get(i).toString();
            }
          }
          else if (vecJDBCTargetName.get(i).toString().equalsIgnoreCase("User"))
            this.strJDBCDefaultTargetParameterUser = vecJDBCTargetValue.get(i).toString();
          else if (vecJDBCTargetName.get(i).toString().equalsIgnoreCase("Password"))
            this.strJDBCDefaultTargetParameterPassword = vecJDBCTargetValue.get(i).toString();
        }
      }
      isOK=true;
    }
    try {
      inStream.reset();
    } catch (IOException e) {
      this.logger.write("normal", "Sorry, an error occurred: " + e);
      LoaderException le = new LoaderException("IOException: ",
                                               (Throwable)e);
      throw le;
    }
    this.logger.write("full", "\tparseImportJDBCDefaultParameters method is finished.");
    return isOK;
  }

  //set and get methods
  /**
   * Set Logger of this class.
   * @param logger Logger object which is used to write to log and standard output.
   */
  public void setLogger(Logger logger) {
    this.logger = logger;
  }

  /**
   * Set ConfigReader of this class.
   * @param configReader defines ConfigReader object which is used to store data
   * about source driver.
   */
  public void setConfigReaderSource(ConfigReader configReader) {
    this.configReaderSource = configReader;
  }


  /**
   * This method set the absolute path to loader job file
   * @param loaderJobName is relativ path to loader job file
   */
  public void setLoaderJobPath(String loaderJobName){
    File file=new File(loaderJobName);
    this.loaderJobFile=file.getAbsoluteFile().getParent()+System.getProperty("file.separator");
  }

  /**
   * This method make absoulte path from relative path
   * @param urlToDatabase relative path
   * @return absolute path
   * @throws LoaderException
   */
  public String getAbsolutePathFromDatabaseURL(String urlToDatabase) throws LoaderException{

    if(configReaderSource.getFileSystemDatabase()==true){
      String urlPrefix = configReaderSource.getConnectionPrefix();
      String pathToDatabase=urlToDatabase.substring(urlPrefix.length());
      File file=new File(pathToDatabase);
      if (!file.isAbsolute()){
        pathToDatabase=this.loaderJobFile + pathToDatabase;
        File absolutePath=new File(pathToDatabase);
        try {
          pathToDatabase=absolutePath.getCanonicalPath();
        }
        catch (Exception ex) {
          this.logger.write("normal", ex.getMessage());
          LoaderException le = new LoaderException("Exception: ",(Throwable)ex);
          throw le;
        }
        urlToDatabase = urlToDatabase.substring(0, (urlPrefix.length())) +
                        pathToDatabase;
      }
    }
    return urlToDatabase;
  }


//source parameters
  /**
   * Method return JDBCSourceParameterConnection,or if that
   * parameter is not defined method will return default value which is
   * strJDBCDefaultSourceParameterConnection
   * @return JDBCSourceParameterConnection
   * @throws LoaderException
   */
  public String getJDBCSourceParameterConnection() throws LoaderException {
    String parseUrl="";
    try{
      if (!this.strJDBCSourceParameterConnection.equals("")) {
        parseUrl= getAbsolutePathFromDatabaseURL(this.strJDBCSourceParameterConnection);
      } else {
        parseUrl= getAbsolutePathFromDatabaseURL(this.strJDBCDefaultSourceParameterConnection);
      }
    }catch(LoaderException ex){
      throw ex;
    }
    return parseUrl;
  }

  /**
   * This method set value of strJDBCSourceParameterConnection parameter
   * @param arg set JDBCSouceParameterConnection to value of arg
   */
  public void setJDBCSourceParameterConnection(String arg) {
    this.strJDBCSourceParameterConnection = arg;
  }

  private String getJDBCDefaultSourceParameterConnection() {
    return this.strJDBCDefaultSourceParameterConnection;
  }

  /**
   * Method return JDBCSourceParameterDriver,or if that
   * parameter is not defined method will return default value which is
   * strJDBCDefaultSourceParameterDriver
   * @return JDBCSourceParameterDriver
   */
  public String getJDBCSourceParameterDriver() {
    if( !this.strJDBCSourceParameterDriver.equals("") )
      return this.strJDBCSourceParameterDriver;
    else
      return this.strJDBCDefaultSourceParameterDriver;
  }

  /**
   * This method set value of strJDBCSourceParameterDriver parameter
   * @param arg set JDBCSouceParameterDriver to value of arg
   */
  public void setJDBCSourceParameterDriver(String arg) {
    this.strJDBCSourceParameterDriver = arg;
  }

  private String getJDBCDefaultSourceParameterDriver() {
    return this.strJDBCDefaultSourceParameterDriver;
  }

  /**
   * Method return JDBCSourceParameterDriver,or if that
   * parameter is not defined method will return default value which is
   * strJDBCDefaultSourceParameterDriver
   * @return JDBCSourceParameterDriver
   */
  public String getJDBCSourceParameterPassword() {
    if( !this.strJDBCSourceParameterPassword.equals("") )
      return this.strJDBCSourceParameterPassword;
    else
      return this.strJDBCDefaultSourceParameterPassword;
  }

  /**
   * This method set value of strJDBCSourceParameterPassword parameter
   * @param arg setJDBCSourceParameterPassword to value of arg
   */
  public void setJDBCSourceParameterPassword(String arg) {
    this.strJDBCSourceParameterPassword = arg;
  }

  private String getJDBCDefaultSourceParameterPassword() {
    return this.strJDBCDefaultSourceParameterPassword;
  }

  /**
   * Method return JDBCSourceParameterUser,or if that
   * parameter is not defined method will return default value which is
   * strJDBCDefaultSourceParameterUser
   * @return JDBCSourceParameterUser
   */
  public String getJDBCSourceParameterUser() {
    if( !this.strJDBCSourceParameterUser.equals("") )
      return this.strJDBCSourceParameterUser;
    else
      return this.strJDBCDefaultSourceParameterUser;
  }

  /**
   * This method set value of strJDBCSourceParameterUser parameter
   * @param arg set JDBCSourceParameterUser to value of arg
   */
  public void setJDBCSourceParameterUser(String arg) {
    this.strJDBCSourceParameterUser = arg;
  }

  private String getJDBCDefaultSourceParameterUser() {
    return this.strJDBCDefaultSourceParameterUser;
  }


//targets parameters
  /**
   * Method return JDBCTargetParameterConnection,or if that
   * parameter is not defined method will return default value which is
   * strJDBCTargetParameterConnection
   * @return JDBCTargetParameterConnection
   */
  public String getJDBCTargetParameterConnection() {
    if( !this.strJDBCTargetParameterConnection.equals("") )
      return this.strJDBCTargetParameterConnection;
    else
      return this.strJDBCDefaultTargetParameterConnection;
  }

  /**
   * This method set value of strJDBCTargetParameterConnection parameter
   * @param arg set JDBCTargetParameterConnection to value of arg
   */
  public void setJDBCTargetParameterConnection(String arg) {
    this.strJDBCTargetParameterConnection = arg;
  }

  private String getJDBCDefaultTargetParameterConnection() {
    return this.strJDBCDefaultTargetParameterConnection;
  }

  /**
   * Method return JDBCTargetParameterDriver,or if that
   * parameter is not defined method will return default value which is
   * strJDBCDefaultTargetParameterDriver
   * @return JDBCTargetParameterDriver
   */
  public String getJDBCTargetParameterDriver() {
    if( !this.strJDBCTargetParameterDriver.equals("") )
      return this.strJDBCTargetParameterDriver;
    else
      return this.strJDBCDefaultTargetParameterDriver;
  }

  /**
   * This method set value of strJDBCTargetParameterDriver parameter
   * @param arg set JDBCTargetParameterDriver to value of arg
   */
  public void setJDBCTargetParameterDriver(String arg) {
    this.strJDBCTargetParameterDriver = arg;
  }

  private String getJDBCDefaultTargetParameterDriver() {
    return this.strJDBCDefaultTargetParameterDriver;
  }

  /**
   * Method return JDBCTargetParameterPassword,or if that
   * parameter is not defined method will return default value which is
   * strJDBCDefaultTargetParameterPassword
   * @return JDBCTargetParameterPassword
   */
  public String getJDBCTargetParameterPassword() {
    if( !this.strJDBCTargetParameterPassword.equals("") )
      return this.strJDBCTargetParameterPassword;
    else
      return this.strJDBCDefaultTargetParameterPassword;
  }

  /**
   * This method set value of strJDBCTargetParameterPassword parmeter
   * @param arg set JDBCTargetParameterPassword to value of arg
   */
  public void setJDBCTargetParameterPassword(String arg) {
    this.strJDBCTargetParameterPassword = arg;
  }

  private String getJDBCDefaultTargetParameterPassword() {
    return this.strJDBCDefaultTargetParameterPassword;
  }

  /**
   * Method return JDBCTargetParameterPassword,or if that
   * parameter is not defined method will return default value which is
   * strJDBCDefaultTargetParameterPassword
   * @return JDBCTargetParameterPassword
   */
  public String getJDBCTargetParameterUser() {
    if( !this.strJDBCTargetParameterUser.equals("") )
      return this.strJDBCTargetParameterUser;
    else
      return this.strJDBCDefaultTargetParameterUser;
  }

  /**
   * This method set value of strJDBCTargetParameterUser parameter
   * @param arg set JDBCTargetParameterUser to value of arg
   */
  public void setJDBCTargetParameterUser(String arg) {
    this.strJDBCTargetParameterUser = arg;
  }

  private String getJDBCDefaultTargetParameterUser() {
    return this.strJDBCDefaultTargetParameterUser;
  }

  private String getDBVendorDefault() {
    return this.strDbVendorDefault;
  }

  private String getDriverNameDefault() {
    return this.strDriverNameDefault;
  }

  private String getTargetDbVendorDefault() {
    return this.strTargetDbVendorDefault;
  }

  private String getTargetDriverNameDefault() {
    return this.strTargetDriverNameDefault;
  }

  /**
   * Method return TargetDBVendor,or if that
   * parameter is not defined method will return default value which is
   * strTargetDBVendorDefault
   * @return TargetDBVendor
   */
  public String getTargetDBVendor() {
    if( !this.strTargetDbVendor.equals("") )
      return this.strTargetDbVendor;
    else
      return this.strTargetDbVendorDefault;
  }

  /**
   * Method return TargetDriverName,or if that
   * parameter is not defined method will return default value which is
   * strTargetDriverNameDefault
   * @return TargetDriverName
   */
  public String getTargetDriverName() {
    if( !this.strTargetDriverName.equals("") )
      return this.strTargetDriverName;
    else{
      if(this.strTargetDbVendor.equalsIgnoreCase("")&&this.strTargetDriverName.equalsIgnoreCase("")){
        return this.strTargetDriverNameDefault;
      }else
        return "";
    }
  }

  /**
   * Method return DriverName for source database,or if that
   * parameter is not defined method will return default value which is
   * strDriverNameDefault
   * @return DriverName
   */
  public String getDriverName() {
    if( !this.strDriverName.equals("") )
      return this.strDriverName;
    else{
      if(this.strDbVendor.equalsIgnoreCase("")&&this.strDriverName.equalsIgnoreCase("")){
        return this.strDriverNameDefault;
      }else
        return "";
    }
  }

  /**
   * Method return DbVendor for source database,or if that
   * parameter is not defined method will return default value which is
   * strDbVendorDefault
   * @return DbVendor
   */
  public String getDbVendor() {
    if( !this.strDbVendor.equals("") )
      return this.strDbVendor;
    else
      return this.strDbVendorDefault;
  }

}