/**
  LoaderJobAttrReader - Class used for reading attributes from  tag.
    Copyright (C) 2002-2003  Together
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 LoaderJobAttrReader.java
 Date: 20.5.2003.
 @version 1.0.0
 @author: Zoran Milakovic zoran@prozone.co.yu
 */

package org.webdocwf.util.loader;

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.webdocwf.util.loader.logging.Logger;

/**
 * Class used for reading attribute values from tags in XML file.
 *
 * @author     Zoran Milakovic
 * @version    1.1
 */
public class LoaderJobAttrReader {

//parametri loaderJob elementa
  boolean bDefaultOnErrorContinue = false;
  boolean bDefaultCommit = true;
  String strDefaultUserID = "";
  String strDefaultLogDir = "";
  String strDefaultLogFile = "default";
  String strDefaultVendorConfig = "OctopusDBVendors.xml";
  boolean bDefaultObjectIDAutoCreate = false;
  int iDefaultObjectIDStartValue = 1;
  int iDefaultCommitCount = 100;
  int iLoaderJobCommitCount = 0;
  boolean bDefaultOidLogic = true;

  String strDefaultLogMode = "normal";
  int iDefaultObjectIDIncrement = 20;
  String strDefaultObjectIDTableName = "objectid";
  String strDefaultObjectIDColumnName = "next";
  String strDefaultObjectIDNameColumnName = "";
  String strDefaultObjectIDNameColumnValue = "";
  String strDefaultTableMode = "Query";

  //rale
  String strDefaultDataCutOff = "";
  String strLogTableName = "";
  String strLogTable = "";
  String strLogColumnName = "";
  String strLogRowNumber = "";
  String strLogOriginalValue = "";
  String strLogNewValue = "";
  String strLogImportDefinitionName = "";
  String strLogOperationName = "";
  String strLogTypeName = "";

  private Logger logger;
  String strLogClassName = "";
  String strPathToLoggerConf = "";
  String setFetchSize = "";
  String setCursorName = "";

  /**
   * Method importLoaderJobAttributes is used to analyse import XML file
       * about attributes of root element(loaderJob). Values of these attributes puts in global variables.
   * @param inputStream Data from inputXML file which is converted into InputStream.
   * @throws LoaderException
   */
  public void importLoaderJobAttributes(InputStream inputStream) throws LoaderException {
    Document doc = null;
    this.logger.write("full", "\timportLoaderJobAttributes method is started.");
    try {
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = null;
      db = dbf.newDocumentBuilder();
      doc = db.parse(inputStream);
    }
    catch (Exception e) {
      this.logger.write("normal", "Sorry, an error occurred: " + e);
      LoaderException le = new LoaderException("Exception: ",
                           (Throwable)e);
      throw le;
//            System.exit(1);
    }
    if (doc != null) {
      NodeList tagRestartCounter = doc.getElementsByTagName("loaderJob");
      if (tagRestartCounter.getLength() != 0) {
        NamedNodeMap attrs = tagRestartCounter.item(0).getAttributes();
        Node nodeResult = attrs.getNamedItem("logMode");
        if (nodeResult != null)
          this.strDefaultLogMode = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("objectIDIncrement");
        String strIncrement = "";
        if (nodeResult != null)
          strIncrement = nodeResult.getNodeValue();
        if (!strIncrement.equals(""))
          this.iDefaultObjectIDIncrement = Integer.parseInt(strIncrement);

        nodeResult = attrs.getNamedItem("objectIDTableName");
        if (nodeResult != null)
          this.strDefaultObjectIDTableName = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("objectIDColumnName");
        if (nodeResult != null)
          this.strDefaultObjectIDColumnName = nodeResult.getNodeValue();

        //log table
        nodeResult = attrs.getNamedItem("dataCutOff");
        if (nodeResult != null)
          this.strDefaultDataCutOff = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logTableName");
        if (nodeResult != null)
          this.strLogTableName = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logTable");
        if (nodeResult != null)
          this.strLogTable = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logColumnName");
        if (nodeResult != null)
          this.strLogColumnName = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logRowNumber");
        if (nodeResult != null)
          this.strLogRowNumber = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logOriginalValue");
        if (nodeResult != null)
          this.strLogOriginalValue = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logNewValue");
        if (nodeResult != null)
          this.strLogNewValue = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logImportDefinitionName");
        if (nodeResult != null)
          this.strLogImportDefinitionName = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logOperationName");
        if (nodeResult != null)
          this.strLogOperationName = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logTypeName");
        if (nodeResult != null)
          this.strLogTypeName = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("objectIDNameColumnName");
        if (nodeResult != null)
          this.strDefaultObjectIDNameColumnName = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("objectIDNameColumnValue");
        if (nodeResult != null)
          this.strDefaultObjectIDNameColumnValue = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("onErrorContinue");
        if (nodeResult != null)
          this.bDefaultOnErrorContinue = (new Boolean(nodeResult.getNodeValue())).booleanValue();

        nodeResult = attrs.getNamedItem("commit");
        if (nodeResult != null)
          this.bDefaultCommit = (new Boolean(nodeResult.getNodeValue())).booleanValue();

        nodeResult = attrs.getNamedItem("userID");
        if (nodeResult != null)
          this.strDefaultUserID = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logDir");
        if (nodeResult != null)
          this.strDefaultLogDir = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("logFile");
        if (nodeResult != null)
          this.strDefaultLogFile = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("vendorConfig");
        if (nodeResult != null)
          this.strDefaultVendorConfig = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("objectIDAutoCreate");
        if (nodeResult != null)
          this.bDefaultObjectIDAutoCreate = (new Boolean(nodeResult.getNodeValue())).booleanValue();

        nodeResult = attrs.getNamedItem("objectIDStartValue");
        if (nodeResult != null)
          this.iDefaultObjectIDStartValue = Integer.parseInt(nodeResult.getNodeValue());

        nodeResult = attrs.getNamedItem("commitCount");
        if (nodeResult != null)
          this.iLoaderJobCommitCount = Integer.parseInt(nodeResult.getNodeValue());

        nodeResult = attrs.getNamedItem("oidLogic");
        if (nodeResult != null)
          this.bDefaultOidLogic = (new Boolean(nodeResult.getNodeValue())).booleanValue();

        nodeResult = attrs.getNamedItem("tableMode");
        if (nodeResult != null)
          this.strDefaultTableMode = nodeResult.getNodeValue();

//        nodeResult = attrs.getNamedItem("logClassName");
//        if (nodeResult != null)
//          this.strLogClassName = nodeResult.getNodeValue();

//        nodeResult = attrs.getNamedItem("pathToLoggerConf");
//        if (nodeResult != null)
//          this.strPathToLoggerConf = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("returnCode");
        if (nodeResult != null) {
          if (!ReturnCode.isParameter)
            ReturnCode.setDefaultErrorReturnCode(Integer.parseInt(nodeResult.getNodeValue()));
        }

        nodeResult = attrs.getNamedItem("setFetchSize");
        if (nodeResult != null)
          this.setFetchSize = nodeResult.getNodeValue();

        nodeResult = attrs.getNamedItem("setCursorName");
        if (nodeResult != null)
          this.setCursorName = nodeResult.getNodeValue();

      }
    }
    try {
      inputStream.reset();
    }
    catch (IOException e) {
      this.logger.write("normal", "Sorry, an error occurred: " + e);
      LoaderException le = new LoaderException("IOException: ",
                           (Throwable)e);
      throw le;
    }
    this.logger.write("full", "\timportLoaderJobAttributes method is finished.");
  }

  /**
   * This method set setCursorName parameter
   * @param setCursorName is value of parameter
   */
  public void setSetCursorName(String setCursorName) {
    this.setCursorName = setCursorName;
  }

  /**
   * This method read value from setCursorName parameter
   * @return value attribute.
   */
  public String getSetCursorName() {
    return this.setCursorName;
  }

  /**
   * This method set setFetchSize parameter
   * @param setFetchSize is value of parameter
   */
  public void setSetFetchSize(String setFetchSize) {
    this.setFetchSize = setFetchSize;
  }

  /**
   * This method read value from setFetchSize parameter
   * @return value attribute.
   */
  public String getSetFetchSize() {
    return this.setFetchSize;
  }

  /**
   * Set Logger which is used for write in log file.
   * @param logger Logger object.
   */
  public void setLogger(Logger logger) {
    this.logger = logger;
  }

  /**
   * This method read value from strDefaultTableMode parameter
   * @return default value of TableMode attribute.
   */
  public String getDefaultTableMode() {
    return this.strDefaultTableMode;
  }

  /**
   * This method reads value from strDefaultDataCutOff parameter
   * @return value of parameter
   */
  public String getDefaultDataCutOff() {
    return this.strDefaultDataCutOff;
  }

  /**
   * This method read value from strDefaultObjectIDNameColumnValue parameter
   * @return default value of ObjectIDNameColumnValue attribute.
   */
  public String getDefaultObjectIDNameColumnValue() {
    return this.strDefaultObjectIDNameColumnValue;
  }

  /**
   * This method read value from strDefaultObjectIDNameColumnName parameter
   * @return default value of ObjectIDNameColumnName attribute.
   */
  public String getDefaultObjectIDNameColumnName() {
    return this.strDefaultObjectIDNameColumnName;
  }

  /**
   * This method read value from strDefaultObjectIDColumnName parameter
   * @return default value of ObjectIDColumnName attribute.
   */
  public String getDefaultObjectIDColumnName() {
    return this.strDefaultObjectIDColumnName;
  }

  /**
   * This method read value from strDefaultObjectIDTableName parameter
   * @return default value of ObjectIDTableName attribute.
   */
  public String getDefaultObjectIDTableName() {
    return this.strDefaultObjectIDTableName;
  }

  /**
   * This method read value from iDefaultObjectIDIncrement parameter
   * @return default value of ObectIDIncrement attribute.
   */
  public int getDefaultObjectIDIncrement() {
    return this.iDefaultObjectIDIncrement;
  }

  /**
   * This method read value from strDefaultLogMode parameter
   * @return default value of LogMode attribute.
   */
  public String getDefaultLogMode() {
    return this.strDefaultLogMode;
  }
  /**
	 * This method set value for strDefaultLogMode parameter
	 */
	public void setDefaultLogMode(String logMode) {
	  this.strDefaultLogMode = logMode;
	}

  /**
   * This method read value from bDefaultOidLogic parameter
   * @return default value of OidLogic attribute.
   */
  public boolean getDefaultOidLogic() {
    return this.bDefaultOidLogic;
  }

  /**
   * This method read value from iDefaultCommitCount parameter
   * @return default value of CommitCount attribute.
   */
  public int getDefaultCommitCount() {
    return this.iDefaultCommitCount;
  }

  /**
   * This method read value from iLoaderJobCommitCount parameter
   * @return default value of iLoaderJobCommitCount attribute.
   */
  public int getLoaderJobCommitCount() {
    return this.iLoaderJobCommitCount;
  }

  /**
   * This method read value from iDefaultObjectIDStartValue parameter
   * @return default value of ObjectIDStartValue attribute.
   */
  public int getDefaultObjectIDStartValue() {
    return this.iDefaultObjectIDStartValue;
  }

  /**
   * This method read value from bDefaultObjectIDAutoCreate parameter
   * @return default value of ObjectIDAutoCreate attribute.
   */
  public boolean getDefaultObjectIDAutoCreate() {
    return this.bDefaultObjectIDAutoCreate;
  }

  /**
   * This method read value from strDefaultVendorConfig parameter
   * @return default value of VendorConfig attribute.
   */
  public String getDefaultVendorConfig() {
    return this.strDefaultVendorConfig;
  }

  /**
   * This method read value from strDefaultLogFile parameter
   * @return default value of LogFile attribute.
   */
  public String getDefaultLogFile() {
    return this.strDefaultLogFile;
  }

  /**
   * This method read value from strDefaultLogDir parameter
   * @return default value of LogDir attribute.
   */
  public String getDefaultLogDir() {
    return this.strDefaultLogDir;
  }

  /**
   * This method read value from strDefaultUserID parameter
   * @return default value of UserID attribute.
   */
  public String getDefaultUserID() {
    return this.strDefaultUserID;
  }

  /**
   * This method read value from bDefaultCommit parameter
   * @return default value of Commit attribute.
   */
  public boolean getDefaultCommit() {
    return this.bDefaultCommit;
  }

  /**
   * This method read value from bDefaultOnErrorContinue parameter
   * @return default value of OnErrorContinue attribute.
   */
  public boolean getDefaultOnErrorContinue() {
    return this.bDefaultOnErrorContinue;
  }

  public void setDefaultOnErrorContinue(boolean value) {
    this.bDefaultOnErrorContinue=value;
  }




  /**
   * This method read value from parameter strLogTableName
   * @return value of parameter
   */
  public String getLogTableName() {
    return this.strLogTableName;
  }

  /**
   * This method read value from parameter strLogTable
   * @return value of parameter
   */
  public String getLogTable() {
    return this.strLogTable;
  }

  /**
   * This method read value from parameter strLogColumnName
   * @return value of parameter
   */
  public String getLogColumnName() {
    return this.strLogColumnName;
  }

  /**
   * This method read value from parameter strLogRowNumber
   * @return value of parameter
   */
  public String getLogRowNumber() {
    return this.strLogRowNumber;
  }

  /**
   * This method read value from parameter strLogOriginalValue
   * @return value of parameter
   */
  public String getLogOriginalValue() {
    return this.strLogOriginalValue;
  }

  /**
   * This method read value from parameter strLogNewValue
   * @return value of parameter
   */
  public String getLogNewValue() {
    return this.strLogNewValue;
  }

  /**
   * This method read value from parameter strLogImportDefinitionName
   * @return value of parameter
   */
  public String getLogImportDefinitionName() {
    return this.strLogImportDefinitionName;
  }

  /**
   * This method read value from parameter strLogOperationName
   * @return value of parameter
   */
  public String getLogOperationName() {
    return this.strLogOperationName;
  }

  /**
   * This method read value from parameter strLogTypeName
   * @return value of parameter
   */
  public String getLogTypeName() {
    return this.strLogTypeName;
  }

}
