/*
  Loader - tool for transfering data from one JDBC source to another and
  doing transformations during copy.
    Copyright (C) 2002  Together
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 LoaderXIncluder.java
 Date: 22.09.2001.
 @version 1.0
 @author:
 Milosevic Sinisa
 */


package org.webdocwf.util.loader;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import org.apache.xerces.parsers.SAXParser;
import org.webdocwf.util.loader.logging.Logger;
import org.xml.sax.AttributeList;
import org.xml.sax.HandlerBase;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * LoaderXIncluder.java
 * This class parses a document (text or xml file) and writes the documents
 * contents back to standard output.
 */
public class LoaderXIncluder
    extends HandlerBase {
  private Writer out;
  private String encoding;
  private int level = 0;
  //counter for dtd file
  private int counter1 = 1;

  //counter for sql files
  private int counter2 = 0;

  //counter for xml (import definition) file
  private int counter3 = 0;

  //counter for line before dtd file
  private int counter4 = 0;

  //counter for LoaderJob.xml (part) file
  private int counter5 = 0;

  //for create and drop database
  private int counter6 = 0;

  private boolean firstPartXml = true;

  private String loaderJobPath="";

  /** Main program entry point.
   * @param argv represents input parmeters
   */
  public static void main(String argv[]) {
    if (argv.length == 0) {
      System.out.println("Usage: java LoaderXIncluder uri");
      System.out.println(" where uri is the URI of your XML document.");
      System.out.println(" Sample: java LoaderXIncluder demo.xml");
      System.exit(1);
    }
    LoaderXIncluder s1 = new LoaderXIncluder();
    s1.parseURI(argv[0]);
  }

  /**
   * Constructor of class without parameters.
   */
  public LoaderXIncluder() {
  }

  /**
   * Construct object LoaderXIncluder with associated outputStream and encoding.
   * @param out - OutputStream where will be written final XML.
   * @param encoding - encoding String representation of encoding.
   * @throws UnsupportedEncodingException
   */
  public LoaderXIncluder(OutputStream out, String encoding) throws UnsupportedEncodingException {
    this.out = new OutputStreamWriter(out, encoding);
    this.encoding = encoding;
  }

  /**
   * Construct object LoaderXIncluder with associated outputStream.
   * Class uses default "UTF-8" encoding.
   * @param out - OutputStream where will be written final XML.
   */
  public LoaderXIncluder(OutputStream out) {
    try {
      this.out = new OutputStreamWriter(out, "UTF8");
      this.encoding = "UTF-8";
    }
    catch (UnsupportedEncodingException e) {}
  }

  /**
   * Method parseUri parses a file "uri" and writes the file
   * contents back to standard output including contents of 'include' files .
   * @param uri - Name of XML file.
   */
  public void parseURI(String uri) {
    try {
      SAXParser parser = new SAXParser();
      parser.setDocumentHandler(this);
      parser.setEntityResolver(this);
      parser.setFeature("http://xml.org/sax/features/validation", false);
      parser.setErrorHandler(this);
//        System.out.println("Loader loads XML file : "+ uri+"\n");
      Logger.getCentralLogger().write("normal", "Loader loads XML file : " + uri);

      File f = new File(uri);
      this.loaderJobPath=f.getCanonicalFile().getParent();
      URL xml = f.toURL();

      InputStreamReader xmlStream = new InputStreamReader(xml.openStream());
      BufferedReader reader = new BufferedReader(xmlStream);
      String intro = "";
      while ( (intro = reader.readLine()) != null) {
        if (intro.indexOf("include") == -1)
          counter6++;
        else
          break;
      }
      counter6 = counter6 - 4;
      LocationOfException.setJdbcNumber(counter6);
      reader.close();
      xmlStream.close();
      parser.parse(uri);
    }
    catch (Exception e) {
//      System.err.println();
      e.printStackTrace();
      Logger.getCentralLogger().write("normal", e.getMessage());
    }
  }

  boolean hasDTD = false;
  /** Processing DOCTYPE deklaration
   * @param publicId is public ID of document
   * @param systemId is system ID of document
   * @return null
   */
  public InputSource resolveEntity(String publicId, String systemId) {
    hasDTD = true;
    try {
      if (systemId != null) {
        out.write("<!DOCTYPE loaderJob SYSTEM \"" + systemId + "\">" + "\n");
        counter4++;
      } else if (publicId != null) {
        out.write("<!DOCTYPE loaderJob PUBLIC \"" + publicId + "\">" + "\n");
        counter4++;
      }
    }
    catch (Exception e) {
      System.err.println(e);
    }
    return null;
  }

  /** Processing instruction.
   *@param target is target
   *@param data is target data
   */
  public void processingInstruction(String target, String data) {
    try {
      out.write("<?");
      out.write(target);
      if (data != null && data.length() > 0) {
        out.write(' ');
        out.write(data);
      }
      out.write("?>");
      counter4++;
    }
    catch (IOException e) {
      System.err.println(e);
    }
  }

  /** Start document. */
  public void startDocument() {
    if (level == 0) {
      try {
        out.write("<?xml version=\"1.0\"?>\r\n");
        counter4++;
      }
      catch (IOException e) {
        System.err.println(e);
      }
      LocationOfException.setIntroNumber(counter4);
    }
  }

  /** Start element.
   * @param name is name of the tag
   * @param atts is attributes of the tag
   */
  public void startElement(String name, AttributeList atts) {
    try {
      //if DTD or SCHEMA is not defined use default
      if (name.equalsIgnoreCase("loaderJob")) {
      	/*
        if (!hasDTD) {
          String OCTOPUS_HOME = System.getProperty("OCTOPUS_HOME");
          if (OCTOPUS_HOME != null) {
            if (! (OCTOPUS_HOME.endsWith("\\") || OCTOPUS_HOME.endsWith("/")))
              OCTOPUS_HOME += "/";
            String url = "file:///" + OCTOPUS_HOME + "XmlTransform/xml/dtd/loaderJob.dtd";
            out.write("<!DOCTYPE loaderJob SYSTEM \"" + url + "\">" + "\n");
          } else {
            URL dtdURL = this.getClass().getClassLoader().getResource("xml/dtd/loaderJob.dtd");
            InputStreamReader dtdStream = new InputStreamReader(dtdURL.openStream());
            BufferedReader reader = new BufferedReader(dtdStream);
            StringBuffer sb = new StringBuffer();
            String nextLine = "";
            boolean firstLine = true;
            while ( (nextLine = reader.readLine()) != null) {
              if (!firstLine) {
                sb.append(nextLine + "\n");
                counter1++;
              } else
                firstLine = false;
            }
            out.write("<!DOCTYPE loaderJob [" + sb.toString() + "\n]>");
            counter1++;
            LocationOfException.setDtdNumber(counter1);
          }
        }
        */
      }
      if (name.equalsIgnoreCase("include")) {
        String href = atts.getValue("href");
        if (href == null) {
          System.out.println("Missing href attribute in include Element");
          System.exit(1);
        }

        File file=new File(href);
        if(!file.isAbsolute()){
          String tmp=this.loaderJobPath + System.getProperty("file.separator") + href;
          File fileHref=new File(tmp);
          href = fileHref.getCanonicalPath();
        }

        String parse = atts.getValue("parse");
        if (parse == null)
          parse = "xml";
        if (parse.equals("text")) {
          String encoding = atts.getValue("encoding");
          includeTextDocument(href, encoding);
        } else if (parse.equals("xml")) {
          level++;
          includeXMLDocument(href);
          level--;
        } else {
          System.out.println("Illegal value for parse attribute: " + parse);
          Logger.getCentralLogger().write("normal", "Illegal value for parse attribute: " + parse);
          System.exit(1);
        }
      } else if (name.equalsIgnoreCase("definitionInclude")) {
      } else {
        out.write("<" + name);
				if (name.equals("loaderJob")) {
					out.write(" xmlns='http://www.objectweb.org'");
					out.write(" xmlns:xsi=\'http://www.w3.org/2001/XMLSchema-instance\'");
					out.write(" xsi:schemaLocation='http://www.objectweb.org http://octopus.objectweb.org/doc/XmlTransform/xml/xmlschema/loaderJob.xsd'");
					out.write(" ");
				}
//                if(name.startsWith("jdbc"))
//                   counter5++;
//                if(!name.startsWith("jdbc") && counter5!=0 && firstPartXml==true){
//                  LocationOfException.setJdbcNumber(counter5);
//                  firstPartXml=false;
//                  counter5=0;
//                }
//System.out.println("atts="+atts);
        for (int i = 0; i < atts.getLength(); i++) {
          out.write(" ");
          out.write(atts.getName(i));
          out.write("='");
          String value = atts.getValue(i);
          // + 4 allows space for one entitiy reference.
          // If there's more than that, then the StringBuffer
          // will automatically expand
          // Need to use character references if the encoding
          // can't support the character
          StringBuffer encodedValue = new StringBuffer(value.length() + 4);
          for (int j = 0; j < value.length(); j++) {
            char c = value.charAt(j);
            if (c == '&')
              encodedValue.append("&amp;");
            else if (c == '<')
              encodedValue.append("&lt;");
            else if (c == '>')
              encodedValue.append("&gt;");
            else if (c == '\'')
              encodedValue.append("&apos;");
            else
              encodedValue.append(c);
          }
          out.write(encodedValue.toString());
          out.write("'");
        }
        out.write(">");
      }
    }
    catch (IOException e) {
      System.err.println(e);
      Logger.getCentralLogger().write("normal", e.getMessage());
    }
    catch (SAXException ex) {
      System.err.println(ex);
      Logger.getCentralLogger().write("normal", ex.getMessage());
    }
  }

  /** Characters.
   * @param ch is array of characters
   * @param start is int
   * @param length is length of the input parameters
   * @throws SAXException
   */
  public void characters(char ch[], int start, int length) throws SAXException {
    try {
      for (int i = 0; i < length; i++) {
        char c = ch[start + i];
        if (c == '&')
          out.write("&amp;");
        else if (c == '<')
          out.write("&lt;");
        else if (c == '\n') {
          counter2++;
          out.write(c);
        } else
          out.write(c);
      }
    }
    catch (IOException e) {
      System.err.println(e);
      Logger.getCentralLogger().write("normal", e.getMessage());
    }
  }

  /** Ignorable whitespace.
   * @param ch is array of characters
   * @param start is int
   * @param length is length of the input parameters
   */
  public void ignorableWhitespace(char ch[], int start, int length) {
    try {
      this.characters(ch, start, length);
    }
    catch (SAXException e) {
      System.err.println(e);
      Logger.getCentralLogger().write("normal", e.getMessage());
    }
  }

  /** End element.
   * @param name is name of the tag
   */
  public void endElement(String name) {
    if (!name.equalsIgnoreCase("include") && !name.equalsIgnoreCase("definitionInclude")) {
//            if(name.startsWith("jdbc")&&name.endsWith("s"))
//              counter5++;
      try {
        out.write("</");
        out.write(name);
        out.write(">");
      }
      catch (IOException e) {
        System.err.println(e);
        Logger.getCentralLogger().write("normal", e.getMessage());
      }
    }
  }

  /** End document. */
  public void endDocument() {
    try {
      out.flush();
    }
    catch (IOException e) {
      System.err.println(e);
      Logger.getCentralLogger().write("normal", e.getMessage());
    }
  }

  //
  // ErrorHandler methods
  //
  /** Warning.
   * @param ex is SAXParseException exception
   */
  public void warning(SAXParseException ex) {
    System.err.println("[Warning] " + getLocationString(ex) + ": " + ex.getMessage());
    Logger.getCentralLogger().write("normal",
        "[Warning] " + getLocationString(ex) + ": " + ex.getMessage());
  }

  /** Error.
   * @param ex is SAXParseException exception
   */
  public void error(SAXParseException ex) {
    System.err.println("[Error] " + getLocationString(ex) + ": " + ex.getMessage());
    Logger.getCentralLogger().write("normal",
        "[Error] " + getLocationString(ex) + ": " + ex.getMessage());
  }

  /** Fatal error.
   * @param ex is SAXParseException exception
   * @throws SAXException
   */
  public void fatalError(SAXParseException ex) throws SAXException {
    System.err.println("[Fatal Error] " + getLocationString(ex) + ": " + ex.getMessage());
    Logger.getCentralLogger().write("normal",
        "[Fatal Error] " + getLocationString(ex) + ": " + ex.getMessage());
    throw ex;
  }

  /** Returns a string of the location.
   * @param ex is SAXParseException exception
   * @return string
   */
  private String getLocationString(SAXParseException ex) {
    StringBuffer str = new StringBuffer();
    String systemId = ex.getSystemId();
    if (systemId != null) {
      int index = systemId.lastIndexOf('/');
      if (index != -1)
        systemId = systemId.substring(index + 1);
      str.append(systemId);
    }
    str.append(':');
    str.append(ex.getLineNumber());
    str.append(':');
    str.append(ex.getColumnNumber());
    return str.toString();
  }

  /**
   * <p>
   * This utility method reads a document at a specified URL
   * and fires off calls to <code>characters()</code>.
   * It's used to include files with <code>parse="text"</code>
   * </p>
   *
   * @param  url          URL of the document that will be read
   * @param  encoding     Encoding of the document; e.g. UTF-8,
   *                      ISO-8859-1, etc.
   * @throws SAXException if the requested document cannot
   * be downloaded from the specified URL
   * or if the encoding is not recognized
   */
  private void includeTextDocument(String url, String encoding) throws SAXException {
    if (encoding == null || encoding.trim().equals(""))
      encoding = "UTF-8";
    File sourceXml = new File(url);
    URL source;
    counter2 = 1;
    try {
      source = sourceXml.toURL();
    }
    catch (MalformedURLException e) {
      Logger.getCentralLogger().write("normal", "Unresolvable URL :" + e.getMessage());
      throw new SAXException("Unresolvable URL :", e);
    }
    try {
      URLConnection uc = source.openConnection();
      String encodingFromHeader = uc.getContentEncoding();
      if (encodingFromHeader != null)
        encoding = encodingFromHeader;
      InputStream in = new BufferedInputStream(uc.getInputStream());
      InputStreamReader reader = new InputStreamReader(in, encoding);
      char[] c = new char[1024];
      while (true) {
        int charsRead = reader.read(c, 0, 1024);
        if (charsRead == -1)
          break;
        if (charsRead > 0)
          this.characters(c, 0, charsRead);
      }
      counter2 = counter2 + 4;
      LocationOfException.setFileLineNumber(counter2, url);
//            System.out.println("Sql file "+url+"ima broj linija :"+counter2);
      counter2 = 0;
    }
    catch (UnsupportedEncodingException e) {
      Logger.getCentralLogger().write("normal", "Unsupported encoding: " + encoding);
      throw new SAXException("Unsupported encoding: " + encoding, e);
    }
    catch (IOException e) {
      Logger.getCentralLogger().write("normal", "Document not found: " + source.toExternalForm());
      throw new SAXException("Document not found: " + source.toExternalForm(), e);
    }
  }

  /**
   * <p>
   * This utility method reads a document at a specified URL
   * and fires off calls to various <code>DocumentHandler</code> methods.
   * It's used to include files with <code>parse="xml"</code>
   * </p>
   *
   * @param  url          URL of the document that will be read
   * @throws SAXException if the requested document cannot
   be downloaded from the specified URL.
   */
  private void includeXMLDocument(String url) throws SAXException {
    File sourceXml = new File(url);
    URL source;
    InputStream includeXML = null;
//        int counter3=0;
    try {
      source = sourceXml.toURL();
      includeXML = source.openStream();
      InputStreamReader isr = new InputStreamReader(includeXML);
      BufferedReader br = new BufferedReader(isr);
      while (br.readLine() != null)
        counter3++;
      counter3 = counter3 + 2;
      LocationOfException.setFileLineNumber(counter3, url);
//            System.out.println("xml file " + url+" ima broj linija :"+counter3);
      counter3 = 0;
//            LocationOfException.setJdbcNumber(counter2);

    }
    catch (MalformedURLException e) {
      Logger.getCentralLogger().write("normal", "Unresolvable URL :" + e.getMessage());
      throw new SAXException("Unresolvable URL :", e);
    }
    catch (IOException e) {
      Logger.getCentralLogger().write("normal", "Error in opening input stream :" + e.getMessage());
      throw new SAXException("Error in opening input stream :", e);
    }

    SAXParser includeParser = new SAXParser();
    includeParser.setDocumentHandler(this);
    includeParser.setEntityResolver(this);
    includeParser.setErrorHandler(this);
    try {
      includeParser.parse(url);
    }
    catch (Exception e) {
      System.err.println(e);
      Logger.getCentralLogger().write("normal", e.getMessage());
    }
  }


}
