/*
  OctopusClassLoader
    Copyright (C) 2002-2003  Together
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 Loader.java
 Date: 03.03.2003.
 @version 2.1 alpha
 @author:
 Zoran Milakovic zoran@prozone.co.yu
 */


package org.webdocwf.util.loader;

import java.io.*;
import java.net.*;
import java.lang.reflect.*;

/**
 *
 * OctopusClassLoader add files to System Class Loader
 * @author Zoran Milakovic
 * @version 1.0
 */
public class OctopusClassLoader {

  private static final Class[] parameters = new Class[] {URL.class};

  /**
   * This method add file to System class loader
   * @param s defines the path to file object (string)
   * @throws IOException
   */
  public static void addFile(String s) throws IOException {

    File f = new File(s);

    addFile(f);

  } //end method

  /**
   * This method add file to System class loader
   * @param f defines the file object
   * @throws IOException
   */
  public static void addFile(File f) throws IOException {

    addURL(f.toURL());

  } //end method


  /**
   * This method add file to System class loader
   * @param u defines the url object
   * @throws IOException
   */
  public static void addURL(URL u) throws IOException {

    URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();

    Class sysclass = URLClassLoader.class;

    try {

      Method method = sysclass.getDeclaredMethod("addURL", parameters);

      method.setAccessible(true);

      method.invoke(sysloader, new Object[] {u});

    }
    catch (Throwable t) {

      t.printStackTrace();

      throw new IOException("Error, could not add URL to system classloader");

    } //end try catch

  } //end method

} //end class
