/**
  Transformation - Transformations in Octopus Loader.
    Copyright (C) 2002-2004  Together
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 Transformation.java
 Date: 05.04.2004.
 @version 1.0
 @author: Milosevic Sinisa sinisa@prozone.co.yu
 */

package org.webdocwf.util.loader.transformation;

import java.lang.reflect.Constructor;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.webdocwf.util.loader.ConfigReader;
import org.webdocwf.util.loader.LoaderException;
import org.webdocwf.util.loader.OctopusXMLUtil;
import org.webdocwf.util.loader.logging.Logger;

/**
 *
 * Transformation - transformations in Octopus Loader.
 */
public class Transformation {

  
  
  private String transformationName; 
  private String transformationClassName;
  private String transformatorConfigName;
  
  private Vector sourceColumnNames = new Vector();

  private List transformationTargetColumns = new ArrayList();
  
  private Transformer transformer;
  private Logger logger;
  private JavaScriptEvaluator jsEvaluator;
  private Element transformationDocFragment;


  public Transformation(String name, String className, String configString, Element doc) throws Exception {

      this.transformationName = name;
      this.transformationClassName = className;
      this.transformatorConfigName = configString;
      this.transformationDocFragment = doc;
			String javaScriptExpression = ""; 
			
	    NodeList childNodes = this.transformationDocFragment.getElementsByTagName("javaScript");
	    if ( childNodes.item(0) != null ){
					javaScriptExpression = childNodes.item(0).getFirstChild().getNodeValue();
	    }
      try {
				init();
			  if (javaScriptExpression != ""){
					//instantiate JavaScriptEvaluator for transformation with javaScript
					jsEvaluator = new JavaScriptEvaluator();
					jsEvaluator.setExpression(javaScriptExpression);
					jsEvaluator.setVariableNames(this.sourceColumnNames);
					this.transformer = (Transformer) jsEvaluator;
      	}else{
				 //instantiate transformer
				 Class transformatorClass = Class.forName(this.transformationClassName);
			   Class[] ArgClassArray = new Class[] { };
			   Object[] ArgObject = new Object[] { };
			   Constructor transConstructor = transformatorClass.getDeclaredConstructor(ArgClassArray);
			   this.transformer = (Transformer)(transConstructor.newInstance(ArgObject));
				}
			 if(transformatorConfigName != null && this.transformer != null) {
				this.transformer.configure(transformatorConfigName);
			 }
				
      }catch(Exception e) {
      	throw new LoaderException("Error during transformation!",e);
      }
  }

  public Transformation(String name, String className, Element doc) throws Exception {
      new Transformation(name,className,null,doc);
  }

  /**
   * This method set logger object
   */
  public void setLogger(Logger logger) {
		if (this.transformer instanceof JavaScriptEvaluator){
			jsEvaluator.setLogger(logger);      
		}else{
    	this.logger = logger;
		}

  }
  
  	private void init() {
		initSourceColumns();
		initTargetColumns();
	}



  /**
   * @param doc represents Object document
   * @param importJob represents current import job
   */
  private void initSourceColumns() {
//ZK change this because of problem with reading xml file, because of method importValue which reads all data from <importDefinition>tag 
	if (this.transformationDocFragment!=null){
		this.sourceColumnNames = OctopusXMLUtil.importValueForTransform(this.transformationDocFragment, "sourceColumn","name"); 
	}
  }

  private void initTargetColumns() {
	Vector targetColumnNames = OctopusXMLUtil.importValueForTransform(this.transformationDocFragment, "targetColumn","name");
	Vector targetTableNames = OctopusXMLUtil.importValueForTransform(this.transformationDocFragment, "targetColumn","tableName");
	Vector targetTableIDs = OctopusXMLUtil.importValueForTransform(this.transformationDocFragment, "targetColumn","tableID");    
	Vector targetValueModes = OctopusXMLUtil.importValueForTransform(this.transformationDocFragment, "targetColumn","valueMode");
	for(int i = 0; i < targetColumnNames.size(); i++) {
		this.transformationTargetColumns.add(new TransformationTargetColumn(
		(String)targetColumnNames.get(i),
		(String)targetTableNames.get(i),
		(String)targetTableIDs.get(i),
		(String)targetValueModes.get(i)
		));
	}
	
  }


  /**
   * Returns Vector with source column names
   * @return vector
   */
  public Vector getSourceColumnNames() {
    return this.sourceColumnNames;
  }

  /**
   * Returns Vector with target column names
   * @return vector
   */
  public Vector getTargetColumnNames() {
    Vector retVal = new Vector();
    for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
		retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getName() );
    }
    return retVal;
  }
  
  /**
	 * Returns Vector with target column names
	 * @param index logical table index
	 * @return vector
	 */
	public Vector getTargetColumnNames(int index) {
	  Vector retVal = new Vector();
	  for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
	  	  if( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getTableID().equals( String.valueOf(index) ) )
		  	retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getName() );
	  }
	  
	  return retVal;
	}
	
	/**
	* Returns Vector with target column types
	* @param index logical table index
	* @return vector
	*/
   public Vector getTargetColumnTypes(int index) {
	 Vector retVal = new Vector();
	 for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
		 if( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getTableID().equals( String.valueOf(index) ) )
		   retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getType() );
//		 else 
//		 	retVal.add(null);
	 }
	 return retVal;
   }
   
   /**
   * Returns Vector with target column types on ordered places, and 
   * add null if in this place logic table is different than specified
   * @param index logical table index
   * @return vector
   */
  public Vector getOrderedTargetColumnTypes(int index) {
	Vector retVal = new Vector();
	for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
		if( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getTableID().equals( String.valueOf(index) ) )
		  retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getType() );
		else 
		  retVal.add(null);
	}
	return retVal;
  }
   
   /**
   * Returns Vector with target value modes
   * @param index logical table index
   * @return vector
   */
  public Vector getTargetValueModes(int index) {
	Vector retVal = new Vector();
	for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
		if( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getTableID().equals( String.valueOf(index) ) )
		  retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getValueMode() );
	}
	return retVal;
  }
   
   /**
	  * Returns Vector with target key column names
	  * @param index logical table index
	  * @return vector
	  */ 
	 public Vector getTargetKeyColumnNames(int index) {
	   Vector retVal = new Vector();
	   for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
			if( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getTableID().equals( String.valueOf(index) ) ) {
			   if( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getValueMode().equals("Key") )
				   retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getName() );
			}
	   }
	   return retVal;
	 }
  
  /**
	 * Returns Vector with target column names for specified table
	 * @param tableName name of table
	 * @return vector
	 */
	public Vector getTargetColumnNames(String tableName) {
	  Vector retVal = new Vector();
	  for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
	  		if( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getTableName().equals(tableName) )
		  		retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getName() );
	  }
	  return retVal;
	}


  /**
   * Returns Vector with target key column names
   * @return vector
   */ 
  public Vector getTargetKeyColumnNames() {
	Vector retVal = new Vector();
	for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
		if( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getValueMode().equals("Key") )
			retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getName() );
	}
	return retVal;
  }

  /**
   * Returns Vector with target value mode
   * @return vector
   */ 
  public Vector getTargetValueMode() {
	Vector retVal = new Vector();
	for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
		retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getValueMode() );
	}
	return retVal;
  }


  /**
   * Returns Vector with target column types
   * @return vector
   */
//  public Vector getTargetColumnTypes() {
//	Vector retVal = new Vector();
//	for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
//		retVal.add( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getValueMode() );
//	}
//	return retVal; 
//  }

 
  /**
   * This method read value of sub counter parameter
   * @param sourceValues values to transform
   * @return vector transformed values
   * @throws SQLException
   */
  public Vector transformValues(Vector sourceValues) throws Exception{
    Vector results = new Vector();
    try {  
	    results.addAll(this.transformer.transformValue(sourceValues));
    } catch (Exception except) {
      throw except;
    }
    return results;

}
  /**
   * This method returns transformation name
   * @return String which represents transformation name
   */
  public String getName() {
    return this.transformationName;
  }


  /**
   * Method transformationColumnTypes is used to put types of transformation columns into
   * global vector sorted in target tables. If there is an error, Exception
   * "SQLException" or "NullPointerException" is thrown.
   * @param c Connection to target database.
   * @param firstColumn is first column
   * @param columnsSuportedTarget is true if driver for target database supports getColumns method
   * @param configReaderTarget is ConfigReader object for target database
   * @throws SQLException Constructs an SQLException object with a reason.
   * @throws NullPointerException Constructs a NullPointerException with the specified detail message.
   */
  public void transformationColumnTypes(
  		Connection c,
      int firstColumn, 
      boolean columnsSuportedTarget, ConfigReader configReaderTarget) throws SQLException, NullPointerException {
    	int iCnt = 0;
    try {
    List tableNames = this.getTargetTableNames();
   	for(int k = 0; k < tableNames.size(); k++) {
	      Vector columnNames = this.getTargetColumnNames(tableNames.get(k).toString());
	      Statement stmtTrans = c.createStatement();
	      //Vector typs = new Vector();
	      Vector subTyps = new Vector();
	      String strQuery = "select ";
				ResultSet rsetTrans=null;
	      if (columnNames.size() != 0) {
	        for (int i = 0; i < columnNames.size(); i++) {
	            strQuery +=  columnNames.get(i).toString() +
	                ", ";
	          }
	          strQuery = strQuery.substring(0, strQuery.length() - 2);
	          strQuery += " from " + tableNames.get(k).toString();
	       if (columnsSuportedTarget){
						rsetTrans = c.getMetaData().getColumns( c.getCatalog(), null, tableNames.get(k).toString(), "%" );
						String columnName = "";
						String columnType = "";
						while(rsetTrans.next()){
							columnName = rsetTrans.getString(3+firstColumn);
							columnType = rsetTrans.getString(5+firstColumn);
							for (int j = 0; j < columnNames.size(); j++) {									
									if( columnNames.get(j).toString().equalsIgnoreCase( columnName ) ){
									this.setType(tableNames.get(k).toString(),columnNames.get(j).toString(),columnType);
									//typs.add(columnType);
							  }
							}
						}		
	       }else{//TODO ZK ADDED stmtConstant.setMaxRows(1). Place this as parameter in conf file, like maxRowsSuported
					if (configReaderTarget.getMaxRowsSupported()){
						stmtTrans.setMaxRows(1);
	    		}
						rsetTrans = stmtTrans.executeQuery(strQuery);
						for (int j = 0; j <  columnNames.size(); j++) {
								this.setType( tableNames.get(k).toString(),
								columnNames.get(j).toString(),
								rsetTrans.getMetaData().getColumnTypeName(j + firstColumn) );
							//typs.add(rsetConstant.getMetaData().getColumnTypeName(j + firstColumn));
						}
			  }		
				rsetTrans.close();
	    }
				stmtTrans.close();
   	}
//	   this.hmTargetColumnTypes.addAll(typs);
    }
    catch (SQLException ex) {
      throw ex;
    }
    catch (NullPointerException ex) {
      throw ex;
    }
  }
  
  public List getTargetTableNames() {
  	List retVal = new ArrayList();
	Element transformation = this.transformationDocFragment;
   NodeList nodeList = transformation.getChildNodes();
   String tableName = "";
   for(int i = 0; i < nodeList.getLength(); i++) {
		if( nodeList.item(i).getNodeType() != Node.ELEMENT_NODE )
			continue;
		   if( !nodeList.item(i).getNodeName().equals("targetColumns"))
				continue;
		Element targetColumns = (Element)nodeList.item(i);
		NodeList targetColumnElements = targetColumns.getElementsByTagName("targetColumn");
		for(int j = 0; j < targetColumnElements.getLength();j++) { 
		   Element targetColumn = (Element)targetColumnElements.item(j);
		   tableName = targetColumn.getAttribute("tableName");
		   if( !retVal.contains(tableName) )  
		   	retVal.add(tableName);
		}
   }
   return retVal;
  }
  
  public String getTargetTableID() {
	Element transformation = this.transformationDocFragment;
   NodeList nodeList = transformation.getChildNodes();
   for(int i = 0; i < nodeList.getLength(); i++) {
   	if( nodeList.item(i).getNodeType() != Node.ELEMENT_NODE )
   		continue;
	   if( !nodeList.item(i).getNodeName().equals("targetColumns"))
			continue;
   	Element targetColumns = (Element)nodeList.item(i);
	   Element targetColumn = (Element)targetColumns.getElementsByTagName("targetColumn").item(0); 
	   return targetColumn.getAttribute("tableID");
   }
   return null;
	}
  
   private void setType(String tableName, String columnName, String type) {
		for(int i = 0; i < this.transformationTargetColumns.size(); i++) {
			if( ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getTableName().equals(tableName) && 
				 ((TransformationTargetColumn)this.transformationTargetColumns.get(i)).getName().equals(columnName)  )
			((TransformationTargetColumn)this.transformationTargetColumns.get(i)).setType(type);
	   }
   }
  
  /**
   * This method reset all variables
   */
  public void reset() {
      this.sourceColumnNames = new Vector();
      this.transformationTargetColumns = new ArrayList();
  }
  
  public void release() throws Exception{ 	
  	this.transformer.release();
  }
}
