/*
* LoaderTest.java   Sept 01, 2002
*
* Sample JUnit test using Loader for creating test database and
* inserting data into it.
*
 */

package test.org.webdocwf.util.loader;


import java.sql.Connection;
import java.sql.DriverManager;

import org.webdocwf.util.loader.test.LoaderTestCase;
import org.webdocwf.util.loader.test.DatabaseOperation;
import org.webdocwf.util.loader.test.CreateDatabaseOperation;
import org.webdocwf.util.loader.test.DropDatabaseOperation;
import org.webdocwf.util.loader.test.LoaderOperation;
import org.webdocwf.util.loader.Loader;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import junit.framework.TestResult;

/**
 * @author Sinisa Milosevic
 * @version $Revision: 1.4 $
 */
public class LoaderTest3 extends LoaderTestCase
{

  public LoaderTest3(String name)
  {
    super(name);
  }

  /**
   * Returns the test database connection.
   * @throws Exception
   * @return jdbc connection
   */
  public Connection getConnection() throws Exception
  {

    Class driverClass = Class.forName("org.hsqldb.jdbcDriver");
    Connection jdbcConnection = DriverManager.getConnection(
        "jdbc:hsqldb:test/LoaderTest3/LoaderTest3","sa","");

    return jdbcConnection;
  }

  /**
   * Returns the name of test database.
   * @throws Exception
   * @return string
   */

  public String getDatabaseName() throws Exception
  {
    return "LoaderTest3";
  }


  /**
   * Returns the test Loader class (loaderjob).
   * @throws Exception
   * @return Loader object
   */
  public Loader getLoader() throws Exception
  {
    
    showHeader();
    Loader loadJob= new Loader("modules/Octopus/src/testdata/ObjectLoader/CreateTables3.xml");
    loadJob.setUserID("admin");
    loadJob.setLogDirName("test");
    loadJob.setLogFileName("LoaderTest3.txt");


    return loadJob;
  }
  
  private static boolean isHeaderShown = false;
  private void showHeader() {
  	if( !this.isHeaderShown ) {
  	System.out.println();	
        System.out.println("******************************************************");
        System.out.println("   Executing test: test3 - ");
        System.out.println("******************************************************");		
        this.isHeaderShown = true;
  	}
  }

  /**
   * Returns the database operations executed in test setup. First operation will be
   * executed dbOperation[0], then dbOperation[1]...
   * @throws Exception
   * @return dbOperation parameter
   */
  public DatabaseOperation[] getSetUpOperation() throws Exception
  {
//        Creating test database.....
    DatabaseOperation[] dbOperation = new DatabaseOperation[3];
//        dbOperation[0]=new CreateDatabaseOperation(getDatabaseName());

//        Creating tables.....
    dbOperation[0]=new LoaderOperation(getLoader());

//		    Inserting data.....
		  Loader loadJob1= new Loader("modules/Octopus/src/testdata/ObjectLoader/InsertData.xml");
    loadJob1.setUserID("admin");
    loadJob1.setLogDirName("test");
    loadJob1.setLogFileName("LoaderTest3.txt");
    dbOperation[1]=new LoaderOperation(loadJob1);

//		    Creating indexes, foreign keys.....
		  Loader loadJob2= new Loader("modules/Octopus/src/testdata/ObjectLoader/CreateIndex.xml");
    loadJob2.setLogDirName("test");
    loadJob2.setLogFileName("LoaderTest3_CreateIndex.txt");
    dbOperation[2]=new LoaderOperation(loadJob2);


    return dbOperation;
  }

  /**
   * Returns the database operation executed in test cleanup.
   *  First operation will be executed dbOperation[0], then dbOperation[1]...
   * @throws Exception
   * @return dbOperation parameter
   */
  public DatabaseOperation[] getTearDownOperation() throws Exception
  {
//		    Deleting test database.....
    DatabaseOperation[] dbOperation = new DatabaseOperation[1];
    dbOperation[0]=DatabaseOperation.DO_NOTHING;

    return dbOperation;
  }


  public void testMe() throws Exception
  {
   
  }


  public static Test suite() {
    return new TestSuite(LoaderTest3.class);
  }


  public static void main(String args[]) {
    
//    	junit.textui.TestRunner.run(suite());
    TestResult result= (new LoaderTest3("testMe 3")).run();

  }



}




