
/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.generator;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.enhydra.xml.ElementImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.webdocwf.util.loader.LoaderException;
import org.webdocwf.util.loader.logging.Logger;
import org.webdocwf.util.loader.logging.StandardLogger;

/**
 * DomlDesignReader class retrieves the input data from doml file, and placed them in to
 * ImportDefinition class.
 * @author Radoslav Dutina
 * @version 1.0
 */
public class DomlDesignReader {

  private ElementImpl searchDocument;
  private Document documentXml;
  private String databaseType=null;

  private String tableName=null;

  private String notUsingOid=null;
  private String columnName=null;
  private String isConstant=null;
  private String isPrimaryKey=null;

  private String size=null;
  private String canBeNull=null;
  private String dbType=null;

  private String indexId=null;
  private String indexUnique=null;
  private String indexColumnId=null;

  private String constraint=null;
  private String foreignKeyColumn=null;
  private String reference=null;
  private String foreignTable=null;

  private static List listColumnNames=null;
  private static List listTargetTableNames=null;
  private static List listTargetTableID=null;
  private static List listColumnType=null;
  private static List listColumnLenght=null;
  private static List listAllowNulls=null;

  private static List listPrimaryKeys=null;
  private static List listIndexVariables=null;
  private static List listForeignVariables=null;

  private ImportDefinitionAttributes importDefinitionAttributes=new ImportDefinitionAttributes();
  private RelationshipsAttributes relationshipsAttributes=new RelationshipsAttributes();
  private MappingTypeData mappingTypeData=null;

  private StringTokenizer st;
  private String oidDbType=null;
  private String oidDbSize=null;

  private Logger logger;
  /**
   * Construct object DomlDesignReader with associated parameters.
   * @param generatorParameters represents the references to InputParameter object.
   * @param document is the object of Document class.
   * @param root is the object of Element class.
   * @throws LoaderException
   */

  public DomlDesignReader(Document document, Element root, InputParameters generatorParameters)
      throws LoaderException{
		setLogger();
		this.logger.write("normal", "DomlDesignReader is started.");
    DocumentBuilderFactory factoryXml = DocumentBuilderFactory.newInstance();
    try {
      File file = new File( generatorParameters.getDomlPath());
      DocumentBuilder builderXml = factoryXml.newDocumentBuilder();
      try {
        this.documentXml = builderXml.parse( file );
      }catch( Exception e ) {
        String msg="Exception in class DomlDesignReader: Error has occurred when trying to parse file!";
        LoaderException le=new LoaderException(msg+"\n"+e.getMessage()+"\n", (Throwable)e);
				this.logger.write("full", "Exception in class DomlDesignReader: Error has occurred when trying to parse file!"+"\n"+le.getStackTraceAsString());
        throw le;
      }
      this.searchDocument = (ElementImpl)ElementImpl.newInstance(documentXml);

      NodeList databaseName = searchDocument.getElementsByTagName("database");
      if(databaseName.getLength()!=0){
        databaseType=databaseName.item(0).getAttributes().item(0).getNodeValue();
        //28.08.2003
//        generatorParameters.setSourceType(databaseType.toString());

        NodeList tableList=searchDocument.getElementsByTagName("table");
        int countOid=0;
        int counterConstraint=0;
        if(tableList.getLength()!=0){
          for(int i=0;i<tableList.getLength();i++){
            String maxConstraintLength=generatorParameters.getMaxConstraintLength();
            int maxConstraintInt=0;
            if(!maxConstraintLength.equalsIgnoreCase("") && !maxConstraintLength.equalsIgnoreCase("-1")){
              maxConstraintInt=Integer.parseInt(maxConstraintLength);
            }
            listColumnNames=new ArrayList();
            listTargetTableNames=new ArrayList();
            listTargetTableID=new ArrayList();
            listColumnType=new ArrayList();
            listColumnLenght=new ArrayList();
            listAllowNulls=new ArrayList();

            listPrimaryKeys=new ArrayList();
            listIndexVariables=new ArrayList();
            listForeignVariables=new ArrayList();

//            MappingOidAndVersion mappingOidAndVersion=new MappingOidAndVersion(generatorParameters);

            st=new StringTokenizer(generatorParameters.getOidColumnType());
            int countOidToken=0;
            //e.g. oidDb=decimal (19,0)
            while(st.hasMoreTokens()){
              if(countOidToken==0)
                oidDbType=st.nextToken();
              else
                oidDbSize=(st.nextToken()).substring(1,5);
              countOidToken++;
            }

            tableName=((ElementImpl)tableList.item(i)).getAttribute("dbTableName");
            notUsingOid=((ElementImpl)tableList.item(i)).getAttribute("notUsingOid");
            //System.out.println("Working...."+tableName);
						this.logger.write("normal", "Working...."+tableName);
            //BufferOctopusClass.getInstance().writeToBuffer("Working...."+tableName);
            //if notUsingOid is false, we must add columns oid and version
            boolean addOid=false;
            if(notUsingOid.equalsIgnoreCase("")||notUsingOid.equalsIgnoreCase("false"))
              addOid=true;

            importDefinitionAttributes.setName(tableName);
            importDefinitionAttributes.setTableName(tableName);

            NodeList indexList=((ElementImpl)tableList.item(i)).getElementsByTagName("index");
            if(indexList.getLength()!=0){
              for(int l=0;l<indexList.getLength();l++){
                indexId=((ElementImpl)indexList.item(l)).getAttribute("id");
                indexUnique=((ElementImpl)indexList.item(l)).getAttribute("unique");

                NodeList indexColumn=((ElementImpl)indexList.item(l)).getElementsByTagName("indexColumn");
                indexColumnId="";
                for(int j=0;j<indexColumn.getLength();j++){
                indexColumnId+=((ElementImpl)indexColumn.item(j)).getAttribute("id")+",";
                }
                indexColumnId=indexColumnId.substring(0,indexColumnId.length()-1);

                if(indexUnique.equalsIgnoreCase("false")||indexUnique.equalsIgnoreCase("")){
                  listIndexVariables.add("1");
                }else{
                  listIndexVariables.add("0");
                }
                //index lenght restrictions
                String indexName=indexId+"_"+tableName;
                if( maxConstraintInt>0 && indexName.length()>maxConstraintInt ){
                  String newIndexName=indexName.substring(0,maxConstraintInt-String.valueOf(counterConstraint).length());
                  indexName=newIndexName+String.valueOf(counterConstraint);
                  counterConstraint++;
                }
                listIndexVariables.add(indexName);
                listIndexVariables.add(indexColumnId);
              }
            }

            NodeList columnList=((ElementImpl)tableList.item(i)).getElementsByTagName("column");
            if(columnList.getLength()!=0){
              for(int j=0;j<columnList.getLength();j++){
                columnName=((ElementImpl)columnList.item(j)).getAttribute("id");

                listColumnNames.add(columnName);
                listTargetTableNames.add(tableName);
                listTargetTableID.add("0");

                isConstant=((ElementImpl)columnList.item(j)).getAttribute("isConstant");
                isPrimaryKey=((ElementImpl)columnList.item(j)).getAttribute("isPrimaryKey");
                //primary key restrictions
                if(isPrimaryKey.equalsIgnoreCase("true")){
                  String primaryKeyName=tableName+"_"+columnName;
                  if( maxConstraintInt>0 && primaryKeyName.length()>maxConstraintInt ){
                    String newPrimaryName=primaryKeyName.substring(0,maxConstraintInt-String.valueOf(counterConstraint).length());
                    primaryKeyName=newPrimaryName+String.valueOf(counterConstraint);
                    counterConstraint++;
                  }
                  listPrimaryKeys.add(primaryKeyName);
                  listPrimaryKeys.add(columnName);
                }

                NodeList typeConstraint=((ElementImpl)columnList.item(j)).getElementsByTagName(
                    "referenceObject");
                if(typeConstraint.getLength()!=0){
                  for(int m=0;m<typeConstraint.getLength();m++){
                    constraint=((ElementImpl)typeConstraint.item(m)).getAttribute("constraint");
                    foreignKeyColumn=((ElementImpl)typeConstraint.item(m)).getAttribute(
                        "foreignKeyColumn");
                    reference=((ElementImpl)typeConstraint.item(m)).getAttribute("reference");

                    NodeList allTable=searchDocument.getElementsByTagName("table");
                    for(int n=0;n<allTable.getLength();n++){
                      String ID=((ElementImpl)allTable.item(n)).getAttribute("id");
                      if(ID.equalsIgnoreCase(reference)){
                        foreignTable=((ElementImpl)allTable.item(n)).getAttribute("dbTableName");
                        break;
                      }
                    }
                    //foreign key restrictions (lenght)
                    String foreignKeyName=tableName+"_"+columnName;
                    if( maxConstraintInt>0 && foreignKeyName.length()>maxConstraintInt ){
                      String newFKName=foreignKeyName.substring(0,maxConstraintInt-String.valueOf(counterConstraint).length());
                      foreignKeyName=newFKName+String.valueOf(counterConstraint);
                      counterConstraint++;
                      }

                    if(foreignKeyColumn.equalsIgnoreCase("true")){
                      listForeignVariables.add(tableName);
//                      listForeignVariables.add(tableName+"_"+columnName);
                      listForeignVariables.add(foreignKeyName);
                      listForeignVariables.add(columnName);
                      listForeignVariables.add(foreignTable);
                      listForeignVariables.add(foreignKeyColumn);
                    }else{
                      //oid column
                      listForeignVariables.add(tableName);
//                      listForeignVariables.add(tableName+"_"+columnName);
                      listForeignVariables.add(foreignKeyName);
                      listForeignVariables.add(columnName);
                      listForeignVariables.add(foreignTable);
                      listForeignVariables.add(generatorParameters.getOidColumnName());

                    }
                    NodeList typeList=((ElementImpl)columnList.item(j)).getElementsByTagName(
                        "type");
                    if(typeList.getLength()!=0){
                      for(int k=0;k<typeList.getLength();k++){
                        size=((ElementImpl)typeList.item(k)).getAttribute("size");
                        canBeNull=((ElementImpl)typeList.item(k)).getAttribute("canBeNull");
                        dbType=((ElementImpl)typeList.item(k)).getAttribute("dbType");

                        if(canBeNull.equalsIgnoreCase("true")){
                          listAllowNulls.add("");
                        }else{
                          listAllowNulls.add(" NOT NULL");
                        }
                        //if parameter dbType=null, then the primary key is oid column from referenced
                        //table
                        if(dbType.equalsIgnoreCase("none")){
//                          mappingTypeData=new MappingTypeData(oidDbType, generatorParameters);
//                          listColumnType.add(mappingTypeData.getSQLType());
                          listColumnType.add(oidDbType);
                          if(size.equalsIgnoreCase("")||size==null&&

							(generatorParameters.getIsDecimal(mappingTypeData.getSQLType()).equalsIgnoreCase("true"))){
                            listColumnLenght.add(" (19,0) ");
                          }else if (!size.equalsIgnoreCase("")){
                          	 listColumnLenght.add(" ("+size+") ");

                          }else
                            listColumnLenght.add(size);
                        }
                      }
                    }
                  }
                }else {
                  NodeList typeList=((ElementImpl)columnList.item(j)).getElementsByTagName("type");
                  if(typeList.getLength()!=0){
                    for(int k=0;k<typeList.getLength();k++){
                      size=((ElementImpl)typeList.item(k)).getAttribute("size");
                      canBeNull=((ElementImpl)typeList.item(k)).getAttribute("canBeNull");
                      dbType=((ElementImpl)typeList.item(k)).getAttribute("dbType");

                      if(canBeNull.equalsIgnoreCase("true")){
                        listAllowNulls.add("");
                      }else{
                        listAllowNulls.add(" NOT NULL ");
                      }
                      //Mapp the type of the parameter
                      mappingTypeData=new MappingTypeData(dbType, generatorParameters);
                      listColumnType.add(mappingTypeData.getSQLType());
//ZK change this 6.5.2004
//                      if((size.equalsIgnoreCase("")||size==null)&&
//                         (mappingTypeData.getSQLType().equalsIgnoreCase("varchar")||
//                         mappingTypeData.getSQLType().equalsIgnoreCase("nvarchar")||
//                         mappingTypeData.getSQLType().equalsIgnoreCase("longvarchar"))){
//                      listColumnLenght.add("32");
					  if((size.equalsIgnoreCase("")||size==null) &&
					  	(generatorParameters.getHasSize(mappingTypeData.getSQLType()).equalsIgnoreCase("true"))){
						 listColumnLenght.add("32");
					  }else if (!size.equalsIgnoreCase("")){
                         listColumnLenght.add(" ("+size+") ");
                      }else{
                        listColumnLenght.add(size);
                      }
                    }
                  }
                }
              }
            }
            //add column names for columns oid and version
            if(addOid==true&&countOid<tableList.getLength()){
              listColumnNames.add(generatorParameters.getOidColumnName());
              listTargetTableNames.add(tableName);
              listTargetTableID.add("0");
              listAllowNulls.add(" NOT NULL ");
              listColumnLenght.add(" (" + oidDbSize + ") ");
              listColumnType.add(oidDbType);

              listColumnNames.add(generatorParameters.getVersionColumnName());
              listTargetTableNames.add(tableName);
              listTargetTableID.add("0");
              listAllowNulls.add(" NOT NULL ");
              listColumnLenght.add("");
              listColumnType.add(generatorParameters.getVersionColumnType());

              String primaryKeyNameOid=tableName+"_"+generatorParameters.getOidColumnName();
              if( maxConstraintInt>0 && primaryKeyNameOid.length()>maxConstraintInt ){
                String newPrimaryName=primaryKeyNameOid.substring(0,maxConstraintInt-String.valueOf(counterConstraint).length());
                primaryKeyNameOid=newPrimaryName+String.valueOf(counterConstraint);
                counterConstraint++;
              }
              listPrimaryKeys.add(primaryKeyNameOid);
              listPrimaryKeys.add(generatorParameters.getOidColumnName());
            }

            importDefinitionAttributes.setTagSourceColumnName(getColumnNames());
            importDefinitionAttributes.setTagTargetColumnName(getColumnNames());
            importDefinitionAttributes.setTagTargetTableName(getTargetTableNames());
            importDefinitionAttributes.setTagTargetTableID(getTargetTableID());
            importDefinitionAttributes.setTagColumnType(getColumnType());
            importDefinitionAttributes.setTagAllowNulls(getAllowNulls());
            importDefinitionAttributes.setTagColumnLenght(getColumnLenght());

            relationshipsAttributes.setTableName(tableName);
            relationshipsAttributes.setPrimaryKeys(getPrimaryKeys());
            relationshipsAttributes.setIndexVariables(getIndexVariables());
            relationshipsAttributes.setForeignVariables(getForeignVariables());

            WriteSqlFiles writeSql= new WriteSqlFiles(tableName, countOid,
                getImportDefinition(), getRelationshipsAttributes(), generatorParameters);

            WriteImportDefinition writeImportDefinition=new WriteImportDefinition(document,root,
                getImportDefinition(), generatorParameters);
            countOid++;
          }
          //System.out.println("\ndone....");
          //BufferOctopusClass.getInstance().writeToBuffer("\ndone....");
		  		this.logger.write("normal", "\ndone....");
	}
      }else{
        String msg = "Your file don't have database tag!";
        LoaderException le=new LoaderException("Exception:",(Throwable) new Exception(msg));
				this.logger.write("full", "Exception in class DomlDesignReader:"+le.getStackTraceAsString());
        throw le;
      }
    } catch( Exception e ) {
      String msg="Exception in class DomlDesignReader: Error has occurred when trying to set parameters!"+"\n";

      LoaderException le=new LoaderException(msg+e.getMessage(), (Throwable)e);
	  	this.logger.write("full", "Exception in class DomlDesignReader: Error has occurred when trying to set parameters!"+"\n"+ le.getStackTraceAsString());
      throw le;
    }
	this.logger.write("normal", "DomlDesignReader is finished.");
  }

  /**
   * This method read value of importDefinitionAttributes parameters.
   * @return references to ImportDefinitionAttributes objects.
   */
  public ImportDefinitionAttributes getImportDefinition(){
    return importDefinitionAttributes;
  }

  /**
   * This method read value of relationshipsAttributes parameters.
   * @return references to RelationshipsAttributes objects.
   */
  public RelationshipsAttributes getRelationshipsAttributes(){
    return relationshipsAttributes;
  }

  /**
   * This method sets value of listColumnNames parameters.
   * @param list_ColumnNames is value of parameters.
   */
  public static void setColumnNames(String[] list_ColumnNames){
    listColumnNames=Arrays.asList(list_ColumnNames);
  }

  /**
   * This method read value of listColumnNames parameters.
   * @return value of parameters.
   */
  public static String[] getColumnNames(){
    String[] ret=new String[listColumnNames.size()];
    listColumnNames.toArray(ret);
    return ret;
  }

  /**
   * This method sets value of listTargetTableNames parameters.
   * @param list_TargetTableNames is value of parameters.
   */
  public static void setTargetTableNames(String[] list_TargetTableNames){
    listTargetTableNames=Arrays.asList(list_TargetTableNames);
  }

  /**
   * This method read value of listTargetTableNames parameters.
   * @return value of parameters.
   */
  public static String[] getTargetTableNames(){
    String[] ret=new String[listTargetTableNames.size()];
    listTargetTableNames.toArray(ret);
    return ret;
  }

  /**
   * This method sets value of listTargetTableID parameters.
   * @param list_TargetTableID is value of parameter.
   */
  public static void setTargetTableID(String[] list_TargetTableID){
    listTargetTableID=Arrays.asList(list_TargetTableID);
  }

  /**
   * This method read value of listTargetTableID parameters.
   * @return value of parameters.
   */
  public static String[] getTargetTableID(){
    String[] ret=new String[listTargetTableID.size()];
    listTargetTableID.toArray(ret);
    return ret;
  }

  /**
   * This method sets value of listColumnType parameters.
   * @param list_ColumnType is value of parameters.
   */
  public static void setColumnType(String[] list_ColumnType){
    listColumnType=Arrays.asList(list_ColumnType);
  }

  /**
   * This method read value of listColumnType parameters.
   * @return value of parameters.
   */
  public static String[] getColumnType(){
    String[] ret=new String[listColumnType.size()];
    listColumnType.toArray(ret);
    return ret;
  }

  /**
   * This method sets value of listColumnLenght parameters.
   * @param list_ColumnLenght is value of parameter.
   */
  public static void setColumnLenght(String[] list_ColumnLenght){
    listColumnLenght=Arrays.asList(list_ColumnLenght);
  }

  /**
   * This method read value of listColumnLenght parameters.
   * @return value of parameters.
   */
  public static String[] getColumnLenght(){
    String[] ret=new String[listColumnLenght.size()];
    listColumnLenght.toArray(ret);
    return ret;
  }

  /**
   * This method sets value of listAllowNulls parameters.
   * @param list_AllowNulls is value of parameters.
   */
  public static void setAllowNulls(String[] list_AllowNulls){
    listAllowNulls=Arrays.asList(list_AllowNulls);
  }

  /**
   * This method read value of listAllowNulls parameters.
   * @return value of parameters.
   */
  public static String[] getAllowNulls(){
    String[] ret=new String[listAllowNulls.size()];
    listAllowNulls.toArray(ret);
    return ret;
  }


  /**
   * This method sets the value of listPrimaryKeys parameters.
   * @param primary_Keys is value of parameter.
   */
  public static void setPrimaryKeys(String[] primary_Keys){
    listPrimaryKeys=Arrays.asList(primary_Keys);
  }

  /**
   * This method read the value of listPrimaryKeys parameters.
   * @return value of parameter.
   */
  public static String[] getPrimaryKeys(){
    String[] ret=new String[listPrimaryKeys.size()];
    listPrimaryKeys.toArray(ret);
    return ret;
  }

  /**
   * This method sets the value of listIndexVariables parameters.
   * @param index_Variables is value of parameter.
   */
  public static void setIndexVariables(String[] index_Variables){
    listIndexVariables=Arrays.asList(index_Variables);
  }

  /**
   * This method read the value of listIndexVariables parameters.
   * @return value of parameter.
   */
  public static String[] getIndexVariables(){
    String[] ret=new String[listIndexVariables.size()];
    listIndexVariables.toArray(ret);
    return ret;
  }

  /**
   * This method sets the value of listForeignVariables parameters.
   * @param foreign_Variables is the value of parameter.
   */
  public static void setForeignVariables(String[] foreign_Variables){
    listForeignVariables=Arrays.asList(foreign_Variables);
  }

  /**
   * This method read the value of listForeignVariables parameters.
   * @return value of parameter.
   */
  public static String[] getForeignVariables(){
    String[] ret=new String[listForeignVariables.size()];
    listForeignVariables.toArray(ret);
    return ret;
  }
  /**
	   * This method will set logger object
	   * @param logger
	   */
	  private void setLogger() {
		  this.logger = StandardLogger.getCentralLogger();
	  }

}