/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.generator;

import java.util.Vector;

import org.webdocwf.util.loader.LoaderException;
import org.webdocwf.util.loader.logging.Logger;
import org.webdocwf.util.loader.logging.StandardLogger;

/**
 * GenerateSqlForAllVendors class is used for generating all sql statements for
 * all named database Vendors
 * @author Radoslav Dutina
 * @version 1.0
 */
public class GenerateSqlForAllVendors {

    private Logger logger;

    /**
     * Public constructor of GenerateSqlForAllVendors class. Construct object with an
     * associated parameters
     * @param generatorParameters defines object of InputParameters class
     * @throws LoaderException
     */

    public GenerateSqlForAllVendors(InputParameters generatorParameters) throws LoaderException {

        setLogger();
        this.logger.write("normal", "GenerateSqlForAllVendors is started.");
        generateParameters(generatorParameters);
        this.logger.write("normal", "GenerateSqlForAllVendors is finished.");
    }
    /**
     * this method will genarate files for all vendors
     * @param generatorParameters
     * @throws LoaderException
     */
    private void generateParameters(InputParameters generatorParameters) throws LoaderException {
        Vector allVendors = generatorParameters.getAllVendors();

        try {
            for (int i = 0; i < allVendors.size(); i++) {
                String vendorName = allVendors.get(i).toString();

                //System.out.println("Working.... SQL Statements for " + vendorName + " Vendor");
                //if (this.logger != null) {
                    this.logger.write("normal", "Working.... SQL Statements for " + vendorName + " Vendor");
                //}
                //BufferOctopusClass.getInstance().writeToBuffer("Working.... SQL Statements for " + vendorName + " Vendor");

                generatorParameters.setTargetDriverName("");
                generatorParameters.setTargetType(vendorName);
                generatorParameters.setSqlToGenerate();
                String generatorOutputOld = generatorParameters.getGeneratorOutput();
                if (generatorOutputOld.equalsIgnoreCase("") || generatorOutputOld == null)
                    generatorParameters.setGeneratorOutput("SQLForAllVendors/" + vendorName);
                else
                    generatorParameters.setGeneratorOutput(generatorOutputOld + "/SQLForAllVendors/" + vendorName);

                if (generatorParameters.getDomlPath() == null || generatorParameters.getDomlPath().equalsIgnoreCase("")) {
                    CreateIncludeFiles createIncludeFiles = new CreateIncludeFiles(generatorParameters);
                } else {
                    CreateIncludeDomlFiles createIncludeDomlFiles = new CreateIncludeDomlFiles(generatorParameters);
                }
                if (generatorParameters.getGenerateXml().equalsIgnoreCase("true")) {
                    LoadJobWriter loadJobWriter = new LoadJobWriter(generatorParameters);
                }
                generatorParameters.setGeneratorOutput(generatorOutputOld);
            }
        } catch (LoaderException e) {

            LoaderException le = new LoaderException("Error while generate sql statements for all vendors.", e);
            //if (this.logger != null) {
                this.logger.write("full", "Error while generate sql statements for all vendors." + le.getStackTraceAsString());
            //}
            throw le;
        }
    }
    /**
       * This method will set logger object
       * @param logger
       */
    private void setLogger() {
        this.logger = StandardLogger.getCentralLogger();
    }
}