/*
     LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.
    Copyright (C) 2003  Together
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.webdocwf.util.loader.generator;

import java.io.File;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import org.webdocwf.util.loader.LoaderException;
import org.webdocwf.util.loader.logging.Logger;
import org.webdocwf.util.loader.logging.StandardLogger;
/**
 * InputParameters class sets the input parameters.
 * @author Radoslav Dutina
 * @version 1.0
 */
public class InputParameters {

    private String sourceType = "msql";
    private String targetType = "msql";
    private String sourceDataBase = null;
    private String targetDataBase = null;

    private String valueMode = "Overwrite";
    private String generatorOutput = "";
    private String sourceUser = "";
    private String sourcePassword = "";
    private String targetUser = "";
    private String targetPassword = "";
    private String sourceDriverName = "";
    private String targetDriverName = "";
    private String domlPath = "";

    private String packageName = null;

    private String alterTablePrimaryKey = null;
    private String pathToSourceConf = null;
    private String pathToTargetConf = null;

    //  private String path="conf/OctopusDBVendors.xml";
    private String path = "OctopusDBVendors.xml";
    private SearchXmlFile searchXmlFile = null;

    //sql statements only
    private Vector sqlToGenerate = new Vector();
    private boolean generateSomeSql = false;
    private boolean generateSqlForAllVendors = false;

    private Vector allVendors = new Vector();
    private String generateXml = "false";
    private String generateDoml = "false";
    private String restoreMode = "false";

    //optimized modes
    private boolean fullMode = true;
    private String[] excludedTables = null;

    private Vector includeTableList = new Vector();

    private String confJarStructure = "";

    //oid and version columns
    private String oidColumnName = "oid";
    private String versionColumnName = "version";
    private String oidColumnType = "decimal";
    private String versionColumnType = "bigint";

    private String maxConstraintLength = "";

    //ZK added for new attribute hasSize
    private Hashtable htHasSize = new Hashtable();
    private Hashtable htIsDecimal = new Hashtable();
    private Logger logger;
    //ZK end 

    /**
     * Set hasSize values for db vendor 
     */

    public void setHasSize() throws LoaderException {
        try {
            setLogger();
//            if (this.logger != null) {
//                this.logger.write("full", "\tsetHasSize is started.");
//            }
            SearchXmlFile searchXmlFile = new SearchXmlFile("absolute", getPathToSourceConf(), getConfJarStructure());
            this.htHasSize = searchXmlFile.getHasSize();
//            if (this.logger != null) {
//                this.logger.write("full", "\tsetHasSize is finished.");
//            }

        } catch (Exception ex) {
            
            LoaderException le = new LoaderException("Exception:" + ex.getMessage(), ex);
						if (this.logger != null) {
							this.logger.write("full", "Exception:" + le.getStackTraceAsString());
						}
						throw le;
        }
    }
    /**
     * Return boolean if size for this type must be defined by user or not
     * @param sqlType
     * @return String which represents is this type has size or not
     * @throws LoaderException
     */
    public String getHasSize(String sqlType) throws LoaderException {
        try {
            setLogger();
           // this.logger.write("full", "\tgetHasSize is started.");
            if (htHasSize.containsKey((sqlType).toUpperCase())) {
//                if (this.logger != null) {
//                    this.logger.write("full", "\tgetHasSize is finished.");
//                }
                return this.htHasSize.get((sqlType).toUpperCase()).toString();
            } else {
//                if (this.logger != null) {
//                    this.logger.write("full", "\tgetHasSize is finished.");
//                }
                return "false";
            }

        } catch (Exception e) {
            
            LoaderException le = new LoaderException("Exception:", (Throwable) e);
						if (this.logger != null) {
							this.logger.write("full", "Exception:" + le.getStackTraceAsString());
						}
						throw le;
        }

    }
    /**
     * Set decimal types for this DB vendor
     * @throws LoaderException
     */
    public void setIsDecimal() throws LoaderException {
        try {
            setLogger();
//            if (this.logger != null) {
//                this.logger.write("full", "\tsetIsDecimal is started.");
//            }
            SearchXmlFile searchXmlFile = new SearchXmlFile("absolute", getPathToSourceConf(), getConfJarStructure());
            this.htIsDecimal = searchXmlFile.getIsDecimal();

        } catch (Exception ex) {
            
            LoaderException le = new LoaderException("Exception" + ex.getMessage(), ex);
						if (this.logger != null) {
								this.logger.write("full", "Exception:" + le.getStackTraceAsString());
						}
						throw le;
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetIsDecimal is finished.");
//        }
    }
    /**
     * Return boolean if this string represents decimal type or not
     * @param sqlType
     * @return String which represents is this number decimal or not
     * @throws LoaderException
     */
    public String getIsDecimal(String sqlType) throws LoaderException {
        try {
            setLogger();
//            if (this.logger != null) {
//                this.logger.write("full", "\tgetIsDecimal is started.");
//            }
            if (htIsDecimal.containsKey((sqlType).toUpperCase())) {
                //this.logger.write("full", "\tgetIsDecimal is finished.");
                return this.htIsDecimal.get((sqlType).toUpperCase()).toString();
            } else {
//                if (this.logger != null) {
//                    this.logger.write("full", "\tgetIsDecimal is finished.");
//                }
                return "false";
            }
        } catch (Exception e) {
						LoaderException le = new LoaderException("Exception:" + e.getMessage(), e);
            if (this.logger != null) {
                this.logger.write("full", "Exception:" + le.getStackTraceAsString());
            }
            throw le;
        }

    }

    public void setOidAndVersionColumnName() throws LoaderException {

        try {
            setLogger();
//            if (this.logger != null) {
//                this.logger.write("full", "\tsetOidAndVersionColumnName is started.");
//            }
            SearchXmlFile searchXmlFile = new SearchXmlFile("absolute", getPathToTargetConf(), getConfJarStructure());

            this.oidColumnName = searchXmlFile.getOidDbColumn();
            this.versionColumnName = searchXmlFile.getVersionDbColumn();
            this.oidColumnType = searchXmlFile.getOidDB();
            this.versionColumnType = searchXmlFile.getVersionDb();

        } catch (Exception ex) {
						LoaderException le = new LoaderException("Exception:" + ex.getMessage(), ex);
            if (this.logger != null) {
                this.logger.write("full", "Exception:" + le.getStackTraceAsString());
            }
            throw le;
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetOidAndVersionColumnName is finished.");
//        }
    }

    public String getOidColumnName() {
        return this.oidColumnName;
    }

    public String getVersionColumnName() {
        return this.versionColumnName;
    }

    public String getOidColumnType() {
        return this.oidColumnType;
    }

    public String getVersionColumnType() {
        return this.versionColumnType;
    }

    /**
     * This method set value of includeTableList parameter
     * @param include_table_list is value of parameter
     * @throws LoaderException
     */
    public void setIncludeTableList(String include_table_list) throws LoaderException {
        setLogger();
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetIncludeTableList is started.");
//        }
        if (include_table_list.indexOf(",") != -1) {
            String msg = "\tDelimiter for tables in 'Include Table List' parameter is ';' and not ','";
						LoaderException le = new LoaderException("Exception:", new Exception(msg + "\n"));
            if (this.logger != null) {
                this.logger.write("full", "Exception:" + "\tDelimiter for tables in 'Include Table List' parameter is ';' and not ','"+le.getStackTraceAsString());
            }
            throw le;
        }

        StringTokenizer st = new StringTokenizer(include_table_list, ";");
        while (st.hasMoreTokens()) {
            String tableName = st.nextElement().toString();
            if (tableName.length() > 0)
                this.includeTableList.add(tableName);
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetIncludeTableList is finished.");
//        }
    }

    /**
     * This method read value from includeTableList parameter
     * @return value of parameter
     */
    public Vector getIncludeTableList() {
        return this.includeTableList;
    }

    /**
     * This method set value of restroreMode parameter
     * @param restoreMode is value of parameter
     */
    public void setRestoreMode(String restoreMode) {
        this.restoreMode = restoreMode;
    }

    /**
     * This method read value from restoreMode parameter
     * @return value of parameter
     */
    public String getRestoreMode() {
        return this.restoreMode;
    }

    /**
     * This method set value of fullMode parameter
     * @param full_mode is value of parameter
     */
    public void setFullMode(String full_mode) {
        if (full_mode.equalsIgnoreCase("true"))
            fullMode = true;
        else
            fullMode = false;
    }

    /**
     * This method read value of fullMode parameter
     * @return value of parameter
     */
    public boolean getFullMode() {
        return fullMode;
    }

    /**
     * This method set value of generateXml parameter
     * @param generate_Xml is value of parameter
     */
    public void setGenerateXml(String generate_Xml) {
        this.generateXml = generate_Xml;
    }

    /**
     * This method read value of generateXml parameter
     * @return value of parameter
     */
    public String getGenerateXml() {
        return this.generateXml;
    }

    /**
     * This method read value of  generateDoml parameter
     * @return value of parameter
     */
    public String getGenerateDoml() {
        return this.generateDoml;
    }

    /**
     * This method set value of generateDoml parameter
     * @param generate_Doml is value of parameter
     */
    public void setGenerateDoml(String generate_Doml) {
        this.generateDoml = generate_Doml;
    }

    /**
     * This method read value from generateSqlForAllVendors parameter
     * @return value of parameter
     */
    public boolean getSqlForAllVendors() {
        return generateSqlForAllVendors;
    }

    /**
     * This method set the value of allVendors parameter
     * @param generate is value of parameter
     */
    public void setAllVendors(String generate) {
        if (generate.equalsIgnoreCase("true")) {
            generateSqlForAllVendors = true;
            allVendors = searchXmlFile.getAllVendors();
        }
    }

    /**
     * This method read value from allVendors parameter
     * @return value of parameter
     */
    public Vector getAllVendors() {
        return allVendors;
    }

    /**
     * This method read value form generateSqlOnly parameter
     * @return value of parameter
     */
    public boolean isGenerateSql() {
        return generateSomeSql;
    }

    private void setDefaultSqlStmt() {
        sqlToGenerate.add(0, null);
        sqlToGenerate.add(1, null);
        sqlToGenerate.add(2, null);
        sqlToGenerate.add(3, null);
        sqlToGenerate.add(4, null);
        sqlToGenerate.add(5, null);
    }

    /**
     * This method sets the value of sqlToGenerate parameter.
     * @param generate_DropTableStmt is value of parameter
     */
    public void setSqlStmtDropTable(String generate_DropTableStmt) {

        if (generate_DropTableStmt.equalsIgnoreCase("true")) {
            generateSomeSql = true;
            sqlToGenerate.setElementAt("DropTables", 1);
        }
    }

    /**
     * This method sets the value of sqlToGenerate parameter.
     * @param generate_DropIntegrityStmt is value of parameter
     */
    public void setSqlStmtDropIntegrity(String generate_DropIntegrityStmt) {
        if (generate_DropIntegrityStmt.equalsIgnoreCase("true")) {
            generateSomeSql = true;
            sqlToGenerate.setElementAt("DropIntegrity", 0);
        }
    }

    /**
     * This method sets the value of sqlToGenerate parameter.
     * @param generate_CreateTableStmt is value of parameter
     */
    public void setSqlStmtCreateTable(String generate_CreateTableStmt) {
        if (generate_CreateTableStmt.equalsIgnoreCase("true")) {
            generateSomeSql = true;
            sqlToGenerate.setElementAt("CreateTables", 2);
        }
    }

    /**
     * This method sets the value of sqlToGenerate parameter.
     * @param generate_CreatePKStmt is value of parameter
     */
    public void setSqlStmtCreatePK(String generate_CreatePKStmt) {
        if (generate_CreatePKStmt.equalsIgnoreCase("true")) {
            if (!this.alterTablePrimaryKey.equalsIgnoreCase("false")) {
                generateSomeSql = true;
                sqlToGenerate.setElementAt("CreatePrimary", 4);
            }
        }
    }

    /**
     * This method sets the value of sqlToGenerate parameter.
     * @param generate_CreateFKStmt is value of parameter
     */
    public void setSqlStmtCreateFK(String generate_CreateFKStmt) {
        if (generate_CreateFKStmt.equalsIgnoreCase("true")) {
            generateSomeSql = true;
            sqlToGenerate.setElementAt("CreateForeigin", 5);
        }
    }

    /**
     * This method sets the value of sqlToGenerate parameter.
     * @param generate_CreateIndexStmt is value of parameter
     */
    public void setSqlStmtCreateIndex(String generate_CreateIndexStmt) {
        if (generate_CreateIndexStmt.equalsIgnoreCase("true")) {
            generateSomeSql = true;
            sqlToGenerate.setElementAt("CreateIndexes", 3);
        }
    }

    /**
     * This method reads value of sqlToGenerate parameter.
     * @return value of parameter.
     */
    public Vector getSqlToGenerate() {
        return sqlToGenerate;
    }

    /**
     * This method sets value of sqlToGenerate parameter.
     */
    public void setSqlToGenerate() {

        if (this.alterTablePrimaryKey.equalsIgnoreCase("false")) {
            sqlToGenerate.setElementAt(null, 4);
        }
    }

    /**
     * This method reads value of path parameter.
     * @return value of parameter.
     */
    public String getPath() {
        return path;
    }

    /**
     * Construct object InputParameters, and initialise the default value of input parameters.
     * @throws LoaderException
     */
    public InputParameters(String confJarStructure) throws LoaderException {
        //initialization InputParameters
        setDefaultSqlStmt();
        searchXmlFile = new SearchXmlFile("relative", path, confJarStructure);
    }

    /**
     * This method sets the value of pathToSourceConf parameter.
     * @param path_ToSourceConf is the value of parameter.
     */
    public void setPathToSourceConf(String path_ToSourceConf) {
        pathToSourceConf = path_ToSourceConf;
    }

    /**
     * This method read the value of pathToSourceConf parameter.
     * @return the value of parameter.
     */
    public String getPathToSourceConf() {
        return pathToSourceConf;
    }

    /**
     * This method sets the value of sourceType parameter.
     * @param source_Type is the value of parameter.
     * @throws LoaderException
     */
    public void setSourceType(String source_Type) throws LoaderException {
        setLogger();
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetSourceType is started.");
//        }
        if (source_Type != null) {
            if (searchXmlFile.getPathToConf(source_Type) != null) {
                setPathToSourceConf(searchXmlFile.getPathToConf(source_Type));
                sourceType = source_Type;
            } else {
                String msg = "This type of source database (" + source_Type + ") is not supported!" + " See documentation!";
								LoaderException le = new LoaderException("Exception:", new Exception(msg + "\n"));
                if (this.logger != null) {
                    this.logger.write("full", "Exception:" + "This type of source database (" + source_Type + ") is not supported!" + " See documentation!"+le.getStackTraceAsString());
                }
                
                throw le;
            }
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetSourceType is finished.");
//        }
    }

    /**
     * This method read the value of sourceType parameter.
     * @return value of parameter.
     */
    public String getSourceType() {
        return sourceType;
    }

    /**
     * This method sets the value of pathToTargetConf parameter.
     * @param path_ToTargetConf is the value of parameter.
     */
    public void setPathToTargetConf(String path_ToTargetConf) {
        pathToTargetConf = path_ToTargetConf;
    }

    /**
     * This method read the value of pathToTargetConf parameter.
     * @return value of parameter.
     */
    public String getPathToTargetConf() {
        return pathToTargetConf;
    }

    /**
     * This method sets the value of targetType parameter.
     * @param target_Type is the value of parameter.
     * @throws LoaderException
     */
    public void setTargetType(String target_Type) throws LoaderException {
        setLogger();
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetTargetType is started.");
//        }
        if (target_Type != null) {
            //search the OctopusDBVendors.xml file for defined type of database.
            if (searchXmlFile.getPathToConf(target_Type) != null) {
                setPathToTargetConf(searchXmlFile.getPathToConf(target_Type));
                targetType = target_Type;
                setAlterTablePrimaryKey2(pathToTargetConf);
            } else {
                String msg = "This type of target database (" + target_Type + ") is not supported!" + " See documentation!";
								LoaderException le = new LoaderException("Exception:", new Exception(msg));
                if (this.logger != null) {
                    this.logger.write("full", "Exception:" + "This type of target database (" + target_Type + ") is not supported!" + " See documentation!"+le.getStackTraceAsString());
                } 
                throw le;
            }
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetTargetType is finished.");
//        }
    }

    private void setAlterTablePrimaryKey2(String pathToTargetConf) throws LoaderException {
        SearchXmlFile searchXmlFile = new SearchXmlFile("absolute", getPathToTargetConf(), getConfJarStructure());
        if ((searchXmlFile.getAlterTablePrimaryKey(getTargetDriverName()) != null))
            setAlterTablePrimaryKey(searchXmlFile.getAlterTablePrimaryKey(getTargetDriverName()));
    }

    public void setSourceDriverProperties() throws LoaderException {
        setLogger();
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetSourceDriverProperties is started.");
//        }
        SearchXmlFile searchXmlFile = new SearchXmlFile("absolute", getPathToSourceConf(), getConfJarStructure());
        Vector tmpVector = searchXmlFile.getFileSystemDatabase(getSourceDriverName());
        String fileSystemDatabse = tmpVector.get(0).toString();
        String connectionPrefix = tmpVector.get(1).toString();

        if (fileSystemDatabse.equalsIgnoreCase("true")) {
            File file = new File(getSourceDataBase());
            File fileOut = new File(getGeneratorOutput());
            if (!file.isAbsolute()) {
                try {
                    String main = "";
                    if (!getGeneratorOutput().equalsIgnoreCase(""))
                        main = fileOut.getCanonicalPath() + System.getProperty("file.separator") + getSourceDataBase();
                    else
                        main = getSourceDataBase();

                    File mainFile = new File(main);
                    setSourceDataBase(mainFile.getCanonicalPath());
                } catch (Exception ex) {
										LoaderException le = new LoaderException("Exception:" + ex.getMessage(), ex);
                    if (this.logger != null) {
                        this.logger.write("full", "Exception:" + le.getStackTraceAsString());
                    }
                   
                    throw le;
                    //ex.printStackTrace();
                }
            }
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetSourceDriverProperties is finished.");
//        }
    }

    public void setTargetDriverProperties() throws LoaderException {
        setLogger();
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetTargetDriverProperties is started.");
//        }
        SearchXmlFile searchXmlFile = new SearchXmlFile("absolute", getPathToTargetConf(), getConfJarStructure());
        try {
            this.maxConstraintLength = searchXmlFile.getMaxConstraintLength(getTargetDriverName());
        } catch (Exception ex) {
						LoaderException le = new LoaderException("Exception:" + ex.getMessage(), ex);
            this.logger.write("full", "Exception:" + le.getStackTraceAsString());
            throw le;
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetTargetDriverProperties is finished.");
//        }

    }

    public String getMaxConstraintLength() {
        return this.maxConstraintLength;
    }

    /**
     * This method read the value of targetType parameter.
     * @return value of parameter.
     */
    public String getTargetType() {
        return targetType;
    }

    /**
     * This method sets the value of sourceDataBase parameter.
     * @param source_DataBase is the value of parameter.
     * @throws LoaderException
     */
    public void setSourceDataBase(String source_DataBase) throws LoaderException {
        setLogger();
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetSourceDataBase is started.");
//        }
        if (source_DataBase != null) {
            if (this.sourceDriverName.equalsIgnoreCase("microsoft")) {
                int num1 = source_DataBase.indexOf("SelectMethod=");
                int num2 = source_DataBase.indexOf("selectMethod=");
                if (num2 != -1) {
                    //           source_DataBase=source_DataBase.replaceFirst("selectMethod=","SelectMethod=");
                    if (source_DataBase.indexOf("selectMethod=") != -1)
                        source_DataBase = Utils.replaceAll(source_DataBase, "selectMethod=", "SelectMethod=");

                    num1 = 0;
                }
                if (num1 != -1)
                    sourceDataBase = source_DataBase;
                else
                    sourceDataBase = source_DataBase + ";SelectMethod=cursor";
            } else
                sourceDataBase = source_DataBase;
        } else {
            String msg = "You must enter the place where is source database is placed (param -sdb)!";
						LoaderException le = new LoaderException("Exception:", new Exception(msg));
            this.logger.write("full", "Exception:" + "You must enter the place where is source database is placed (param -sdb)!"+le.getStackTraceAsString());
            throw le;
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetSourceDataBase is finished.");
//        }
    }

    /**
     * This method read the value of sourceDataBase parameter.
     * @return value of parameter.
     */
    public String getSourceDataBase() {
        return sourceDataBase;
    }

    /**
     * This method sets the value of targetDataBase parameter.
     * @param target_DataBase is the value of the parameter.
     * @throws LoaderException
     */
    public void setTargetDataBase(String target_DataBase) throws LoaderException {
        setLogger();
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetTargetDataBase is started.");
//        }
        if (target_DataBase != null) {
            if (this.targetDriverName.equalsIgnoreCase("microsoft")) {
                int num1 = target_DataBase.indexOf("SelectMethod=");
                int num2 = target_DataBase.indexOf("selectMethod=");
                if (num2 != -1) {
                    //           target_DataBase=target_DataBase.replaceFirst("selectMethod=","SelectMethod=");
                    if (target_DataBase.indexOf("selectMethod=") != -1)
                        target_DataBase = Utils.replaceAll(target_DataBase, "selectMethod=", "SelectMethod=");
                    num1 = 0;
                }
                if (num1 != -1)
                    targetDataBase = target_DataBase;
                else
                    targetDataBase = target_DataBase + ";SelectMethod=cursor";
            } else
                targetDataBase = target_DataBase;
        } else {
            String msg = "You must enter the place where is target database is placed (param -tdb)!";
						LoaderException le = new LoaderException("Exception:", new Exception(msg));
            if (this.logger != null) {
                this.logger.write("full", "Exception:" + "You must enter the place where is target database is placed (param -tdb)!"+le.getStackTraceAsString());
            }
            throw le;
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetTargetDataBase is finished.");
//        }
    }

    /**
     * This method read the value of targetDataBase parameter.
     * @return value of parameter.
     */
    public String getTargetDataBase() {
        return targetDataBase;
    }

    /**
     * This method sets the value of valueMode parameter.
     * @param value_Mode is the value of the parameter.
     * @throws LoaderException
     */
    public void setValueMode(String value_Mode) throws LoaderException {
        setLogger();
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetValueMode is started.");
//        }
        if (value_Mode != null) {
            if (value_Mode.equalsIgnoreCase("copy")) {
                valueMode = "Overwrite";
            } else if (value_Mode.equalsIgnoreCase("sync")) {
                valueMode = "Update";
            } else {
                String msg = "The possible value of value modes (param -m) are 'copy' or 'sync'.!";
								LoaderException le = new LoaderException("Exception:", new Exception(msg));
                if (this.logger != null) {
                    this.logger.write("full", "Exception:" + "The possible value of value modes (param -m) are 'copy' or 'sync'.!"+le.getStackTraceAsString());
                }
                throw le;
            }
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetValueMode is finished.");
//        }
    }

    /**
     * This method read the value of valueMode parameter.
     * @return value of parameter.
     */
    public String getValueMode() {
        return valueMode;
    }

    /**
     * This method sets the value of generatorOutput parameter.
     * @param generator_Output is the value of the parameter.
     */
    public void setGeneratorOutput(String generator_Output) {
        if (generator_Output != null)
            generatorOutput = generator_Output;
    }

    /**
     * This method read the value of generatorOutput parameter.
     * @return value of parameter.
     */
    public String getGeneratorOutput() {
        return generatorOutput;
    }

    /**
     * This method sets the value of sourceUser parameter.
     * @param source_User is the value of the parameter.
     */
    public void setSourceUser(String source_User) {
        if (source_User != null)
            sourceUser = source_User;
    }

    /**
     * This method read the value of sourceUser parameter.
     * @return value of parameter.
     */
    public String getSourceUser() {
        return sourceUser;
    }

    /**
     * This method sets the value of sourcePassword parameter.
     * @param source_Password is the value of the parameter.
     */
    public void setSourcePassword(String source_Password) {
        if (source_Password != null)
            sourcePassword = source_Password;

    }

    /**
     * This method read the value of sourcePassword parameter.
     * @return value of parameter.
     */
    public String getSourcePassword() {
        return sourcePassword;
    }

    /**
     * This method sets the value of targetUser parameter.
     * @param target_User is the value of the parameter.
     */
    public void setTargetUser(String target_User) {
        if (target_User != null)
            targetUser = target_User;
    }

    /**
     * This method read the value of targetUser parameter.
     * @return value of parameter.
     */
    public String getTargetUser() {
        return targetUser;
    }

    /**
     * This method sets the value of targetPassword parameter.
     * @param target_Password is the value of the parameter.
     */
    public void setTargetPassword(String target_Password) {
        if (target_Password != null)
            targetPassword = target_Password;
    }

    /**
     * This method read the value of targetPassword parameter.
     * @return value of parameter.
     */
    public String getTargetPassword() {
        return targetPassword;
    }

    /**
     * This method sets the value of sourceDriverName parameter.
     * @param source_DriverName is the value of parameter.
     */
    public void setSourceDriverName(String source_DriverName) {
        if (source_DriverName != null)
            sourceDriverName = source_DriverName;
    }

    /**
     * This method read the value of sourceDriverName parameter.
     * @return value of parameter.
     */
    public String getSourceDriverName() {
        return sourceDriverName;
    }

    /**
     * This method sets the value of targetDriverName parameter.
     * @param target_DriverName is the value of the parameter.
     */
    public void setTargetDriverName(String target_DriverName) {
        if (target_DriverName != null)
            targetDriverName = target_DriverName;
    }

    /**
     * This method read the value of targetDriverName parameter.
     * @return value of parameter.
     */
    public String getTargetDriverName() {
        return targetDriverName;
    }

    /**
     * This method sets the value of domlPath parameter.
     * @param doml_Path is the value of the parameter.
     */
    public void setDomlPath(String doml_Path) {
        if (doml_Path != null)
            domlPath = doml_Path;
    }

    /**
     * This method read the value of domlPath parameter.
     * @return value of parameter.
     */
    public String getDomlPath() {
        return domlPath;
    }

    /**
     * This method sets the value of packageName parameter.
     * @param package_Name is the value of parameter.
     *  @throws LoaderException
     */
    public void setPackageName(String package_Name) throws LoaderException {
        setLogger();
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetPackageName is started.");
//        }
        if (generateDoml.equalsIgnoreCase("true")) {
            if (package_Name != null) {
                packageName = package_Name;
            } else {
                String msg = "Sorry, but you must enter the package name (param -pack) for the doml file!";
								LoaderException le = new LoaderException("Exception", new Exception(msg));
                if (this.logger != null) {
                    this.logger.write("full", "Exception:" + "Sorry, but you must enter the package name (param -pack) for the doml file!"+le.getStackTraceAsString());
                }
                throw le;
            }
        }
//        if (this.logger != null) {
//            this.logger.write("full", "\tsetPackageName is finished.");
//        }
    }

    /**
     * This method read the value of packageName parameter.
     * @return value of parameter.
     */
    public String getPackageName() {
        return packageName;
    }

    /**
     * This method sets the value of alterTablePrimaryKey parameter.
     * @param alter_TablePrimaryKey is the value of parameter.
     */
    public void setAlterTablePrimaryKey(String alter_TablePrimaryKey) {
        alterTablePrimaryKey = alter_TablePrimaryKey;
    }

    /**
     * This method read the value of alterTablePrimaryKey parameter.
     * @return value of parameter.ss
     */
    public String getAlterTablePrimaryKey() {
        return alterTablePrimaryKey;
    }

    /**
     * This method set excluded tables
     * @param value
     */
    public void setExcludedTables(String value) {
        StringTokenizer st = new StringTokenizer(value, ",");
        this.excludedTables = new String[st.countTokens()];
        int i = 0;
        while (st.hasMoreTokens()) {
            this.excludedTables[i] = st.nextToken();
            i++;
        }
    }

    /**
     *This method returns excluded tables
     * @return String[] with excluded tables
     */
    public String[] getExcludedTables() {
        return this.excludedTables;
    }

    /**
     *This method set parameter confJar
     */
    public void setConfJarStructure(String confJarStructure) {
        if (confJarStructure != null)
            this.confJarStructure = confJarStructure;
    }

    /**
     *This method returns string with path to conf in jar
     * @return String
     */
    public String getConfJarStructure() {
        return this.confJarStructure;
    }
    /**
    	 * This method will set logger object
    	 * @param logger
    	 */
    private void setLogger() {
        this.logger = StandardLogger.getCentralLogger();
    }
}
