
/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.generator;

import java.util.Vector;

import org.webdocwf.util.loader.BufferOctopusClass;
import org.webdocwf.util.loader.LoaderException;
import org.webdocwf.util.loader.TimeWatch;
import org.webdocwf.util.loader.logging.StandardLogger;

/**
 * LoaderGenerator class dinamicly generates the xml files (LoaderJob.xml and ImportDefinition.xml)
 * from the input data and source database.
 * Also LoaderGenerator creates SQL statements for building and loading an SQL target database.
 * @author Radoslav Dutina
 * @version 1.0
 */
public class LoaderGenerator {

    private InputParameters generatorParameters;
    private StandardLogger logger;
    private String strLogDirName = "";
    private String strLogFileName = "defaultGenerator";
    private String logMode = "normal";
    private TimeWatch timeCounter;
    private String strLoggerParam = "";
    private boolean standardLogger;

    /**
     * This method set sourceDataBase
     * @param sourceDataBase
     * @throws LoaderException
     */
    public void setSourceDataBase(String sourceDataBase) throws LoaderException {
        this.generatorParameters.setSourceDataBase(sourceDataBase);
    }

    /**
     * This method return sourceDataBase
     * @return sourceDataBase
     * @throws LoaderException
     */

    public String getSourceDataBase() throws LoaderException {
        return this.generatorParameters.getSourceDataBase();
    }
    /**
     * This method set targetDataBase
     * @param targetDataBase
     * @throws LoaderException
     */

    public void setTargetDataBase(String targetDataBase) throws LoaderException {
        this.generatorParameters.setTargetDataBase(targetDataBase);
    }

    /**
     * This method return targetDataBase
     * @return targetDataBase
     * @throws LoaderException
     */
    public String getTargetDataBase() throws LoaderException {
        return this.generatorParameters.getTargetDataBase();
    }

    /**
     * This method set sourceType
     * @param sourceType
     * @throws LoaderException
     */
    public void setSourceType(String sourceType) throws LoaderException {
        this.generatorParameters.setSourceType(sourceType);
    }

    /**
     * This method return sourceType
     * @return sourceType
     * @throws LoaderException
     */
    public String getSourceType() throws LoaderException {
        return this.generatorParameters.getSourceType();
    }

    /**
     * This method set targetType
     * @param targetType
     * @throws LoaderException
     */
    public void setTargetType(String targetType) throws LoaderException {
        this.generatorParameters.setTargetType(targetType);
    }

    /**
     * This method return targetType
     * @return String which is target type
     * @throws LoaderException
     */
    public String getTargetType() throws LoaderException {
        return this.generatorParameters.getTargetType();
    }

    /**
     * This method set sourceDriverName
     * @param sourceDriverName
     * @throws LoaderException
     */
    public void setSourceDriverName(String sourceDriverName) throws LoaderException {
        this.generatorParameters.setSourceDriverName(sourceDriverName);
    }

    /**
     * This method return sourceDriverName
     * @return sourceDriverName
     * @throws LoaderException
     */
    public String getSourceDriverName() throws LoaderException {
        return this.generatorParameters.getSourceDriverName();
    }

    /**
     * This method set targetDriverName
     * @param targetDriverName
     * @throws LoaderException
     */
    public void setTargetDriverName(String targetDriverName) throws LoaderException {
        this.generatorParameters.setTargetDriverName(targetDriverName);
    }

    /**
     * This method return targetDriverName
     * @return targetDriverName
     * @throws LoaderException
     */
    public String getTargetDriverName() throws LoaderException {
        return this.generatorParameters.getTargetDriverName();
    }

    /**
     * This method set valueMode
     * @param valueMode
     * @throws LoaderException
     */
    public void setValueMode(String valueMode) throws LoaderException {
        this.generatorParameters.setValueMode(valueMode);
    }

    /**
     * This method returns valueMode
     * @return String which is value mode
     * @throws LoaderException
     */
    public String getValueMode() throws LoaderException {
        return this.generatorParameters.getValueMode();
    }

    /**
     * This method set generatorOutput
     * @param generatorOutput
     * @throws LoaderException
     */
    public void setGeneratorOutput(String generatorOutput) throws LoaderException {
        this.generatorParameters.setGeneratorOutput(generatorOutput);
    }

    /**
     * This method return generatorOutput
     * @return generatorOutput
     * @throws LoaderException
     */
    public String getGeneratorOutput() throws LoaderException {
        return this.generatorParameters.getGeneratorOutput();
    }

    /**
     * This method set sourceUser
     * @param sourceUser
     * @throws LoaderException
     */
    public void setSourceUser(String sourceUser) throws LoaderException {
        this.generatorParameters.setSourceUser(sourceUser);
    }

    /**
     * This method return sourceUser
     * @return sourceUser
     * @throws LoaderException
     */
    public String getSourceUser() throws LoaderException {
        return this.generatorParameters.getSourceUser();
    }

    /**
     * This method set targetUser
     * @param targetUser
     * @throws LoaderException
     */
    public void setTargetUser(String targetUser) throws LoaderException {
        this.generatorParameters.setTargetUser(targetUser);
    }

    /**
     * This method return targetUser
     * @return targetUser
     * @throws LoaderException
     */
    public String getTargetUser() throws LoaderException {
        return this.generatorParameters.getTargetUser();
    }
    /**
     * This method set sourcePassword
     * @param sourcePassword
     * @throws LoaderException
     */
    public void setSourcePassword(String sourcePassword) throws LoaderException {
        this.generatorParameters.setSourcePassword(sourcePassword);
    }

    /**
     * This method return sourcePassword
     * @return sourcePassword
     * @throws LoaderException
     */
    public String getSourcePassword() throws LoaderException {
        return this.generatorParameters.getSourcePassword();
    }

    /**
     * This method set targetPassword
     * @param targetPassword
     * @throws LoaderException
     */
    public void setTargetPassword(String targetPassword) throws LoaderException {
        this.generatorParameters.setTargetPassword(targetPassword);
    }

    /**
     * This method set targetPassword
     * @return targetPassword
     * @throws LoaderException
     */
    public String getTargetPassword() throws LoaderException {
        return this.generatorParameters.getTargetPassword();
    }

    /**
     * This method set domlPath
     * @param domlPath
     * @throws LoaderException
     */
    public void setDomlPath(String domlPath) throws LoaderException {
        this.generatorParameters.setDomlPath(domlPath);
    }

    /**
     * This method return domlPath
     * @return domlPath
     * @throws LoaderException
     */
    public String getDomlPath() throws LoaderException {
        return this.generatorParameters.getDomlPath();
    }

    /**
     * This method set packageName
     * @param packageName
     * @throws LoaderException
     */
    public void setPackageName(String packageName) throws LoaderException {
        this.generatorParameters.setPackageName(packageName);
    }

    /**
     * This method return packageName
     * @return packageName
     * @throws LoaderException
     */
    public String getPackageName() throws LoaderException {
        return this.generatorParameters.getPackageName();
    }
    /**
     * This method set generateDropTableStmt
     * @param generateDropTableStmt
     * @throws LoaderException
     */
    public void setGenerateDropTableStmt(boolean generateDropTableStmt) throws LoaderException {
        this.generatorParameters.setSqlStmtDropTable(new Boolean(generateDropTableStmt).toString());
    }
    /**
     * This method return generateDropTableStmt
     * @return generateDropTableStmt
     * @throws LoaderException
     */
    public boolean getGenerateDropTableStmt() throws LoaderException {
        Vector tempVector = this.generatorParameters.getSqlToGenerate();
        String dropTables = (String) tempVector.elementAt(1);
        if (dropTables.equalsIgnoreCase("DropTables")) {
            return true;
        } else {
            return false;
        }

    }
    /**
     * This method set generateCreateTableStmt
     * @param generateCreateTableStmt
     * @throws LoaderException
     */
    public void setGenerateCreateTableStmt(boolean generateCreateTableStmt) throws LoaderException {
        this.generatorParameters.setSqlStmtCreateTable(new Boolean(generateCreateTableStmt).toString());
    }

    /**
     * This method will return boolean which represents if user set "Create Tables" or not
     * @return boolean value
     * @throws LoaderException
     */
    public boolean getGenerateCreateTableStmt() throws LoaderException {
        Vector tempVector = this.generatorParameters.getSqlToGenerate();
        String createTables = (String) tempVector.elementAt(2);
        if (createTables.equalsIgnoreCase("CreateTables")) {
            return true;
        } else {
            return false;
        }
    }
    /**
     * This method set generateDropIntegrityStmt
     * @param generateDropIntegrityStmt
     * @throws LoaderException
     */

    public void setGenerateDropIntegrityStmt(boolean generateDropIntegrityStmt) throws LoaderException {
        this.generatorParameters.setSqlStmtDropIntegrity(new Boolean(generateDropIntegrityStmt).toString());
    }
    /**
     * This method return generateDropIntegrityStmt
     * @return generateDropIntegrityStmt
     * @throws LoaderException
     */
    public boolean getGenerateDropIntegrityStmt() throws LoaderException {
        Vector tempVector = this.generatorParameters.getSqlToGenerate();
        String dropIntegrity = (String) tempVector.elementAt(0);
        if (dropIntegrity.equalsIgnoreCase("DropIntegrity")) {
            return true;
        } else {
            return false;
        }
    }
    /**
     * This method set generateCreatePKStmt
     * @param generateCreatePKStmt
     * @throws LoaderException
     */
    public void setGenerateCreatePKStmt(boolean generateCreatePKStmt) throws LoaderException {
        this.generatorParameters.setSqlStmtCreatePK(new Boolean(generateCreatePKStmt).toString());
    }
    /**
     * This method return generateCreatePKStmt
     * @return GenerateCreatePKStmt
     * @throws LoaderException
     */
    public boolean getGenerateCreatePKStmt() throws LoaderException {
        Vector tempVector = this.generatorParameters.getSqlToGenerate();
        String createPrimary = (String) tempVector.elementAt(4);
        if (createPrimary.equalsIgnoreCase("CreatePrimary")) {
            return true;
        } else {
            return false;
        }
    }
    /**
     * This method set generateCreateFKStmt
     * @param generateCreateFKStmt
     * @throws LoaderException
     */
    public void setGenerateCreateFKStmt(boolean generateCreateFKStmt) throws LoaderException {
        this.generatorParameters.setSqlStmtCreateFK(new Boolean(generateCreateFKStmt).toString());
    }

    /**
     * This method return generateCreateFKStmt
     * @return generateCreateFKStmt
     * @throws LoaderException
     */
    public boolean getGenerateCreateFKStmt() throws LoaderException {
        Vector tempVector = this.generatorParameters.getSqlToGenerate();
        String createForeign = (String) tempVector.elementAt(5);
        if (createForeign.equalsIgnoreCase("CreateForeigin")) {
            return true;
        } else {
            return false;
        }
    }
    /**
     * This method set generateCreateIndexStmt
     * @param generateCreateIndexStmt
     * @throws LoaderException
     */
    public void setGenerateCreateIndexStmt(boolean generateCreateIndexStmt) throws LoaderException {
        this.generatorParameters.setSqlStmtCreateIndex(new Boolean(generateCreateIndexStmt).toString());
    }
    /**
     * This method return generateCreateIndexStmt
     * @return CreateIndexes
     * @throws LoaderException
     */
    public boolean getGenerateCreateIndexStmt() throws LoaderException {
        Vector tempVector = this.generatorParameters.getSqlToGenerate();
        String createIndexes = (String) tempVector.elementAt(3);
        if (createIndexes.equalsIgnoreCase("CreateIndexes")) {
            return true;
        } else {
            return false;
        }
    }
    /**
     * This method set generateSqlForAllVendors
     * @param generateSqlForAllVendors
     * @throws LoaderException
     */
    public void setGenerateSqlForAllVendors(boolean generateSqlForAllVendors) throws LoaderException {
        this.generatorParameters.setAllVendors(new Boolean(generateSqlForAllVendors).toString());
    }

    /**
     * This method return GenerateSqlForAllVendors
     * @return GenerateSqlForAllVendors
     * @throws LoaderException
     */
    public boolean getGenerateSqlForAllVendors() throws LoaderException {
        return this.generatorParameters.getSqlForAllVendors();
    }
    /**
     * This method set generateXml
     * @param generateXml
     * @throws LoaderException
     */
    public void setGenerateXml(boolean generateXml) throws LoaderException {
        this.generatorParameters.setGenerateXml(new Boolean(generateXml).toString());
    }

    /**
     * This method return generateXml
     * @return boolean 
     * @throws LoaderException
     */
    public boolean getGenerateXml() throws LoaderException {
        String tempGenerateXml = this.generatorParameters.getGenerateXml();
        return (new Boolean(tempGenerateXml)).booleanValue();

    }
    /**
     * This method set generateDoml
     * @param generateDoml
     * @throws LoaderException
     */
    public void setGenerateDoml(boolean generateDoml) throws LoaderException {
        this.generatorParameters.setGenerateDoml(new Boolean(generateDoml).toString());
    }

    /**
     * This method return generateDoml
     * @return boolean with value for generateDoml
     * @throws LoaderException
     */
    public boolean getGenerateDoml() throws LoaderException {
        String tempGenerateDoml = this.generatorParameters.getGenerateDoml();
        return (new Boolean(tempGenerateDoml)).booleanValue();
    }

    /**
     * This method return generateSQL
     * @return bollean with value for generateSQL
     * @throws LoaderException
     */
    public boolean getGenerateSQL() throws LoaderException {
        boolean tempGenerateSQL = this.generatorParameters.isGenerateSql();
        return tempGenerateSQL;
    }
    /**
     * This method set fullMode
     * @param fullMode
     * @throws LoaderException
     */
    public void setFullMode(boolean fullMode) throws LoaderException {
        this.generatorParameters.setFullMode(new Boolean(fullMode).toString());
    }

    /**
     * This method return fullMode
     * @return boolean with value for fullMode
     * @throws LoaderException
     */
    public boolean getFullMode() throws LoaderException {
        return this.generatorParameters.getFullMode();
    }

    /**
     * This method set includeTableList
     * @param includeTableList
     * @throws LoaderException
     */
    public void setIncludeTableList(String includeTableList) throws LoaderException {
        this.generatorParameters.setIncludeTableList(includeTableList);
    }

    /**
     * This method return includeTableList
     * @return String which is list with included tables
     * @throws LoaderException
     */
    public String getIncludeTableList() throws LoaderException {
        Vector tmpIncludeTableList = this.generatorParameters.getIncludeTableList();
        return tmpIncludeTableList.toString();
    }

    /**
     * This method set confJarStructure
     * @param confJarStructure
     * @throws LoaderException
     */
    public void setConfJarStructure(String confJarStructure) throws LoaderException {
        this.generatorParameters.setConfJarStructure(confJarStructure);
    }

    /**
     * This method get confJarStructure
     * @return ConfJarStructure
     * @throws LoaderException
     */
    public String getConfJarStructure() throws LoaderException {
        return this.generatorParameters.getConfJarStructure();
    }

    /**
     * This method set restoreMode
     * @param restoreMode
     * @throws LoaderException
     */
    public void setRestoreMode(String restoreMode) throws LoaderException {
        this.generatorParameters.setRestoreMode(restoreMode);
    }

    /**
     * This method return restoreMode
     * @return restoreMode
     * @throws LoaderException
     */
    public String getRestoreMode() throws LoaderException {
        return this.generatorParameters.getRestoreMode();
    }

    
	/**
		* This method return logMode
		* @return logMode
		*/
	   public String getLogMode() {
		   return logMode;
	   }

	   /**
		* This method return strLogDirName
		* @return strLogDirName
		*/
	   public String getStrLogDirName() {
		   return strLogDirName;
	   }

	   /**
		* This method return strLogFileName
		* @return strLogFileName
		*/
	   public String getStrLogFileName() {
		   return strLogFileName;
	   }

	   /**
		* This method set logMode
		* @param logMode
		*/
	   public void setLogMode(String logMode) {
		   this.logMode = logMode;
	   }

	   /**
		* This method set strLogDirName
		* @param strLogDirName
		*/
	   public void setStrLogDirName(String strLogDirName) {
		   this.strLogDirName = strLogDirName;
	   }

	   /**
		* This method set strLogFileName
		* @param strLogFileName
		*/
	   public void setStrLogFileName(String strLogFileName) {
		   this.strLogFileName = strLogFileName;
	   }
   
    /**
     * This method write values for all input parameters to System out
     * @throws LoaderException
     */
    public String inputToString() throws LoaderException {
        String resString = "";
        try {

            resString ="Source type="+ getSourceType()+ "\n" 
                		+ "Source driver=" + getSourceDriverName()+ "\n"
                		+ "Source URL=" + getSourceDataBase()+ "\n"
                    + "Source user="+ getSourceUser()+ "\n"
                    + "Source password="+ getSourcePassword()+ "\n"
                    + "Target type=" + getTargetType()+ "\n"
                    + "Target driver=" + getTargetDriverName() + "\n"
                    + "Target URL=" + getTargetDataBase() + "\n"
                    + "Target user=" + getTargetUser() + "\n"
                    + "Target password=" + getTargetPassword() + "\n"
                    + "Conf file in jar=" + getConfJarStructure() + "\n"
                    + "Doml path=" + getDomlPath()+ "\n"
                    + "Ouput directory=" + getGeneratorOutput() + "\n"
                    + "Include table list=" + getIncludeTableList() + "\n"
                    + "Restore mode=" + getRestoreMode()+ "\n"
                    + "Value mode="+ getValueMode()+ "\n"
                    + "Generate create foreign keys ="+ getGenerateCreateFKStmt()+ "\n"
                    + "Generate create indexes ="+ getGenerateCreateIndexStmt()+ "\n"
                    + "Generate create primary keys ="+ getGenerateCreatePKStmt()+ "\n"
                    + "Generate create tables ="+ getGenerateCreateTableStmt()+ "\n"
                    + "Generate drop foreign keys ="+ getGenerateDropIntegrityStmt()+ "\n"
                    + "Generate drop tables ="+ getGenerateDropTableStmt()+ "\n"
                    + "Generate sql for all vendors ="+ getGenerateSqlForAllVendors()+ "\n"
                    + "Generate doml ="+ getGenerateDoml()+ "\n"
                    + "Package name="+ getPackageName()+ "\n"
                    + "Generate xml ="+ getGenerateXml()+ "\n"
                    + "Full mode=" + getFullMode() + "\n"
										+ "Log mode=" + getLogMode() + "\n"
										+ "Log directory name=" + getStrLogDirName() + "\n"
										+ "Log file name=" + getStrLogFileName() + "\n";

            return resString;
        } catch (Exception e) {
            throw new LoaderException("Exception:" + e.getMessage(), e);
        }
    }

    /**
     * Construct object LoaderGenerator with associated parameters.
     * @param sourceType defines the type of the source data.
     * @param targetType defines the type of the target data.
     * @param sourceDriverName is driver of defined source database.
     * @param targetDriverName is driver of defined  target database.
     * @param sourceDataBase defines the place where you're put the database tables.
     * @param targetDataBase defines the place where the target database tables are.
     * @param valueMode defines the difference from overwrite and update attribute.
     * Default valueMode is Overwrite.
     * @param generatorOutput is optional. That represents the directory, where the
     * Octopus place created files. If this argument don't exists, Octopus place
     * created files in to current directory!
     * @param sourceUser defines the user of the source database.
     * @param sourcePassword defines user password for the source database.
     * @param targetUser defines the user of the target database.
     * @param targetPassword defines user password for the target database.
     * @param domlPath defines if the doml file exists. Default value is false
     * all, application generates xml, sql and doml files as ouput files. If the generateDoc has value
     * doml, application generates only the doml file, and if generateDoc has the value wdoml, application
     * generates xml and sql files.
     * @param packageName defines the package name for the generated doml file.
     * @param generateDropTableStmt defines if you want to generate only Sql statement for DROP TABLE
     * @param generateCreateTableStmt defines if you want to generate only Sql statement for CREATE TABLE
     * @param generateCreatePKStmt defines if you want to generate only Sql statement for ALTER TABLE
     * @param generateCreateFKStmt defines if you want to generate only Sql statement for ALTER TABLE
     * @param generateCreateIndexStmt defines if you want to generate only Sql statement for ALTER TABLE
     * @param generateSqlForAllVendors defines if you want to generate Sql statements for all named
     * database vendors
     * @param generateXml defines if you want to generate xml file as output files
     * @param generateDoml defines if you want to generate doml file as output file
     * @param fullMode defines in which mode you want to generate xml files.
     * @param includeTableList defines the list of tables which you want to include into
     * Generator process
     * @param confJarStructure defines the structure of jar file in which are conf
     * files placed
     * @throws LoaderException
     */

    public LoaderGenerator(String sourceType, String sourceDataBase, 
												    String valueMode, String generatorOutput, String sourceDriverName, 
												    String targetDriverName, String targetDataBase, String targetType, 
												    String sourceUser, String sourcePassword, String targetUser, 
												    String targetPassword, String domlPath, String packageName, String generateDropTableStmt, 
												    String generateDropIntegrityStmt, String generateCreateTableStmt, String generateCreatePKStmt, 
												    String generateCreateFKStmt, String generateCreateIndexStmt, String generateSqlForAllVendors, 
												    String generateXml, String generateDoml, String fullMode, String includeTableList, 
												    String confJarStructure) 
												    throws LoaderException {

        this(sourceType, sourceDataBase, valueMode, generatorOutput, 
        sourceDriverName, targetDriverName, targetDataBase, targetType, 
        sourceUser, sourcePassword, targetUser, targetPassword, domlPath, 
        packageName, generateDropTableStmt, generateDropIntegrityStmt, 
        generateCreateTableStmt, generateCreatePKStmt, generateCreateFKStmt, 
        generateCreateIndexStmt, generateSqlForAllVendors, generateXml, generateDoml, 
        fullMode, "false", includeTableList, confJarStructure);

    }

    /**
     * Construct object LoaderGenerator with associated parameters.
     * @param sourceType defines the type of the source data.
     * @param targetType defines the type of the target data.
     * @param sourceDriverName is driver of defined source database.
     * @param targetDriverName is driver of defined  target database.
     * @param sourceDataBase defines the place where you're put the database tables.
     * @param targetDataBase defines the place where the target database tables are.
     * @param valueMode defines the difference from overwrite and update attribute.
     * Default valueMode is Overwrite.
     * @param generatorOutput is optional. That represents the directory, where the
     * Octopus place created files. If this argument don't exists, Octopus place
     * created files in to current directory!
     * @param sourceUser defines the user of the source database.
     * @param sourcePassword defines user password for the source database.
     * @param targetUser defines the user of the target database.
     * @param targetPassword defines user password for the target database.
     * @param domlPath defines if the doml file exists. Default value is false
     * all, application generates xml, sql and doml files as ouput files. If the generateDoc has value
     * doml, application generates only the doml file, and if generateDoc has the value wdoml, application
     * generates xml and sql files.
     * @param packageName defines the package name for the generated doml file.
     * @param generateDropTableStmt defines if you want to generate only Sql statement for DROP TABLE
     * @param generateCreateTableStmt defines if you want to generate only Sql statement for CREATE TABLE
     * @param generateCreatePKStmt defines if you want to generate only Sql statement for ALTER TABLE
     * @param generateCreateFKStmt defines if you want to generate only Sql statement for ALTER TABLE
     * @param generateCreateIndexStmt defines if you want to generate only Sql statement for ALTER TABLE
     * @param generateSqlForAllVendors defines if you want to generate Sql statements for all named
     * database vendors
     * @param generateXml defines if you want to generate xml file as output files
     * @param generateDoml defines if you want to generate doml file as output file
     * @param fullMode defines in which mode you want to generate xml files.
     * @param restoreMode defines if we use application for restoring the database
     * @param includeTableList defines the list of tables which you want to include into
     * Generator process
     * @param confJarStructure defines the structure of jar file in which are conf files
     * placed.
     * @throws LoaderException
     */

    public LoaderGenerator(String sourceType, String sourceDataBase, String valueMode, 
												    String generatorOutput, String sourceDriverName, String targetDriverName, 
												    String targetDataBase, String targetType, String sourceUser, String sourcePassword, 
												    String targetUser, String targetPassword, String domlPath, String packageName, 
												    String generateDropTableStmt, String generateDropIntegrityStmt, String generateCreateTableStmt, 
												    String generateCreatePKStmt, String generateCreateFKStmt, String generateCreateIndexStmt, 
												    String generateSqlForAllVendors, String generateXml, String generateDoml, String fullMode, 
												    String restoreMode, String includeTableList, String confJarStructure) 
												    throws LoaderException {

        generatorParameters = new InputParameters(confJarStructure);
        generatorParameters.setConfJarStructure(confJarStructure);

        generatorParameters.setSourceType(sourceType);
        //target and source driver name must goes frist, because alterTablePrimaryKey...
        generatorParameters.setTargetDriverName(targetDriverName);
        generatorParameters.setSourceDriverName(sourceDriverName);
        generatorParameters.setTargetType(targetType);
        generatorParameters.setSourceDataBase(sourceDataBase);
        generatorParameters.setTargetDataBase(targetDataBase);
        generatorParameters.setValueMode(valueMode);
        generatorParameters.setGeneratorOutput(generatorOutput);
        generatorParameters.setSourceUser(sourceUser);
        generatorParameters.setSourcePassword(sourcePassword);
        generatorParameters.setTargetUser(targetUser);
        generatorParameters.setTargetPassword(targetPassword);

        if (generateDropTableStmt != null)
            generatorParameters.setSqlStmtDropTable(generateDropTableStmt);
        if (generateDropIntegrityStmt != null)
            generatorParameters.setSqlStmtDropIntegrity(generateDropIntegrityStmt);
        if (generateCreateTableStmt != null)
            generatorParameters.setSqlStmtCreateTable(generateCreateTableStmt);
        if (generateCreatePKStmt != null)
            generatorParameters.setSqlStmtCreatePK(generateCreatePKStmt);
        if (generateCreateFKStmt != null)
            generatorParameters.setSqlStmtCreateFK(generateCreateFKStmt);
        if (generateCreateIndexStmt != null)
            generatorParameters.setSqlStmtCreateIndex(generateCreateIndexStmt);
        if (generateSqlForAllVendors != null)
            generatorParameters.setAllVendors(generateSqlForAllVendors);

        generatorParameters.setDomlPath(domlPath);
        if (generateXml != null)
            generatorParameters.setGenerateXml(generateXml);
        if (generateDoml != null)
            generatorParameters.setGenerateDoml(generateDoml);
        generatorParameters.setPackageName(packageName);
        if (fullMode != null)
            generatorParameters.setFullMode(fullMode);
        if (restoreMode != null)
            generatorParameters.setRestoreMode(restoreMode);
        if (includeTableList != null)
            generatorParameters.setIncludeTableList(includeTableList);

        generatorParameters.setSourceDriverProperties();
        generatorParameters.setTargetDriverProperties();
        generatorParameters.setOidAndVersionColumnName();
        generatorParameters.setHasSize();
        generatorParameters.setIsDecimal();
        //setLogger();

    }

    /**
    	 * Construct object LoaderGenerator with associated parameters.
    	 * @param sourceType defines the type of the source data.
    	 * @param targetType defines the type of the target data.
    	 * @param sourceDriverName is driver of defined source database.
    	 * @param targetDriverName is driver of defined  target database.
    	 * @param sourceDataBase defines the place where you're put the database tables.
    	 * @param targetDataBase defines the place where the target database tables are.
    	 * @param valueMode defines the difference from overwrite and update attribute.
    	 * Default valueMode is Overwrite.
    	 * @param generatorOutput is optional. That represents the directory, where the
    	 * Octopus place created files. If this argument don't exists, Octopus place
    	 * created files in to current directory!
    	 * @param sourceUser defines the user of the source database.
    	 * @param sourcePassword defines user password for the source database.
    	 * @param targetUser defines the user of the target database.
    	 * @param targetPassword defines user password for the target database.
    	 * @param domlPath defines if the doml file exists. Default value is false
    	 * all, application generates xml, sql and doml files as ouput files. If the generateDoc has value
    	 * doml, application generates only the doml file, and if generateDoc has the value wdoml, application
    	 * generates xml and sql files.
    	 * @param packageName defines the package name for the generated doml file.
    	 * @param generateDropTableStmt defines if you want to generate only Sql statement for DROP TABLE
    	 * @param generateCreateTableStmt defines if you want to generate only Sql statement for CREATE TABLE
    	 * @param generateCreatePKStmt defines if you want to generate only Sql statement for ALTER TABLE
    	 * @param generateCreateFKStmt defines if you want to generate only Sql statement for ALTER TABLE
    	 * @param generateCreateIndexStmt defines if you want to generate only Sql statement for ALTER TABLE
    	 * @param generateSqlForAllVendors defines if you want to generate Sql statements for all named
    	 * database vendors
    	 * @param generateXml defines if you want to generate xml file as output files
    	 * @param generateDoml defines if you want to generate doml file as output file
    	 * @param fullMode defines in which mode you want to generate xml files.
    	 * @param restoreMode defines if we use application for restoring the database
    	 * @param includeTableList defines the list of tables which you want to include into
    	 * Generator process
    	 * @param confJarStructure defines the structure of jar file in which are conf files
    	 * @param logMode defines logging level
    	 * @param logDirName defines logging directory
    	 * @param logFileName defines logging file name
    	 * placed.
    	 * @throws LoaderException
    	 */

    public LoaderGenerator(String sourceType, String sourceDataBase, String valueMode, 
												    String generatorOutput, String sourceDriverName, String targetDriverName, String targetDataBase, 
												    String targetType, String sourceUser, String sourcePassword, String targetUser, 
												    String targetPassword, String domlPath, String packageName, String generateDropTableStmt, 
												    String generateDropIntegrityStmt, String generateCreateTableStmt, String generateCreatePKStmt, 
												    String generateCreateFKStmt, String generateCreateIndexStmt, String generateSqlForAllVendors, 
												    String generateXml, String generateDoml, String fullMode, String restoreMode, 
												    String includeTableList, String confJarStructure, String logMode, String logDirName, 
												    String logFileName) 
												    throws LoaderException {

        generatorParameters = new InputParameters(confJarStructure);
        generatorParameters.setConfJarStructure(confJarStructure);

        generatorParameters.setSourceType(sourceType);
        //target and source driver name must goes frist, because alterTablePrimaryKey...
        generatorParameters.setTargetDriverName(targetDriverName);
        generatorParameters.setSourceDriverName(sourceDriverName);
        generatorParameters.setTargetType(targetType);
        generatorParameters.setSourceDataBase(sourceDataBase);
        generatorParameters.setTargetDataBase(targetDataBase);
        generatorParameters.setValueMode(valueMode);
        generatorParameters.setGeneratorOutput(generatorOutput);
        generatorParameters.setSourceUser(sourceUser);
        generatorParameters.setSourcePassword(sourcePassword);
        generatorParameters.setTargetUser(targetUser);
        generatorParameters.setTargetPassword(targetPassword);

        if (generateDropTableStmt != null)
            generatorParameters.setSqlStmtDropTable(generateDropTableStmt);
        if (generateDropIntegrityStmt != null)
            generatorParameters.setSqlStmtDropIntegrity(generateDropIntegrityStmt);
        if (generateCreateTableStmt != null)
            generatorParameters.setSqlStmtCreateTable(generateCreateTableStmt);
        if (generateCreatePKStmt != null)
            generatorParameters.setSqlStmtCreatePK(generateCreatePKStmt);
        if (generateCreateFKStmt != null)
            generatorParameters.setSqlStmtCreateFK(generateCreateFKStmt);
        if (generateCreateIndexStmt != null)
            generatorParameters.setSqlStmtCreateIndex(generateCreateIndexStmt);
        if (generateSqlForAllVendors != null)
            generatorParameters.setAllVendors(generateSqlForAllVendors);

        generatorParameters.setDomlPath(domlPath);
        if (generateXml != null)
            generatorParameters.setGenerateXml(generateXml);
        if (generateDoml != null)
            generatorParameters.setGenerateDoml(generateDoml);
        generatorParameters.setPackageName(packageName);
        if (fullMode != null)
            generatorParameters.setFullMode(fullMode);
        if (restoreMode != null)
            generatorParameters.setRestoreMode(restoreMode);
        if (includeTableList != null)
            generatorParameters.setIncludeTableList(includeTableList);
        if (logMode != null) {
            this.logMode = logMode;
        }
        if (logDirName != null) {
            this.strLogDirName = logDirName;
        }
        if (logFileName != null) {
            this.strLogFileName = logFileName;
        }
        generatorParameters.setSourceDriverProperties();
        generatorParameters.setTargetDriverProperties();
        generatorParameters.setOidAndVersionColumnName();
        generatorParameters.setHasSize();
        generatorParameters.setIsDecimal();

    }

    /**
     * Public constructor of LoaderGenerator class. Constructor set value for sourceDataBase and
     * targetDataBase attribute. Class set all other attributes to the default values.
     * @param sourceDataBase defines the place where you're put the database tables.
     * @param targetDataBase defines the place where the target database tables are.
     * @param sourceType defines the type of the source database.
     * @param targetType defines the type of the target database.
     * @throws LoaderException
     */
    public LoaderGenerator(String sourceDataBase, String targetDataBase, String sourceType, 
    												String targetType) throws LoaderException {

        this(sourceDataBase, targetDataBase, sourceType, targetType, null);

    }

    /**
    * Public constructor of LoaderGenerator class. Constructor set value for sourceDataBase and
    * targetDataBase attribute. Class set all other attributes to the default values.
    * @param sourceDataBase defines the place where you're put the database tables.
    * @param targetDataBase defines the place where the target database tables are.
    * @param sourceType defines the type of the source database.
    * @param targetType defines the type of the target database.
    * @throws LoaderException
    */
    public LoaderGenerator(String sourceDataBase, String targetDataBase, String sourceType, 
    											String targetType, String confJarStructure) throws LoaderException {

        generatorParameters = new InputParameters("");
        generatorParameters.setConfJarStructure(confJarStructure);
        generatorParameters.setSourceDataBase(sourceDataBase);

        generatorParameters.setTargetDataBase(targetDataBase);
        generatorParameters.setSourceType(sourceType);
        generatorParameters.setTargetType(targetType);
        generatorParameters.setSourceDriverProperties();
        generatorParameters.setTargetDriverProperties();
        generatorParameters.setOidAndVersionColumnName();
        generatorParameters.setHasSize();
        generatorParameters.setIsDecimal();
        //setLogger();
    }

    /**
     * Main method controls parameters, it they are OK starts load method, but if they aren't makes alert.
     * @param args represents the input parameters
     *
     * Usage: java org.webdocwf.util.loader.generator.Generator [options]
     * Options:
     * -st sourceType, defines the type of the source database.
     * -tt targetType, defines the type of the target database.
     * -sdn sourceDriverName, defines the driver of defined source database.
     * -tdn targetDriverName, defines the driver of defined  target database.
     * -sdb sourceDataBase, defines the place where you're put the source database tables.
     * -tdb targetDataBase, defines the place where you're put the target database tables.
     * -m valueMode, defines the difference from overwrite and update attribute. If the valueMode
     * has value 'copy', attribute has value 'Overwrite', or if the valueMode has value 'sync', attribute
     * has value 'Update'. Default value of valueMode is 'copy'.
     * -o generatorOutput, is optional. That represent the directory, where the Octopus place created files.
     * If this argument don't exists, Octopus place created files in to current directory!
     * -su sourceUser, defines the user of the source database.
     * -sp sourcePassword, defines the user password for the source database.
     * -tu targetUser, defines the user of the target database.
     * -tp targetPassword, defines the user password for the target database.
     * -doml domlUrl, defines the place where the doml file is placed.
     * -pack packageName, defines the package name for the generated doml file.
     * -xml -xml generateXml, defines if you want to generate xml files as output files
     * -gdoml generateDoml, defines if you want to generate doml file as output file.
     * -sqlCT generateCreateTables defines if you want to generate only Sql statement for CREATE TABLE.
     * -sqlDT generateDropTableStmt defines if you want to generate only Sql statement for DROP TABLE.
     * -sqlCPK generateCreatePrimaryKeys defines if you want to generate only Sql statement for ALTER TABLE ADD CONSTRAINT (PRIMARY KEY).
     * -sqlCFK generateCreateForeigKeys defines if you want to generate only Sql statement for ALTER TABLE ADD CONSTRAINT (FOREIGN KEY).
     * -sqlCI generateCreateIndex defines if you want to generate only Sql statement for CREATE INDEX.
     * -sqlAll generateSqlForAllVendors defines if you want to generate sql statements for all named database vendors.
     * -fm fullMode defines the mode of output xml files. Possible values are true and false.
     * -lm logMode defines logging level
     * -l logDirName defines logging directory
     * -f logFileName defines logging file name
     **/

    public static void main(String args[]) {

        String generatorOutput = null;
        String valueMode = null;
        String domlUrl = null;
        String packageName = null;

        String sourceType = null;
        String sourceDataBase = null;
        String sourceDriverName = null;
        String sourceUser = null;
        String sourcePassword = null;

        String targetType = null;
        String targetDataBase = null;
        String targetDriverName = null;
        String targetUser = null;
        String targetPassword = null;

        //logging
        String logDirName = null;
        String logFileName = null;
        String logMode = null;

        //Output mode
        //main
        String generateXml = null;
        String generateDoml = null;
        //for generate sql
        String generateDropTableStmt = null;
        String generateDropIntegrityStmt = null;
        String generateCreateTableStmt = null;
        String generateCreatePKStmt = null;
        String generateCreateFKStmt = null;
        String generateCreateIndexStmt = null;

        //for genrate sql for all vendors
        String generateSqlForAllVendors = null;
        //optimized modes
        String fullMode = null;
        String restoreMode = null;
        String includeTableList = null;
        String confJarStructure = null;

        if (args.length > 8 && args.length < 53) {
            for (int i = 0; i < args.length - 1; i = i + 1) {
                if (args[i].equalsIgnoreCase("-st"))
                    sourceType = args[++i];
                else if (args[i].equalsIgnoreCase("-sdb"))
                    sourceDataBase = args[++i];
                else if (args[i].equalsIgnoreCase("-m"))
                    valueMode = args[++i];
                else if (args[i].equalsIgnoreCase("-o"))
                    generatorOutput = args[++i];
                else if (args[i].equalsIgnoreCase("-sdn"))
                    sourceDriverName = args[++i];
                else if (args[i].equalsIgnoreCase("-tdn"))
                    targetDriverName = args[++i];
                else if (args[i].equalsIgnoreCase("-tdb"))
                    targetDataBase = args[++i];
                if (args[i].equalsIgnoreCase("-tt"))
                    targetType = args[++i];
                if (args[i].equalsIgnoreCase("-su"))
                    sourceUser = args[++i];
                if (args[i].equalsIgnoreCase("-sp"))
                    sourcePassword = args[++i];
                if (args[i].equalsIgnoreCase("-tu"))
                    targetUser = args[++i];
                if (args[i].equalsIgnoreCase("-tp"))
                    targetPassword = args[++i];
                if (args[i].equalsIgnoreCase("-doml"))
                    domlUrl = args[++i];
                if (args[i].equalsIgnoreCase("-pack"))
                    packageName = args[++i];
                if (args[i].equalsIgnoreCase("-xml"))
                    generateXml = args[++i];
                if (args[i].equalsIgnoreCase("-gdoml"))
                    generateDoml = args[++i];
                if (args[i].equalsIgnoreCase("-sqlCT"))
                    generateCreateTableStmt = args[++i];
                if (args[i].equalsIgnoreCase("-sqlDT"))
                    generateDropTableStmt = args[++i];
                if (args[i].equalsIgnoreCase("-sqlDI"))
                    generateDropIntegrityStmt = args[++i];
                if (args[i].equalsIgnoreCase("-sqlCPK"))
                    generateCreatePKStmt = args[++i];
                if (args[i].equalsIgnoreCase("-sqlCFK"))
                    generateCreateFKStmt = args[++i];
                if (args[i].equalsIgnoreCase("-sqlCI"))
                    generateCreateIndexStmt = args[++i];
                if (args[i].equalsIgnoreCase("-sqlAll"))
                    generateSqlForAllVendors = args[++i];
                if (args[i].equalsIgnoreCase("-fm"))
                    fullMode = args[++i];
                if (args[i].equalsIgnoreCase("-rm"))
                    restoreMode = args[++i];
                if (args[i].equalsIgnoreCase("-it"))
                    includeTableList = args[++i];
                if (args[i].equalsIgnoreCase("-cjs"))
                    confJarStructure = args[++i];
                if (args[i].equalsIgnoreCase("-lm"))
                    logMode = args[++i];
                else if (args[i].equalsIgnoreCase("-l"))
                    logDirName = args[++i];
                else if (args[i].equalsIgnoreCase("-f"))
                    logFileName = args[++i];
            }
        } else {
            System.out.println("You didn't enter all required parameters, for running the application!");
            BufferOctopusClass.getInstance().writeToBuffer("You didn't enter all required parameters, for running the application!");
            printUsage();
            System.exit(1);
        }
        try {

            LoaderGenerator generator = new LoaderGenerator(sourceType, sourceDataBase, valueMode, generatorOutput, sourceDriverName, targetDriverName, targetDataBase, targetType, sourceUser, sourcePassword, targetUser, targetPassword, domlUrl, packageName, generateDropTableStmt, generateDropIntegrityStmt, generateCreateTableStmt, generateCreatePKStmt, generateCreateFKStmt, generateCreateIndexStmt, generateSqlForAllVendors, generateXml, generateDoml, fullMode, restoreMode, includeTableList, confJarStructure, logMode, logDirName, logFileName);

            generator.generate();
            System.exit(0);

        } catch (LoaderException le) {
            System.out.println(le.getStackTraceAsString());
            BufferOctopusClass.getInstance().writeToBuffer(le.getStackTraceAsString());
            return;
        } catch (Throwable th) {
            BufferOctopusClass.getInstance().writeToBuffer(th.getMessage());
            th.printStackTrace(System.out);
            return;
        }
    }

    static void printUsage() {
        //System.out.println("Usage: java org.webdocwf.util.loader.generator.Generator [options] ");
        BufferOctopusClass.getInstance().writeToBuffer("Usage: java org.webdocwf.util.loader.generator.Generator [options] ");
        //System.out.println(" Options:");
        BufferOctopusClass.getInstance().writeToBuffer(" Options:");
        //System.out.println(" -st sourceType, defines the type of the source database.");
        BufferOctopusClass.getInstance().writeToBuffer(" -st sourceType, defines the type of the source database.");
        //System.out.println(" -tt targetType, defines the type of the target database.");
        BufferOctopusClass.getInstance().writeToBuffer(" -tt targetType, defines the type of the target database.");
        //System.out.println(" -sdn sourceDriverName, represents driver of defined source database.");
        BufferOctopusClass.getInstance().writeToBuffer(" -sdn sourceDriverName, represents driver of defined source database.");
        //    System.out.println(" -tdn targetDriverName, represents driver of defined target database.");
        BufferOctopusClass.getInstance().writeToBuffer(" -tdn targetDriverName, represents driver of defined target database.");
        //    System.out.println(" -sdb sourceDatabase, defines the place where you're put the database tables");
        BufferOctopusClass.getInstance().writeToBuffer(" -sdb sourceDatabase, defines the place where you're put the database tables");
        //    System.out.println(" -tdb targetDataBase defines the place where the target database tables are.");
        BufferOctopusClass.getInstance().writeToBuffer(" -tdb targetDataBase defines the place where the target database tables are.");
        //    System.out.println(" -m valueMode, is the difference from overwrite and update attribute");
        BufferOctopusClass.getInstance().writeToBuffer(" -m valueMode, is the difference from overwrite and update attribute");
        //    System.out.println(" -su sourceUser, defines the user of the source database.");
        BufferOctopusClass.getInstance().writeToBuffer(" -su sourceUser, defines the user of the source database.");
        //    System.out.println(" -sp sourcePassword, defines user password for the source database.");
        BufferOctopusClass.getInstance().writeToBuffer(" -sp sourcePassword, defines user password for the source database.");
        //    System.out.println(" -tu targetUser, defines the user of the target database.");
        BufferOctopusClass.getInstance().writeToBuffer(" -tu targetUser, defines the user of the target database.");
        //    System.out.println(" -tp targetPassword, defines user password for the target database.");
        BufferOctopusClass.getInstance().writeToBuffer(" -tp targetPassword, defines user password for the target database.");
        //    System.out.println(" -doml domlUrl, defines the place where the doml file is placed.");
        BufferOctopusClass.getInstance().writeToBuffer(" -doml domlUrl, defines the place where the doml file is placed.");
        //    System.out.println(" -o generatorOutput, is optional. That represent the directory, where the Octopus place"+
        //                       " created files. If this argument dont exists, Octopus place created files in to current direcory!");
        BufferOctopusClass.getInstance().writeToBuffer(" -o generatorOutput, is optional. That represent the directory, where the Octopus place" + " created files. If this argument dont exists, Octopus place created files in to current direcory!");
        //    System.out.println(" -pack packageName, defines the package name for the generated doml file.");
        BufferOctopusClass.getInstance().writeToBuffer(" -pack packageName, defines the package name for the generated doml file.");

        //    System.out.println(" -xml generateXml, defines if you want to generate xml files as output files. "+
        //                  "    Possible values are true and false.");
        BufferOctopusClass.getInstance().writeToBuffer(" -xml generateXml, defines if you want to generate xml files as output files. " + "    Possible values are true and false.");
        //    System.out.println(" -gdoml generateDoml, defines if you want to generate doml file as output file. "+
        //                      "    Possible values are true and false.");
        BufferOctopusClass.getInstance().writeToBuffer(" -gdoml generateDoml, defines if you want to generate doml file as output file. " + "    Possible values are true and false.");
        //    System.out.println(" -sqlCT generateCreateTables, defines if you want to generate only Sql statement for CREATE TABLE.");
        BufferOctopusClass.getInstance().writeToBuffer(" -sqlCT generateCreateTables, defines if you want to generate only Sql statement for CREATE TABLE.");

        //    System.out.println(" -sqlDT generateDropTableStmt, defines if you want to generate only Sql statement for DROP TABLE.");
        BufferOctopusClass.getInstance().writeToBuffer(" -sqlDT generateDropTableStmt, defines if you want to generate only Sql statement for DROP TABLE.");

        //    System.out.println(" -sqlCPK generateCreatePrimaryKeys, defines if you want to generate only Sql statement for ALTER TABLE ADD CONSTRAINT (PRIMARY KEY).");
        BufferOctopusClass.getInstance().writeToBuffer(" -sqlCPK generateCreatePrimaryKeys, defines if you want to generate only Sql statement for ALTER TABLE ADD CONSTRAINT (PRIMARY KEY).");

        //    System.out.println(" -sqlCFK generateCreateForeigKeys, defines if you want to generate only Sql statement for ALTER TABLE ADD CONSTRAINT (FOREIGN KEY).");
        BufferOctopusClass.getInstance().writeToBuffer(" -sqlCFK generateCreateForeigKeys, defines if you want to generate only Sql statement for ALTER TABLE ADD CONSTRAINT (FOREIGN KEY).");

        //    System.out.println(" -sqlCI generateCreateIndex, defines if you want to generate only Sql statement for CREATE INDEX.");
        BufferOctopusClass.getInstance().writeToBuffer(" -sqlCI generateCreateIndex, defines if you want to generate only Sql statement for CREATE INDEX.");

        //    System.out.println(" -sqlAll generateSqlForAllVendors, defines if you want to generate sql statements for all named database vendors.");
        BufferOctopusClass.getInstance().writeToBuffer(" -sqlAll generateSqlForAllVendors, defines if you want to generate sql statements for all named database vendors.");

        //    System.out.println(" -fm fullMode, defines the mode of output xml files. Possible values are true and false.");
        BufferOctopusClass.getInstance().writeToBuffer("  -fm fullMode, defines the mode of output xml files. Possible values are true and false.");

        //    System.out.println(" -it includeTableList, defines tables which you want to incude into Generator process.");
        BufferOctopusClass.getInstance().writeToBuffer("  -it includeTableList, defines tables which you want to incude into Generator process.");

        //    System.out.println("For more details, see documentation.");
        BufferOctopusClass.getInstance().writeToBuffer("For more details, see documentation.");
        //System.out.println(" -lm defines the default logmode. Possible values are 'none', 'normal' (is the default) and 'full'.");
        BufferOctopusClass.getInstance().writeToBuffer(" -lm defines the default logmode. Possible values are 'none', 'normal' (is the default) and 'full'.");

        //System.out.println(" -l defines the logfile directory. The default is the current working directory.   \n");
        BufferOctopusClass.getInstance().writeToBuffer(" -v defines variables used in variable columns.  \n");

        //System.out.println(" -f defines the logfile name. The default is 'GeneratorLog-YYYY-MM-DD-HH-mm-SS.txt'. \n");
        BufferOctopusClass.getInstance().writeToBuffer(" -f defines the logfile name. The default is 'GeneratorLog-YYYY-MM-DD-HH-mm-SS.txt'. \n");

    }

    /**
     * This method init logger
     * @throws LoaderException
     */
    private void setLogger() throws LoaderException {

        this.timeCounter = new TimeWatch();
        this.timeCounter.setStartJobTime();

        try {

            this.logger = new StandardLogger();
            this.strLoggerParam = this.strLogDirName + ";" + this.strLogFileName;
            this.standardLogger = true;
            this.logger.configure(strLoggerParam);
            this.logger.setEnabledLogLevels(this.logMode);
        } catch (Exception e) {

            LoaderException le = new LoaderException("Error while instantiate StandardLogger!", e);
            throw le;
        }
    }

    /**
     *
     * Method generate is main method in class LoaderGenerator. It is used to load data from the source
     * table, and starts LoadJobWriter,CreateIncludeFiles and CreateIncludeDomlFiles methods.
     * @throws LoaderException
     */
    public void generate() throws LoaderException {

        try {
            LoadJobWriter loadJobWriter;
            GenerateSqlForAllVendors generateSqlForAllVendors;
            //Zk added for logging
            this.setLogger();

            if (standardLogger == true) {
                this.logger.write("normal", "Default (Standard) Logger is load.");
                this.logger.write("normal", "OctopusGenerator started.");
            }
            if (generatorParameters.getSqlForAllVendors() == true) {

                generateSqlForAllVendors = new GenerateSqlForAllVendors(generatorParameters);

            } else {
                if (generatorParameters.getDomlPath() == null || generatorParameters.getDomlPath().equalsIgnoreCase("")) {
                    CreateIncludeFiles createIncludeFiles = new CreateIncludeFiles(generatorParameters);
                } else {
                    CreateIncludeDomlFiles createIncludeDomlFiles;
                    createIncludeDomlFiles = new CreateIncludeDomlFiles(generatorParameters);
                }
                if (generatorParameters.getGenerateXml().equalsIgnoreCase("true"))
                    loadJobWriter = new LoadJobWriter(generatorParameters);
            }
            if (this.standardLogger == true) {
                this.logger.write("normal", "Duration: " + this.timeCounter.getTotalTime());
                this.logger.write("normal", "OctopusGenerator finished.");
            }
        } catch (Exception e) {
            String msg = "Exception in method generate() in LoaderGenerator class!";
            LoaderException le = new LoaderException(msg + "\n" + e.getMessage(), (Throwable) e);
            if (standardLogger == true) {
                this.logger.write("full", "Exception in method generate() in LoaderGenerator class!" + "\n" + le.getStackTraceAsString());
            }
            throw le;

        }

    }
   
}