
/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.generator;


import org.webdocwf.util.loader.LoaderException;
import org.webdocwf.util.loader.logging.Logger;
import org.webdocwf.util.loader.logging.StandardLogger;

/**
 *
 * MappingJavaType class mapping the jdbc type of data in to java type of data.
 * @author Radoslav Dutina
 * @version 1.0
 */
public class MappingJavaType {

    private String javaType = null;
    private String jdbcType = null;
		private Logger logger;
    /**
     * Construct object MappingJavaType with associated parameters.
     * @param sqlDataType is the type which you wont to map.
     * @param generatorParameters represents the references to InputParameter object.
     * @throws LoaderException
     */
    public MappingJavaType(InputParameters generatorParameters, String sqlDataType) throws LoaderException {
				setLogger();
				this.logger.write("full", "MappingJavaType is started.");
        try {
            SearchXmlFile searchXmlFileTarget = new SearchXmlFile("absolute", 
            																											 generatorParameters.getPathToTargetConf(), 
            																											 generatorParameters.getConfJarStructure());
            //for cource conf file.
            jdbcType = searchXmlFileTarget.getJDBCFromSQLType(sqlDataType);

            SearchXmlFile searchXmlFile = new SearchXmlFile("relative", 
																									           generatorParameters.getPath(), 
																									           generatorParameters.getConfJarStructure());

            setJavaType(searchXmlFile.getJAVAType(jdbcType));

        } catch (Exception e) {
            String msg = "Exception in Class MappingJavaType:";
						LoaderException le = new LoaderException(msg + "\n" + e.getMessage(), (Throwable) e);
						this.logger.write("full", "Exception in Class MappingJavaType."+"\n"+le.getStackTraceAsString());
            throw le;
        }
		this.logger.write("full", "MappingJavaType is finished.");
    }

    /**
     * This method sets the value of javaType parameter.
     * @param java_Type is the value of parameter.
     */
    public void setJavaType(String java_Type) {
        javaType = java_Type;
    }

    /**
     * This method reads value of javaType parameter.
     * @return value of parameter.
     */
    public String getJavaType() {
        return this.javaType;
    }
	/**
		  * This method will set logger object
		  * @param logger
		  */
		 private void setLogger() {
			 this.logger = StandardLogger.getCentralLogger();
		 }
}