
/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.generator;


import org.webdocwf.util.loader.LoaderException;
import org.webdocwf.util.loader.logging.Logger;
import org.webdocwf.util.loader.logging.StandardLogger;

/**
 *
 * MappingTypeData class mapping the type of data, which depends on the type of database.
 * @author Radoslav Dutina
 * @version 1.0
 */
public class MappingTypeData {

    private String jdbcType = null;
    private String sqlType = null;
    private String msg = null;
    private Logger logger;

    /**
     * Construct object MappingTypeData with associated parameters.
     * @param dataType is the type which you wont to map.
     * @param generatorParameters represents the references to InputParameter object.
     * @throws LoaderException
     */

    public MappingTypeData(String dataType, InputParameters generatorParameters) throws LoaderException {
				setLogger();
				this.logger.write("full", "MappingTypeData is started.");
        SearchXmlFile searchXmlFileSource = new SearchXmlFile("absolute", generatorParameters.getPathToSourceConf(), generatorParameters.getConfJarStructure());
        //for cource conf file.
        jdbcType = searchXmlFileSource.getJDBCFromSQLType(dataType);

        SearchXmlFile searchXmlFileTarget = new SearchXmlFile("absolute", generatorParameters.getPathToTargetConf(), generatorParameters.getConfJarStructure());
        //for target conf file.
        if (!searchXmlFileTarget.getSQLFromJDBCType(jdbcType).equalsIgnoreCase("none")) {
            setSQLType(searchXmlFileTarget.getSQLFromJDBCType(jdbcType));
        } else {
            msg = "You must adjust " + generatorParameters.getPathToTargetConf() + " file." + " This type of data (" + jdbcType + ") from source database don't have mapping in target database!";
						LoaderException le = new LoaderException("Exception:", new Exception(msg));
						this.logger.write("full", "Exception in class MappingJavaData."+"\n"+le.getStackTraceAsString());
            throw le;
        }
				this.logger.write("full", "dataType is " + dataType);
				this.logger.write("full", "jdbcType is " + jdbcType);
				this.logger.write("full", "SQLType is " + searchXmlFileTarget.getSQLFromJDBCType(jdbcType));
				this.logger.write("full", "MappingTypeData is finished.");
    }

    /**
     * This method read the value of sqlType parameter.
     * @return value of parameter.
     */
    public String getSQLType() {
        return this.sqlType;
    }

    /**
     * This method sets the value of sqlType parameter.
     * @param sql_Type is the value of parameter.
     */
    public void setSQLType(String sql_Type) {
        sqlType = sql_Type;
    }
    /**
    		  * This method will set logger object
    		  * @param logger
    		  */
    private void setLogger() {
        this.logger = StandardLogger.getCentralLogger();
    }
}
