
/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.generator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import java.io.File;

import java.io.*;
import java.util.*;

import org.enhydra.xml.*;
import org.webdocwf.util.loader.LoaderException;
import org.webdocwf.util.loader.logging.Logger;
import org.webdocwf.util.loader.logging.StandardLogger;

/**
 *
 * SearchXmlFile class search the named xml document.
 * @author Radoslav Dutina
 * @version 1.0
 */
public class SearchXmlFile {

    private SearchElement searchDocument;
    private Document document;
    private String fileName;
    private String jdbcType = null;
    private String sqlType = null;
    private String javaType = null;
    private String oidDbType = null;
    private String versionDbType = null;
    private String oidDbColumnName = null;
    private String versionDbColumnName = null;
    private InputStream is = null;
    private File file = null;
    private String octopusPath = null;
    private String msg = null;
    private Vector allVendors = new Vector();
    private Logger logger = null;
    /**
     * Construct object SearchXmlFile with associated parameters.
     * @param typePath defines the type of the path. Path can be relative, absolute and jar.
     * If the parameter has value jar, then the xml file is placed in jar file.
     * @param path is the path to named xml file.
     * @throws LoaderException
     */
    public SearchXmlFile(String typePath, String path, String confJarStructure) throws LoaderException {
				
				
				setLogger();
//				if (this.logger!=null){
//        	this.logger.write("full", "SearchXmlFile is started.");
//				}
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        try {
            octopusPath = System.getProperty("OCTOPUS_HOME");
            if (octopusPath == null) {
                if (confJarStructure != null && !confJarStructure.equalsIgnoreCase("")) {
                    is = getClass().getClassLoader().getResourceAsStream(confJarStructure + "/" + path);
                } else {
                    is = getClass().getClassLoader().getResourceAsStream("xml/conf/" + path);
                }
            } else {
                //make the diference if the path is apsolute or relative!
                if (typePath.equalsIgnoreCase("relative")) {
                    fileName = octopusPath + "/conf/" + path;
                } else if (typePath.equalsIgnoreCase("absolute")) {
                    fileName = path;
                }
            }
            if (fileName != null) {
                file = new File(fileName);
                if (!file.exists()) {
                    msg = "This " + fileName + " file doesn't exists!!";
                    LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
										if (this.logger!=null){
											this.logger.write("full", "Exception in SearchXmlFile." + le.getStackTraceAsString());
										}
                    throw le;
                }
            }

            DocumentBuilder builder = factory.newDocumentBuilder();

            try {
                if (octopusPath != null)
                    document = builder.parse(file);
                else
                    document = builder.parse(is);

            } catch (Exception e) {
                if (fileName != null)
                    msg = "Exception has occurred when application try to parse " + fileName + " file." + "\n";
                else
                    msg = "Exception has occurred when application try to parse " + path + " file placed in jar file." + "\n";
								LoaderException le = new LoaderException("Exception:" + msg + e.getMessage(), (Throwable) e);
								if (this.logger!=null){
									this.logger.write("full", "Exception in SearchXmlFile." + msg  +"\n" + le.getStackTraceAsString());
								}
                
                throw le;
            }

        } catch (Exception e) {
            String msg = "Exception in SearchXmlFile.";
						LoaderException le = new LoaderException(msg + "\n" + e.getMessage(), (Throwable) e);
						if (this.logger!=null){
            	this.logger.write("full", "Exception in SearchXmlFile." + "\n" + le.getStackTraceAsString());
						}
            throw le;
        }
//				if (this.logger!=null){
//        	this.logger.write("full", "SearchXmlFile is finished.");
//				}

    }

    /**
     * This method read value of allVendors parameter
     * @return value of parameter
     */
    public Vector getAllVendors() {
		setLogger();
//		if (this.logger!=null){
//        this.logger.write("full", "\tgetAllVendors is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList databaseVendors = searchDocument.getSubElementsByTagName("Database/Vendor");
        if (databaseVendors.getLength() != 0) {
            for (int i = 0; i < databaseVendors.getLength(); i++) {
                String vendorName = ((Element) databaseVendors.item(i)).getAttribute("name");
                String vendorPath = getPathToConf(vendorName);
                allVendors.add(vendorName);
            }
        }
//		if (this.logger!=null){
//        this.logger.write("full", "\tgetAllVendors is finished.");
//		}
        return allVendors;
    }

    /**
     * This method read value of allVendorsPath parameter
     * @return value of parameter
     */
    public Vector getAllVendorsPath() {
		setLogger();
//		if (this.logger!=null){
//        this.logger.write("full", "\tgetAllVendorsPath is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        Vector allVendorsPath = new Vector();
        NodeList databaseVendors = searchDocument.getSubElementsByTagName("Database/Vendor");
        if (databaseVendors.getLength() != 0) {
            for (int i = 0; i < databaseVendors.getLength(); i++) {
                String vendorName = ((Element) databaseVendors.item(i)).getAttribute("name");
                String vendorPath = getPathToConf(vendorName);
                allVendorsPath.add(vendorPath);
            }
        }
//		if (this.logger!=null){
//        this.logger.write("full", "\tgetAllVendorsPath is finished.");
//		}
        return allVendorsPath;
    }

    /**
     * This method read value of drivers parameter
     * @return value of parameter
     */
    public Vector getAllDriversForVendor() {
		setLogger();
//		if (this.logger!=null){
//        this.logger.write("full", "\tgetAllDriversForVendor is started.");
//		}
        Vector allDriversForVendor = new Vector();
        Vector drivers = new Vector();
        Vector metaDataDriver = new Vector();
        String sourceVendor = "false";
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList driverTag = searchDocument.getSubElementsByTagName("Driver");

        for (int i = 0; i < driverTag.getLength(); i++) {
            String driverName = ((Element) driverTag.item(i)).getAttribute("name");
            allDriversForVendor.add(driverName);

            NodeList metaData = ((Element) driverTag.item(i)).getElementsByTagName("MetaData");
            String supportMetaData = ((Element) metaData.item(0)).getAttribute("value");

            if (supportMetaData.equalsIgnoreCase("true")) {
                metaDataDriver.add(driverName);
                sourceVendor = "true";
            }
        }
        drivers.add(allDriversForVendor);
        drivers.add(metaDataDriver);
        drivers.add(sourceVendor);
//		if (this.logger!=null){
//        this.logger.write("full", "\tgetAllDriversForVendor is finished.");
//		}
        return drivers;

    }

    /**
     * This method search the named xml document, and read the value of pathToConf parameter
     * which represents the path of the conf file, for the named database.
     * @param database_Type is the type of the database.
     * @return value of paramter.
     */
    public String getPathToConf(String database_Type) {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetPathToConf is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        //ZK 18.6 2004 commented this. It was problem with reading xml file from jar.
        //if(octopusPath!=null){
        NodeList databaseType = searchDocument.getSubElementsByCondition("Database/Vendor@name=" + database_Type);
        if (databaseType.getLength() != 0) {
            String pathToConf = databaseType.item(0).getFirstChild().getNodeValue();
//			if (this.logger!=null){
//						this.logger.write("full", "\tgetPathToConf is finished.");
//			}
            return pathToConf;
        } else {
//			if (this.logger!=null){
//						this.logger.write("full", "\tgetPathToConf is finished.");
//			}
            return null;
        }
				
    }

    /**
     * This method search the named xml document, and read the value of driverClassName parameter,
     * which represents driver class for the named driver.
     * @param driverName is the name of the driver.
     * @param jdbcParameters is reference to JdbcParameters object.
     * @param generatorParameters is object of InputParameters class
     * @throws LoaderException
     */
    public void getClassName(String driverName, JdbcParameters jdbcParameters, InputParameters generatorParameters) throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetClassName is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList driverTag = null;
        if (driverName.equalsIgnoreCase("")) {
            driverTag = searchDocument.getSubElementsByTagName("Driver");
        } else {
            driverTag = searchDocument.getSubElementsByCondition("Driver@name=" + driverName);
        }
        if (driverTag.getLength() != 0) {
            NodeList classNameTag = ((Element) driverTag.item(0)).getElementsByTagName("ClassName");
            if (classNameTag.getLength() == 0) {
                if (fileName != null)
                    msg = "In " + fileName + " file for this database, you don't have ClassName tag!";
                else
                    msg = "In " + octopusPath + " ( in .jar) file for this database, you don't have ClassName tag!";
										
										LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
										if (this.logger!=null){
														this.logger.write("full", "Exception:"+"\n"+le.getStackTraceAsString());
										}
                
                throw le;
            }
            jdbcParameters.setDriverClassName(((Element) classNameTag.item(0)).getAttribute("value"));

            NodeList connectionTag = ((Element) driverTag.item(0)).getElementsByTagName("Connection");
            if (connectionTag.getLength() == 0) {
                if (fileName != null)
                    msg = "In " + fileName + " file for this database, you don't have Connection tag!";
                else
                    msg = "In " + octopusPath + " ( in .jar) file for this database, you don't have Connection tag!";
								LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
								if (this.logger!=null){
										this.logger.write("full", "Exception:"+"\n"+le.getStackTraceAsString());
								}
                throw le;
            }
            jdbcParameters.setConnection(((Element) connectionTag.item(0)).getAttribute("value"));
            //      NodeList alterTableTag= ((Element)driverTag.item(0)).getElementsByTagName("AlterTablePrimaryKey");
            //      if(alterTableTag.getLength()==0){
            //        if(fileName!=null)
            //          msg= "In "+fileName+" file for this database, you don't have AlterTablePrimaryKey tag!";
            //        else
            //          msg= "In "+octopusPath+" ( in .jar) file for this database, you don't have AlterTablePrimaryKey tag!";
            //        LoaderException le = new LoaderException(msg);
            //        throw le;
            //      }
            //      jdbcParameters.setAlterTablePrimaryKey(((Element)alterTableTag.item(0)).getAttribute("value"));
            //      generatorParameters.setAlterTablePrimaryKey(((Element)alterTableTag.item(0)).getAttribute("value"));
        } else {
            if (fileName != null)
                msg = "In " + fileName + " file for this database, specified driver name (" + driverName + ") is not valid!";
            else
            msg = "In " + octopusPath + " ( in .jar) file for this database, specified driver name (" + driverName + ") is not valid!";
						LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
						if (this.logger!=null){
									this.logger.write("full", "Exception:"+"\n"+le.getStackTraceAsString());
						}
            
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetClassName is finished.");
//		}
    }

    public Vector getFileSystemDatabase(String driverName) throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetFileSystemDatabase is started.");
//		}
        Vector driverProperties = new Vector();
        String strFileSystemDatabase = "";
        String strConnection = "";

        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList driverTag = null;
        if (driverName.equalsIgnoreCase("")) {
            driverTag = searchDocument.getSubElementsByTagName("Driver");
        } else {
            driverTag = searchDocument.getSubElementsByCondition("Driver@name=" + driverName);
        }
        if (driverTag.getLength() != 0) {
            //FileSystemDatabase tag
            NodeList fileSystemDatabaseTag = ((Element) driverTag.item(0)).getElementsByTagName("FileSystemDatabase");
            if (fileSystemDatabaseTag.getLength() == 0) {
                if (fileName != null)
                    msg = "In " + fileName + " file for this database, you don't have FileSystemDatabase tag!";
                else
                    msg = "In " + octopusPath + " ( in .jar) file for this database, you don't have FileSystemDatabase tag!";
                LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
								if (this.logger!=null){
									this.logger.write("full", "Exception:"+le.getStackTraceAsString());
								}
                throw le;
            } else {
                strFileSystemDatabase = ((Element) fileSystemDatabaseTag.item(0)).getAttribute("value");
            }

            //Connection
            NodeList connectionTag = ((Element) driverTag.item(0)).getElementsByTagName("Connection");
            if (connectionTag.getLength() == 0) {
                if (fileName != null)
                    msg = "In " + fileName + " file for this database, you don't have Connection tag!";
                else
                    msg = "In " + octopusPath + " ( in .jar) file for this database, you don't have Connection tag!";
                LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
								if (this.logger!=null){
									this.logger.write("full", "Exception:"+le.getStackTraceAsString());
								}
            
                throw le;
            } else {
                strConnection = ((Element) connectionTag.item(0)).getAttribute("value");
            }

        } else {
            if (fileName != null)
                msg = "In " + fileName + " file for this database, specified driver name (" + driverName + ") is not valid!";
            else
                msg = "In " + octopusPath + " ( in .jar) file for this database, specified driver name (" + driverName + ") is not valid!";
						LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Exception:"+le.getStackTraceAsString());
						}
            throw le;
        }
        driverProperties.add(strFileSystemDatabase);
        driverProperties.add(strConnection);
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetFileSystemDatabase is finished.");
//		}
        return driverProperties;
    }

    public String getMaxConstraintLength(String driverName) throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetMaxConstraintLength is started.");
//		}
        String maxConstraintLength = "-1";
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList driverTag = null;
        if (driverName.equalsIgnoreCase("")) {
            driverTag = searchDocument.getSubElementsByTagName("Driver");
        } else {
            driverTag = searchDocument.getSubElementsByCondition("Driver@name=" + driverName);
        }
        if (driverTag.getLength() != 0) {
            try {
                NodeList maxConstraintLengthTag = ((Element) driverTag.item(0)).getElementsByTagName("MaxConstraintLength");

                maxConstraintLength = ((Element) maxConstraintLengthTag.item(0)).getAttribute("value");

            } catch (Exception ex) {
                if (fileName != null)
                    msg = "In " + fileName + " file for this database, you don't have maxConstraintLenght tag!";
                else
                    msg = "In " + octopusPath + " ( in .jar) file for this database, you don't have maxConstraintLenght tag!";
								LoaderException le = new LoaderException("Exception:", new Exception(msg));
								if (this.logger!=null){
									this.logger.write("full", "Exception:"+"\n"+le.getStackTraceAsString());
								}
                
                throw le;
            }

        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetMaxConstraintLength is finished.");
//		}
        return maxConstraintLength;
    }

    /**
     * This method set value of alterTablePrimaryKey parameter
     * @param driverName is name of the driver
     * @return value of parameter
     * @throws LoaderException
     */
    public String getAlterTablePrimaryKey(String driverName) throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetAlterTablePrimaryKey is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList driverTag = null;
        if (driverName.equalsIgnoreCase("")) {
            driverTag = searchDocument.getSubElementsByTagName("Driver");
        } else {
            driverTag = searchDocument.getSubElementsByCondition("Driver@name=" + driverName);
        }
        if (driverTag.getLength() != 0) {
            NodeList alterTableTag = ((Element) driverTag.item(0)).getElementsByTagName("AlterTablePrimaryKey");
            if (alterTableTag.getLength() == 0) {
                if (fileName != null)
                    msg = "In " + fileName + " file for this database, you don't have AlterTablePrimaryKey tag!";
                else
                    msg = "In " + octopusPath + " ( in .jar) file for this database, you don't have AlterTablePrimaryKey tag!";
								LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
								if (this.logger!=null){
									this.logger.write("full", "Exception:"+le.getStackTraceAsString());
								}
                throw le;
            } else {
                String ret = ((Element) alterTableTag.item(0)).getAttribute("value");
//				if (this.logger!=null){
//								this.logger.write("full", "\tgetAlterTablePrimaryKey is finished.");
//				}
                return ret;
            }
			
        } else {
            if (fileName != null)
                msg = "In " + fileName + " file for this database, specified driver name (" + driverName + ") is not valid!";
            else
                msg = "In " + octopusPath + " ( in .jar) file for this database, specified driver name (" + driverName + ") is not valid!";
            LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Exception:"+le.getStackTraceAsString());
						}
            throw le;
        }
		
    }

    /**
     * This method search the named xml document, and read the value of jdbcType parameter,
     * which represents the jdbc type of data of the named  sql type of data.
     * @param sql_Type is the sql type of data.
     * @return jdbc type of data.
     * @throws LoaderException
     */
    public String getJDBCFromSQLType(String sql_Type) throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetJDBCFromSQLType is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        //TODO zoran added this line
        sql_Type = Utils.replaceAll(sql_Type, " ", "_");
        NodeList SQLTypeTag = searchDocument.getSubElementsByTagName("SQLType/" + sql_Type.toUpperCase());
        if (SQLTypeTag.getLength() != 0) {
            jdbcType = SQLTypeTag.item(0).getFirstChild().getNodeValue();
        } else {
            String msg = "This type of data (" + sql_Type + ") doesn't exists in the source table conf file!";
						LoaderException le = new LoaderException("Exception:", new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Exception:"+le.getStackTraceAsString());
						}
            
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetJDBCFromSQLType is finished.");
//		}
        return jdbcType;
    }

    //ZK added this method 5.5 2004
    /**
    	 * This method search the named xml document, and read the value of hasSize parameter
    	 * @return hasSize.
    	 * @throws LoaderException
    	 */
    public Hashtable getHasSize() throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetHasSize is started.");
//		}
        Hashtable resHashtable = new Hashtable();
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList SQLTypesTagX = searchDocument.getSubElementsByTagName("SQLType");
        NodeList SQLTypesTag = SQLTypesTagX.item(0).getChildNodes();
        if (SQLTypesTag.getLength() != 0) {
            for (int i = 0; i < SQLTypesTag.getLength(); i++) {
                if (!(SQLTypesTag.item(i) instanceof org.enhydra.xml.SearchElement))
                    continue;
                String hasSize = ((Element) SQLTypesTag.item(i)).getAttribute("hasSize");
                if (hasSize == null || hasSize.equalsIgnoreCase(""))
                    hasSize = "false";
                resHashtable.put(((Element) SQLTypesTag.item(i)).getNodeName(), hasSize);
            }
        } else {
						String msg = "Exception in method getHasSize in class SearchXmlFile.";
            LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Error:"+le.getStackTraceAsString());
						}
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetHasSize is finished.");
//		}
        return resHashtable;
    }
    /**
      * This method search the named xml document, and read the value of isDecimal parameter
    	* @return hasSize.
    	* @throws LoaderException
    	*/
    public Hashtable getIsDecimal() throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetIsDecimal is started.");
//		}
        Hashtable resHashtable = new Hashtable();
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList SQLTypesTagX = searchDocument.getSubElementsByTagName("SQLType");
        NodeList SQLTypesTag = SQLTypesTagX.item(0).getChildNodes();
        if (SQLTypesTag.getLength() != 0) {
            for (int i = 0; i < SQLTypesTag.getLength(); i++) {
                if (!(SQLTypesTag.item(i) instanceof org.enhydra.xml.SearchElement))
                    continue;
                String hasSize = ((Element) SQLTypesTag.item(i)).getAttribute("isDecimal");
                if (hasSize == null || hasSize.equalsIgnoreCase(""))
                    hasSize = "false";
                resHashtable.put(((Element) SQLTypesTag.item(i)).getNodeName(), hasSize);
            }
        } else {
						String msg = "Exception in method getIsDecimal in class SearchXmlFile.";
            LoaderException le = new LoaderException("Exception:", new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Error:"+le.getStackTraceAsString());
						}
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetIsDecimal is finished.");
//		}
        return resHashtable;
    }
    /**
     * This method search the named xml document, and read the value of sqlType parameter,
     * which represents the sql type of data of the named jdbc type of data.
     * @param jdbc_Type is the jdbc type of data.
     * @return the sql type of data.
     * @throws LoaderException
     */
    public String getSQLFromJDBCType(String jdbc_Type) throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetSQLFromJDBCType is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList JDBCTypeTag = searchDocument.getSubElementsByTagName("JDBCType/" + jdbc_Type.toUpperCase());
        if (JDBCTypeTag.getLength() != 0) {
            sqlType = JDBCTypeTag.item(0).getFirstChild().getNodeValue();
        } else {
            String msg = "This type of data (" + jdbc_Type + ") doesn't exists on the target table conf file!";
            LoaderException le = new LoaderException("Exception:", new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Error:"+le.getStackTraceAsString());
						}
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetSQLFromJDBCType is finished.");
//		}
        return sqlType;
    }

    /**
     * This method search the named xml document, and read the value of javaType parameter,
     * which represents the java type of data of the named jdbc type of data.
     * @param java_Type is the jdbc type of data.
     * @return java type of data.
     * @throws LoaderException
     */
    public String getJAVAType(String java_Type) throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetJAVAType is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        java_Type = Utils.replaceAll(java_Type, " ", "_");
        NodeList JAVATypeTag = searchDocument.getSubElementsByTagName("JAVAType/" + java_Type.toUpperCase());
        if (JAVATypeTag.getLength() != 0) {
            javaType = JAVATypeTag.item(0).getFirstChild().getNodeValue();
        } else {
            String msg = "This type of data (" + java_Type + ") doesn't exists in Java Type class!";
            LoaderException le = new LoaderException("Exception:", new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Error:"+le.getStackTraceAsString());
						}
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetJAVAType is finished.");
//		}
        return javaType;
    }

    /**
     * This method search the named xml document, and read the value of oidDbType parameter, which
     * represents the oid type for the named database.
     * @return oid type.
     * @throws LoaderException
     */
    public String getOidDB() throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetOidDB is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList OidDb = searchDocument.getSubElementsByTagName("OidDbType");
        if (OidDb.getLength() != 0) {
            this.oidDbType = OidDb.item(0).getFirstChild().getNodeValue();
        } else {
            String msg = "You must cofigure your conf file for this Oid type!";
            LoaderException le = new LoaderException("Exception:", new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Error:"+le.getStackTraceAsString());
						}
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetOidDB is finished.");
//		}
        return oidDbType;
    }

    /**
     * This method search the named xml document, and read the value of versionDbType parameter, which
     * represents the version type of the named database.
     * @return version type.
     * @throws LoaderException
     */
    public String getVersionDb() throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetVersionDb is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList VersionDb = searchDocument.getSubElementsByTagName("VersionDbType");
        if (VersionDb.getLength() != 0) {
            this.versionDbType = VersionDb.item(0).getFirstChild().getNodeValue();
        } else {
            String msg = "You must cofigure yoor conf file for this Version type!";
            LoaderException le = new LoaderException("Exception:", new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Error:"+le.getStackTraceAsString());
						}
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetVersionDb is finished.");
//		}
        return versionDbType;
    }

    /**
     * This method search the named xml document, and read the value of oidDbColumnName parameter,
     * which represents the oid name for the named database.
     * @return oid name.
     * @throws LoaderException
     */
    public String getOidDbColumn() throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetOidDbColumn is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList OidDbColumn = searchDocument.getSubElementsByTagName("OidDbColumnName");
        if (OidDbColumn.getLength() != 0) {
            this.oidDbColumnName = OidDbColumn.item(0).getFirstChild().getNodeValue();
        } else {
            String msg = "You must cofigure yoor conf file for this Oid name!";
            LoaderException le = new LoaderException("Exception:", new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Error:"+le.getStackTraceAsString());
						}
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetOidDbColumn is finished.");
//		}
        return oidDbColumnName;
    }

    /**
     * This method search the named xml document, and read the value of versionDbColumnName parameter,
     * which represents the version name of the named database.
     * @return version name.
     * @throws LoaderException
     */
    public String getVersionDbColumn() throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetVersionDbColumn is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList VersionDbColumn = searchDocument.getSubElementsByTagName("VersionDbColumnName");
        if (VersionDbColumn.getLength() != 0) {
            this.versionDbColumnName = VersionDbColumn.item(0).getFirstChild().getNodeValue();
        } else {
            String msg = "You must cofigure yoor conf file for this Version name!";
            LoaderException le = new LoaderException("Exception:", new Exception(msg));
						if (this.logger!=null){
							this.logger.write("full", "Error:"+le.getStackTraceAsString());
						}
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetVersionDbColumn is finished.");
//		}
        return versionDbColumnName;
    }

    /**
     * This method search the named xml document, and read the value of name (Driver) parameter,
     * which represents the driver name.
     * @return value of parameter.
     */
    public String getDriverName() {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetDriverName is started.");
//		}
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList driverTag = searchDocument.getSubElementsByTagName("Driver");
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetDriverName is finished.");
//		}
        return ((Element) driverTag.item(0)).getAttribute("name");

    }

    public String getExcludedTables() throws LoaderException {
		setLogger();
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetExcludedTables is started.");
//		}
        String ret = "";
        searchDocument = (SearchElement) SearchElement.newInstance(document);
        NodeList excludeTableTag = searchDocument.getSubElementsByTagName("ExcludeTables");
        if (excludeTableTag.getLength() != 0) {
            if (excludeTableTag.item(0).getFirstChild() != null)
                ret = excludeTableTag.item(0).getFirstChild().getNodeValue();
            else
                ret = "";
        } else {
            String msg = "ExcludeTables tag doesn't exist in source conf file!";
            LoaderException le = new LoaderException("Exception:", (Throwable) new Exception(msg));
						if (this.logger!=null){
								this.logger.write("full", "Error:"+le.getStackTraceAsString());
						}
            throw le;
        }
//		if (this.logger!=null){
//				this.logger.write("full", "\tgetExcludedTables is finished.");
//		}
        return ret;
    }
    /**
    	  * This method will set logger object
    	  * @param logger
    	  */
    private void setLogger() {
        this.logger = StandardLogger.getCentralLogger();
    }
}
