/*
     LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.
    Copyright (C) 2003  Together
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.webdocwf.util.loader.generator;

import java.util.*;

import org.webdocwf.util.loader.logging.Logger;
import org.webdocwf.util.loader.logging.StandardLogger;

/**
 *
 * SqlStatements class creates the sql statements for building target database.
 * @author Radoslav Dutina
 * @version 1.0
 */
public class SqlStatements {

    private List list = new ArrayList();
		private List hsqlPKlist = new ArrayList();
    //  private String[] uncompatibileTypes = {
    //      "int", "integer", "datetime", "smalldatetime",
    //      "money", "smallmoney", "bigint", "smallint", "timestamp", "tinyint",
    //      "ntext", "bit", "int2", "int4", "int8", "bytea",
    //      "text", "real", "date", "time", "tiny", "long", "double", "identity",
    //      "image","longvarchar","long varchar","nvarchar","ntext",
    //      "binary", "blob", "graphic", "longraw", "long raw", "byte"};
    private String[] targetDBWithoutAlterTable = { "xml", "csv", "excel" };

    private boolean incompatible = false;
    private boolean incompatibleDBTarget = false;
    private boolean incompatibleDBSource = false;
    private Logger logger = null;
    /**
     * Construct object SqlStatements with associated parameters.
     * @param nameSQL is the name of created sql file.
     * @param tableName is name of the table form which we retrieve data.
     * @param importDefinitionAttributes is references to ImportDefinitionAttributes object.
     * @param relationshipsAttributes is references to RelationshipsAttributes object.
     * @param k is the counter, whish are used for restricting same operations.
         * @param generatorParameters represents the references to InputParameter object.
     */

    public SqlStatements(String nameSQL, String tableName, ImportDefinitionAttributes importDefinitionAttributes, RelationshipsAttributes relationshipsAttributes, int k, InputParameters generatorParameters) {
        setLogger();
        this.logger.write("full", "SqlStatements is started.");
        String targetDBType = generatorParameters.getTargetType();
        String sourceDBType = generatorParameters.getSourceType();
        checkDBTypeTarget(targetDBType);
        checkDBTypeSource(sourceDBType);

        if (nameSQL.equalsIgnoreCase("CreateTables")) {
            list.add("Create table" + " " + tableName);
            list.add("(");
            if (generatorParameters.getSourceType().equalsIgnoreCase("csv")) {
                for (int i = 0; i < importDefinitionAttributes.getTagSourceColumnName().length; i++) {
                    if (i == importDefinitionAttributes.getTagSourceColumnName().length - 1)
                        list.add(importDefinitionAttributes.getTagSourceColumnName()[i] + "  " + "VARCHAR" + " (254) " + "    ");
                    else
                        list.add(importDefinitionAttributes.getTagSourceColumnName()[i] + "  " + "VARCHAR" + " (254) " + "    ,");
                }
                list.add(");");
                list.add("");
                list.add("");
                list.add("");
            } else if (generatorParameters.getSourceType().equalsIgnoreCase("access")) {
                String end = "    ,";
                for (int i = 0; i < importDefinitionAttributes.getTagSourceColumnName().length; i++) {
                    if (i == importDefinitionAttributes.getTagSourceColumnName().length - 1)
                        end = "    ";
                    if (importDefinitionAttributes.getTagColumnLenght()[i].equalsIgnoreCase(""))
                        list.add(importDefinitionAttributes.getTagSourceColumnName()[i] + "  " + (importDefinitionAttributes.getTagColumnType()[i]).toUpperCase() + importDefinitionAttributes.getTagAllowNulls()[i] + end);
                    else
                        list.add(importDefinitionAttributes.getTagSourceColumnName()[i] + "  " + (importDefinitionAttributes.getTagColumnType()[i]).toUpperCase() + "(" + importDefinitionAttributes.getTagColumnLenght()[i] + ")  " + importDefinitionAttributes.getTagAllowNulls()[i] + end);
                }
                list.add(");");
                list.add("");
                list.add("");
                list.add("");
            } else {
                String end = "  ";
                for (int i = 0; i < importDefinitionAttributes.getTagSourceColumnName().length; i++) {
                    boolean pk = true;
                    for (int j = 0; j < relationshipsAttributes.getPrimaryKeys().length; j = j + 2) {
                        if (relationshipsAttributes.getPrimaryKeys()[j + 1].equalsIgnoreCase(importDefinitionAttributes.getTagSourceColumnName()[i])) {
                            pk = false;
                            break;
                        }
                    }
                    if (i != importDefinitionAttributes.getTagSourceColumnName().length - 1)
                        end = "    ,";
                    else
                        end = "    ";

                    //          checkType(importDefinitionAttributes.getTagColumnType()[i]);
                    //incompatible  atibile is parameter, which decide if we should put the
                    //length parameter in to sql expression.
                    //          if (!incompatible) {
                    if (generatorParameters.getAlterTablePrimaryKey().equalsIgnoreCase("true")) {
                        list.add(importDefinitionAttributes.getTagSourceColumnName()[i] + "  " + (importDefinitionAttributes.getTagColumnType()[i]).toUpperCase() //+ "("
                        +importDefinitionAttributes.getTagColumnLenght()[i] //+
                        //")  " 
                        +importDefinitionAttributes.getTagAllowNulls()[i] + end);
                    } else {
                        //pk parameter is used to decide if the fild in table is primary key.
                        //If it is, then we put the fild in to expression.
                        if (pk) {
                            //parameter isn't the primary key.
                            list.add(importDefinitionAttributes.getTagSourceColumnName()[i] + "  " + (importDefinitionAttributes.getTagColumnType()[i]).toUpperCase()
                            // + "("
                            +importDefinitionAttributes.getTagColumnLenght()[i] +
                            // ")  " 
                            // +
                            importDefinitionAttributes.getTagAllowNulls()[i] + end);
                        } else {
													//TODO 23.8 2004 ZK added because of hsql problem with primary keys on 2,3,... columns.
													//Change this to support other databases, not only hsql. 
                        		if (generatorParameters.getTargetType().equalsIgnoreCase("HypersonicSQL")){
//															
																hsqlPKlist.add(importDefinitionAttributes.getTagSourceColumnName()[i]);
																list.add(importDefinitionAttributes.getTagSourceColumnName()[i] + "  " + (importDefinitionAttributes.getTagColumnType()[i]).toUpperCase()
															  +importDefinitionAttributes.getTagColumnLenght()[i] 
																+", ");
                        		}else{
                        		//end
                            //parameter is the primary key.
                            list.add(importDefinitionAttributes.getTagSourceColumnName()[i] + "  " + (importDefinitionAttributes.getTagColumnType()[i]).toUpperCase() //+ "("
                            +importDefinitionAttributes.getTagColumnLenght()[i] //+
                            //")  " 
                            +"PRIMARY KEY " + end);
                        		}
                        }
                    }
                    //         }
                    //          else {
                    //            if (generatorParameters.getAlterTablePrimaryKey().equalsIgnoreCase(
                    //                "true")) {
                    //              list.add(importDefinitionAttributes.getTagSourceColumnName()[i] +
                    //                       "  " +
                    //                       (importDefinitionAttributes.getTagColumnType()[i]).
                    //                       toUpperCase() + " "
                    //                       + importDefinitionAttributes.getTagAllowNulls()[i]
                    //                       + end);
                    //            }
                    //            else {
                    //              if (pk) {
                    //                list.add(importDefinitionAttributes.getTagSourceColumnName()[i] +
                    //                         "  " +
                    //                         (importDefinitionAttributes.getTagColumnType()[i]).
                    //                         toUpperCase() + " "
                    //                         + importDefinitionAttributes.getTagAllowNulls()[i]
                    //                         + end);
                    //              }
                    //              else {
                    //                list.add(importDefinitionAttributes.getTagSourceColumnName()[i] +
                    //                         "  " +
                    //                         (importDefinitionAttributes.getTagColumnType()[i]).
                    //                         toUpperCase() + " "
                    //                         + " PRIMARY KEY "
                    //                         + end);
                    //              }
                    //            }
                    //          }
                }
								//TODO 23.8 2004 ZK added because of hsql problem with primary keys on 2,3,... columns.
								//Change this to support other databases, not only hsql. 
								if (generatorParameters.getTargetType().equalsIgnoreCase("HypersonicSQL")){
									String columnNames = "";
									columnNames = columnNames+" (";
									
									for (int i = 0; i < hsqlPKlist.size()-1; i++) {
										columnNames = columnNames + hsqlPKlist.get(i) + ",";
				                        
				          }
								  if (hsqlPKlist.size() >= 2){
								  		columnNames = columnNames + hsqlPKlist.get(hsqlPKlist.size()-1) + ")";
								  }else{
											columnNames = columnNames + hsqlPKlist.get(0) + ")";
								  }
									list.add(", CONSTRAINT" + "  " +importDefinitionAttributes.getTableName()+"_PK" +"  "
									 +"PRIMARY KEY " + columnNames);
								}
								//end
                list.add(");");
                list.add("");
                list.add("");
                list.add("");
            }
        } else if (nameSQL.equalsIgnoreCase("CreatePrimary")) {
            list.add("");
            if (incompatibleDBSource) {
                if (k == 0) {
                    list.add("");
                }
            } else if (incompatibleDBTarget) {
                if (k == 0) {
                    list.add("");
                }
            } else {
                if (relationshipsAttributes.getPrimaryKeys().length != 0) {
                    int primaryCount = relationshipsAttributes.getPrimaryKeys().length;
                    for (int i = 0; i < primaryCount; i = i + 2) {
                        String arrayOfPrimaryNames = relationshipsAttributes.getPrimaryKeys()[i + 1];
                        for (int m = i + 2; m < primaryCount; m = m + 2) {
                            if (relationshipsAttributes.getPrimaryKeys()[i].equalsIgnoreCase(relationshipsAttributes.getPrimaryKeys()[m])) {

                                arrayOfPrimaryNames = arrayOfPrimaryNames + "," + relationshipsAttributes.getPrimaryKeys()[m + 1];
                                i = i + 2;
                            } else
                                break;
                        }

                        list.add("ALTER TABLE " + " " + relationshipsAttributes.getTableName() + " " + "ADD CONSTRAINT " + relationshipsAttributes.getPrimaryKeys()[i] + " PRIMARY KEY" + "(" +
                        //relationshipsAttributes.getPrimaryKeys()[i+1]+") ;");
                        arrayOfPrimaryNames + ") ;");
                    }
                } else {
                    list.add("");
                }
            } //end of main else
        } else if (nameSQL.equalsIgnoreCase("CreateIndex")) {
            list.add("");
            if (incompatibleDBSource) {
                if (k == 0) {
                    list.add("");
                }
            } else if (incompatibleDBTarget) {
                if (k == 0) {
                    list.add("");
                }
            } else {
                if (relationshipsAttributes.getIndexVariables().length != 0) {
                    int indexCount = relationshipsAttributes.getIndexVariables().length;
                    for (int i = 0; i < indexCount; i = i + 3) {
                        String arrayOfIndexNames = relationshipsAttributes.getIndexVariables()[i + 2];
                        for (int m = i + 3; m < indexCount; m = m + 3) {
                            if (relationshipsAttributes.getIndexVariables()[i + 1].equalsIgnoreCase(relationshipsAttributes.getIndexVariables()[m + 1])) {
                                arrayOfIndexNames = arrayOfIndexNames + "," + relationshipsAttributes.getIndexVariables()[m + 2];
                                i = i + 3;
                            } else
                                break;
                        }
                        if (relationshipsAttributes.getIndexVariables()[i].equalsIgnoreCase("0") || relationshipsAttributes.getIndexVariables()[i].equalsIgnoreCase("1")) {
                            //unique mey have the value  0 or 1
                            int j = Integer.parseInt(relationshipsAttributes.getIndexVariables()[i]);
                            String unique;
                            if (j == 1)
                                unique = "";
                            else
                                unique = "UNIQUE";
                            list.add("CREATE " + unique + " INDEX " + relationshipsAttributes.getIndexVariables()[i + 1] +
                            //" ON "+tableName+"("+relationshipsAttributes.getIndexVariables()[i+2]+") ;");
                            " ON " + tableName + "(" + arrayOfIndexNames + ") ;");

                        } else if (relationshipsAttributes.getIndexVariables()[i].equalsIgnoreCase("true") || relationshipsAttributes.getIndexVariables()[i].equalsIgnoreCase("false")) {
                            //unique mey have the value true or false
                            String unique;
                            String j = relationshipsAttributes.getIndexVariables()[i];
                            if (j.equalsIgnoreCase("true"))
                                unique = "";
                            else
                                unique = "UNIQUE";
                            list.add("CREATE " + unique + " INDEX " + relationshipsAttributes.getIndexVariables()[i + 1] +
                            //" ON "+tableName+"("+relationshipsAttributes.getIndexVariables()[i+2]+") ;");
                            " ON " + tableName + "(" + arrayOfIndexNames + ") ;");
                        }
                    }
                } else {
                    //relationshipsAttributes.getIndexVariables().length==0
                    list.add("");
                }
            }
        } else if (nameSQL.equalsIgnoreCase("CreateIntegrity")) {
            list.add("");
            if (incompatibleDBSource) {
                if (k == 0) {
                    list.add("");
                }
            } else if (incompatibleDBTarget) {
                if (k == 0) {
                    list.add("");
                }
            } else {
                //TODO ZK added relationshipsAttributes.getForeignVariables().length >= 5 because problems with c-jdbc
                if (relationshipsAttributes.getForeignVariables().length != 0 && relationshipsAttributes.getForeignVariables().length >= 5) {

                    for (int i = 0; i < relationshipsAttributes.getForeignVariables().length; i = i + 5) {

                        list.add("ALTER TABLE " + relationshipsAttributes.getForeignVariables()[i] + " ADD CONSTRAINT " + relationshipsAttributes.getForeignVariables()[i + 1] + " FOREIGN KEY (" + relationshipsAttributes.getForeignVariables()[i + 2] + ")" + " REFERENCES " + relationshipsAttributes.getForeignVariables()[i + 3] + " (" + relationshipsAttributes.getForeignVariables()[i + 4] + ") ;");
                    }
                } else {
                    list.add("");
                }
            }
        } else if (nameSQL.equalsIgnoreCase("DropTables")) {
            list.add("");
            list.add("DROP TABLE " + tableName + " ;");
            list.add("");
        } else if (nameSQL.equalsIgnoreCase("DropIntegrity")) {
            //TODO ZK added relationshipsAttributes.getForeignVariables().length >= 2 because problems with c-jdbc
            if (relationshipsAttributes.getForeignVariables().length != 0 && relationshipsAttributes.getForeignVariables().length >= 2) {

                for (int i = 0; i < relationshipsAttributes.getForeignVariables().length; i = i + 5) {
                    list.add("");
                    list.add("ALTER TABLE " + relationshipsAttributes.getForeignVariables()[i] + " DROP CONSTRAINT " + relationshipsAttributes.getForeignVariables()[i + 1] + " ;");
                    list.add("");
                }

            } else {
                list.add("");
            }

        }
        this.logger.write("full", "SqlStatements is finished.");
    }

    /**
     * This method sets the value of list patameter.
     * @param create_Stream is the value of parameter.
     */
    public void setCreateStream(String[] create_Stream) {
        list = Arrays.asList(create_Stream);
    }

    /**
     * This method read the value of list patameter.
     * @return value of parameter.
     */
    public String[] getCreateStream() {
        String[] ret = new String[list.size()];
        list.toArray(ret);
        return ret;
    }

    //  /**
    //   * This method check the type of named data type.
    //   * @param type is data type which we wont to check.
    //   * @return true or false.
    //   */
    //  private boolean checkType(String type) {
    //    for (int i = 0; i < uncompatibileTypes.length; i++) {
    //      if (type.toLowerCase().endsWith(uncompatibileTypes[i])) {
    //        this.incompatible = true;
    //        break;
    //      }
    //      else {
    //        this.incompatible = false;
    //      }
    //    }
    //    return incompatible;
    //  }

    private boolean checkDBTypeTarget(String type) {
        for (int i = 0; i < targetDBWithoutAlterTable.length; i++) {
            if (type.equalsIgnoreCase(targetDBWithoutAlterTable[i])) {
                this.incompatibleDBTarget = true;
                break;
            } else {
                this.incompatibleDBTarget = false;
            }
        }
        return incompatibleDBTarget;
    }

    private boolean checkDBTypeSource(String type) {
        for (int i = 0; i < targetDBWithoutAlterTable.length; i++) {
            if (type.equalsIgnoreCase(targetDBWithoutAlterTable[i])) {
                this.incompatibleDBSource = true;
                break;
            } else {
                this.incompatibleDBSource = false;
            }
        }
        return incompatibleDBSource;
    }

    /**
    			* This method will set logger object
    			* @param logger
    			*/
    private void setLogger() {
        this.logger = StandardLogger.getCentralLogger();
    }
}
