
/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.generator;

import org.webdocwf.util.loader.logging.Logger;
import org.webdocwf.util.loader.logging.StandardLogger;

/**
 *
 * TableAttributes class stores the value of table attributes, which are needed for
 * creating ImportDefinition.xml file.
 * @author Radoslav Dutina
 * @version 1.0
 */
public class TableAttributes {

    private String tableName = "Table1";
    private String tableID = "0";
    private String insert = "true";
    private String tableMode = "Query";
    private String oidLogic = "true";
    private Logger logger;

    /**
     * Construct object TableAttributes with associated parameters.
     * @param tableName is name of the table form which we retrieve data.
     * @param valueMode valueMode defines the difference from overwrite and update attribute.
     */
    public TableAttributes(String tableName, String valueMode) {
        setLogger();
        this.logger.write("full", "TableAttributes is started.");
        this.tableName = tableName;
        this.tableID = "0";
        this.insert = "true";
        this.tableMode = "Query";
        if (valueMode.equalsIgnoreCase("Overwrite")) {
            this.oidLogic = "false";
        } else {
            this.oidLogic = "true";
        }
		this.logger.write("full", "TableAttributes is finished.");
    }

    /**
     * This method sets the value of tableName parameter.
     * @param table_Name is the value of parameter.
     */
    public void setTableName(String table_Name) {
        tableName = table_Name;
    }

    /**
     * This method read the value of tableName parameter.
     * @return value of parameter.
     */
    public String getTableName() {
        return tableName;
    }

    /**
     * This method sets the value of tableID parameter.
     * @param table_ID is the value of parameter.
     */
    public void setTableID(String table_ID) {
        tableID = table_ID;
    }

    /**
     * This method read the value of tableID parameter.
     * @return value of parameter.
     */
    public String getTableID() {
        return tableID;
    }

    /**
     * This method sets the value of insert parameter.
     * @param _insert is the value of parameter.
     */
    public void setInsert(String _insert) {
        insert = _insert;
    }

    /**
     * This method read the value of insert parameter.
     * @return value of parameter.
     */
    public String getInsert() {
        return insert;
    }

    /**
     * This method sets the value of tableMode parameter.
     * @param table_Mode is the value of parameter.
     */
    public void setTableMode(String table_Mode) {
        tableMode = table_Mode;
    }

    /**
     * This method read the value of tableMode parameter.
     * @return value of parameter.
     */
    public String getTableMode() {
        return tableMode;
    }

    /**
     * This method sets the value of oidLogic parameter.
     * @param oid_Logic is the value of parameter.
     */
    public void setOidLogic(String oid_Logic) {
        oidLogic = oid_Logic;
    }

    /**
     * This method read the value of oidLogic parameter.
     * @return value of parameter.
     */
    public String getOidLogic() {
        return oidLogic;
    }
    /**
    				  * This method will set logger object
    				  * @param logger
    				  */
    private void setLogger() {
        this.logger = StandardLogger.getCentralLogger();
    }
}