
/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.generator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.webdocwf.util.loader.LoaderException;
import org.webdocwf.util.loader.logging.Logger;
import org.webdocwf.util.loader.logging.StandardLogger;

/**
 *
 * WriteImportDefinition class creates ImportDefinition.xml file.
 * @author Radoslav Dutina
 * @version 1.0
 */
public class WriteImportDefinition {
    private Logger logger;

    /**
     * Construct object WriteImportDefinition with associated parameters.
     * @param document is the org.w3c.dom.Document object, which is created in CreateIncludeFile class.
     * @param root is the root element from document object.
     * @param importDefinitionAttributes is references to ImportDefinitionAttributes object.
     * @param generatorParameters represents the references to InputParameter object.
     * @throws LoaderException
     */

    public WriteImportDefinition(Document document, Element root, ImportDefinitionAttributes importDefinitionAttributes, InputParameters generatorParameters) throws LoaderException {
				setLogger();
				this.logger.write("full", "WriteImportDefinition is started.");
        if (!generatorParameters.getFullMode()) {

            Element childRoot1 = (Element) document.createElement("copyTable");
            root.appendChild(childRoot1);
            childRoot1.setAttribute("name", importDefinitionAttributes.getName());
            childRoot1.setAttribute("sourceTableName", importDefinitionAttributes.getName());
            childRoot1.setAttribute("targetTableName", importDefinitionAttributes.getName());
            //      childRoot1.setAttribute("logMode", importDefinitionAttributes.getLogMode());
            childRoot1.setAttribute("objectIDIncrement", importDefinitionAttributes.getObjectIDIncrementid());
            childRoot1.setAttribute("oidLogic", "false");
            childRoot1.setAttribute("onErrorContinue", "false");

        } else {
            Element childRoot1 = (Element) document.createElement("importDefinition");
            root.appendChild(childRoot1);
            childRoot1.setAttribute("name", importDefinitionAttributes.getName());
            childRoot1.setAttribute("tableName", importDefinitionAttributes.getTableName());
            //childRoot1.setAttribute("objectIDIncrement", importDefinitionAttributes.getObjectIDIncrementid());
            //      childRoot1.setAttribute("logMode", importDefinitionAttributes.getLogMode());

            Element childRoot2 = (Element) document.createElement("valueColumns");
            childRoot1.appendChild(childRoot2);

            for (int i = 0; i < importDefinitionAttributes.getTagSourceColumnName().length; i++) {
                if (generatorParameters.getSourceType().equalsIgnoreCase("csv")) {
                    Element childRoot3 = (Element) document.createElement("valueColumn");
                    childRoot2.appendChild(childRoot3);
                    childRoot3.setAttribute("sourceColumnName", importDefinitionAttributes.getTagSourceColumnName()[i]);
                    childRoot3.setAttribute("targetTableName", importDefinitionAttributes.getTagTargetTableName()[i]);
                    childRoot3.setAttribute("targetColumnName", importDefinitionAttributes.getTagTargetColumnName()[i]);
                    childRoot3.setAttribute("targetTableID", importDefinitionAttributes.getTagTargetTableID()[i]);
                    childRoot3.setAttribute("valueMode", generatorParameters.getValueMode());
                } else {
                    if (!generatorParameters.getValueMode().equalsIgnoreCase("Overwrite")) {
                        if (!importDefinitionAttributes.getTagSourceColumnName()[i].equalsIgnoreCase("oid") && !importDefinitionAttributes.getTagSourceColumnName()[i].equalsIgnoreCase("version")) {
                            boolean fk = true;
                            for (int j = 0; j < RelationshipsAttributes.getForeignVariables().length; j = j + 5) {
                                if (RelationshipsAttributes.getForeignVariables()[j + 2].equalsIgnoreCase(importDefinitionAttributes.getTagSourceColumnName()[i])) {
                                    fk = false;
                                    break;
                                }
                            }
                            if (fk) {
                                Element childRoot3 = (Element) document.createElement("valueColumn");
                                childRoot2.appendChild(childRoot3);
                                childRoot3.setAttribute("sourceColumnName", importDefinitionAttributes.getTagSourceColumnName()[i]);
                                childRoot3.setAttribute("targetTableName", importDefinitionAttributes.getTagTargetTableName()[i]);
                                childRoot3.setAttribute("targetColumnName", importDefinitionAttributes.getTagTargetColumnName()[i]);
                                childRoot3.setAttribute("targetTableID", importDefinitionAttributes.getTagTargetTableID()[i]);
                                childRoot3.setAttribute("valueMode", generatorParameters.getValueMode());

                            }
                        }
                    } else {
//                        if (!importDefinitionAttributes.getTagSourceColumnName()[i].equalsIgnoreCase("version")) {
                            if (!importDefinitionAttributes.getTagColumnType()[i].toLowerCase().endsWith("identity")) {
                                Element childRoot3 = (Element) document.createElement("valueColumn");
                                childRoot2.appendChild(childRoot3);
                                childRoot3.setAttribute("sourceColumnName", importDefinitionAttributes.getTagSourceColumnName()[i]);
                                childRoot3.setAttribute("targetTableName", importDefinitionAttributes.getTagTargetTableName()[i]);
                                childRoot3.setAttribute("targetColumnName", importDefinitionAttributes.getTagTargetColumnName()[i]);
                                childRoot3.setAttribute("targetTableID", importDefinitionAttributes.getTagTargetTableID()[i]);
                                childRoot3.setAttribute("valueMode", generatorParameters.getValueMode());
                            }
//                        } else {
//                            Element childRoot5 = (Element) document.createElement("constantColumns");
//                            childRoot1.appendChild(childRoot5);
//                            Element childRoot3 = (Element) document.createElement("constantColumn");
//                            childRoot5.appendChild(childRoot3);
//                            childRoot3.setAttribute("constantValue", "0");
//                            childRoot3.setAttribute("targetTableName", importDefinitionAttributes.getTagTargetTableName()[i]);
//                            childRoot3.setAttribute("targetColumnName", "version");
//                            childRoot3.setAttribute("targetTableID", "0");
//                            childRoot3.setAttribute("valueMode", generatorParameters.getValueMode());
//                        }
                    }
                }
            }

            if (!generatorParameters.getSourceType().equalsIgnoreCase("csv")) {
                int j = 0;
                if (generatorParameters.getValueMode().equalsIgnoreCase("Overwrite")) {
                    //
                } else {
                    //valueMode=Overwrite
                    if (RelationshipsAttributes.getForeignVariables().length != 0) {
                        Element childRoot100 = (Element) document.createElement("relationColumns");
                        childRoot1.appendChild(childRoot100);
                        for (int i = 0; i < RelationshipsAttributes.getForeignVariables().length; i = i + 5) {
                            Element childRoot101 = (Element) document.createElement("relationColumn");
                            childRoot100.appendChild(childRoot101);
                            childRoot101.setAttribute("relationSourceTableName", RelationshipsAttributes.getForeignVariables()[i + 3]);
                            childRoot101.setAttribute("relationTargetTableName", importDefinitionAttributes.getTableName());
                            childRoot101.setAttribute("relationTargetColumnName", RelationshipsAttributes.getForeignVariables()[i + 2]);

                            if (i > 5 && RelationshipsAttributes.getForeignVariables()[i + 3].equalsIgnoreCase(RelationshipsAttributes.getForeignVariables()[(i + 3) - 5])) {
                                j = j + 1;
                            } else {
                                j = 0;
                            }
                            childRoot101.setAttribute("relationSourceTableID", "" + j);
                            childRoot101.setAttribute("relationTargetTableID", "0");
                            childRoot101.setAttribute("relationMode", generatorParameters.getValueMode());
                        }
                    }
                }

                int k = 0;
                int l = 0;
                if (RelationshipsAttributes.getForeignVariables().length != 0) {
                    Element childRoot200 = (Element) document.createElement("tables");
                    childRoot1.appendChild(childRoot200);
                    for (int i = 0; i < RelationshipsAttributes.getForeignVariables().length; i = i + 5) {
                        TableAttributes tableAttributes = new TableAttributes(importDefinitionAttributes.getTableName(), generatorParameters.getValueMode());
                        if (l == 0) {
                            //in the first loop
                            Element childRoot201 = (Element) document.createElement("table");
                            childRoot200.appendChild(childRoot201);
                            childRoot201.setAttribute("tableName", tableAttributes.getTableName());
                            childRoot201.setAttribute("tableID", "" + k);
                            childRoot201.setAttribute("insert", tableAttributes.getInsert());
                            childRoot201.setAttribute("tableMode", tableAttributes.getTableMode());
                            childRoot201.setAttribute("oidLogic", tableAttributes.getOidLogic());
                            l++;
                            k++;
                        }
                        if (generatorParameters.getValueMode().equalsIgnoreCase("Overwrite")) {
                            //
                        } else {
                            //valueMode=Overwrite
                            if (RelationshipsAttributes.getForeignVariables()[i + 3].equalsIgnoreCase(importDefinitionAttributes.getTableName())) {
                                //k=k+1
                                Element childRoot202 = (Element) document.createElement("table");
                                childRoot200.appendChild(childRoot202);
                                childRoot202.setAttribute("tableName", tableAttributes.getTableName());
                                childRoot202.setAttribute("tableID", "" + k);
                                childRoot202.setAttribute("insert", "false");
                                childRoot202.setAttribute("tableMode", tableAttributes.getTableMode());
                                childRoot202.setAttribute("oidLogic", tableAttributes.getOidLogic());
                                k++; //
                            } else {
                                k = 0;
                                if (i > 5 && RelationshipsAttributes.getForeignVariables()[i + 3].equalsIgnoreCase(RelationshipsAttributes.getForeignVariables()[(i + 3) - 5])) {
                                    k = k + 1;
                                } else {
                                    k = 0;
                                }
                                Element childRoot205 = (Element) document.createElement("table");
                                childRoot200.appendChild(childRoot205);
                                childRoot205.setAttribute("tableName", RelationshipsAttributes.getForeignVariables()[i + 3]);
                                childRoot205.setAttribute("tableID", "" + k);
                                childRoot205.setAttribute("insert", "false");
                                childRoot205.setAttribute("tableMode", tableAttributes.getTableMode());
                                childRoot205.setAttribute("oidLogic", tableAttributes.getOidLogic());
                            }
                        }
                    }
                } else {
                    //RelationshipsAttributes.getForeignVariables().length==0
                    Element childRoot200 = (Element) document.createElement("tables");
                    childRoot1.appendChild(childRoot200);
                    Element childRoot203 = (Element) document.createElement("table");
                    childRoot200.appendChild(childRoot203);
                    TableAttributes tableAttributes = new TableAttributes(importDefinitionAttributes.getTableName(), generatorParameters.getValueMode());
                    childRoot203.setAttribute("tableName", tableAttributes.getTableName());
                    childRoot203.setAttribute("tableID", "" + k);
                    childRoot203.setAttribute("insert", tableAttributes.getInsert());
                    childRoot203.setAttribute("tableMode", tableAttributes.getTableMode());
                    childRoot203.setAttribute("oidLogic", tableAttributes.getOidLogic());
                }
            } else {
                //for csv tables
                Element childRoot200 = (Element) document.createElement("tables");
                childRoot1.appendChild(childRoot200);
                Element childRoot201 = (Element) document.createElement("table");
                childRoot200.appendChild(childRoot201);
                TableAttributes tableAttributes = new TableAttributes(importDefinitionAttributes.getTableName(), generatorParameters.getValueMode());
                childRoot201.setAttribute("tableName", tableAttributes.getTableName());
                childRoot201.setAttribute("tableID", tableAttributes.getTableID());
                childRoot201.setAttribute("insert", tableAttributes.getInsert());
                childRoot201.setAttribute("tableMode", tableAttributes.getTableMode());
                childRoot201.setAttribute("oidLogic", tableAttributes.getOidLogic());

            }
        }
		this.logger.write("full", "WriteImportDefinition is finished.");
    }
    /**
    				* This method will set logger object
    				* @param logger
    				  */
    private void setLogger() {
        this.logger = StandardLogger.getCentralLogger();
    }
}
