
/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.wizard;

import java.io.*;

/**
 *
 * OctopusGenerator class sets the input parameters and start the process (LoaderGenerator)
 * @author Radoslav Dutina
 * @version 1.0
 */
public class OctopusGenerator {
  protected WizardFrame wizard = null;
  private OctopusGeneratorData initData;

  /**
   * Construct the object of OctopusGenerator class with associated parameter
   * @param initData is object of LoaderGeneratorEntry which contain input parameters
   */
  public OctopusGenerator(OctopusGeneratorData initData) {
    this.initData=initData;
  }

  /**
   * This method start the process (LoaderGenerator)
   * @return process
   * @throws Exception
   */
  public String[] generateAll() throws Exception {
    int exit = 0;
    Process process = null;
    String tmp[] = new String[55];
    String as[]=null;
    try {
      ErrorReader errorReader;

      BufferedReader buffer;
      BufferedReader error;
      String s;
      int param = 0;
      if (!initData.getAdditionalPaths().equalsIgnoreCase("")) {
        AddClassPath.setClassPathString(initData.getAdditionalPaths());
      }
      if (!initData.getValueMode().equalsIgnoreCase("")) {
        tmp[param] = "-m";
        param++;
        tmp[param] = initData.getValueMode();
        param++;
      }
      if (!initData.getGeneratorOutput().equalsIgnoreCase("")) {
        tmp[param] = "-o";
        param++;
        tmp[param] = initData.getGeneratorOutput();
        param++;
      }
      if (!initData.getSourceDatabase().equalsIgnoreCase("")) {
        tmp[param] = "-sdb";
        param++;
        tmp[param] = initData.getSourceDatabase();
        param++;
      }
      if (!initData.getSourceType().equalsIgnoreCase("")) {
        tmp[param] = "-st";
        param++;
        tmp[param] = initData.getSourceType();
        param++;
      }
      if (!initData.getSourceUser().equalsIgnoreCase("")) {
        tmp[param] = "-su";
        param++;
        tmp[param] = initData.getSourceUser();
        param++;
      }
      if (!initData.getSourcePassword().equalsIgnoreCase("")) {
        tmp[param] = "-sp";
        param++;
        tmp[param] = initData.getSourcePassword();
        param++;
      }
      if (!initData.getTargetDatabase().equalsIgnoreCase("")) {
        tmp[param] = "-tdb";
        param++;
        tmp[param] = initData.getTargetDatabase();
        param++;
      }
      if (!initData.getTargetType().equalsIgnoreCase("")) {
        tmp[param] = "-tt";
        param++;
        tmp[param] = initData.getTargetType();
        param++;
      }
      if (!initData.getTargetUser().equalsIgnoreCase("")) {
        tmp[param] = "-tu";
        param++;
        tmp[param] = initData.getTargetUser();
        param++;
      }
      if (!initData.getTargetPassword().equalsIgnoreCase("")) {
        tmp[param] = "-tp";
        param++;
        tmp[param] = initData.getTargetPassword();
        param++;
      }
      if (!initData.getDomlPath().equalsIgnoreCase("")) {
        tmp[param] = "-doml";
        param++;
        tmp[param] = initData.getDomlPath();
        param++;
      }
      if (!initData.getPackageName().equalsIgnoreCase("")) {
        tmp[param] = "-pack";
        param++;
        tmp[param] = initData.getPackageName();
        param++;
      }
      if (!initData.getSourceDriverName().equalsIgnoreCase("")) {
        tmp[param] = "-sdn";
        param++;
        tmp[param] = initData.getSourceDriverName();
        param++;
      }
      if (!initData.getTargetDriverName().equalsIgnoreCase("")) {
        tmp[param] = "-tdn";
        param++;
        tmp[param] = initData.getTargetDriverName();
        param++;
      }
      if (initData.getXmlGenerate().equalsIgnoreCase("true")) {
        tmp[param] = "-xml";
        param++;
        tmp[param] = initData.getXmlGenerate();
        param++;
      }
      if (initData.getDomlGenerate().equalsIgnoreCase("true")) {
        tmp[param] = "-gdoml";
        param++;
        tmp[param] = initData.getDomlGenerate();
        param++;
      }
      if (initData.getSqlStmtDropTable().equalsIgnoreCase("true")) {
        tmp[param] = "-sqlDT";
        param++;
        tmp[param] = initData.getSqlStmtDropTable();
        param++;
      }
      if (initData.getSqlStmtDropIntegrity().equalsIgnoreCase("true")) {
        tmp[param] = "-sqlDI";
        param++;
        tmp[param] = initData.getSqlStmtDropIntegrity();
        param++;
      }

      if (initData.getSqlStmtCreateTable().equalsIgnoreCase("true")) {
        tmp[param] = "-sqlCT";
        param++;
        tmp[param] = initData.getSqlStmtCreateTable();
        param++;
      }
      if (initData.getSqlStmtCreatePK().equalsIgnoreCase("true")) {
        tmp[param] = "-sqlCPK";
        param++;
        tmp[param] = initData.getSqlStmtCreatePK();
        param++;
      }
      if (initData.getSqlStmtCreateFK().equalsIgnoreCase("true")) {
        tmp[param] = "-sqlCFK";
        param++;
        tmp[param] = initData.getSqlStmtCreateFK();
        param++;
      }
      if (initData.getSqlStmtCreateIndex().equalsIgnoreCase("true")) {
        tmp[param] = "-sqlCI";
        param++;
        tmp[param] = initData.getSqlStmtCreateIndex();
        param++;
      }
      if (initData.getSqlStmtForAllVendors().equalsIgnoreCase("true")) {
        tmp[param] = "-sqlAll";
        param++;
        tmp[param] = initData.getSqlStmtForAllVendors();
        param++;
      }
      if (initData.getFullMode().equalsIgnoreCase("true")) {
        tmp[param] = "-fm";
        param++;
        tmp[param] = initData.getFullMode();
        param++;
      }else{
        tmp[param] = "-fm";
        param++;
        tmp[param] = initData.getFullMode();
        param++;
      }
      if (!initData.getIncludeTableList().equalsIgnoreCase("")) {
        tmp[param] = "-it";
        param++;
        tmp[param] = initData.getIncludeTableList();
        param++;
      }
      if (!initData.getConfJarStructure().equalsIgnoreCase("")) {
        tmp[param] = "-cjs";
        param++;
        tmp[param] = initData.getConfJarStructure();
        param++;
        
			}
		if (!initData.getLogMode().equalsIgnoreCase("")) {
				tmp[param] = "-lm";
				param++;
				tmp[param] = initData.getLogMode();
				param++;
        
			}
		if (!initData.getLogFileDir().equalsIgnoreCase("")) {
				tmp[param] = "-l";
				param++;
				tmp[param] = initData.getLogFileDir();
				param++;
        
			}
		if (!initData.getLogFileName().equalsIgnoreCase("")) {
				tmp[param] = "-f";
				param++;
				tmp[param] = initData.getLogFileName();
				param++;
        
		}



      as=new String[param];
      for (int i=0;i<param;i++){
        if(!tmp[i].equalsIgnoreCase("null"))
          as[i] = tmp[i];
      }
    } catch (Exception e) {
      e.printStackTrace();
      System.out.println(e.getMessage());
      throw new Exception(e.getMessage());

    }
    return as;
  }

}