/*
LoaderGenerator - tool for generated xml, sql and doml file needed for Octopus.


    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.webdocwf.util.loader.wizard;

// Java core packages
import java.util.*;
import java.net.*;
import java.io.*;

// Java extension packages
import javax.swing.*;
import javax.swing.event.*;

public class OctopusHelpPane extends JEditorPane {

   private List history = new ArrayList();
   private int historyIndex;

   // WebBrowserPane constructor
   public OctopusHelpPane()
   {
      // disable editing to enable hyperlinks
      setEditable( false );
      goToURL(getClass().getResource("HelpPages/main.html"));

   }

   // display given URL and add it to history
   public void goToURL( URL url )
   {
      displayPage( url );

      history.add( url );
      historyIndex = history.size() - 1;
   }

   // display next history URL in editorPane
   public URL forward()
   {
      historyIndex++;
      if ( historyIndex >= history.size() )
         historyIndex = history.size() - 1;

      URL url = ( URL ) history.get( historyIndex );
      displayPage( url );

      return url;
   }

   // display previous history URL in editorPane
   public URL back()
   {
      historyIndex--;

      // do not go past beginning of history
      if ( historyIndex < 0 )
         historyIndex = 0;

      // display previous URL
      URL url = ( URL ) history.get( historyIndex );
      displayPage( url );

      return url;
   }

   // display given URL in JEditorPane
   private void displayPage( URL pageURL )
   {
      // display URL
      try {
         setPage( pageURL );
      }

      // handle exception reading from URL
      catch ( IOException ioException ) {
         ioException.printStackTrace();
      }
   }
}