package org.webdocwf.util.loader.wizard;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.UIManager;


public class WizardFrame extends JFrame {
	private WizardFrame wizardFrame;
	private JDesktopPane desktop;
	private JPopupMenu popUp;
	private static boolean startOctopusLoader = false;
	private static String pathToXmlFile = "";
	private static boolean startOctopusProject = false;
	private static String pathToProjectFile = "";
	private static String confJarStructure = "";
	private static int defaultProjectNameCounter = 1;
	private static final String DEFAULT_PROJECT_NAME = "Untitled project";

	public WizardFrame(String title) {
		super(title);
		wizardFrame = this;
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		desktop = new JDesktopPane();
		setContentPane(desktop);

		// Install our custom desktop manager.
		//ZK commented this
		//desktop.setDesktopManager(new SampleDesktopMgr());
		desktop.setBackground(new Color(210, 210, 210));

		createMenuBar();
		createPopupMenu();

		if (startOctopusLoader) {
			try {
				OctopusProjectFrame octopusProject = createOctopusProjectFrame(DEFAULT_PROJECT_NAME+defaultProjectNameCounter++);
				//redefine settings
				octopusProject.getLoaderPanel().setOctopusLoaderInit(
					new OctopusLoaderData(),
					pathToXmlFile,
					confJarStructure);
				octopusProject.setActiveTab(1);
				desktop.add(octopusProject);
				octopusProject.setVisible(true);
				octopusProject.requestFocus();
			} catch (Exception ex) {
				String message = "Error while starting OctopusWizard . Message : ";
				message += ex.getMessage();
				JOptionPane.showMessageDialog(desktop, message + "\n", "Error", 0);
				System.exit(0);
			}
		} else if( startOctopusProject ) {
				try {
				OctopusProperties properties = new OctopusProperties();
				InputStream is=new FileInputStream(pathToProjectFile);
		 		properties.load(is);
				String name = properties.getOctopusProperty("ProjectName");
				if (name == null || name.trim().equals(""))
									name = DEFAULT_PROJECT_NAME+defaultProjectNameCounter++;
				OctopusProjectFrame octopusProject = createOctopusProjectFrame(name);
				confJarStructure = properties.getOctopusProperty("Generator.ConfJarStructure");
				OctopusGeneratorData generatorData = new OctopusGeneratorData(confJarStructure);
				
				//set data
				setGeneratorData(properties,generatorData);
				
				octopusProject.getGeneratorPanel().setOctopusGeneratorInit(
					generatorData,
					confJarStructure);
				//set data
				OctopusLoaderData loaderData = new OctopusLoaderData();
				setLoaderData(properties,loaderData);
//				
								
				
				octopusProject.getLoaderPanel().setOctopusLoaderInit(
					loaderData,
					null,
					confJarStructure);
				
				desktop.add(octopusProject);
				octopusProject.setVisible(true);
				octopusProject.requestFocus();
			} catch (Exception ex) {
				String message = "Error while starting OctopusWizard . Message : ";
				message += ex.getMessage();
				JOptionPane.showMessageDialog(desktop, message + "\n", "Error", 0);
				System.exit(0);
			}
		}
	}

	protected void createMenuBar() {
		JMenuBar mb = new JMenuBar();
		JMenu menu = new JMenu("Project");
		menu.setMnemonic('P');
		JMenu menu2 = new JMenu("IDEOptions");
		menu2.setMnemonic('I');
		JMenu menu3 = new JMenu("Help");
		menu3.setMnemonic('H');

		menu.add(new NewOctopusProjectItem()); // add Octopus Project frame as new project
		menu.add(new OpenOctopusProjectItem()); // add Octopus Project frame as open project
		menu.add(new ExitOctopusApplicationItem()); // add exit
		menu2.add(new LnFActionMetal()); // add look and feel option
		menu2.add(new LnFActionMotif());
		menu2.add(new LnFActionWindows());
		menu2.add(new LnFActionJava());
		menu3.add(new OctopusHelpAction("Help"));
		menu3.addSeparator();
		menu3.add(new OctopusAboutAction());

		setJMenuBar(mb);
		mb.add(menu);
		mb.add(menu2);
		mb.add(menu3);
	}


	public void createPopupMenu() {
		JMenuItem menuItem;

		//Create the popup menu.
		JPopupMenu popup = new JPopupMenu();
		JMenuItem popupItem = new JMenuItem("Start new Octopus project");
		popupItem.setAction(new NewOctopusProjectItem());
		JMenuItem popupItem2 = new JMenuItem("Open existing Octopus project");
		popupItem2.setAction(new OpenOctopusProjectItem());
		JMenuItem popupItem3 = new JMenuItem("Exit Octopus Application");
		popupItem3.setAction(new ExitOctopusApplicationItem());

		popup.add(popupItem);
		popup.add(popupItem2);
		popup.add(popupItem3);

		//Add listener to the destop area so the popup menu can come up.
		MouseListener popupListener = new PopupListener(popup);
		desktop.addMouseListener(popupListener);
	}

	public static void main(String[] args) {

		OctopusSplash octopusSplash = new OctopusSplash("splash_octopus.gif", null);

		if (args.length > 0 && args.length < 5) {
			for (int i = 0; i < args.length - 1; i = i + 1) {
				if (args[i].equalsIgnoreCase("-sol")) {
					pathToXmlFile = args[++i];
					if (!pathToXmlFile.equalsIgnoreCase(""))
						startOctopusLoader = true;
				}
				if (args[i].equalsIgnoreCase("-sop")) {
					pathToProjectFile = args[++i];
					if (!pathToProjectFile.equalsIgnoreCase(""))
						startOctopusProject = true;
				}
				if (args[i].equalsIgnoreCase("-cjs")) {
					confJarStructure = args[++i];
				}
			}
		}
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			e.printStackTrace();
		}
		WizardFrame wizardFrame = new WizardFrame("Enhydra Octopus Application");

//		wizardFrame.addWindowListener(new WindowAdapter() {
//			public void windowClosing(WindowEvent we) {
//				System.exit(0);
//			}
//		});

		wizardFrame.setIconImage(
			new ImageIcon(WizardFrame.class.getResource("images/Enhydra16.gif")).getImage());

		Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
		wizardFrame.setBounds(0, 0, dimension.width - 20, dimension.height - 40);

		try {
			Thread.sleep(2500);
		} catch (Exception e) {
			e.printStackTrace();
		}
		OctopusSplash.getSplash().hideSplash();
		octopusSplash = null;
		wizardFrame.setVisible(true);
	}

	private OctopusProjectFrame createOctopusProjectFrame(String name) throws Exception {
		
		//create menu
		JMenuBar mb = new JMenuBar();
		JMenu menu = new JMenu("Application");
		menu.setMnemonic('A');
		JMenuItem startGenerator = new JMenuItem("Start Generator");
		JMenuItem startLoader = new JMenuItem("Start Loader");
		JMenuItem save = new JMenuItem("Save project");
		JMenuItem stop = new JMenuItem("Close project");
		JMenuItem help = new JMenuItem("Help");
		menu.add(startGenerator);
		menu.add(startLoader);
		menu.addSeparator();
		menu.add(save);
		menu.add(stop);
		menu.addSeparator();
		menu.add(help);
		mb.add(menu); 
		
		//create toolbar
		JButton startGeneratorButton =
					OctopusProjectFrame.createOctopusButton(
						"Start Generator",
						OctopusGeneratorPanel.toolBarButton);
		JButton startLoaderButton =
			OctopusProjectFrame.createOctopusButton(
				"Start Loader",
				OctopusGeneratorPanel.toolBarButton);
		JButton stopButton =
			OctopusProjectFrame.createOctopusButton("Stop", OctopusGeneratorPanel.toolBarButton);
		JButton saveButton =
			OctopusProjectFrame.createOctopusButton("Save", OctopusGeneratorPanel.toolBarButton);
		JButton helpButton =
			OctopusProjectFrame.createOctopusButton("Help", OctopusGeneratorPanel.toolBarButton);
		JToolBar toolBar = new JToolBar("Octopus toolbar");
		toolBar.add(startGeneratorButton);
//		toolBar.addSeparator();
		toolBar.add(startLoaderButton);
		toolBar.addSeparator();
		toolBar.add(saveButton);
//		toolBar.addSeparator();
		toolBar.add(stopButton);
		toolBar.addSeparator();
		toolBar.add(helpButton);
		//create generator panel
		OctopusGeneratorPanel generatorPanel = new OctopusGeneratorPanel();
		generatorPanel.setOctopusGeneratorInit(
			new OctopusGeneratorData(confJarStructure),
			confJarStructure);
		//create loader panel
		OctopusLoaderPanel loaderPanel = new OctopusLoaderPanel();
		loaderPanel.setOctopusLoaderInit(new OctopusLoaderData(), null, confJarStructure);
		//create octopus project frame
		OctopusProjectFrame octopusProject =
			new OctopusProjectFrame(name, loaderPanel, generatorPanel, toolBar);
		octopusProject.getRootPane().setJMenuBar(mb);
		//set actions for menu
		startGenerator.setAction(new StartOctopusGeneratorAction(octopusProject, "Start Generator"));
		startLoader.setAction(new StartOctopusLoaderAction(octopusProject, "Start Loader"));
		stop.setAction(new CloseOctopusProjectAction(octopusProject, "Close"));
		save.setAction(new SaveProjectAction(octopusProject, "Save"));
		help.setAction(new OctopusHelpAction("Help"));
		//set actions for toolbar
		startGeneratorButton.setAction(new StartOctopusGeneratorAction(octopusProject, ""));
		startLoaderButton.setAction(new StartOctopusLoaderAction(octopusProject, ""));
		stopButton.setAction(new CloseOctopusProjectAction(octopusProject, ""));
		saveButton.setAction(new SaveProjectAction(octopusProject, ""));
		helpButton.setAction(new OctopusHelpAction(""));
		//return frame
		return octopusProject;
	}

	private class ExitOctopusApplicationItem extends AbstractAction {
		/**
		 * Action which creates new frame
		 */
		public ExitOctopusApplicationItem() {
			putValue(NAME, "Exit");
			putValue(
				ACCELERATOR_KEY,
				KeyStroke.getKeyStroke('X', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			putValue(
			SMALL_ICON,
			new ImageIcon(
				getClass().getClassLoader().getResource(
					"org/webdocwf/util/loader/" + "wizard/images/Stop.gif")));
			putValue(SHORT_DESCRIPTION, "Exit Enhydra Octopus Application");
			putValue(LONG_DESCRIPTION, "Exit Enhydra Octopus Application");
			putValue(MNEMONIC_KEY, new Integer('X'));
		}

		public void actionPerformed(ActionEvent e) {
			try {
				if( 
							JOptionPane.showConfirmDialog(null, "Are you are sure want to exit?", "Exit", JOptionPane.INFORMATION_MESSAGE) == 0 
							) 
							{
								System.exit(0);
							}
			} catch (Exception ex) {
				String message =
					"Error while creating action for ExitOctopusApplicationItem menu item. Message :"
						+ ex.getMessage();
				JOptionPane.showMessageDialog(desktop, message + "\n", "Error", 0);
			}

		}
	}

	private class NewOctopusProjectItem extends AbstractAction {
		/**
		 * Action which creates new frame
		 */
		public NewOctopusProjectItem() {
			putValue(NAME, "New Octopus Project");
			putValue(
				ACCELERATOR_KEY,
				KeyStroke.getKeyStroke('N', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			putValue(
			SMALL_ICON,
			new ImageIcon(
				getClass().getClassLoader().getResource(
					"org/webdocwf/util/loader/" + "wizard/images/New.gif")));
			putValue(SHORT_DESCRIPTION, "Octopus project");
			putValue(LONG_DESCRIPTION, "Start Octopus project");
			putValue(MNEMONIC_KEY, new Integer('N'));
		}

		public void actionPerformed(ActionEvent e) {
			try {
				String name = (String) JOptionPane.showInputDialog(null,
						"Project name : ",
						"New project...",
						JOptionPane.INFORMATION_MESSAGE,
						null,
						null,
						DEFAULT_PROJECT_NAME+defaultProjectNameCounter++);
			 if( name != null ) {
				if (name == null || name.trim().equals(""))
					name = DEFAULT_PROJECT_NAME+defaultProjectNameCounter++;
				OctopusProjectFrame octopusProject = createOctopusProjectFrame(name);
				desktop.add(octopusProject);
				octopusProject.setVisible(true);
				octopusProject.requestFocus();
			 } else {
			 	//cancel pressed
			 }

			} catch (Exception ex) {
				String message =
					"Error while creating action for NewOctopusProject menu item. Message :"
						+ ex.getMessage();
				JOptionPane.showMessageDialog(desktop, message + "\n", "Error", 0);
			}

		}
	}

	private class OpenOctopusProjectItem extends AbstractAction {
		/**
		 * Action which creates new frame
		 */
		public OpenOctopusProjectItem() {
			putValue(NAME, "Open existing Octopus Project");
			putValue(
				ACCELERATOR_KEY,
				KeyStroke.getKeyStroke('O', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			putValue(
			SMALL_ICON,
			new ImageIcon(
				getClass().getClassLoader().getResource(
					"org/webdocwf/util/loader/" + "wizard/images/Open.gif")));
			putValue(SHORT_DESCRIPTION, "Octopus project");
			putValue(LONG_DESCRIPTION, "Open Octopus project");
			putValue(MNEMONIC_KEY, new Integer('0'));
		}

		public void actionPerformed(ActionEvent e) {
			try {
				File current = new File(System.getProperty("user.dir"));
				JFileChooser chooser = new JFileChooser(current);
				chooser.setDialogTitle("Open Octopus project ...");
				chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				chooser.setFileFilter(
					(javax
						.swing
						.filechooser
						.FileFilter) new OctopusFileFilter(OctopusFileFilter
						.OPF));
				chooser.setAcceptAllFileFilterUsed(false);

				chooser.setApproveButtonText("Open");
				int v = chooser.showOpenDialog(desktop);

				switch (v) {
					case JFileChooser.APPROVE_OPTION :
						if (chooser.getSelectedFile() != null) {
							openProject(chooser.getSelectedFile());
						}
						break;
					case JFileChooser.CANCEL_OPTION :
					case JFileChooser.ERROR_OPTION :
						chooser.removeAll();
						chooser = null;
						current = null;

				}

			} catch (Exception ex) {
				String message =
					"Error while creating action for OpenOctopusProject menu item. Message :"
						+ ex.getMessage();
				JOptionPane.showMessageDialog(desktop, message + "\n", "Error", 0);
			}
		}
			
			private void openProject(File projectFile) throws Exception {
				OctopusProperties properties = new OctopusProperties();
				InputStream is=new FileInputStream(projectFile);
		 		properties.load(is);
				String name = properties.getOctopusProperty("ProjectName");
				if (name == null || name.trim().equals(""))
									name = DEFAULT_PROJECT_NAME+defaultProjectNameCounter++;
				OctopusProjectFrame octopusProject = createOctopusProjectFrame(name);
				confJarStructure = properties.getOctopusProperty("Generator.ConfJarStructure");
				OctopusGeneratorData generatorData = new OctopusGeneratorData(confJarStructure);
				setGeneratorData(properties,generatorData);
				
				octopusProject.getGeneratorPanel().setOctopusGeneratorInit(
					generatorData,
					confJarStructure);
				//set data
				OctopusLoaderData loaderData = new OctopusLoaderData();
				setLoaderData(properties,loaderData);
								
				
				octopusProject.getLoaderPanel().setOctopusLoaderInit(
					loaderData,
					null,
					confJarStructure);
				
				desktop.add(octopusProject);
				octopusProject.setVisible(true);
				octopusProject.requestFocus();
			}
	}

	private class LnFActionMetal extends AbstractAction {

		public LnFActionMetal() {

			putValue(NAME, "Metal Look&Feel");
			putValue(
				ACCELERATOR_KEY,
				KeyStroke.getKeyStroke('M', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			putValue(SHORT_DESCRIPTION, "Metal Look & Feel");
			putValue(LONG_DESCRIPTION, "Set Metal Look & Feel Model");
			putValue(MNEMONIC_KEY, new Integer('M'));
		}

		public void actionPerformed(ActionEvent e) {
			LnFListener lnf = new LnFListener(wizardFrame);
			lnf.actionPerformed(e);
		}
	}

	private class LnFActionMotif extends AbstractAction {

		public LnFActionMotif() {

			putValue(NAME, "Motif Look&Feel");
			putValue(
				ACCELERATOR_KEY,
				KeyStroke.getKeyStroke('F', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			putValue(SHORT_DESCRIPTION, "Motif Look & Feel");
			putValue(LONG_DESCRIPTION, "Set Motif Look & Feel Model");
			putValue(MNEMONIC_KEY, new Integer('F'));
		}

		public void actionPerformed(ActionEvent e) {
			LnFListener lnf = new LnFListener(wizardFrame);
			lnf.actionPerformed(e);
		}
	}

	private class LnFActionWindows extends AbstractAction {

		public LnFActionWindows() {

			putValue(NAME, "Windows Look&Feel");
			putValue(
				ACCELERATOR_KEY,
				KeyStroke.getKeyStroke('W', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			putValue(SHORT_DESCRIPTION, "Windows Look & Feel");
			putValue(LONG_DESCRIPTION, "Set Windows Look & Feel Model");
			putValue(MNEMONIC_KEY, new Integer('W'));
		}

		public void actionPerformed(ActionEvent e) {
			LnFListener lnf = new LnFListener(wizardFrame);
			lnf.actionPerformed(e);
		}
	}

	private class LnFActionJava extends AbstractAction {

		public LnFActionJava() {

			putValue(NAME, "Java Look&Feel");
			putValue(
				ACCELERATOR_KEY,
				KeyStroke.getKeyStroke('J', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			putValue(SHORT_DESCRIPTION, "Java Look & Feel");
			putValue(LONG_DESCRIPTION, "Set Java Look & Feel Model");
			putValue(MNEMONIC_KEY, new Integer('J'));
		}

		public void actionPerformed(ActionEvent e) {
			LnFListener lnf = new LnFListener(wizardFrame);
			lnf.actionPerformed(e);
		}
	}

	class PopupListener extends MouseAdapter {
		JPopupMenu popup;

		PopupListener(JPopupMenu popupMenu) {
			popup = popupMenu;
		}

		public void mousePressed(MouseEvent e) {
			showPopup(e);
		}

		public void mouseReleased(MouseEvent e) {
			showPopup(e);
		}

		private void showPopup(MouseEvent e) {
			if (e.isPopupTrigger()) {
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}

	private class OctopusHelpAction extends AbstractAction {

		/**
		 * Action which creates new help frame
		 */
		public OctopusHelpAction(String label) {
			putValue(NAME, label);
			putValue(
				SMALL_ICON,
				new ImageIcon(
					getClass().getClassLoader().getResource(
						"org/webdocwf/util/loader/" + "wizard/images/Help16.gif")));
			putValue(SHORT_DESCRIPTION, "Help for running the program");
			putValue(LONG_DESCRIPTION, "Help for running the program");
			putValue(
				ACCELERATOR_KEY,
				KeyStroke.getKeyStroke('H', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			putValue(MNEMONIC_KEY, new Integer('H'));
		}

		/**
		 * Creating event for new windows help frame
		 * @param e is creating events
		 */
		public void actionPerformed(ActionEvent e) {
			try {
				OctopusGeneratorHelpFrame helpFrame = new OctopusGeneratorHelpFrame();
				helpFrame.setIconImage(
					new ImageIcon(
						getClass().getClassLoader().getResource(
							"org/webdocwf/util/loader/" + "wizard/images/Enhydra16.gif"))
						.getImage());
				helpFrame.setVisible(true);
			} catch (Exception ex) {
				String message =
					"Error while creating action for OctopusHelp menu item. Message :"
						+ ex.getMessage();
				JOptionPane.showMessageDialog(desktop, message + "\n", "Error", 0);
				System.exit(0);
			}
		}
	}

	private OctopusAboutFrame createOctopusAboutFrame() {
		OctopusAboutFrame frame = new OctopusAboutFrame(this.desktop);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		return frame;
	}

	private class OctopusAboutAction extends AbstractAction {
		/**
		 * Action which creates new about frame
		 */
		public OctopusAboutAction() {
			putValue(NAME, "About");
			putValue(
				SMALL_ICON,
				new ImageIcon(
					getClass().getClassLoader().getResource(
						"org/webdocwf/util/loader/" + "wizard/images/About16.gif")));
			putValue(SHORT_DESCRIPTION, "About the program");
			putValue(LONG_DESCRIPTION, "About the program");
			putValue(
				ACCELERATOR_KEY,
				KeyStroke.getKeyStroke('A', Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
			putValue(MNEMONIC_KEY, new Integer('A'));
		}

		/**
		 * Creating event for new windows about frame
		 * @param e is creating events
		 */
		public void actionPerformed(ActionEvent e) {
			try {
				OctopusAboutFrame aboutFrame = createOctopusAboutFrame();
				//        aboutFrame.setIconImage(new ImageIcon(getClass().getClassLoader().getResource("org/webdocwf/util/loader/"+
				//            "wizard/images/Enhydra16.gif")).getImage());
				desktop.add(aboutFrame);
				aboutFrame.setVisible(true);

			} catch (Exception ex) {
				String message =
					"Error while creating action for OctopusAbout menu item. Message :"
						+ ex.getMessage();
				JOptionPane.showMessageDialog(desktop, message + "\n", "Error", 0);
				System.exit(0);
			}
		}
	}

	private void setLoaderData(OctopusProperties properties, OctopusLoaderData loaderData) throws Exception {
		try {
			loaderData.setAdditionalPaths(properties.getOctopusProperty("Loader.AdditionalPaths"));
			loaderData.setCommitCount(properties.getOctopusProperty("Loader.CommitCount"));
			loaderData.setConfJarStructure(properties.getOctopusProperty("Loader.ConfJarStructure"));
			loaderData.setIncludeTables(properties.getOctopusProperty("Loader.IncludeTables"));
			loaderData.setLogFileDir(properties.getOctopusProperty("Loader.LogFileDir"));
			loaderData.setLogFileName(properties.getOctopusProperty("Loader.LogFileName"));
			loaderData.setLogMode(properties.getOctopusProperty("Loader.LogMode"));
			loaderData.setOnErrorCon(properties.getOctopusProperty("Loader.ErrorConntinue"));
			loaderData.setPathToXml(properties.getOctopusProperty("Loader.PathToXml"));
			loaderData.setRestartIndicator(properties.getOctopusProperty("Loader.RestartIndicator"));
			loaderData.setReturnCode(properties.getOctopusProperty("Loader.ReturnCode"));
			loaderData.setUserId(properties.getOctopusProperty("Loader.UserId"));
			loaderData.setVariables(properties.getOctopusProperty("Loader.Variables"));
			loaderData.setVendorConf(properties.getOctopusProperty("Loader.VendorConf"));
		}catch(Exception e) {
			throw e;
		}
	}

	private void setGeneratorData(OctopusProperties properties, OctopusGeneratorData generatorData) throws Exception {
		try {
			generatorData.setAdditionalPaths(properties.getOctopusProperty("Generator.AdditionalPaths"));
			generatorData.setConfJarStructure(properties.getOctopusProperty("Generator.ConfJarStructure"));
			generatorData.setDomlGenerate(properties.getOctopusProperty("Generator.DomlGenerate"));
			generatorData.setDomlPath(properties.getOctopusProperty("Generator.DomlPath"));
			generatorData.setFullMode(properties.getOctopusProperty("Generator.FullMode"));
			generatorData.setGeneratorOutput(properties.getOctopusProperty("Generator.GeneratorOutput"));
			generatorData.setIncludeTableList(properties.getOctopusProperty("Generator.IncludeTableList"));
			generatorData.setPackageName(properties.getOctopusProperty("Generator.PackageName"));
			generatorData.setSourceDatabase(properties.getOctopusProperty("Generator.SourceDatabase"));
			generatorData.setSourceDriverName(properties.getOctopusProperty("Generator.SourceDriverName"));
			generatorData.setSourcePassword(properties.getOctopusProperty("Generator.SourcePassword"));
			generatorData.setSourceType(properties.getOctopusProperty("Generator.SourceType"));
			generatorData.setSourceUser(properties.getOctopusProperty("Generator.SourceUser"));
			generatorData.setSqlGenerate(properties.getOctopusProperty("Generator.SqlGenerate"));
			generatorData.setSqlStmtCreateFK(properties.getOctopusProperty("Generator.SqlStmtCreateFK"));
			generatorData.setSqlStmtCreateIndex(properties.getOctopusProperty("Generator.SqlStmtCreateIndex"));
			generatorData.setSqlStmtCreatePK(properties.getOctopusProperty("Generator.SqlStmtCreatePK"));
			generatorData.setSqlStmtCreateTable(properties.getOctopusProperty("Generator.SqlStmtCreateTable"));
			generatorData.setSqlStmtDropIntegrity(properties.getOctopusProperty("Generator.SqlStmtDropIntegrity"));
			generatorData.setSqlStmtDropTable(properties.getOctopusProperty("Generator.SqlStmtDropTable"));
			generatorData.setSqlStmtForAllVendors(properties.getOctopusProperty("Generator.SqlStmtForAllVendors"));
			generatorData.setTargetDatabase(properties.getOctopusProperty("Generator.TargetDatabase"));
			generatorData.setTargetDriverName(properties.getOctopusProperty("Generator.TargetDriverName"));
			generatorData.setTargetPassword(properties.getOctopusProperty("Generator.TargetPassword"));
			generatorData.setTargetType(properties.getOctopusProperty("Generator.TargetType"));
			generatorData.setTargetUser(properties.getOctopusProperty("Generator.TargetUser"));
			generatorData.setValueMode(properties.getOctopusProperty("Generator.ValueMode"));
			generatorData.setXmlGenerate(properties.getOctopusProperty("Generator.XmlGenerate"));
			
			generatorData.setLogMode(properties.getOctopusProperty("Generator.LogMode"));
			generatorData.setLogFileDir(properties.getOctopusProperty("Generator.LogFileDir"));
			generatorData.setLogFileName(properties.getOctopusProperty("Generator.LogFileName"));
		}catch(Exception e) {
			throw e;
		}
	}


}