/*

	 Loader - tool for transfering data from one JDBC source to another and 
	 doing transformations during copy.

    Copyright (C) 2002  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/*
 * DatabaseOperation.java   Aug 30, 2002
 * Sinisa Milosevic  sinisami@eunet.yu
 *
 */

package org.webdocwf.util.loader.test;

import java.sql.Connection;
import org.webdocwf.util.loader.Loader;

import java.sql.SQLException;

/**
 * Defines the interface contract for operations performed on the database.
 *
 * @author sinisa Milosevic
 * @version $Revision: 1.1 $
 */
public abstract class DatabaseOperation
{
    public static final String NONE = "NONE";
    public static final String CREATE = "CREATE";
    public static final String DROP = "DROP";
    public static final String LOADER = "LOADER";
    public static DatabaseOperation DO_NOTHING = new DummyAction();


    /**
     * Executes this operation on the specified database using the specified
     * connection to the database.
     *
     * @param conn the database connection.
     */
    public abstract void execute(Connection conn) throws SQLException;


/**
     * Returns type of database operation
     *
     */
    public abstract String getDatabaseOperationType();


    private static class DummyAction extends DatabaseOperation
    {
        public void execute(Connection conn)
        {
        }

        public String getDatabaseOperationType()
        {
        	return DatabaseOperation.NONE;
        }
    }
}






