/*

	 Loader - tool for transfering data from one JDBC source to another and 
	 doing transformations during copy.

    Copyright (C) 2002  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/*
 * DropDatabaseOperation.java   Aug 30, 2002
 * Sinisa Milosevic  sinisami@eunet.yu
 *
 */


package org.webdocwf.util.loader.test;

import java.sql.Connection;
import java.sql.Statement;
import org.webdocwf.util.loader.test.DatabaseOperation;
import org.webdocwf.util.loader.Loader;
import org.webdocwf.util.loader.LoaderException;

import java.sql.SQLException;

/**
 * Deletes entire database using sql command (DROP DATABASE) or specified loader job.
 *
 * @author Sinisa Milosevic
 * @version $Revision: 1.1 $
 */
public class DropDatabaseOperation extends DatabaseOperation
{
	
	private String databaseName=null;
	
    DropDatabaseOperation()
    {
    }
    
    public DropDatabaseOperation(String name)
    {
    	databaseName=name;
    }

    ////////////////////////////////////////////////////////////////////////////
    // DatabaseOperation class

    /**
     * Executes this operation on the specified database using the specified
     * connection to the database.
     *
     * @param conn the database connection.
     */

    public void execute(Connection conn) throws SQLException
    {
        Statement stmt = conn.createStatement();
        
        if(databaseName!=null){
        	try
        	{
           stmt.execute("DROP DATABASE "+databaseName);
        	}
        	finally
        	{
            stmt.close();
        	}
        }
        			
    }

    
        /**
     * Returns type of database operation
     *
     */
    public String getDatabaseOperationType()
    {
    	return DatabaseOperation.DROP;
    }	


}







