/*
    Copyright (C) 2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package org.enhydra.xml;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;



/**
 * @author Tweety
 *
 * A class representing a node in a meta-data tree, which implements
 * the <a href="../../../../api/org/w3c/dom/NodeList.html">
 *
 * <p> Namespaces are ignored in this implementation.  The terms "tag
 * name" and "node name" are always considered to be synonymous.
 *
 * @version 1.0
 */
public class NodeListImpl implements NodeList {

	/**
	 * List of <code>Node</code>s.
	 */
	List nodes;


	/**
	 * Constructs an empty <code>NodeListImpl</code>.
	 */
	public NodeListImpl() {
		this.nodes = new ArrayList();
	}


	/**
	 * Constructs <code>NodeListImpl</code> with the given list of nodes.
	 *
	 * @param nodes list of nodes.
	 */
	public NodeListImpl(List nodes) {
		this.nodes = nodes;
	}


	/**
	 * Returns the count of nodes.
	 *
	 * @return the count of nodes.
	 */
	public int getLength() {
		return nodes.size();
	}


	/**
	 * Returns the <code>Node</code> with the given index.
	 *
	 * @param index node index.
	 *
	 * @return the <code>Node</code> with the given index.
	 */
	public Node item(int index) {
		if (index < 0 || index > nodes.size()) {
			return null;
		}
		return (Node) nodes.get(index);
	}


	/**
	 * Appends the given list to the end of the existing one.
	 *
	 * @param list list to add, as <code>NodeListImpl</code>.
	 *
	 * @return this as </code>NodeList</code>.
	 */
	public NodeList append(NodeListImpl list) {
		this.nodes.addAll(list.nodes);
		return this;
	}

}