/**
    Copyright (C) 2002-2003  Together

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


package org.webdocwf.util.i18njdbc;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Vector;

/**
 * This class implements the PreparedStatement interface for the I18nJdbc driver.
 *
 * @author     Zoran Milakovic
 * @author		 Zeljko Kovacevic
 */
public class I18nPreparedStatement implements PreparedStatement {

  private I18nConnection connection;
  private Vector resultSets = new Vector();
  private String sqlForPrepare = "";
  private String sqlPrepared = "";
  private int paramCount = 0;
  private ArrayList parameters = new ArrayList();
  private ArrayList binaryStreamParameters = new ArrayList();
  private I18nProperties properties;
/** Name used for prepared statement parameters */
  public static String PREPARE_SEPARATOR = "~@#NEXTPARAMETER#@~";
  private String sql;

  public I18nPreparedStatement(I18nConnection connection, String preparedSql) {
    DriverManager.println("I18nJdbc - I18nStatement() - connection=" + connection);
    this.sqlForPrepare = preparedSql;
    this.sqlPrepared = preparedSql;
		this.properties = new I18nProperties();
    this.paramCount = countParameters(preparedSql);
    for ( int i = 0; i < paramCount; i++ )
      parameters.add(null);
    this.connection = connection;
    try {
     if(!connection.getAutoCommit()){}
//        write=new I18nWriter(
//              null,            
//              connection.getExtension()
//              );
    }
    catch (Exception ex) {
    ex.printStackTrace();
    }
  }

  private int countParameters(String sql) {
    int count = 0;
    int index = sql.indexOf(PREPARE_SEPARATOR);
    while( index != -1 ) {
      count++;
      sql = sql.substring(index+1);
      index = sql.indexOf(PREPARE_SEPARATOR);
    }
    return count;
  }

  private boolean prepareSql() throws SQLException {
    boolean retVal = true;
    for (int i = 0; i < parameters.size(); i++) {
      int index = sqlPrepared.indexOf(PREPARE_SEPARATOR);
      String val;
      if (index != -1) {
        if( parameters.get(i) == null )
          val = "null";
        else
          val = parameters.get(i).toString();
        sqlPrepared = sqlPrepared.substring(0, index) +
            val +
            sqlPrepared.substring(index + PREPARE_SEPARATOR.length());
      }
    }
    if (sqlPrepared.indexOf(PREPARE_SEPARATOR) != -1)
      throw new SQLException(
          "All ? in prepared query has to be replaced with values.");
    else if (parameters.size() < this.paramCount)
      throw new SQLException(
          "Number of setted parameters is less than number of parameters in statement.");
    else if (parameters.size() > this.paramCount)
      throw new SQLException(
          "Number of setted parameters is greater than number of parameters in statement.");

    return retVal;
  }

  /**
     *Sets the maxFieldSize attribute of the I18nStatement object
     *
     * @param  p0                The new maxFieldSize value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setMaxFieldSize(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the maxRows attribute of the I18nStatement object
     *
     * @param  p0                The new maxRows value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setMaxRows(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the escapeProcessing attribute of the I18nStatement object
     *
     * @param  p0                The new escapeProcessing value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setEscapeProcessing(boolean p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the queryTimeout attribute of the I18nStatement object
     *
     * @param  p0                The new queryTimeout value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setQueryTimeout(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the cursorName attribute of the I18nStatement object
     *
     * @param  p0                The new cursorName value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setCursorName(String p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the fetchDirection attribute of the I18nStatement object
     *
     * @param  p0                The new fetchDirection value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setFetchDirection(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Sets the fetchSize attribute of the I18nStatement object
     *
     * @param  p0                The new fetchSize value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void setFetchSize(int p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the maxFieldSize attribute of the I18nStatement object
     *
     * @return                   The maxFieldSize value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getMaxFieldSize() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the maxRows attribute of the I18nStatement object
     *
     * @return                   The maxRows value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getMaxRows() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the queryTimeout attribute of the I18nStatement object
     *
     * @return                   The queryTimeout value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getQueryTimeout() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the warnings attribute of the I18nStatement object
     *
     * @return                   The warnings value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public SQLWarning getWarnings() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the resultSet attribute of the I18nStatement object
     *
     * @return                   The resultSet value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public ResultSet getResultSet() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the updateCount attribute of the I18nStatement object
     *
     * @return                   The updateCount value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getUpdateCount() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the moreResults attribute of the I18nStatement object
     *
     * @return                   The moreResults value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public boolean getMoreResults() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the fetchDirection attribute of the I18nStatement object
     *
     * @return                   The fetchDirection value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getFetchDirection() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the fetchSize attribute of the I18nStatement object
     *
     * @return                   The fetchSize value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getFetchSize() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the resultSetConcurrency attribute of the I18nStatement object
     *
     * @return                   The resultSetConcurrency value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getResultSetConcurrency() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the resultSetType attribute of the I18nStatement object
     *
     * @return                   The resultSetType value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int getResultSetType() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Gets the connection attribute of the I18nStatement object
     *
     * @return                   The connection value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public Connection getConnection() throws SQLException
    {
        return connection;
    }


    /**
     *Description of the Method
     *
     * @param  sql               Description of Parameter
     * @return                   Description of the Returned Value
     * @exception  SQLException  Description of Exception
     * @since
     */
	public ResultSet executeQuery(String sql) throws SQLException
	{
		DriverManager.println("I18nJdbc - I18nStatement:executeQuery() - sql= " + sql);
		I18nSqlParser parser = new I18nSqlParser();
		this.sql = sql;
		try
			{
				parser.parse(this);
			}
		catch (Exception e)
			{
				throw new SQLException("Syntax Error. " + e.getMessage());
			}

	String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
	File checkFile = new File(fileName);
	if (!checkFile.exists())
	{
	  throw new SQLException("Cannot open data file '" + fileName + "'  !");
	}

	if (!checkFile.canRead())
	{
	  throw new SQLException("Data file '" + fileName + "'  not readable !");
	}
   
	try
	{

				String[] xxx = parser.getColumnNames();
			  String[] yyy = connection.getColumnNames();
			  boolean isOK = true;
			  for(int i=0; i< xxx.length; i++) {
			  if(!xxx[i].endsWith("*")) {
			   out:
			  for(int j=0; j< yyy.length; j++) {
				 if(xxx[i].equalsIgnoreCase(yyy[j])) {
					isOK=true;
					break out;
				  }
				  else
					isOK=false;
				}
			   if(!isOK)
				throw new SQLException("Column '" + xxx[i] + "' not found.");
			  }
			  }
	}
	catch (Exception e)
	{
		if( I18nDriver.DEBUG )
			e.printStackTrace();
	  throw new SQLException("Error reading data file. Message was: " + e);
	}
	//load properties file
	try {
		this.properties.load(new FileInputStream(checkFile));
	} catch (Exception e) {
		throw new SQLException("Error while loading properties");
	}

		I18nResultSet resultSet = new I18nResultSet(this,
					parser.getTableName(), parser.getColumnNames(), parser.getWhereColumnNames(),
					parser.getWhereColumnValues());
	resultSets.add(resultSet);
	return resultSet;
  }



    /**
     *Description of the Method
     *
     * @param  sql               Description of Parameter
     * @return                   Description of the Returned Value
     * @exception  SQLException  Description of Exception
     * @since
     */
	public int executeUpdate(String sql) throws SQLException
   {
   int updated=0;
   DriverManager.println("I18nJdbc - I18nStatement:executeUpdate() - sql= " + sql);
   I18nSqlParser parser = new I18nSqlParser();
   this.sql = sql;
   try
   {
	 parser.parse(this);
   }
   catch (Exception e)
   {
	 throw new SQLException("Syntax Error. " + e.getMessage());
   }
	 if(parser.sqlType.equals(I18nSqlParser.SELECT))
		   throw new SQLException("Not supported SELECT statement - use executeQuery method");
	 else if (parser.sqlType.equals(I18nSqlParser.CREATE_TABLE)) {
			   throw new SQLException("Not supported CREATE TABLE statement - use execute method");
	 }
	 else if (parser.sqlType.equals(I18nSqlParser.INSERT)) {
		  String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
		   File checkFile = new File(fileName);

		   if (!checkFile.exists())
		   {
		   throw new SQLException("Cannot open data file '" + fileName + "'  !");
		   }

		   if (!checkFile.canWrite())
		   {
		   throw new SQLException("Data file '" + fileName + "'  is read only !");
		   }
			 try
			 {
			   String[] xxx = parser.getColumnNames();
							   String[] yyy = connection.getColumnNames();
			   boolean isOK = true;
			   for(int i=0; i< xxx.length; i++) {
				 if(!xxx[i].endsWith("*")) {
				 out:
				 for(int j=0; j< yyy.length; j++) {
				   if(xxx[i].equalsIgnoreCase(yyy[j])) {
					 isOK=true;
					 break out;
				   }
				 else
				   isOK=false;
				 }
			   if(!isOK)
				 throw new SQLException("Column '" + xxx[i] + "' not found.");
			   }
			 }
				try {
						   String[] colValues = parser.columnValues;
						   String[] colNames = parser.columnNames;
						   String keyColumn=connection.getNameColumn();
						   String key;
						   String value;
							 if (colNames.length!=colValues.length){
										throw new Exception("Number of columns and number of values are different!");
							 }			
						   if (colNames[0].equalsIgnoreCase(keyColumn)){
							   key = colValues[0];
							   value = colValues[1];
						   }else{
							 key = colValues[1];
							 value = colValues[0];
						   }
						   this.properties.load(new FileInputStream(checkFile));
						   if (properties.containsKey(key)==true){
								   throw new Exception("Key allready exist in the property file. Key must be unique!");
						   }else{
							   this.properties.put(key,value);
							   this.properties.store(checkFile);
								 updated++;
						   }
							
					   } catch (Exception e) {
						  throw new SQLException("Error writing data to property file."+e.getMessage());
					   }
		   }
		   catch (Exception e)
		   {
		   throw new SQLException("Error writing data file. Message was: " + e);
		   }

	 }
	else if (parser.sqlType.equals(I18nSqlParser.UPDATE)) {

			   String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
		   File checkFile = new File(fileName);

		   if (!checkFile.exists())
		   {
		   throw new SQLException("Cannot open data file '" + fileName + "'  !");
		   }

		   if (!checkFile.canWrite())
		   {
		   throw new SQLException("Data file '" + fileName + "'  is read only !");
		   }

		   try
		   {
			   String[] xxx = parser.getColumnNames();
			   String[] yyy = connection.getColumnNames();
			   boolean isOK = true;
			   for(int i=0; i< xxx.length; i++) {
				 if(!xxx[i].endsWith("*")) {
				   out:
				 for(int j=0; j< yyy.length; j++) {
				   if(xxx[i].equalsIgnoreCase(yyy[j])) {
					 isOK=true;
					 break out;
				   }
				 else
				   isOK=false;
			   }
			   if(!isOK)
				 throw new SQLException("Column '" + xxx[i] + "' not found.");
				 }
			   }
               
					   try {
							  String[] colValues = parser.columnValues;
							  String[] colNames = parser.columnNames;
							  String[] colWhereNames = parser.columnWhereNames;
							  String[] colWhereValues = parser.columnWhereValues;
							  String keyColumn=connection.getNameColumn();
								String valueColumn=connection.getValueColumn();
							  String key = "";
							  String value = "";
							  boolean paramsOK = true;
							  //pick up values which will be updated.This block is used if firs column is 
							  // key column
								  if (colNames[0].equalsIgnoreCase(keyColumn)){
									  if (colValues.length==1){
									   key = colValues[0];
									  }else if (colValues.length==2){
										   key = colValues[0];
										   value = colValues[1];
									  }
							   //pick up values which will be updated.This block is used if first column is 
							   //value column
								  }else if (colNames[0].equalsIgnoreCase(valueColumn)){
										   if (colValues.length==1){
											   value = colValues[0];
										   }else if (colValues.length==2){
											   value = colValues[0];
											   key = colValues[1];
										   }
								  }
								 
								  this.properties.load(new FileInputStream(checkFile));
									if (!(colWhereNames[0].equalsIgnoreCase(keyColumn) || colWhereNames[0].equalsIgnoreCase(valueColumn))){
											throw new SQLException("Error in sql statement. Wrong column name!");
									}
								   if ((!key.equalsIgnoreCase("")) && (!value.equalsIgnoreCase(""))){
									   Enumeration enum = this.properties.keys();
										   while(enum.hasMoreElements()){
												   String oldKey = enum.nextElement().toString();
												   String oldValue = this.properties.getProperty(oldKey);
												   //update key and value if this key is equal with key from where claus and also column name from where claus is equal with key column
												   //or update key and value if this value is equal with value from where claus and also column name from where claus is equal with value column
												   if ((colWhereValues[0].equalsIgnoreCase(oldKey) && keyColumn.equals(colWhereNames[0]))
														   ||(colWhereValues[0].equalsIgnoreCase(oldValue) && valueColumn.equals(colWhereNames[0]))){							
													   this.properties.remove(oldKey);
													   this.properties.put(key, value);
													   updated++;
											   }	
										   }
								    	
									   }else if (!value.equalsIgnoreCase("")){
							   	
										   Enumeration enum = this.properties.keys();
											   while(enum.hasMoreElements()){
												   String oldKey = enum.nextElement().toString();
												   String oldValue = this.properties.getProperty(oldKey); 
												   //update value if this key is equal with key from where claus and also column name from where claus is equal with key column
												   //or update value if this value is equal with value from where claus and also column name from where claus is equal with value column                                             
													   if ((colWhereValues[0].equalsIgnoreCase(oldKey) && keyColumn.equals(colWhereNames[0]))
															||(colWhereValues[0].equalsIgnoreCase(oldValue) && valueColumn.equals(colWhereNames[0]))){
														   this.properties.put(oldKey,value);
														   updated++;
													   }
											   } 
								   }else if (!key.equalsIgnoreCase("")){
										
											   Enumeration enum = this.properties.keys();
											   while(enum.hasMoreElements()){
												   String oldKey = enum.nextElement().toString();
												   String oldValue = this.properties.getProperty(oldKey);
												   //update key if this key is equal with key from where claus and also column name from where claus is equal with key column
												   //or update key if this value is equal with value from where claus and also column name from where claus is equal with value column
												   if ((colWhereValues[0].equalsIgnoreCase(oldKey) && keyColumn.equals(colWhereNames[0]))
															   ||(colWhereValues[0].equalsIgnoreCase(oldValue) && valueColumn.equals(colWhereNames[0]))){
													   this.properties.remove(oldKey);
													   this.properties.put(key, oldValue);
													   updated++;
												   }	
											   } 
							
								   }
							  
						  this.properties.store(checkFile);
						   //}
						  } catch (Exception e) {
							 throw new SQLException("Error writing data to property file."+e.getMessage());
						  }
									
		   }
		   catch (Exception e)
		   {
		   throw new SQLException("Error writing data file. Message was: " + e);
		   }

	   }else if (parser.sqlType.equals(I18nSqlParser.DELETE)){
					String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
					File checkFile = new File(fileName);
			
					if (!checkFile.exists())
					{
					throw new SQLException("Cannot open data file '" + fileName + "'  !");
					}
			
					if (!checkFile.canWrite())
					{
					throw new SQLException("Data file '" + fileName + "'  is read only !");
					}
					   try
						   {
							 
////						   try {
							   boolean deleteAll=false;
							   String[] colWhereNames = parser.columnWhereNames;
							   String[] colWhereValues = parser.columnWhereValues;
							 	 String keyColumn=connection.getNameColumn();
							   String valueColumn=connection.getValueColumn();
							   this.properties.load(new FileInputStream(checkFile));
								 if (!(colWhereNames[0].equalsIgnoreCase(keyColumn) || colWhereNames[0].equalsIgnoreCase(valueColumn))){
											throw new SQLException("Error in sql statement. Wrong column name!");
								 }
							   Enumeration enum = this.properties.keys();
							   if (colWhereNames.length==0){
								   deleteAll=true;
							   }
							   if  (!deleteAll){
								   while(enum.hasMoreElements()){
									   String oldKey = enum.nextElement().toString();
									   String oldValue = this.properties.getProperty(oldKey);
											                
											   if ((colWhereValues[0].equalsIgnoreCase(oldKey) && keyColumn.equals(colWhereNames[0]))
													||(colWhereValues[0].equalsIgnoreCase(oldValue) && valueColumn.equals(colWhereNames[0]))){
												   this.properties.remove(oldKey);
												   this.properties.store(checkFile);
												   updated++;				
											   }
											
								   }
							   }else{
									   this.properties.clear();
									   this.properties.store(checkFile);	
							   } 
						  } catch (Exception e) {
							 throw new SQLException("Error deleting data from property file."+e.getMessage());
					   }

	   }
	 return updated;

 }


    /**
     * Releases this <code>Statement</code> object's database
     * and JDBC resources immediately instead of waiting for
     * this to happen when it is automatically closed.
     * It is generally good practice to release resources as soon as
     * you are finished with them to avoid tying up database
     * resources.
     * <P>
     * Calling the method <code>close</code> on a <code>Statement</code>
     * object that is already closed has no effect.
     * <P>
     * <B>Note:</B> A <code>Statement</code> object is automatically closed
     * when it is garbage collected. When a <code>Statement</code> object is
     * closed, its current <code>ResultSet</code> object, if one exists, is
     * also closed.
     *
     * @exception SQLException if a database access error occurs
     */
    public void close() throws SQLException {
      // close all result sets
      for(Enumeration i = resultSets.elements(); i.hasMoreElements(); ) {
        I18nResultSet resultSet = (I18nResultSet) i.nextElement();
        resultSet.close();
      }
     
    }

    /**
     *Description of the Method
     *
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void cancel() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Description of the Method
     *
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void clearWarnings() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Description of the Method
     *
     * @param  sql               Description of Parameter
     * @return                   Description of the Returned Value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public boolean execute(String sql) throws SQLException
  {
    I18nSqlParser parser = new I18nSqlParser();
    if( binaryStreamParameters.size() != 0 )
          parser.setBinaryStreamList( binaryStreamParameters );
    this.sql = sql;
    try
    {
      parser.parse(this);
    }
    catch (Exception e)
    {
      throw new SQLException("Syntax Error. " + e.getMessage());
    }
      if(parser.sqlType.equals(I18nSqlParser.SELECT))
              throw new SQLException("Not supported SELECT statement - use executeQuery method");
      else if (parser.sqlType.equals(I18nSqlParser.INSERT))
              executeUpdate(sql);
       else if (parser.sqlType.equals(I18nSqlParser.CREATE_TABLE)) {
             String fileName = connection.getPath() + parser.getTableName() + connection.getExtension();
            File checkFile = new File(fileName);

            if (checkFile.exists())
            {
              throw new SQLException("Data file '" + fileName + "'already exists  !");
            }
           
            try
            {
							checkFile.createNewFile();
            
            }
            catch (Exception e)
            {
              throw new SQLException("Error reading data file. Message was: " + e);
            }
        }
        return true;

  }

  /**
   * Set String as parameter in sql statement.
   * @param parameterIndex
   * @param value
   * @throws SQLException
   */
  public void setString(int parameterIndex, String value) throws SQLException {
    if( value == null ) {
      I18nDriver.log("value = null object");
      parameters.set(parameterIndex - 1, value);
    } else {
      value = Utils.replaceAll(value, "'", I18nSqlParser.QUOTE_ESCAPE);
	  	I18nDriver.log("value = " + value);
      parameters.set(parameterIndex - 1, "'" + value + "'");
    }
  }


  /**
   *
   * @param parameterIndex
   * @param value
   * @throws SQLException
   */
  public void setBytes(int parameterIndex, byte[] value) throws SQLException {
    binaryStreamParameters.add(Utils.bytesToHexString(value));
    String paramName = I18nSqlParser.BINARY_STREAM_OBJECT+""+binaryStreamParameters.size();
    parameters.set(parameterIndex-1, paramName);
  }

    /**
     *Adds a feature to the Batch attribute of the I18nStatement object
     *
     * @param  p0                The feature to be added to the Batch attribute
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void addBatch(String p0) throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Description of the Method
     *
     * @exception  SQLException  Description of Exception
     * @since
     */
    public void clearBatch() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }


    /**
     *Description of the Method
     *
     * @return                   Description of the Returned Value
     * @exception  SQLException  Description of Exception
     * @since
     */
    public int[] executeBatch() throws SQLException
    {
        throw new SQLException("Not Supported !");
    }

    //---------------------------------------------------------------------
    // JDBC 3.0
    //---------------------------------------------------------------------

    public boolean getMoreResults(int current) throws SQLException {
        throw new UnsupportedOperationException("Statement.getMoreResults(int) unsupported");
    }

    public ResultSet getGeneratedKeys() throws SQLException {
        throw new UnsupportedOperationException("Statement.getGeneratedKeys() unsupported");
    }

    public int executeUpdate(String sql, int autoGeneratedKeys) throws SQLException {
        throw new UnsupportedOperationException("Statement.executeUpdate(String,int) unsupported");
    }

    public int executeUpdate(String sql, int[] columnIndexes) throws SQLException {
        throw new UnsupportedOperationException("Statement.executeUpdate(String,int[]) unsupported");
    }

    public int executeUpdate(String sql, String[] columnNames) throws SQLException {
        throw new UnsupportedOperationException("Statement.executeUpdate(String,String[]) unsupported");
    }

    public boolean execute(String sql, int autoGeneratedKeys) throws SQLException {
        throw new UnsupportedOperationException("Statement.execute(String,int) unsupported");
    }

    public boolean execute(String sql, int[] columnIndexes) throws SQLException {
        throw new UnsupportedOperationException("Statement.execute(String,int[]) unsupported");
    }

    public boolean execute(String sql, String[] columnNames) throws SQLException {
        throw new UnsupportedOperationException("Statement.execute(String,String[]) unsupported");
    }

    public int getResultSetHoldability() throws SQLException {
        throw new UnsupportedOperationException("Statement.getResultSetHoldability() unsupported");
    }



  public ResultSet executeQuery() throws SQLException {
    if( !prepareSql())
        throw new SQLException("Error with prepared statement  !");
    return executeQuery(this.sqlPrepared);

  }
  public int executeUpdate() throws SQLException {
    if( !prepareSql())
        throw new SQLException("Error with prepared statement  !");
    executeUpdate(this.sqlPrepared);
    return 0;
  }





  public void setNull(int parameterIndex, int sqlType) throws SQLException {
    this.setString(parameterIndex, null );
  }

  public void setBoolean(int parameterIndex, boolean value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setByte(int parameterIndex, byte value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setShort(int parameterIndex, short value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setInt(int parameterIndex, int value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setLong(int parameterIndex, long value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setFloat(int parameterIndex, float value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setDouble(int parameterIndex, double value) throws SQLException {
    this.setString(parameterIndex, String.valueOf(value) );
  }

  public void setBigDecimal(int parameterIndex, BigDecimal value) throws SQLException {
    this.setString( parameterIndex, value.toString() );
  }
  public void setDate(int parameterIndex, Date x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setDate() not yet implemented.");
  }
  public void setTime(int parameterIndex, Time x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setTime() not yet implemented.");
  }
  public void setTimestamp(int parameterIndex, Timestamp x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setTimestamp() not yet implemented.");
  }
  public void setAsciiStream(int parameterIndex, InputStream x, int length) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setAsciiStream() not yet implemented.");
  }
  public void setUnicodeStream(int parameterIndex, InputStream x, int length) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setUnicodeStream() not yet implemented.");
  }

  public void clearParameters() throws SQLException {
    this.sqlPrepared = this.sqlForPrepare;
    for(int i = 0; i < parameters.size(); i++)
      parameters.set(i, null);
  }
  public void setObject(int parameterIndex, Object x, int targetSqlType, int scale) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setObject() not yet implemented.");
  }
  public void setObject(int parameterIndex, Object x, int targetSqlType) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setObject() not yet implemented.");
  }
  public void setObject(int parameterIndex, Object x) throws SQLException {
    if( x == null )
       x = new String("null");
     setString( parameterIndex, x.toString() );
  }
  public boolean execute() throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method execute() not yet implemented.");
  }
  public void addBatch() throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method addBatch() not yet implemented.");
  }
  public void setCharacterStream(int parameterIndex, Reader reader, int length) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setCharacterStream() not yet implemented.");
  }
  public void setRef(int i, Ref x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setRef() not yet implemented.");
  }

  public void setClob(int i, Clob x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setClob() not yet implemented.");
  }
  public void setArray(int i, Array x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setArray() not yet implemented.");
  }
  public ResultSetMetaData getMetaData() throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method getMetaData() not yet implemented.");
  }
  public void setDate(int parameterIndex, Date x, Calendar cal) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setDate() not yet implemented.");
  }
  public void setTime(int parameterIndex, Time x, Calendar cal) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setTime() not yet implemented.");
  }
  public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException("Method setTimestamp() not yet implemented.");
  }
  public void setNull(int paramIndex, int sqlType, String typeName) throws SQLException {
    this.setString(paramIndex, null );
  }

  public void setURL(int parameterIndex, URL x) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException(
        "Method setURL() not yet implemented.");
  }

  public ParameterMetaData getParameterMetaData() throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException(
        "Method getParameterMetaData() not yet implemented.");
  }

  public void setBinaryStream(int parameterIndex, InputStream value, int length) throws
      SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException(
        "Method setBinaryStream() not yet implemented.");
  }

  public void setBlob(int parameterIndex, Blob value) throws SQLException {
    /**@todo Implement this java.sql.PreparedStatement method*/
    throw new java.lang.UnsupportedOperationException(
        "Method setBlob() not yet implemented.");
  }

  public String getSqlStatement() {
      return sql;
    }
    
	public I18nProperties getProperties() {
		   return properties;
	   }

}